@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -52,13 +52,13 @@ add_commit_string(char *comstr, const char *str)
 				 "add_commit_string comstr: len:%d (%p)\n", 
 				 buflen + strlen(str) + 1, comstr); 
 
-	comstr = (char *)realloc(comstr, buflen + strlen(str) + 1);
+	comstr = uim_realloc(comstr, buflen + strlen(str) + 1);
 	comstr[buflen] = '\0';
 
 	debug_printf(DEBUG_NOTE,
 				 "add_commit_string comstr: %s (%p)\n", comstr, comstr); 
 
-	strcat(comstr, str);
+	strlcat(comstr, str, buflen + strlen(str) + 1);
   }
 
   debug_printf(DEBUG_NOTE,
