@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,11 +34,15 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef UIM_AGENT_COMMIT_H
-#define UIM_AGENT_COMMIT_H
+#ifndef UIM_EMACS_COMMIT_H
+#define UIM_EMACS_COMMIT_H
+
+#include <config.h>
 
 #include <stdlib.h>
 
+#include <uim/uim.h>
+
 #include "output.h"
 
 char *add_commit_string(char *comstr, const char *str);
