@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -69,12 +69,8 @@ update_context_encoding(uim_agent_context *ua)
   /* discard current context */
   clear_candidate(ua->cand);
   clear_preedit(ua->pe);
-  uim_release_context(ua->context);
-  
-  ua->context = create_context(ua->encoding, ua);
-
-  update_context_im(ua);
   
+  uim_set_client_encoding(ua->context, ua->encoding);
 }
 
 
@@ -106,10 +102,10 @@ switch_context_im(uim_agent_context *ua, const char *im)
   encoding = get_im_encoding(im);
 
   /* update IM name */
-  if (ua->im) free(ua->im);
+  free(ua->im);
 
   if (im)
-	ua->im = strdup(im);
+	ua->im = uim_strdup(im);
   else
 	ua->im = NULL;
 
@@ -128,11 +124,11 @@ switch_context_im(uim_agent_context *ua, const char *im)
 	debug_printf(DEBUG_NOTE, 
 				 "different encoding %s %s\n", ua->encoding, encoding);
 
-	if (ua->encoding) free(ua->encoding);
-	ua->encoding = strdup(encoding);
+	free(ua->encoding);
+	ua->encoding = uim_strdup(encoding);
 
 	update_context_encoding(ua);
-
+	update_context_im(ua);
   }
 
   uim_prop_list_update(ua->context);
@@ -152,7 +148,7 @@ switch_context_im_all(const char *im)
   update_default_engine(im);
 
   /* check focus state when change IM of current application */
-  quot_im_name = (char *)malloc(strlen(im) + 2);
+  quot_im_name = uim_malloc(strlen(im) + 2);
   quot_im_name[0] = '\'';
   quot_im_name[1] = '\0';
   strcat(quot_im_name, im);
@@ -221,21 +217,21 @@ create_uim_agent_context(const char *encoding)
 
   debug_printf(DEBUG_NOTE, "create_uim_agent_context\n");
 
-  ret = (uim_agent_context *)malloc(sizeof(uim_agent_context));
+  ret = uim_malloc(sizeof(uim_agent_context));
 
   if (encoding) {
-	ret->encoding = strdup(encoding);
+	ret->encoding = uim_strdup(encoding);
   } else {
 	if (debug_level > 0)
-	  ret->encoding = strdup("EUC-JP");
+	  ret->encoding = uim_strdup("EUC-JP");
 	else
-	  ret->encoding = strdup("UTF-8");
+	  ret->encoding = uim_strdup("UTF-8");
   }
 
   ret->context = create_context(ret->encoding, ret);
 
   if ((im = uim_get_default_im_name(setlocale(LC_ALL, NULL))))
-	ret->im = strdup(im);
+	ret->im = uim_strdup(im);
   else
 	ret->im = NULL;
 
@@ -259,7 +255,7 @@ new_uim_agent_context(int id, const char *encoding)
   
   debug_printf(DEBUG_NOTE, "add_uim_agent_context(%d)\n", id);
 
-  ptr = (uim_agent_context_list *)malloc(sizeof(uim_agent_context_list));
+  ptr = uim_malloc(sizeof(uim_agent_context_list));
 
   ptr->agent_context = create_uim_agent_context(encoding);
   ptr->next = NULL;
@@ -310,6 +306,7 @@ release_uim_agent_context(int context_id)
 	  /* free others */
 	  free(ua->encoding);
 	  free(ua->im);
+	  free(ua->prop->list);
 	  free(ua->prop);
 	  free(ua->comstr);
 
@@ -324,6 +321,7 @@ release_uim_agent_context(int context_id)
 	  else
 		agent_context_list_head = ptr->next;
 
+	  free(ua);
 	  free(ptr);
 
 	  return context_id;
@@ -386,13 +384,13 @@ update_context_configuration(uim_agent_context *ua)
   debug_printf(DEBUG_NOTE, "update_context_configuration\n");
   
   /* update IM name */
-  if (ua->im) free(ua->im);
-  ua->im = strdup(uim_get_current_im_name(ua->context));
+  free(ua->im);
+  ua->im = uim_strdup(uim_get_current_im_name(ua->context));
 
   debug_printf(DEBUG_NOTE, "ua->im %s\n", ua->im);
 
-  if (ua->encoding) free(ua->encoding);
-  ua->encoding = strdup(get_im_encoding(ua->im));
+  free(ua->encoding);
+  ua->encoding = uim_strdup(get_im_encoding(ua->im));
 
   debug_printf(DEBUG_NOTE, "ua->encoding %s\n", ua->encoding);
 
