@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,8 +35,10 @@
 */
 
 
-#ifndef UIM_AGENT_DEBUG_H
-#define UIM_AGENT_DEBUG_H
+#ifndef UIM_EMACS_DEBUG_H
+#define UIM_EMACS_DEBUG_H
+
+#include <config.h>
 
 #include <stdio.h>
 #include <stdarg.h>
