@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -64,14 +64,14 @@ search_im_encoding(const char *im)
 im_encoding *
 new_im_encoding(const char *im, const char *encoding)
 {
-  im_encoding *im_enc = (im_encoding *)malloc(sizeof(im_encoding));
+  im_encoding *im_enc = uim_malloc(sizeof(im_encoding));
 
   debug_printf(DEBUG_NOTE, "new_im_encoding\n");
 
-  im_enc->im = strdup(im);
+  im_enc->im = uim_strdup(im);
 
   if (encoding != NULL)
-	im_enc->encoding = strdup( encoding );
+	im_enc->encoding = uim_strdup( encoding );
   else
 	im_enc->encoding = NULL;
 
@@ -104,10 +104,10 @@ set_im_encoding(const char *im, const char *encoding)
   im_enc = search_im_encoding(im);
 
   if (im_enc != NULL) {
-	if (im_enc->encoding) free(im_enc->encoding);
+	free(im_enc->encoding);
 
 	if (encoding)
-	  im_enc->encoding = strdup(encoding);
+	  im_enc->encoding = uim_strdup(encoding);
 	else
 	  im_enc->encoding = NULL;
 
