@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -45,8 +45,8 @@ int default_engine_updated = 0;
 void
 update_default_engine(const char *engine_name)
 {
-  if (default_engine_name) free(default_engine_name);
-  default_engine_name = strdup(engine_name);
+  free(default_engine_name);
+  default_engine_name = uim_strdup(engine_name);
 
   default_engine_updated = 1;
 }
