@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,10 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef UIM_AGENT_IM_H
-#define UIM_AGENT_IM_H
+#ifndef UIM_EMACS_IM_H
+#define UIM_EMACS_IM_H
+
+#include <config.h>
 
 #include <string.h>
 #include <locale.h>
