@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,8 +34,10 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef UIM_AGENT_KEY_H
-#define UIM_AGENT_KEY_H
+#ifndef UIM_EMACS_KEY_H
+#define UIM_EMACS_KEY_H
+
+#include <config.h>
 
 #include <string.h>
 
@@ -43,7 +45,7 @@
 
 #include "uim-el-types.h"
 
-void convert_keyname_a2e(char *keyname, const char *name);
+void convert_keyname_a2e(char *keyname, const char *name, size_t len);
 enum UKey convert_keyname_e2u( const char *keyname );
 
 #endif
