@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -40,7 +40,7 @@ preedit *
 create_preedit()
 {
   preedit *pe;
-  pe = (preedit *)malloc(sizeof(preedit));
+  pe = uim_malloc(sizeof(preedit));
   pe->valid = 0;
   pe->head = pe->tail = NULL;
 
@@ -54,7 +54,7 @@ add_preedit(preedit *pe, int attr, const char *str)
 
   pe->valid = 1;
 
-  pb = (preedit_buffer *)malloc(sizeof(preedit_buffer));
+  pb = uim_malloc(sizeof(preedit_buffer));
 
   if (pe->head == NULL) {
 	pe->head = pb;
@@ -65,8 +65,7 @@ add_preedit(preedit *pe, int attr, const char *str)
   }
 
   if (strlen(str) > 0) {
-	pb->str = (char *)malloc(strlen(str) + 1);
-	strcpy(pb->str, str);
+	pb->str = uim_strdup(str);
 	pe->length += strlen(str);
   } else {
 	pb->str = NULL;
