@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,7 +34,7 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -49,8 +49,6 @@
 
 #include "uim-el-agent.h"
 
-
-
 /* called when owner buffer is killed  */
 static int
 cmd_release(int context_id)
@@ -420,7 +418,7 @@ process_command(int serial, int cid, char *cmd)
 
 
 static int
-analyze_keyvector(char *vector, uim_key *ukey, char *keyname)
+analyze_keyvector(char *vector, uim_key *ukey, char *keyname, size_t keyname_len)
 {
   char *p1, *p2, *c;
   int key;
@@ -523,9 +521,9 @@ analyze_keyvector(char *vector, uim_key *ukey, char *keyname)
 			if (*p1 >= 'A' && *p1 <= 'Z')
 			  /* if the first character is upper case, 
 				 it can be considered as XEmacs style abbrev */
-			  convert_keyname_a2e(keyname, p1);
+				convert_keyname_a2e(keyname, p1, keyname_len);
 			else
-			  strcpy(keyname, p1);
+			strlcpy(keyname, p1, keyname_len);
 
 			ukey->key = convert_keyname_e2u(keyname);
 		  }
@@ -660,7 +658,14 @@ main(int argc, char *argv[])
 
 	fflush(stdout);
 
-	fgets(buf, sizeof(buf), stdin);
+	if (fgets(buf, sizeof (buf), stdin) == NULL) {
+	  if (feof(stdin))
+	    debug_printf(DEBUG_NOTE, "unexpected EOF\n");
+	  else
+	    debug_printf(DEBUG_ERROR, "failed to read command: %s\n",
+			 strerror (errno));
+	  goto QUIT;
+	}
 
 	p1 = buf;
 	serial = -1;
@@ -721,7 +726,7 @@ main(int argc, char *argv[])
 	  keyname[0] = '\0';
 
 
-	  if (analyze_keyvector(p1, &ukey, keyname) > 0) {
+	  if (analyze_keyvector(p1, &ukey, keyname, sizeof(keyname)) > 0) {
 
 	  	a_printf("( %d %d ", serial, cid);
 		if (process_keyvector(serial, cid, ukey, keyname) < 0)
