@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2006-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,6 +35,7 @@
 */
 
 
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -197,7 +198,10 @@ process_message(char *msg)
 }
 
 
-static void
+/**
+ * @return 1 if success, 0 if error.
+ */
+static int
 read_command()
 {
   ssize_t len;
@@ -206,14 +210,21 @@ read_command()
   debug_printf(DEBUG_NOTE, "read command\n");
 
   do {
-	if ((len = read(STDIN_FILENO, rbuf, sizeof(rbuf) - 1)) == -1)
-	  debug_printf(DEBUG_NOTE, "stdin has corrupted\n");
+	len = read(STDIN_FILENO, rbuf, sizeof(rbuf) - 1);
+	if (len == -1) {
+	  debug_printf(DEBUG_NOTE, "stdin is corrupt: %s\n", strerror (errno));
+	  return 0;
+	}
+	if (len == 0) {
+	  debug_printf(DEBUG_NOTE, "unexpected EOF\n");
+	  return 0;
+	}
 
 	rbuf[len] = '\0';
 
 	if (strlen(cmdbuf) + len + 1 > cmdbuf_len) {
 	  cmdbuf_len += DEFAULT_MESSAGE_SIZE;
-	  cmdbuf = (char *)realloc(cmdbuf, cmdbuf_len);
+	  cmdbuf = uim_realloc(cmdbuf, cmdbuf_len);
 	  debug_printf(DEBUG_NOTE, "cmdbuf has extended\n");
 	}
 
@@ -221,6 +232,7 @@ read_command()
 
   } while (!command_exists_in_cmdbuf());
 
+  return 1;
 }
 
 
@@ -276,7 +288,7 @@ main(int argc, char *argv[])
   a_printf("OK\n");
 
   cmdbuf_len = DEFAULT_MESSAGE_SIZE;
-  cmdbuf = (char *)malloc(cmdbuf_len);
+  cmdbuf = uim_malloc(cmdbuf_len);
   cmdbuf[0] = '\0';
 
   while (1) {
@@ -290,7 +302,8 @@ main(int argc, char *argv[])
 	debug_printf(DEBUG_NOTE, "data arrive\n");
 
 	if (FD_ISSET(STDIN_FILENO, &rfds)) {
-	  read_command();
+	  if (!read_command())
+	    goto QUIT;
 	}
 
 	if (FD_ISSET(helper_fd, &rfds)) {
@@ -307,6 +320,7 @@ main(int argc, char *argv[])
 	fflush(NULL);
   }
 
+ QUIT:
   uim_quit();
   return 0;
 }
