@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
+;;  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -612,8 +612,15 @@
 	      ;;(setq translated-vector input-vector)
 	      (setq translated-vector nil)
 	      (throw 'fmap-loop t))
+
+	  (if (boundp 'input-decode-map)
+	      (setq translated (lookup-key input-decode-map
+					   input-vector-main)))
 	  
-	  (setq translated (lookup-key function-key-map input-vector-main))
+	  (if (or (not translated)
+		  (integerp translated))
+	      (setq translated (lookup-key function-key-map
+					   input-vector-main)))
 
 	  (if (and (or (not translated)
 		       (integerp translated))
