@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2008 uim Project http://code.google.com/p/uim/
+;;  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -179,34 +179,68 @@
   
   )
 
+;; 
+;; Disable other minor-mode keymaps while preedit strings or candidates 
+;; are displayed.
+;;
+(defun uim-disable-other-minor-mode-map ()
+  (when (not uim-minor-mode-map-alist)
+    (setq uim-minor-mode-map-alist minor-mode-map-alist)
+    (setq minor-mode-map-alist (list (assq 'uim-mode minor-mode-map-alist)))
+
+    (when (and uim-emacs (>= emacs-major-version 22)) 
+      (setq uim-emulation-mode-map-alists emulation-mode-map-alists)
+      (setq emulation-mode-map-alists nil)))
+  )
+
+(defun uim-enable-other-minor-mode-map ()
+  (when uim-minor-mode-map-alist
+    (setq minor-mode-map-alist uim-minor-mode-map-alist)
+    (setq uim-minor-mode-map-alist nil)
+
+    (when (and uim-emacs (>= emacs-major-version 22)) 
+      (setq emulation-mode-map-alists uim-emulation-mode-map-alists)
+      (setq uim-emulation-mode-map-alists nil)))
+  )
+
 
 ;;
-;; Switch keymap
+;; Change keymap of uim-mode
+;;
+(defun uim-set-keymap (map)
+  (setcdr (assq 'uim-mode minor-mode-map-alist) 
+	  map))
+
+;;
+;; Return current keymap of uim-mode
+;;
+(defun uim-get-keymap ()
+  (cdr (assq 'uim-mode minor-mode-map-alist)))
+
+;;
+;; Set normal keymap
 ;;
 (defun uim-enable-mode-keymap ()
-  (setcdr (assq 'uim-mode minor-mode-map-alist) 
-	  uim-mode-map))
+  (uim-set-keymap uim-mode-map))
 
-(defun uim-disable-mode-keymap ()
-  (setcdr (assq 'uim-mode minor-mode-map-alist) 
-	  nil))
+;;
+;; Disable and return current keymap
+;;
+(defun uim-disable-keymap ()
+  (let (map)
+    (setq map (uim-get-keymap))
+    (uim-set-keymap uim-dummy-map)
+    map)
+  )
 
+;;
+;; Set preedit keymap
+;;
 (defun uim-enable-preedit-keymap ()
   (when (not uim-preedit-keymap-enabled)
+    (setq uim-preedit-keymap-enabled t))
 
-    (setq uim-preedit-keymap-enabled t)
-
-    (setcdr (assq 'uim-mode minor-mode-map-alist) 
-	    uim-preedit-map) 
-
-    ;; disable other keymaps
-    (when (not uim-minor-mode-map-alist)
-      (setq uim-minor-mode-map-alist minor-mode-map-alist)
-      (setq minor-mode-map-alist (list (assq 'uim-mode minor-mode-map-alist)))
-
-      (when (and uim-emacs (>= emacs-major-version 22)) 
-	(setq uim-emulation-mode-map-alists emulation-mode-map-alists)
-	(setq emulation-mode-map-alists nil)))
+  (uim-set-keymap uim-preedit-map)
 
     (when uim-xemacs
       ;; disable toolbar and menubar
@@ -214,29 +248,22 @@
       (setq toolbar-map uim-toolbar-map)
       (uim-xemacs-save-menubar))
     )
-  )
 
+;;
+;; Unset preedit keymap
+;;
 (defun uim-disable-preedit-keymap ()
+
   (when uim-preedit-keymap-enabled
-    (setq uim-preedit-keymap-enabled nil)
-
-    ;; enable other keymaps
-    (when uim-minor-mode-map-alist
-      (setq minor-mode-map-alist uim-minor-mode-map-alist)
-      (setq uim-minor-mode-map-alist nil)
-
-      (when (and uim-emacs (>= emacs-major-version 22)) 
-	(setq emulation-mode-map-alists uim-emulation-mode-map-alists)
-	(setq uim-emulation-mode-map-alists nil)))
+    (setq uim-preedit-keymap-enabled nil))
 
+  (uim-enable-other-minor-mode-map)
     (uim-enable-mode-keymap)
 
     (when uim-xemacs
       ;;enable toolbar and menubar
       (kill-local-variable 'toolbar-map)
       (uim-xemacs-restore-menubar))
-
-    )
   )
 
 
