@@ -1,5 +1,5 @@
 ;; 
-;;  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+;;  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 ;;
 ;;  All rights reserved.
 ;;
@@ -109,8 +109,9 @@
   )
 
 (defun uim-leim-input-method-activate-hook ()
-  (setq current-input-method-title uim-leim-mode-line-string)
+  ;; Don't set input-method-title here.
 
+  ;; However, leave code fore xemacs since I cannot test with xemacs for now.
   (if uim-xemacs
       (uim-leim-xemacs-modeline-init))
   )
