@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -85,6 +85,7 @@ static void update_cb(void *ptr);
 static void mode_update_cb(void *ptr, int mode);
 static void prop_list_update_cb(void *ptr, const char *str);
 static struct preedit_tag *dup_preedit(struct preedit_tag *p);
+static void init_candidate(int nr, int display_limit);
 static void make_page_strs(void);
 static int numwidth(int n);
 static int index2page(int index);
@@ -133,13 +134,13 @@ void init_callbacks(void)
   if (g_opt.statusline_width != UNDEFINED && g_opt.statusline_width <= s_max_width) {
     s_max_width = g_opt.statusline_width;
   }
-  s_commit_str = strdup("");
-  s_candidate_str = strdup("");
-  s_statusline_str = strdup("");
+  s_commit_str = uim_strdup("");
+  s_candidate_str = uim_strdup("");
+  s_statusline_str = uim_strdup("");
   s_candidate_col = UNDEFINED;
-  s_index_str = strdup("");
+  s_index_str = uim_strdup("");
   s_mode = uim_get_current_mode(g_context);
-  s_label_str = strdup("");
+  s_label_str = uim_strdup("");
   s_preedit = create_preedit();
   uim_set_preedit_cb(g_context, clear_cb, pushback_cb, update_cb);
   uim_set_mode_cb(g_context, mode_update_cb);
@@ -219,19 +220,22 @@ int end_callbacks(void)
   free(s_index_str);
 
   if (s_candidate.nr != UNDEFINED) {
-    s_statusline_str = strdup(s_candidate.page_strs[s_candidate.page]);
+    if (s_candidate.page_strs[s_candidate.page])
+      s_statusline_str = uim_strdup(s_candidate.page_strs[s_candidate.page]);
+    else
+      s_statusline_str = uim_strdup("");
     if (s_candidate.index != UNDEFINED) {
       set_candidate();
     } else {
-      s_candidate_str = strdup("");
+      s_candidate_str = uim_strdup("");
       s_candidate_col = UNDEFINED;
-      s_index_str = strdup("");
+      s_index_str = uim_strdup("");
     }
   } else {
-    s_statusline_str = strdup("");
-    s_candidate_str = strdup("");
+    s_statusline_str = uim_strdup("");
+    s_candidate_str = uim_strdup("");
     s_candidate_col = UNDEFINED;
-    s_index_str = strdup("");
+    s_index_str = uim_strdup("");
   }
 
   return TRUE;
@@ -243,12 +247,12 @@ int end_callbacks(void)
  */
 char *get_commit_str(void)
 {
-  char *return_value = strdup(s_commit_str);
+  char *return_value = uim_strdup(s_commit_str);
 
   assert(!s_start_callbacks);
 
   free(s_commit_str);
-  s_commit_str = strdup("");
+  s_commit_str = uim_strdup("");
   return return_value;
 }
 
@@ -259,7 +263,7 @@ char *get_commit_str(void)
 char *get_statusline_str(void)
 {
   assert(!s_start_callbacks);
-  return strdup(s_statusline_str);
+  return uim_strdup(s_statusline_str);
 }
 
 /*
@@ -269,7 +273,7 @@ char *get_statusline_str(void)
 char *get_candidate_str(void)
 {
   assert(!s_start_callbacks);
-  return strdup(s_candidate_str);
+  return uim_strdup(s_candidate_str);
 }
 
 /*
@@ -289,7 +293,7 @@ int get_candidate_col(void)
 char *get_index_str(void)
 {
   assert(!s_start_callbacks);
-  return strdup(s_index_str);
+  return uim_strdup(s_index_str);
 }
 
 /*
@@ -335,8 +339,7 @@ char *get_mode_str(void)
 
   assert(!s_start_callbacks);
 
-  str = malloc(strlen(s_im_str) + strlen(s_label_str) + strlen("[]") + 1);
-  sprintf(str, "%s[%s]", s_im_str, s_label_str);
+  uim_asprintf(&str, "%s[%s]", s_im_str, s_label_str);
   strhead(str, s_max_width);
 
   return str;
@@ -360,19 +363,10 @@ static void switch_app_global_im_cb(void *ptr, const char *name)
 static void switch_system_global_im_cb(void *ptr, const char *name)
 {
   char *buf;
-  int len = 0;
 
-#define HEADER_FORMAT "im_change_whole_desktop\n%s\n"
-
-  len += strlen(HEADER_FORMAT);
-  len += name ? strlen(name) : 0;
-
-  buf = malloc(len);
-  snprintf(buf, len, HEADER_FORMAT, name ? name : "");
+  uim_asprintf(&buf, "im_change_whole_desktop\n%s\n", name ? name : "");
   uim_helper_send_message(g_helper_fd, buf);
   free(buf);
-
-#undef HEADER_FORMAT
 }
 
 /*
@@ -380,18 +374,15 @@ static void switch_system_global_im_cb(void *ptr, const char *name)
  * s_candidate.nr = nr()
  * s_candidate.limit = display_limit(ɽ)
  * s_candidate.cand_colΰݤ롣
- * make_page_strsƤ֡
+ * s_candidate.page = 0 (initial page)
  */
 static void activate_cb(void *ptr, int nr, int display_limit)
 {
   debug2(("activate_cb(nr = %d display_limit = %d)\n", nr, display_limit));
   start_callbacks();
   reset_candidate();
-  s_candidate.nr = nr;
-  s_candidate.limit = display_limit;
-  s_candidate.page = 0;
-  s_candidate.cand_col = malloc(nr * sizeof(int));
-  make_page_strs();
+  init_candidate(nr, display_limit);
+  make_page_strs(); /* setup first page */
 }
 
 /*
@@ -400,15 +391,25 @@ static void activate_cb(void *ptr, int nr, int display_limit)
  */
 static void select_cb(void *ptr, int index)
 {
+  int current_index;
+  int current_page;
+
   debug2(("select_cb(index = %d)\n", index));
   return_if_fail(s_candidate.nr != UNDEFINED);
   return_if_fail(0 <= index && index < s_candidate.nr);
-  if (s_candidate.index == index) {
+
+  current_index = s_candidate.index;
+  current_page = s_candidate.page;
+  if (current_index == index) {
     return;
   }
   start_callbacks();
   s_candidate.index = index;
   s_candidate.page = index2page(index);
+  if (s_candidate.page != current_page &&
+      s_candidate.page_strs[s_candidate.page] == NULL) {
+    make_page_strs();
+  }
 }
 
 /*
@@ -431,6 +432,8 @@ static void shift_page_cb(void *ptr, int direction)
   return_if_fail(0 <= index && index < s_candidate.nr);
   s_candidate.page = page;
   s_candidate.index = index;
+  if (s_candidate.page_strs[page] == NULL)
+    make_page_strs();
   uim_set_candidate_index(g_context, s_candidate.index);
 }
 
@@ -448,22 +451,24 @@ static void deactivate_cb(void *ptr)
 
 void commit_cb(void *ptr, const char *commit_str)
 {
+  char *oldstr;
+
   debug2(("commit_cb(commit_str = \"%s\")\n", commit_str));
   return_if_fail(commit_str != NULL);
   if (strlen(commit_str) == 0) {
     return;
   }
   start_callbacks();
-  s_commit_str = realloc(s_commit_str, strlen(s_commit_str) + strlen(commit_str) + 1);
-  strcat(s_commit_str, commit_str);
+
+  oldstr = s_commit_str;
+  uim_asprintf(&s_commit_str, "%s%s", oldstr, commit_str);
+  free(oldstr);
 }
 
 static void clear_cb(void *ptr)
 {
   start_callbacks();
-  if (s_preedit != NULL) {
-    free_preedit(s_preedit);
-  }
+  free_preedit(s_preedit);
   s_preedit = create_preedit();
   s_preedit->cursor = UNDEFINED;
   debug2(("clear_cb()\n"));
@@ -504,7 +509,7 @@ static void pushback_cb(void *ptr, int attr, const char *str)
       int *rval = width2byte2(str, 1);
       int first_char_byte = rval[0];
       int first_char_width = rval[1];
-      char *first_char = malloc(first_char_byte + 1);
+      char *first_char = uim_malloc(first_char_byte + 1);
       strlcpy(first_char, str, first_char_byte + 1);
       cursor = FALSE;
       pushback_cb(NULL, attr - UPreeditAttr_Reverse, first_char);
@@ -517,15 +522,14 @@ static void pushback_cb(void *ptr, int attr, const char *str)
     }
     /* attrƱʸդ­ */
     if (s_preedit->nr_psegs > 0 && s_preedit->pseg[s_preedit->nr_psegs - 1].attr == attr) {
-      /* char *tmp_str = s_preedit->pseg[s_preedit->nr_psegs - 1].str; */
-      /* s_preedit->pseg[s_preedit->nr_psegs - 1].str = malloc(strlen(tmp_str) + strlen(str) + 1); */
-      s_preedit->pseg[s_preedit->nr_psegs - 1].str = realloc(s_preedit->pseg[s_preedit->nr_psegs - 1].str, strlen(s_preedit->pseg[s_preedit->nr_psegs - 1].str) + strlen(str) + 1);
-      /* sprintf(s_preedit->pseg[s_preedit->nr_psegs - 1].str, "%s%s", tmp_str, str); */
-      strcat(s_preedit->pseg[s_preedit->nr_psegs - 1].str, str);
+      char *tmp_str = s_preedit->pseg[s_preedit->nr_psegs - 1].str;
+
+      uim_asprintf(&s_preedit->pseg[s_preedit->nr_psegs - 1].str, "%s%s", tmp_str, str);
+      free(tmp_str);
     } else {
-      s_preedit->pseg = realloc(s_preedit->pseg,
+      s_preedit->pseg = uim_realloc(s_preedit->pseg,
           sizeof(struct preedit_segment_tag) * (s_preedit->nr_psegs + 1));
-      s_preedit->pseg[s_preedit->nr_psegs].str = strdup(str);
+      s_preedit->pseg[s_preedit->nr_psegs].str = uim_strdup(str);
       s_preedit->pseg[s_preedit->nr_psegs].attr = attr;
       s_preedit->nr_psegs++;
     }
@@ -558,7 +562,7 @@ static void prop_list_update_cb(void *ptr, const char *str)
 {
   char *line;
   int error = TRUE; /* str  "" ΤȤerrorˤ*/
-  char *labels = strdup("");
+  char *labels = uim_strdup("");
   char *dup_str;
 
   const char *enc;
@@ -567,7 +571,7 @@ static void prop_list_update_cb(void *ptr, const char *str)
   debug(("prop_list_update_cb\n"));
   debug2(("str = %s", str));
 
-  dup_str = line = strdup(str);
+  dup_str = line = uim_strdup(str);
 
   while (line[0] != '\0') {
     int i;
@@ -576,6 +580,9 @@ static void prop_list_update_cb(void *ptr, const char *str)
     char *label;
     int label_width;
     int max_label_width = 0;
+    char *pad;
+    int padlen;
+    char *oldlabels;
 
     error = TRUE;
 
@@ -636,11 +643,15 @@ static void prop_list_update_cb(void *ptr, const char *str)
     }
 
     label_width = strwidth(label);
-    labels = realloc(labels, strlen(labels) + strlen(label) + (max_label_width - label_width) + 1);
-    for (i = 0; i < (max_label_width - label_width); i++) {
-      strcat(labels, " ");
-    }
-    strcat(labels, label);
+
+    oldlabels = labels;
+    padlen = max_label_width - label_width;
+    pad = uim_malloc(padlen + 1);
+    memset(pad, ' ', padlen);
+    pad[padlen] = '\0';
+    uim_asprintf(&labels, "%s%s%s", oldlabels, label, pad);
+    free(oldlabels);
+    free(pad);
   }
 
 loop_end:
@@ -666,8 +677,7 @@ loop_end:
   }
 
   enc = get_enc();
-  message_buf = malloc(strlen("prop_list_update\ncharset=") + strlen(enc) + strlen("\n") + strlen(str) + 1);
-  sprintf(message_buf, "prop_list_update\ncharset=%s\n%s", enc, str);
+  uim_asprintf(&message_buf, "prop_list_update\ncharset=%s\n%s", enc, str);
   uim_helper_send_message(g_helper_fd, message_buf);
   free(message_buf);
   debug(("prop_list_update_cb send message\n"));
@@ -678,7 +688,7 @@ loop_end:
  */
 struct preedit_tag *create_preedit(void)
 {
-  struct preedit_tag *p = malloc(sizeof(struct preedit_tag));
+  struct preedit_tag *p = uim_malloc(sizeof(struct preedit_tag));
   p->nr_psegs = 0;
   p->width = 0;
   p->cursor = 0;
@@ -698,9 +708,7 @@ void free_preedit(struct preedit_tag *p)
   for (i = 0; i < p->nr_psegs; i++) {
     free(p->pseg[i].str);
   }
-  if (p->pseg != NULL) {
-    free(p->pseg);
-  }
+  free(p->pseg);
   free(p);
 }
 
@@ -712,14 +720,54 @@ static struct preedit_tag *dup_preedit(struct preedit_tag *p)
   int i;
   struct preedit_tag *dup_p = create_preedit();
   *dup_p = *p;
-  dup_p->pseg = malloc(sizeof(struct preedit_segment_tag) * (p->nr_psegs));
+  dup_p->pseg = uim_malloc(sizeof(struct preedit_segment_tag) * (p->nr_psegs));
   for (i = 0; i < p->nr_psegs; i++) {
     dup_p->pseg[i].attr = p->pseg[i].attr;
-    dup_p->pseg[i].str = strdup(p->pseg[i].str);
+    dup_p->pseg[i].str = uim_strdup(p->pseg[i].str);
   }
   return dup_p;
 }
 
+/*
+ * initialize contents of s_candidate
+ *
+ * s_candidate.page_strs = array of page string
+ * s_candidate.page2index = head index of candidates at the page
+ * s_candidate.cand_col = place of a candidate
+ * s_candidate.nr_pages = total page number
+ * s_candidate.index_col = column of the currently selected candidate
+ */
+static void init_candidate(int nr, int display_limit)
+{
+  int nr_virtual_pages = 0;
+  int i;
+
+  s_candidate.nr = nr;
+  s_candidate.limit = display_limit;
+  s_candidate.page = 0;
+  s_candidate.cand_col = uim_malloc(nr * sizeof(int));
+
+  assert(s_candidate.nr != UNDEFINED);
+  assert(s_candidate.limit != UNDEFINED);
+  assert(s_candidate.cand_col != NULL);
+
+  if (s_candidate.limit)
+    nr_virtual_pages = (s_candidate.nr - 1) / s_candidate.limit + 1;
+  else
+    nr_virtual_pages = 1;
+
+  s_candidate.nr_pages = nr_virtual_pages;
+  s_candidate.page2index = uim_realloc(s_candidate.page2index, nr_virtual_pages * sizeof(int));
+  s_candidate.index_col = uim_realloc(s_candidate.index_col, nr_virtual_pages * sizeof(int));
+  s_candidate.page_strs = uim_realloc(s_candidate.page_strs, nr_virtual_pages * sizeof(char *));
+
+  for (i = 0; i < nr_virtual_pages; i++) {
+    s_candidate.page2index[i] = i * s_candidate.limit;
+    s_candidate.index_col[i] = UNDEFINED;
+    s_candidate.page_strs[i] = NULL;
+  }
+}
+
 /*
  * s_candidate.page_strs = ڡʸ
  * ʸs_max_widthۤȤϡϤ߽Ф򼡤Υڡ˰ܤ
@@ -733,22 +781,29 @@ static struct preedit_tag *dup_preedit(struct preedit_tag *p)
 static void make_page_strs(void)
 {
   /* NULLreallocƥߤˤʤʤ褦 */
-  char *page_str = strdup("");
+  char *page_str = uim_strdup("");
   int page_byte = 0;
   int page_width = 0;
   int index_in_page = 0;
+  char *old_str;
 
-  int index;
+  int index, page, start, nr_in_virtual_page;
 
   assert(s_candidate.nr != UNDEFINED);
   assert(s_candidate.limit != UNDEFINED);
   assert(s_candidate.cand_col != NULL);
 
-  s_candidate.nr_pages = 0;
+  page = s_candidate.page;
+  start = s_candidate.page2index[page];
+  if (s_candidate.limit && (s_candidate.nr - start) > s_candidate.limit)
+    nr_in_virtual_page = s_candidate.limit;
+  else
+    nr_in_virtual_page = s_candidate.nr - start;
 
-  for (index = 0; index < s_candidate.nr; index++) {
+  for (index = start; index < (start + nr_in_virtual_page); index++) {
     /* A:  S:  D:  F:  J:  K:  L:  [Ĥ 227] */
-    int next = FALSE;
+    int next = FALSE; /* flag whether to finish page */
+    int add_extra_page = FALSE;
     /* "[10/20]"  */
     int index_width;
     uim_candidate cand = uim_get_candidate(g_context, index, index_in_page);
@@ -757,7 +812,7 @@ static void make_page_strs(void)
     int cand_label_width = strwidth(cand_str_label);
     int cand_width = cand_label_width + strlen(":") + strwidth(cand_str_cand) + strlen(" ");
     int cand_byte = strlen(cand_str_label) + strlen(":") + strlen(cand_str_cand) + strlen(" ");
-    char *cand_str = malloc(cand_byte + 1);
+    char *cand_str;
 
     if (g_opt.ddskk) {
       index_width = strlen("[xxxx ]") + numwidth(s_candidate.nr - index - 1);
@@ -765,16 +820,10 @@ static void make_page_strs(void)
       index_width = strlen("[/]") + numwidth(index + 1) + numwidth(s_candidate.nr);
     }
 
-    sprintf(cand_str, "%s:%s ", cand_str_label, cand_str_cand);
+    uim_asprintf(&cand_str, "%s:%s ", cand_str_label, cand_str_cand);
     uim_candidate_free(cand);
     free(cand_str_cand);
 
-    if (index_in_page == 0) {
-      s_candidate.page2index = realloc(s_candidate.page2index, (s_candidate.nr_pages + 1) * sizeof(int));
-      s_candidate.page2index[s_candidate.nr_pages] = index;
-      s_candidate.index_col = realloc(s_candidate.index_col, (s_candidate.nr_pages + 1) * sizeof(int));
-    }
-
     if (page_width + cand_width + index_width > s_max_width && index_in_page != 0) {
       /* Ϥ߽ФΤǼΥڡ˰ܤ */
       index--;
@@ -784,6 +833,7 @@ static void make_page_strs(void)
         index_width = strlen("[/]") + numwidth(index + 1) + numwidth(s_candidate.nr);
       }
       next = TRUE;
+      add_extra_page = TRUE;
     } else {
 
       s_candidate.cand_col[index] = page_width + cand_label_width + strlen(":");
@@ -826,47 +876,66 @@ static void make_page_strs(void)
 
       page_width += cand_width;
       page_byte += cand_byte;
-      page_str = realloc(page_str, page_byte + 1);
-      strcat(page_str, cand_str);
+      old_str = page_str;
+      uim_asprintf(&page_str, "%s%s", old_str, cand_str);
+      free(old_str);
 
       index_in_page++;
-      if (index_in_page == s_candidate.limit || index + 1 == s_candidate.nr) {
+      if (index_in_page == s_candidate.limit || (index + 1 - start) == nr_in_virtual_page) {
         next = TRUE;
       }
     }
 
-    if (next) {
+    if (next) { /* do fix the page */
       if (index_width == UNDEFINED) {
-        s_candidate.index_col[s_candidate.nr_pages] = UNDEFINED;
+        s_candidate.index_col[page] = UNDEFINED;
       } else {
         int index_byte = index_width + 2/* utf-8 */;
-        char *index_str = malloc(index_byte + 1);
+        char *index_str;
         int i;
         if (g_opt.ddskk) {
-          sprintf(index_str, "[%s %d]", s_nokori_str, s_candidate.nr - index - 1);
+          uim_asprintf(&index_str, "[%s %d]", s_nokori_str, s_candidate.nr - index - 1);
         } else {
-          sprintf(index_str, "[%d/%d]", index + 1, s_candidate.nr);
+          uim_asprintf(&index_str, "[%d/%d]", index + 1, s_candidate.nr);
           for (i = 0; i < numwidth(index + 1); i++) {
             index_str[1 + i] = ' ';
           }
           index_str[i] = '-';
         }
         assert(page_width + index_width <= s_max_width);
-        s_candidate.index_col[s_candidate.nr_pages] = page_width + strlen("[");
+        s_candidate.index_col[page] = page_width + strlen("[");
         page_byte += index_byte;
-        page_str = realloc(page_str, page_byte + 1);
-        strcat(page_str, index_str);
+        old_str = page_str;
+        uim_asprintf(&page_str, "%s%s", old_str, index_str);
+        free(old_str);
         free(index_str);
       }
-      s_candidate.page_strs = realloc(s_candidate.page_strs, (s_candidate.nr_pages + 1) * sizeof(char *));
-      s_candidate.page_strs[s_candidate.nr_pages] = strdup(page_str);
-      s_candidate.nr_pages++; 
+      free(s_candidate.page_strs[page]);
+      s_candidate.page_strs[page] = uim_strdup(page_str);
+      page++; 
 
       page_byte = 0;
       page_width = 0;
       index_in_page = 0;
       free(page_str);
-      page_str = strdup("");
+      page_str = uim_strdup("");
+    }
+    if (add_extra_page) {
+      int i;
+
+      s_candidate.nr_pages++;
+      s_candidate.page2index = uim_realloc(s_candidate.page2index, s_candidate.nr_pages * sizeof(int));
+      s_candidate.index_col = uim_realloc(s_candidate.index_col, s_candidate.nr_pages * sizeof(int));
+      s_candidate.page_strs = uim_realloc(s_candidate.page_strs, s_candidate.nr_pages * sizeof(char *));
+
+      for (i = s_candidate.nr_pages - 1; i >= page; i--) {
+        s_candidate.page2index[i] = s_candidate.page2index[i - 1];
+        s_candidate.index_col[i] = s_candidate.index_col[i - 1];
+        s_candidate.page_strs[i] = s_candidate.page_strs[i - 1];
+      }
+      s_candidate.page2index[page] = index + 1;
+      s_candidate.index_col[page] = UNDEFINED;
+      s_candidate.page_strs[page] = NULL;
     }
     free(cand_str);
   }
@@ -947,7 +1016,7 @@ static void set_candidate(void)
   int index_width;
 
   if (s_candidate.index_col[s_candidate.page] == UNDEFINED) {
-    s_index_str = strdup("");
+    s_index_str = uim_strdup("");
     index_width = 0;
   } else {
     /* üθΥǥå */
@@ -956,13 +1025,17 @@ static void set_candidate(void)
     int right_edge_cand_index_width = numwidth(right_edge_cand_index + 1);
     /* ߤθΥǥå */
     int cand_index_width = numwidth(s_candidate.index + 1);
-    int i;
-    s_index_str = malloc(right_edge_cand_index_width + 1);
-    for (i = 0; i < right_edge_cand_index_width - cand_index_width; i++) {
-      s_index_str[i] = ' ';
-    }
-    s_index_str[i] = '\0';
-    sprintf(s_index_str, "%s%d", s_index_str, s_candidate.index + 1);
+    int padlen = right_edge_cand_index_width - cand_index_width;
+    char *pad;
+
+    if (padlen < 0)
+      padlen = 0;
+    pad = uim_malloc(padlen + 1);
+    memset(pad, ' ', padlen);
+    pad[padlen] = '\0';
+    uim_asprintf(&s_index_str, "%s%d", pad, s_candidate.index + 1);
+    free(pad);
+
     if (g_opt.ddskk) {
       index_width = strlen("[xxxx ]") + numwidth(s_candidate.nr - s_candidate.index - 1);
     } else {
@@ -973,12 +1046,12 @@ static void set_candidate(void)
 
   s_candidate_col = s_candidate.cand_col[s_candidate.index];
   if (s_candidate_col == UNDEFINED) {
-    s_candidate_str = strdup("");
+    s_candidate_str = uim_strdup("");
     return;
   }
   cand = uim_get_candidate(g_context, s_candidate.index, 0);
   if (uim_candidate_get_cand_str(cand) == NULL) {
-    s_candidate_str = strdup("");
+    s_candidate_str = uim_strdup("");
     s_candidate_col = UNDEFINED;
     uim_candidate_free(cand);
     return;
@@ -993,6 +1066,8 @@ static void set_candidate(void)
 
 void callbacks_winch(void)
 {
+  int current_index;
+
   start_callbacks();
 
   s_max_width = g_win->ws_col;
@@ -1021,7 +1096,14 @@ void callbacks_winch(void)
     free(s_candidate.index_col);
     s_candidate.index_col = NULL;
   }
+  if (s_candidate.cand_col != NULL) {
+    free(s_candidate.cand_col);
+    s_candidate.cand_col = NULL;
+  }
 
+  current_index = s_candidate.index;
+  init_candidate(s_candidate.nr, s_candidate.limit);
+  s_candidate.index = current_index;
   make_page_strs();
 }
 
