@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -267,7 +267,7 @@ static void start_preedit(void)
       }
     }
     debug2(("s_head.row = %d s_head.col = %d\n", s_head.row, s_head.col));
-    s_line2width = malloc(sizeof(int));
+    s_line2width = uim_malloc(sizeof(int));
     s_line2width[0] = s_head.col;
     s_preedit_lines = 1;
   }
@@ -352,16 +352,16 @@ static void draw_statusline(int force, int restore, int visible, int draw_backgr
 
   /* staticѿν 1󤷤¹Ԥʤ */
   if (statusline_str == NULL) {
-    statusline_str = strdup("");
+    statusline_str = uim_strdup("");
   }
   if (candidate_str == NULL) {
-    candidate_str = strdup("");
+    candidate_str = uim_strdup("");
   }
   if (index_str == NULL) {
-    index_str = strdup("");
+    index_str = uim_strdup("");
   }
   if (mode_str == NULL) {
-    mode_str = strdup("");
+    mode_str = uim_strdup("");
   }
 
   prev_statusline_str = statusline_str;
@@ -973,11 +973,9 @@ static void erase_preedit(void)
 {
   s_prev_preedit_lines = s_preedit_lines;
   s_preedit_lines = 1;
-  if (s_prev_line2width != NULL) {
-    free(s_prev_line2width);
-  }
+  free(s_prev_line2width);
   s_prev_line2width = s_line2width;
-  s_line2width = malloc(sizeof(int));
+  s_line2width = uim_malloc(sizeof(int));
   s_line2width[0] = s_head.col;
   erase_prev_preedit();
 }
@@ -988,12 +986,10 @@ static void set_line2width(struct preedit_tag *preedit)
   int line_width = s_head.col;
   int lineno = 0;
 
-  if (s_prev_line2width != NULL) {
-    free(s_prev_line2width);
-  }
+  free(s_prev_line2width);
   s_prev_line2width = s_line2width;
   s_prev_preedit_lines = s_preedit_lines;
-  s_line2width = malloc(sizeof(int));
+  s_line2width = uim_malloc(sizeof(int));
   s_preedit_lines = 1;
 
   for (i = 0; i < preedit->nr_psegs; i++) {
@@ -1001,7 +997,7 @@ static void set_line2width(struct preedit_tag *preedit)
     int seg_w = strwidth(seg_str);
     while (line_width + seg_w >= g_win->ws_col) {
       if (lineno + 1 == s_preedit_lines) {
-        s_line2width = realloc(s_line2width, sizeof(int) * ++s_preedit_lines);
+        s_line2width = uim_realloc(s_line2width, sizeof(int) * ++s_preedit_lines);
       }
       if (line_width + seg_w > g_win->ws_col) {
         int *byte_width = width2byte(seg_str, g_win->ws_col - line_width);
@@ -1020,7 +1016,7 @@ static void set_line2width(struct preedit_tag *preedit)
   }
   if (s_preedit_lines > s_prev_preedit_lines) {
     int i;
-    s_prev_line2width = realloc(s_prev_line2width, sizeof(int) * s_preedit_lines);
+    s_prev_line2width = uim_realloc(s_prev_line2width, sizeof(int) * s_preedit_lines);
     for (i = s_prev_preedit_lines; i < s_preedit_lines; i++) {
       s_prev_line2width[i] = 0;
     }
