@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -164,7 +164,7 @@ void init_escseq(const struct attribute_tag *attr_uim)
   escseq2n2(orig_pair, &s_orig_fore_num, &s_orig_back_num);
   s_enter_uim_mode = attr2escseq(&s_attr_uim);
   if (s_enter_uim_mode != NULL) {
-    s_enter_uim_mode = strdup(s_enter_uim_mode);
+    s_enter_uim_mode = uim_strdup(s_enter_uim_mode);
   }
 
   fixtty();
@@ -779,6 +779,7 @@ static void change_background_attr(struct attribute_tag *from, struct attribute_
 static const char *attr2escseq(const struct attribute_tag *attr)
 {
   static char escseq[20];
+  char numstr[20];
   int add_semicolon = FALSE;
   if (!attr->underline && !attr->standout && !attr->bold && !attr->blink
       && attr->foreground == FALSE && attr->background == FALSE) {
@@ -787,47 +788,49 @@ static const char *attr2escseq(const struct attribute_tag *attr)
   strlcpy(escseq, "\033[", sizeof(escseq));
   if (attr->underline && s_enter_underline_num != NULL) {
     add_semicolon = TRUE;
-    strcat(escseq, s_enter_underline_num);
+    strlcat(escseq, s_enter_underline_num, sizeof(escseq));
   }
   if (attr->standout && s_enter_standout_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_enter_standout_num);
+    strlcat(escseq, s_enter_standout_num, sizeof(escseq));
   }
   if (attr->bold && s_bold_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_bold_num);
+    strlcat(escseq, s_bold_num, sizeof(escseq));
   }
   if (attr->blink && s_blink_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_blink_num);
+    strlcat(escseq, s_blink_num, sizeof(escseq));
   }
   if (attr->foreground != FALSE) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    sprintf(escseq + strlen(escseq), "%d", attr->foreground);
+    snprintf(numstr, sizeof(numstr), "%d", attr->foreground);
+    strlcat(escseq, numstr, sizeof(escseq));
   }
   if (attr->background != FALSE) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     }
-    sprintf(escseq + strlen(escseq), "%d", attr->background);
+    snprintf(numstr, sizeof(numstr), "%d", attr->background);
+    strlcat(escseq, numstr, sizeof(escseq));
   }
-  strcat(escseq, "m");
+  strlcat(escseq, "m", sizeof(escseq));
   debug2(("attr2escseq underline = %d standout = %d bold = %d blink = %d fore = %d back = %d\n",
       attr->underline, attr->standout, attr->bold, attr->blink,
       attr->foreground, attr->background));
@@ -853,7 +856,7 @@ static void set_attr(const char *str, int len)
     /* s_escseq_buf ˤ'\0'ϴޤޤʤ */
     int escseq_buf_len = strlen(s_escseq_buf);
     must_free = TRUE;
-    free_str = str = malloc(escseq_buf_len + len + 1);
+    free_str = str = uim_malloc(escseq_buf_len + len + 1);
     memcpy((char *)str, s_escseq_buf, escseq_buf_len);
     memcpy((char *)str + escseq_buf_len, tmp_str, len);
     ((char *)str)[escseq_buf_len + len] = '\0';
@@ -866,7 +869,7 @@ static void set_attr(const char *str, int len)
     str++;
     if (str == end) {
       int escseq_buf_len = end - start;
-      s_escseq_buf = malloc(escseq_buf_len + 1);
+      s_escseq_buf = uim_malloc(escseq_buf_len + 1);
       memcpy(s_escseq_buf, start, escseq_buf_len);
       s_escseq_buf[escseq_buf_len] = '\0';
     }
@@ -874,7 +877,7 @@ static void set_attr(const char *str, int len)
     else if (str[0] == '[') {
       int nr_params = 1;
       /* ^[[1;2;31, 2, 3 */
-      int *params = malloc(sizeof(int));
+      int *params = uim_malloc(sizeof(int));
       params[0] = 0;
       str++;
 
@@ -889,7 +892,7 @@ static void set_attr(const char *str, int len)
         }
         if (str[0] == ';') {
           nr_params++;
-          params = realloc(params, sizeof(int) * nr_params);
+          params = uim_realloc(params, sizeof(int) * nr_params);
           params[nr_params - 1] = 0;
           str++;
         }
@@ -897,7 +900,7 @@ static void set_attr(const char *str, int len)
 
       if (str == end) {
         int escseq_buf_len = end - start;
-        s_escseq_buf = malloc(escseq_buf_len + 1);
+        s_escseq_buf = uim_malloc(escseq_buf_len + 1);
         memcpy(s_escseq_buf, start, escseq_buf_len);
         s_escseq_buf[escseq_buf_len] = '\0';
       }
@@ -1053,7 +1056,7 @@ void put_erase(int n)
     return;
   }
 
-  spaces = malloc(n + 1);
+  spaces = uim_malloc(n + 1);
   for (i = 0; i < n; i++) {
     spaces[i] = ' ';
   }
@@ -1197,7 +1200,7 @@ char *cut_padding(const char *escseq)
   if (escseq == NULL) {
     return NULL;
   }
-  cut_str = malloc(strlen(escseq) + 1);
+  cut_str = uim_malloc(strlen(escseq) + 1);
   for (i = 0, j = 0; escseq[i] != '\0';) {
     if (escseq[i] == '$' && escseq[i + 1] == '<') {
       int i2 = i + 1;
