@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -122,7 +122,7 @@ void unget_stdin(const char *str, int count)
     return;
   }
   debug(("unget count = %d s_buf_size = %d\n", count, s_buf_size));
-  s_unget_buf = realloc(s_unget_buf, s_buf_size + count);
+  s_unget_buf = uim_realloc(s_unget_buf, s_buf_size + count);
   memcpy(s_unget_buf + s_buf_size, str, count);
   s_buf_size += count;
 }
