@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,8 +31,8 @@
 
 */
 
-#ifndef READ_H
-#define READ_H
+#ifndef UIM_FEP_READ_H
+#define UIM_FEP_READ_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
