@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -112,9 +112,9 @@ const char *usersockname(const char *file)
 
   if (!get_ud_path(sock_dir, sizeof(sock_dir))) {
     sendline("uim-fep cannot create directory");
-    exit(EXIT_FAILURE);
+    /* exit(EXIT_FAILURE); */
   }
-  snprintf(buf, UNIX_PATH_MAX, "%s/%s", sock_dir, filebuf);
+  snprintf(buf, sizeof(buf), "%s/%s", sock_dir, filebuf);
 
   return buf;
 }
