@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -228,7 +228,7 @@ static const char *get_default_im_name(void)
 int main(int argc, char **argv)
 {
   /* command will be execed on pty */
-  const char **command = malloc(sizeof(const char *) * (argc + 1));
+  const char **command = uim_malloc(sizeof(const char *) * (argc + 1));
   char *engine;
   char *sock_path = NULL; /* Socket for backtick */
   int gnu_screen = FALSE;
@@ -260,7 +260,7 @@ int main(int argc, char **argv)
   command[1] = NULL;
 
   init_str();
-  engine = strdup(get_default_im_name());
+  engine = uim_strdup(get_default_im_name());
 
   while ((op = getopt(argc, argv, "e:s:u:b:w:t:C:f:SXciodKvh")) != -1) {
     int i;
@@ -311,7 +311,7 @@ int main(int argc, char **argv)
         break;
 
       case 'u':
-        engine = strdup(optarg);
+        engine = uim_strdup(optarg);
         break;
 
       case 'c':
@@ -417,7 +417,7 @@ opt_end:
 
   if (!get_ud_path(uim_dir, sizeof(uim_dir))) {
     sendline("uim-fep cannot make directory");
-    return EXIT_FAILURE;
+    /* return EXIT_FAILURE; */
   }
 
   sty_str = getenv("STY");
@@ -1025,7 +1025,7 @@ static void recover_loop(void)
  */
 static struct winsize *get_winsize(void)
 {
-  struct winsize *win = malloc(sizeof(struct winsize));
+  struct winsize *win = uim_malloc(sizeof(struct winsize));
   ioctl(g_win_in, TIOCGWINSZ, win);
   if (g_opt.status_type == LASTLINE) {
     win->ws_row--;
