@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -80,11 +80,23 @@ caret_state_indicator_timeout(gpointer data)
 }
 
 static gint
+#if GTK_CHECK_VERSION(2, 90, 0)
+caret_state_indicator_paint_window(GtkWidget *window, cairo_t *cr)
+#else
 caret_state_indicator_paint_window(GtkWidget *window)
+#endif
 {
-  gtk_paint_flat_box(window->style, window->window, GTK_STATE_NORMAL,
-		     GTK_SHADOW_OUT, NULL, GTK_WIDGET(window), "tooltip",
-		     0, 0, -1, -1);
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gtk_render_frame(gtk_widget_get_style_context(window), cr,
+             0, 0,
+             gtk_widget_get_allocated_width(window),
+             gtk_widget_get_allocated_height(window));
+#else
+  gtk_paint_flat_box(gtk_widget_get_style(window),
+             gtk_widget_get_window(window),
+             GTK_STATE_NORMAL, GTK_SHADOW_OUT, NULL, GTK_WIDGET(window),
+             "tooltip", 0, 0, -1, -1);
+#endif
 
   return FALSE;
 }
@@ -122,9 +134,15 @@ caret_state_indicator_new(void)
 			      DEFAULT_WINDOW_HEIGHT);
   gtk_widget_set_app_paintable(window, TRUE);
 
+#if GTK_CHECK_VERSION(2, 90, 0)
+  g_signal_connect(window, "draw",
+		   G_CALLBACK(caret_state_indicator_paint_window), 
+		   NULL);
+#else
   g_signal_connect(window, "expose_event",
 		   G_CALLBACK(caret_state_indicator_paint_window), 
 		   NULL);
+#endif
   g_signal_connect(window, "destroy",
 		   G_CALLBACK(caret_state_indicator_destroy_cb), 
 		   NULL);
