@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,7 +37,6 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <gtk/gtkimcontext.h>
 #include <gtk/gtkimmodule.h>
 #include <gdk/gdkkeysyms.h>
 #ifdef GDK_WINDOWING_X11
@@ -56,9 +55,11 @@
 #include "uim/uim-im-switcher.h"
 #include "uim/gettext.h"
 #include "uim/uim-scm.h"
+#include "uim/counted-init.h"
 
 #include "gtk-im-uim.h"
 #include "uim-cand-win-gtk.h"
+#include "uim-cand-win-tbl-gtk.h"
 #include "caret-state-indicator.h"
 #include "key-util-gtk.h"
 #ifdef GDK_WINDOWING_X11
@@ -319,7 +320,11 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
 static void
 remove_cur_toplevel()
 {
+#if GTK_CHECK_VERSION(2, 18, 0)
+  if (cur_toplevel && gtk_widget_is_toplevel(cur_toplevel)) {
+#else
   if (cur_toplevel && GTK_WIDGET_TOPLEVEL(cur_toplevel)) {
+#endif
     if (cur_key_press_handler_id)
       g_signal_handler_disconnect(cur_toplevel, cur_key_press_handler_id);
     if (cur_key_release_handler_id)
@@ -356,7 +361,11 @@ update_cur_toplevel(IMUIMContext *uic)
 
   if (uic->widget) {
     GtkWidget *toplevel = gtk_widget_get_toplevel(uic->widget);
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (toplevel && gtk_widget_is_toplevel(toplevel)) {
+#else
     if (toplevel && GTK_WIDGET_TOPLEVEL(toplevel)) {
+#endif
       if (cur_toplevel != toplevel) {
 	remove_cur_toplevel();
 	cur_toplevel = toplevel;
@@ -391,13 +400,17 @@ on_client_widget_grab_notify(GtkWidget *widget, gboolean was_grabbed, IMUIMConte
     grab_widget = gtk_grab_get_current();
     if (!grab_widget) {
       if (cur_toplevel && GTK_IS_WINDOW(cur_toplevel)) {
-	GtkWindowGroup *group;
-	GtkWindow *window;
+        GtkWindowGroup *group;
+        GtkWindow *window;
 	
-	window = GTK_WINDOW(cur_toplevel);
-	group = window->group;
-	if (group && group->grabs)
-	  grab_widget = GTK_WIDGET(group->grabs->data);
+        window = GTK_WINDOW(cur_toplevel);
+        group = gtk_window_get_group(window);
+#if GTK_CHECK_VERSION(2, 22, 0)
+        grab_widget = gtk_window_group_get_current_grab(group);
+#else
+        if (group && group->grabs)
+          grab_widget = GTK_WIDGET(group->grabs->data);
+#endif
       }
     }
   }
@@ -475,12 +488,20 @@ index_changed_cb(UIMCandWinGtk *cwin, IMUIMContext *uic)
 static void
 layout_candwin(IMUIMContext *uic)
 {
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gint x, y, width, height;
+#else
   gint x, y, width, height, depth;
+#endif
 
   g_return_if_fail(uic);
 
   if (uic->win && uic->cwin) {
+#if GTK_CHECK_VERSION(2, 90, 0)
+    gdk_window_get_geometry(uic->win, &x, &y, &width, &height);
+#else
     gdk_window_get_geometry(uic->win, &x, &y, &width, &height, &depth);
+#endif
     gdk_window_get_origin(uic->win, &x, &y);
     uim_cand_win_gtk_layout(uic->cwin, x, y, width, height);
   }
@@ -629,6 +650,9 @@ update_prop_list_cb(void *ptr, const char *str)
   IMUIMContext *uic = (IMUIMContext *)ptr;
   GString *prop_list;
   uim_bool show_state;
+  char *show_state_with;
+  uim_bool show_state_mode;
+  uim_bool show_state_mode_on;
 
   if (uic != focused_context || disable_focused_context)
     return;
@@ -640,22 +664,34 @@ update_prop_list_cb(void *ptr, const char *str)
   g_string_free(prop_list, TRUE);
 
   show_state = uim_scm_symbol_value_bool("bridge-show-input-state?");
-  if (show_state && uic->win) {
-    gint timeout;
-    gint x, y;
-    GString *label;
+  show_state_with = uim_scm_c_symbol(uim_scm_symbol_value("bridge-show-with?"));
+  show_state_mode = (strcmp(show_state_with, "mode") == 0);
+  show_state_mode_on = uim_scm_symbol_value_bool("bridge-show-input-state-mode-on?");
 
-    gdk_window_get_origin(uic->win, &x, &y);
-    label = get_caret_state_label_from_prop_list(str);
-    caret_state_indicator_update(uic->caret_state_indicator, x, y, label->str);
-    g_string_free(label, TRUE);
-    timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+  if (uic->win) {
+    if (show_state && !(show_state_mode && !show_state_mode_on)) {
+      gint timeout;
+      gint x, y;
+      GString *label;
 
-    if (timeout != 0)
-      caret_state_indicator_set_timeout(uic->caret_state_indicator,
+      gdk_window_get_origin(uic->win, &x, &y);
+      label = get_caret_state_label_from_prop_list(str);
+      caret_state_indicator_update(uic->caret_state_indicator, x, y, label->str);
+      g_string_free(label, TRUE);
+      if (strcmp(show_state_with, "time") == 0)
+	timeout = uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+      else
+	timeout = 0;
+
+      if (timeout != 0)
+	caret_state_indicator_set_timeout(uic->caret_state_indicator,
 					timeout * 1000);
-    gtk_widget_show_all(uic->caret_state_indicator);
+      gtk_widget_show_all(uic->caret_state_indicator);
+    } else if (show_state_mode && !show_state_mode_on) {
+      gtk_widget_hide(uic->caret_state_indicator);
+    }
   }
+  free(show_state_with);
 }
 
 #if IM_UIM_USE_NEW_PAGE_HANDLING
@@ -704,6 +740,18 @@ cand_activate_cb(void *ptr, int nr, int display_limit)
 
   uic->cwin_is_active = TRUE;
 
+  /* XXX: adjust display_limit for UIMCandWinTblGtk */
+  if (UIM_IS_CAND_WIN_TBL_GTK(uic->cwin) && nr > display_limit) {
+    uim_candidate c;
+    const char *s;
+    c = uim_get_candidate(uic->uc, 0, 9999);
+    s = uim_candidate_get_annotation_str(c);
+#define LEN_DISPLAY_LIMIT 14
+    if (strncmp(s, "display_limit=", LEN_DISPLAY_LIMIT) == 0) {
+      display_limit = atoi(s + LEN_DISPLAY_LIMIT);
+    }
+    uim_candidate_free(c);
+  }
 #if !IM_UIM_USE_NEW_PAGE_HANDLING
   for (i = 0; i < nr; i++) {
     cand = uim_get_candidate(uic->uc, i, display_limit ? i % display_limit : i);
@@ -1307,21 +1355,21 @@ im_uim_reset(GtkIMContext *ic)
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 #if !defined(WORKAROUND_BROKEN_RESET_IN_GTK)
   uim_reset_context(uic->uc);
+  clear_cb(uic);
+  update_cb(uic);
 #else
   if (uic == focused_context) {
     uim_focus_out_context(uic->uc);
     uim_focus_in_context(uic->uc);
   } else {
     uim_reset_context(uic->uc);
+    clear_cb(uic);
+    update_cb(uic);
   }
 #endif
 #ifdef GDK_WINDOWING_X11
   im_uim_compose_reset(uic->compose);
 #endif
-#if !defined(WORKAROUND_BROKEN_RESET_IN_GTK)
-  clear_cb(uic);
-  update_cb(uic);
-#endif
 }
 
 static void
@@ -1365,6 +1413,20 @@ im_uim_set_client_window(GtkIMContext *ic, GdkWindow *w)
   update_client_widget(uic);
 }
 
+static UIMCandWinGtk *
+im_uim_create_cand_win_gtk()
+{
+  UIMCandWinGtk *cwin;
+  char *candwinprog = uim_scm_symbol_value_str("uim-candwin-prog");
+  if (candwinprog && !strncmp(candwinprog, "uim-candwin-tbl", 15)) {
+    cwin = UIM_CAND_WIN_GTK(uim_cand_win_tbl_gtk_new());
+  } else {
+    cwin = uim_cand_win_gtk_new();
+  }
+  free(candwinprog);
+  return cwin;
+}
+
 static void
 im_uim_init(IMUIMContext *uic)
 {
@@ -1378,7 +1440,7 @@ im_uim_init(IMUIMContext *uic)
   uic->nr_psegs = 0;
   uic->prev_preedit_len = 0;
 
-  uic->cwin = uim_cand_win_gtk_new();
+  uic->cwin = im_uim_create_cand_win_gtk();
 #if IM_UIM_USE_TOPLEVEL
   cwin_list = g_list_append(cwin_list, uic->cwin);
 #endif
@@ -1575,12 +1637,16 @@ handle_key_on_toplevel(GtkWidget *widget, GdkEventKey *event, gpointer data)
     if (rv)
       return FALSE;
 
+#if !GTK_CHECK_VERSION(2, 90, 0)
+    /* FIXME: Can't compile with GSEAL_ENABLE */
     if (GTK_IS_TEXT_VIEW(uic->widget))
       GTK_TEXT_VIEW(uic->widget)->need_im_reset = TRUE;
     else if (GTK_IS_ENTRY(uic->widget)) {
-      if (GTK_ENTRY(uic->widget)->editable)
-	GTK_ENTRY(uic->widget)->need_im_reset = TRUE;
+      /* FIXME: Can't compile with GSEAL_ENABLE */
+      if (gtk_editable_get_editable(GTK_EDITABLE(uic->widget)))
+        GTK_ENTRY(uic->widget)->need_im_reset = TRUE;
     }
+#endif
     return TRUE;
   }
 
@@ -1591,7 +1657,7 @@ handle_key_on_toplevel(GtkWidget *widget, GdkEventKey *event, gpointer data)
 void
 im_module_init(GTypeModule *type_module)
 {
-  if (uim_init() == -1)
+  if (uim_counted_init() == -1)
     return;
 
   context_list.next = (IMUIMContext *)&context_list;
@@ -1624,5 +1690,5 @@ im_module_exit(void)
 #ifdef GDK_WINDOWING_X11
   im_uim_release_compose_tree();
 #endif
-  uim_quit();
+  uim_counted_quit();
 }
