@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,7 +37,6 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
-#include <gtk/gtkimcontext.h>
 #include <gtk/gtkimmodule.h>
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -54,6 +53,9 @@
 #define IM_UIM_USE_SNOOPER      0
 #define IM_UIM_USE_TOPLEVEL     1
 
+/* enable per page candidates handling */
+#define IM_UIM_USE_NEW_PAGE_HANDLING	1
+
 typedef struct _IMUIMContext {
   struct _GtkIMContext parent;
   struct _GtkIMContext *slave;
