@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,6 +34,7 @@
 /* example-start entry entry.c */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 
 void enter_callback(GtkWidget *, GtkWidget *);
@@ -51,15 +52,15 @@ void enter_callback( GtkWidget *widget,
 void entry_toggle_editable( GtkWidget *checkbutton,
                             GtkWidget *entry )
 {
-  gtk_entry_set_editable(GTK_ENTRY(entry),
-			 GTK_TOGGLE_BUTTON(checkbutton)->active);
+  gtk_editable_set_editable(GTK_EDITABLE(entry),
+			 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton)));
 }
 
 void entry_toggle_visibility( GtkWidget *checkbutton,
                               GtkWidget *entry )
 {
   gtk_entry_set_visibility(GTK_ENTRY(entry),
-			 GTK_TOGGLE_BUTTON(checkbutton)->active);
+			 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton)));
 }
 
 int main( int   argc,
@@ -71,33 +72,34 @@ int main( int   argc,
     GtkWidget *entry;
     GtkWidget *button;
     GtkWidget *check;
+    gint position;
 
     GTimer *tim = g_timer_new();
 
-    gtk_set_locale();
     gtk_init (&argc, &argv);
 
     g_timer_start(tim);
 
     /* create a new window */
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_widget_set_usize( GTK_WIDGET (window), 200, 100);
+    gtk_widget_set_size_request( GTK_WIDGET (window), 200, 100);
     gtk_window_set_title(GTK_WINDOW (window), "GTK Entry");
-    gtk_signal_connect(GTK_OBJECT (window), "delete_event",
-                       (GtkSignalFunc) gtk_exit, NULL);
+    g_signal_connect(G_OBJECT (window), "delete_event",
+                       G_CALLBACK(exit), NULL);
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_container_add (GTK_CONTAINER (window), vbox);
     gtk_widget_show (vbox);
 
-    entry = gtk_entry_new_with_max_length (50);
-    gtk_signal_connect(GTK_OBJECT(entry), "activate",
-		       GTK_SIGNAL_FUNC(enter_callback),
+    entry = gtk_entry_new ();
+    gtk_entry_set_max_length (GTK_ENTRY(entry), 50);
+    g_signal_connect(G_OBJECT(entry), "activate",
+		       G_CALLBACK(enter_callback),
 		       entry);
     gtk_entry_set_text (GTK_ENTRY (entry), "hello");
-    gtk_entry_append_text (GTK_ENTRY (entry), " world");
-    gtk_entry_select_region (GTK_ENTRY (entry),
-			     0, GTK_ENTRY(entry)->text_length);
+    gtk_editable_insert_text (GTK_EDITABLE (entry), " world", -1, &position);
+    gtk_editable_select_region (GTK_EDITABLE (entry),
+			     0, gtk_entry_get_text_length(GTK_ENTRY(entry)));
     gtk_box_pack_start (GTK_BOX (vbox), entry, TRUE, TRUE, 0);
     gtk_widget_show (entry);
 
@@ -107,24 +109,28 @@ int main( int   argc,
                                   
     check = gtk_check_button_new_with_label("Editable");
     gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT(check), "toggled",
-			GTK_SIGNAL_FUNC(entry_toggle_editable), entry);
+    g_signal_connect (G_OBJECT(check), "toggled",
+			G_CALLBACK(entry_toggle_editable), entry);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);
     gtk_widget_show (check);
     
     check = gtk_check_button_new_with_label("Visible");
     gtk_box_pack_start (GTK_BOX (hbox), check, TRUE, TRUE, 0);
-    gtk_signal_connect (GTK_OBJECT(check), "toggled",
-			GTK_SIGNAL_FUNC(entry_toggle_visibility), entry);
+    g_signal_connect (G_OBJECT(check), "toggled",
+			G_CALLBACK(entry_toggle_visibility), entry);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);
     gtk_widget_show (check);
                                    
     button = gtk_button_new_with_label ("Close");
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC(gtk_exit),
-			       GTK_OBJECT (window));
+    g_signal_connect_swapped (G_OBJECT (button), "clicked",
+			       G_CALLBACK(exit),
+			       G_OBJECT (window));
     gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION(2, 18, 0)
+    gtk_widget_set_can_default (button, TRUE);
+#else
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+#endif
     gtk_widget_grab_default (button);
     gtk_widget_show (button);
     
