@@ -1,6 +1,6 @@
 /*
 
-  copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -38,9 +38,6 @@
 #include <stdlib.h>
 #include <uim/uim.h>
 #include <uim/uim-scm.h>
-#if HAVE_EBLIB
-#include "uim-eb.h"
-#endif /* HAVE_EBLIB */
 
 #define NR_CANDIDATES 20 /* FIXME! not used yet */
 #define DEFAULT_MIN_WINDOW_WIDTH 80
@@ -65,6 +62,10 @@ static void	uim_cand_win_gtk_map		(GtkWidget *widget);
 static void	uim_cand_win_gtk_unmap		(GtkWidget *widget);
 static void	uim_cand_win_gtk_create_sub_window(UIMCandWinGtk *cwin);
 static void	uim_cand_win_gtk_layout_sub_window(UIMCandWinGtk *cwin);
+static void	uim_cand_win_gtk_real_set_index		(UIMCandWinGtk *cwin,
+							 gint index);
+static void	uim_cand_win_gtk_real_set_page		(UIMCandWinGtk *cwin,
+							 gint page);
 
 static gboolean	tree_selection_change		(GtkTreeSelection *selection,
 						 GtkTreeModel *model,
@@ -134,6 +135,9 @@ uim_cand_win_gtk_class_init (UIMCandWinGtkClass *klass)
 
   widget_class->map   = uim_cand_win_gtk_map;
   widget_class->unmap = uim_cand_win_gtk_unmap;
+
+  klass->set_index = uim_cand_win_gtk_real_set_index;
+  klass->set_page = uim_cand_win_gtk_real_set_page;
 }
 
 void
@@ -208,22 +212,22 @@ uim_cand_win_gtk_init (UIMCandWinGtk *cwin)
 		    G_CALLBACK(tree_selection_changed), cwin);
 
   renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "scale", 0.8, NULL);
+  g_object_set(renderer, "scale", 0.8, (const gchar *)NULL);
 
   column = gtk_tree_view_column_new_with_attributes("No",
 						    renderer,
 						    "text", COLUMN_HEADING,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(cwin->view), column);
   gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
   renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "scale", 1.2, NULL);
+  g_object_set(renderer, "scale", 1.2, (const gchar *)NULL);
   /*  g_object_set(renderer, "size-points", 20.0, NULL); */
   column = gtk_tree_view_column_new_with_attributes("Text",
 						    renderer,
 						    "text", COLUMN_CANDIDATE,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(cwin->view), column);
   gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(cwin->view), TRUE);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(cwin->view), FALSE);
@@ -312,8 +316,8 @@ uim_cand_win_gtk_new (void)
   return UIM_CAND_WIN_GTK(obj);
 }
 
-static void
-update_label(UIMCandWinGtk *cwin)
+void
+uim_cand_win_gtk_update_label(UIMCandWinGtk *cwin)
 {
   char label_str[20];
 
@@ -352,14 +356,14 @@ tree_selection_change(GtkTreeSelection *selection,
 		    cand_win_gtk_signals[INDEX_CHANGED_SIGNAL], 0);
     }
 
-    update_label(cwin);
+    uim_cand_win_gtk_update_label(cwin);
 
     if (cwin->candidate_index < 0)
       return FALSE;
     else
       return TRUE;
   } else {
-    update_label(cwin);
+    uim_cand_win_gtk_update_label(cwin);
 
     return TRUE;
   }
@@ -374,36 +378,11 @@ tree_selection_changed(GtkTreeSelection *selection,
   UIMCandWinGtk *cwin = UIM_CAND_WIN_GTK(data);
 
   if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-    gchar *annotation = NULL;
-#if HAVE_EBLIB
-    /* FIXME! This is a ad-hoc solution to advance
-       annotation related discussion. */
-    if (uim_scm_symbol_value_bool("eb-enable-for-annotation?")) {
-      gchar *cand = NULL;
+    char *annotation = NULL;
 
-      gtk_tree_model_get(model, &iter,
-			 COLUMN_CANDIDATE, &cand,
-			 -1);
-      if (cand && *cand) {
-	char *book;
-	uim_eb *ueb = NULL;
-
-	book = uim_scm_symbol_value_str("eb-dic-path");
-	if (book && *book)
-	  ueb = uim_eb_new(book);
-	if (ueb) {
-	  annotation = uim_eb_search_text(ueb, cand);
-	  uim_eb_destroy(ueb);
-	}
-	free(book);
-      }
-      g_free(cand);
-    }
-#else /* HAVE_EB_LIB */
     gtk_tree_model_get(model, &iter,
 		       COLUMN_ANNOTATION, &annotation,
 		       -1);
-#endif /* HAVE_EB_LIB */
 
     if (annotation && *annotation) {
       if (!cwin->sub_window.window)
@@ -420,7 +399,7 @@ tree_selection_changed(GtkTreeSelection *selection,
 	cwin->sub_window.active = FALSE;
       }
     }
-    g_free(annotation);
+    free(annotation);
   } else {
     if (cwin->sub_window.window) {
       gtk_widget_hide(cwin->sub_window.window);
@@ -563,7 +542,7 @@ uim_cand_win_gtk_set_candidates(UIMCandWinGtk *cwin,
         gtk_list_store_set(store, &ti,
 			   COLUMN_HEADING,    uim_candidate_get_heading_label(cand),
 			   COLUMN_CANDIDATE,  uim_candidate_get_cand_str(cand),
-			   COLUMN_ANNOTATION, NULL, /*uim_candidate_get_annotation(cand),*/
+			   COLUMN_ANNOTATION, uim_candidate_get_annotation_str(cand),
 			   TERMINATOR);
       } else {
 #if 0
@@ -587,7 +566,7 @@ uim_cand_win_gtk_set_candidates(UIMCandWinGtk *cwin,
 
   uim_cand_win_gtk_set_page(cwin, 0);
 
-  update_label(cwin);
+  uim_cand_win_gtk_update_label(cwin);
 }
 
 void
@@ -628,7 +607,7 @@ uim_cand_win_gtk_set_page_candidates(UIMCandWinGtk *cwin,
       gtk_list_store_set(store, &ti,
 			 COLUMN_HEADING,    uim_candidate_get_heading_label(cand),
 			 COLUMN_CANDIDATE,  uim_candidate_get_cand_str(cand),
-			 COLUMN_ANNOTATION, NULL, /*uim_candidate_get_annotation(cand),*/
+			 COLUMN_ANNOTATION, uim_candidate_get_annotation_str(cand),
 			 TERMINATOR);
     }
   }
@@ -658,6 +637,12 @@ uim_cand_win_gtk_get_index(UIMCandWinGtk *cwin)
 
 void
 uim_cand_win_gtk_set_index(UIMCandWinGtk *cwin, gint index)
+{
+  UIM_CAND_WIN_GTK_GET_CLASS (cwin)->set_index(cwin, index);
+}
+
+static void
+uim_cand_win_gtk_real_set_index(UIMCandWinGtk *cwin, gint index)
 {
   gint new_page;
 
@@ -692,7 +677,7 @@ uim_cand_win_gtk_set_index(UIMCandWinGtk *cwin, gint index)
     GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(cwin->view));
 
     gtk_tree_selection_unselect_all(selection);
-    update_label(cwin);
+    uim_cand_win_gtk_update_label(cwin);
   }
 }
 
@@ -715,6 +700,12 @@ uim_cand_win_gtk_get_page(UIMCandWinGtk *cwin)
 
 void
 uim_cand_win_gtk_set_page(UIMCandWinGtk *cwin, gint page)
+{
+  UIM_CAND_WIN_GTK_GET_CLASS (cwin)->set_page(cwin, page);
+}
+
+static void
+uim_cand_win_gtk_real_set_page(UIMCandWinGtk *cwin, gint page)
 {
   guint len, new_page;
   gint new_index;
@@ -880,17 +871,28 @@ uim_cand_win_gtk_set_cursor_location(UIMCandWinGtk *cwin, GdkRectangle *area)
   cwin->cursor = *area;
 }
 
+#define UIM_ANNOTATION_WIN_WIDTH 200
+#define UIM_ANNOTATION_WIN_HEIGHT 200
 
 static void
 uim_cand_win_gtk_create_sub_window(UIMCandWinGtk *cwin)
 {
-  GtkWidget *window, *scrwin, *text_view;
+  GtkWidget *window, *scrwin, *text_view, *frame;
+  GdkGeometry hints;
 
   if (cwin->sub_window.window)
     return;
 
   cwin->sub_window.window = window = gtk_window_new(GTK_WINDOW_POPUP);
-  gtk_window_set_default_size(GTK_WINDOW(window), 200, 200);
+
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
+
+  hints.min_width = UIM_ANNOTATION_WIN_WIDTH;
+  hints.min_height = UIM_ANNOTATION_WIN_HEIGHT;
+  hints.max_width = UIM_ANNOTATION_WIN_WIDTH;
+  hints.max_height = UIM_ANNOTATION_WIN_HEIGHT;
+  gtk_window_set_geometry_hints(GTK_WINDOW(window), frame, &hints, GDK_HINT_MAX_SIZE | GDK_HINT_MIN_SIZE);
 
   cwin->sub_window.scrolled_window = scrwin = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
@@ -898,11 +900,13 @@ uim_cand_win_gtk_create_sub_window(UIMCandWinGtk *cwin)
 
   cwin->sub_window.text_view = text_view = gtk_text_view_new();
   gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
-  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD);
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD_CHAR);
   gtk_widget_show(text_view);
 
   gtk_container_add(GTK_CONTAINER(scrwin), text_view);
-  gtk_container_add(GTK_CONTAINER(window), scrwin);
+  gtk_container_add(GTK_CONTAINER(frame), scrwin);
+  gtk_container_add(GTK_CONTAINER(window), frame);
+  gtk_widget_show(frame);
   gtk_widget_show(scrwin);
   gtk_widget_show(text_view);
 }
@@ -910,23 +914,49 @@ uim_cand_win_gtk_create_sub_window(UIMCandWinGtk *cwin)
 static void
 uim_cand_win_gtk_layout_sub_window(UIMCandWinGtk *cwin)
 {
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gint x, y, w, h, sw, sh, x2, y2, w2, h2;
+#else
   gint x, y, w, h, d, sw, sh, x2, y2, w2, h2, d2;
+#endif
+  GdkRectangle rect;
+  GtkTreePath *path;
+  GtkTreeViewColumn *focus_column;
 
   if (!cwin->sub_window.window)
     return;
 
-  gdk_window_get_geometry(GTK_WIDGET(cwin)->window,
+  gtk_tree_view_get_cursor(GTK_TREE_VIEW(cwin->view), &path, &focus_column);
+  gtk_tree_view_get_cell_area(GTK_TREE_VIEW(cwin->view), path, NULL, &rect);
+  gtk_tree_path_free(path);
+
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gdk_window_get_geometry(gtk_widget_get_window(GTK_WIDGET(cwin)),
+			  &x, &y, &w, &h);
+#else
+  gdk_window_get_geometry(gtk_widget_get_window(GTK_WIDGET(cwin)),
 			  &x, &y, &w, &h, &d);
-  gdk_window_get_origin(GTK_WIDGET(cwin)->window, &x, &y);
+#endif
+  gdk_window_get_origin(gtk_widget_get_window(GTK_WIDGET(cwin)), &x, &y);
 
   sw = gdk_screen_get_width  (gdk_screen_get_default ());
   sh = gdk_screen_get_height (gdk_screen_get_default ()); 
- gdk_window_get_geometry(cwin->sub_window.window->window,
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gdk_window_get_geometry(gtk_widget_get_window(cwin->sub_window.window),
+			  &x2, &y2, &w2, &h2);
+#else
+  gdk_window_get_geometry(gtk_widget_get_window(cwin->sub_window.window),
 			  &x2, &y2, &w2, &h2, &d2);
+#endif
   if (x + w + w2 > sw)
     x = x - w2;
   else
     x = x + w;
 
+  if ((y + rect.y + h2) > sh)
+    y = sh - h2;
+  else
+    y = y + rect.y;
+
   gtk_window_move(GTK_WINDOW(cwin->sub_window.window), x, y);
 }
