@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -86,6 +86,10 @@ struct _UIMCandWinGtkClass {
 
   /* signals */
   void (*index_changed) (UIMCandWinGtkClass *cwin);
+
+  /* member functions */
+  void (*set_index)		(UIMCandWinGtk *cwin, gint index);
+  void (*set_page)		(UIMCandWinGtk *cwin, gint page);
 };
 
 
@@ -133,6 +137,8 @@ void		uim_cand_win_gtk_set_cursor_location	(UIMCandWinGtk *cwin,
 							 GdkRectangle *area);
 void		uim_cand_win_gtk_get_window_pos_type	(UIMCandWinGtk *cwin);
 
+void		uim_cand_win_gtk_update_label		(UIMCandWinGtk *cwin);
+
 G_END_DECLS
 
 #endif /*UIM_GTK_UIM_CAND_WIN_GTK_H */
