@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -53,6 +53,9 @@
 #define UIM_IS_CANDIDATE_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), UIM_TYPE_CANDIDATE_WINDOW))
 #define UIM_CANDIDATE_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), UIM_TYPE_CANDIDATE_WINDOW, UIMCandidateWindowClass))
 
+#define UIM_ANNOTATION_WIN_WIDTH 200
+#define UIM_ANNOTATION_WIN_HEIGHT 200
+
 typedef struct _UIMCandidateWindow	UIMCandidateWindow;
 typedef struct _UIMCandidateWindowClass	UIMCandidateWindowClass;
 
@@ -78,6 +81,14 @@ struct _UIMCandidateWindow {
 
   gboolean is_active;
   gboolean need_hilite;
+
+  /* sub window */
+  struct sub_window {
+    GtkWidget *window;
+    GtkWidget *scrolled_window;
+    GtkWidget *text_view;
+    gboolean active;
+  } sub_window;
 };
 
 struct _UIMCandidateWindowClass {
@@ -97,6 +108,8 @@ static gint uim_cand_win_gtk_get_index(UIMCandidateWindow *cwin);
 static void uim_cand_win_gtk_set_index(UIMCandidateWindow *cwin, gint index);
 static void uim_cand_win_gtk_set_page(UIMCandidateWindow *cwin, gint page);
 static void uim_cand_win_gtk_set_page_candidates(UIMCandidateWindow *cwin, guint page, GSList *candidates);
+static void uim_cand_win_gtk_create_sub_window(UIMCandidateWindow *cwin);
+static void uim_cand_win_gtk_layout_sub_window(UIMCandidateWindow *cwin);
 
 static void uim_cand_win_gtk_layout(void);
 
@@ -111,16 +124,20 @@ enum {
 enum {
   TERMINATOR = -1,
   COLUMN_HEADING,
-  COLUMN_CANDIDATE
+  COLUMN_CANDIDATE,
+  COLUMN_ANNOTATION,
+  NR_COLUMNS
 };
 
 static void candidate_window_init(UIMCandidateWindow *cwin);
 static void candidate_window_class_init(UIMCandidateWindowClass *klass);
 
+static gboolean tree_selection_change(GtkTreeSelection *selection,
+				      GtkTreeModel *model,
+				      GtkTreePath *path,
+				      gboolean path_currently_selected,
+				      gpointer data);
 static gboolean tree_selection_changed(GtkTreeSelection *selection,
-				       GtkTreeModel *model,
-				       GtkTreePath *path,
-				       gboolean path_currently_selected,
 				       gpointer data);
 
 #if 0
@@ -210,11 +227,11 @@ update_label(UIMCandidateWindow *cwin)
 
 /* copied from uim-cand-win-gtk.c */
 static gboolean
-tree_selection_changed(GtkTreeSelection *selection,
-		       GtkTreeModel *model,
-		       GtkTreePath *path,
-		       gboolean path_currently_selected,
-		       gpointer data)
+tree_selection_change(GtkTreeSelection *selection,
+		      GtkTreeModel *model,
+		      GtkTreePath *path,
+		      gboolean path_currently_selected,
+		      gpointer data)
 {
   /* candidate_window *cwin = data; */
   gint *indicies;
@@ -247,6 +264,48 @@ tree_selection_changed(GtkTreeSelection *selection,
   }
 }
 
+/* copied from uim-cand-win-gtk.c */
+static gboolean
+tree_selection_changed(GtkTreeSelection *selection,
+                       gpointer data)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  /* UIMCandidateWindow *cwin = UIM_CANDIDATE_WINDOW(data); */
+
+  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+    char *annotation = NULL;
+
+    gtk_tree_model_get(model, &iter,
+                       COLUMN_ANNOTATION, &annotation,
+                       -1);
+
+    if (annotation && *annotation) {
+      if (!cwin->sub_window.window)
+        uim_cand_win_gtk_create_sub_window(cwin);
+      gtk_text_buffer_set_text(
+        gtk_text_view_get_buffer(GTK_TEXT_VIEW(cwin->sub_window.text_view)),
+        annotation, -1);
+      uim_cand_win_gtk_layout_sub_window(cwin);
+      gtk_widget_show(cwin->sub_window.window);
+      cwin->sub_window.active = TRUE;
+    } else {
+      if (cwin->sub_window.window) {
+        gtk_widget_hide(cwin->sub_window.window);
+        cwin->sub_window.active = FALSE;
+      }
+    }
+    free(annotation);
+  } else {
+    if (cwin->sub_window.window) {
+      gtk_widget_hide(cwin->sub_window.window);
+      cwin->sub_window.active = FALSE;
+    }
+  }
+
+  return TRUE;
+}
+
 #if 0
 /* copied from uim-cand-win-gtk.c */
 static gboolean
@@ -347,29 +406,31 @@ candidate_window_init(UIMCandidateWindow *cwin)
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(cwin->view));
 
   gtk_tree_selection_set_select_function(selection,
-		  			 tree_selection_changed,
+		  			 tree_selection_change,
 					 cwin,
 					 NULL);
+  g_signal_connect(G_OBJECT(selection), "changed",
+		   G_CALLBACK(tree_selection_changed), cwin);
 
   renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "scale", 0.8, NULL);
+  g_object_set(renderer, "scale", 0.8, (const gchar *)NULL);
 
   column = gtk_tree_view_column_new_with_attributes("No",
 						    renderer,
 						    "text",
 						    COLUMN_HEADING,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column(GTK_TREE_VIEW(cwin->view), column);
 
   renderer = gtk_cell_renderer_text_new();
-  g_object_set(renderer, "scale", 1.2, NULL);
+  g_object_set(renderer, "scale", 1.2, (const gchar *)NULL);
   /* g_object_set(renderer, "size-points", 20.0, NULL); */
   column = gtk_tree_view_column_new_with_attributes("Text",
 						    renderer,
 						    "text",
 						    COLUMN_CANDIDATE,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column(GTK_TREE_VIEW(cwin->view), column);
 
@@ -392,6 +453,11 @@ candidate_window_init(UIMCandidateWindow *cwin)
   cursor_location.y = 0;
   cursor_location.height = 0;
   caret_state_indicator_set_cursor_location(cwin->caret_state_indicator, &cursor_location);
+
+  cwin->sub_window.window = NULL;
+  cwin->sub_window.scrolled_window = NULL;
+  cwin->sub_window.text_view = NULL;
+  cwin->sub_window.active = FALSE;
 }
 
 static void
@@ -462,7 +528,7 @@ candwin_activate(gchar **str)
 
   /* create GtkListStores, and set candidates */
   for (i = 0; i < nr_stores; i++) {
-    GtkListStore *store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+    GtkListStore *store = gtk_list_store_new(NR_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     GSList *node;
 
     g_ptr_array_add(cwin->stores, store);
@@ -475,11 +541,12 @@ candwin_activate(gchar **str)
       GtkTreeIter ti;
       if (node) {
 	gchar *str = node->data;
-	gchar **column = g_strsplit(str, "\t", 2);
+	gchar **column = g_strsplit(str, "\a", 3);
 	gtk_list_store_append(store, &ti);
 	gtk_list_store_set(store, &ti,
 			   COLUMN_HEADING, column[0],
 			   COLUMN_CANDIDATE, column[1],
+			   COLUMN_ANNOTATION, column[2],
 			   TERMINATOR);
 	g_strfreev(column);
 	g_free(str);
@@ -520,8 +587,11 @@ candwin_move(char **str)
 static void
 candwin_show(void)
 {
-  if (cwin->is_active)
+  if (cwin->is_active) {
     gtk_widget_show_all(GTK_WIDGET(cwin));
+    if (cwin->sub_window.active)
+      gtk_widget_show(cwin->sub_window.window);
+  }
 }
 
 static void
@@ -529,6 +599,8 @@ candwin_deactivate(void)
 {
   gtk_widget_hide(GTK_WIDGET(cwin));
   cwin->is_active = FALSE;
+  if (cwin->sub_window.window)
+    gtk_widget_hide(cwin->sub_window.window);
 }
 
 static void
@@ -654,7 +726,7 @@ static void str_parse(gchar *str)
   gchar **tmp;
   gchar *command;
 
-  tmp = g_strsplit(str, "\n", 0);
+  tmp = g_strsplit(str, "\f", 0);
   command = tmp[0];
 
   if (command) {
@@ -665,7 +737,9 @@ static void str_parse(gchar *str)
     } else if (strcmp("show", command) == 0) {
       candwin_show();
     } else if (strcmp("hide", command) == 0) {
-      gtk_widget_hide_all(GTK_WIDGET(cwin));
+      gtk_widget_hide(GTK_WIDGET(cwin));
+      if (cwin->sub_window.window)
+        gtk_widget_hide(cwin->sub_window.window);
     } else if (strcmp("move", command) == 0) {
       candwin_move(tmp);
     } else if (strcmp("deactivate", command) == 0) {
@@ -711,7 +785,7 @@ read_cb(GIOChannel *channel, GIOCondition c, gpointer p)
     strcat(read_buf, buf);
   }
 
-  tmp = g_strsplit(read_buf, "\n\n", 0);
+  tmp = g_strsplit(read_buf, "\f\f", 0);
 
   while (tmp[i]) {
     str_parse(tmp[i]);
@@ -727,8 +801,12 @@ main(int argc, char *argv[])
 {
   GIOChannel *channel;
 
-  gtk_set_locale();
+  /* disable uim context in annotation window */
+  setenv("GTK_IM_MODULE", "gtk-im-context-simple", 1);
+
   gtk_init(&argc, &argv);
+  if (uim_init() < 0)
+    return 0;
 
   init_candidate_win();
 
@@ -738,6 +816,8 @@ main(int argc, char *argv[])
   g_io_channel_unref(channel);
 
   gtk_main();
+  uim_quit();
+
   return 0;
 }
 
@@ -851,10 +931,11 @@ uim_cand_win_gtk_set_page_candidates(UIMCandidateWindow *cwin,
   if (candidates == NULL)
     return;
 
+  cwin->sub_window.active = FALSE;
   len = g_slist_length(candidates);
 
   /* create GtkListStores, and set candidates */
-  store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+  store = gtk_list_store_new(NR_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
   cwin->stores->pdata[page] = store;
   /* set candidates */
@@ -866,11 +947,12 @@ uim_cand_win_gtk_set_page_candidates(UIMCandidateWindow *cwin,
 
     if (node) {
       gchar *str = node->data;
-      gchar **column = g_strsplit(str, "\t", 2);
+      gchar **column = g_strsplit(str, "\a", 3);
       gtk_list_store_append(store, &ti);
       gtk_list_store_set(store, &ti,
 			 COLUMN_HEADING, column[0],
 			 COLUMN_CANDIDATE, column[1],
+			 COLUMN_ANNOTATION, column[2],
 			 TERMINATOR);
 
       g_strfreev(column);
@@ -900,6 +982,97 @@ uim_cand_win_gtk_layout()
     y = cwin->pos_y;
 
   gtk_window_move(GTK_WINDOW(cwin), x, y);
+
+  uim_cand_win_gtk_layout_sub_window(cwin);
+}
+
+/* copied from uim-cand-win-gtk.c */
+static void
+uim_cand_win_gtk_create_sub_window(UIMCandidateWindow *cwin)
+{
+  GtkWidget *window, *scrwin, *text_view, *frame;
+  GdkGeometry hints;
+
+  if (cwin->sub_window.window)
+    return;
+
+  cwin->sub_window.window = window = gtk_window_new(GTK_WINDOW_POPUP);
+
+  frame = gtk_frame_new(NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
+
+  hints.min_width = UIM_ANNOTATION_WIN_WIDTH;
+  hints.min_height = UIM_ANNOTATION_WIN_HEIGHT;
+  hints.max_width = UIM_ANNOTATION_WIN_WIDTH;
+  hints.max_height = UIM_ANNOTATION_WIN_HEIGHT;
+  gtk_window_set_geometry_hints(GTK_WINDOW(window), frame, &hints, GDK_HINT_MAX_SIZE | GDK_HINT_MIN_SIZE);
+
+  cwin->sub_window.scrolled_window = scrwin = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
+                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
+  cwin->sub_window.text_view = text_view = gtk_text_view_new();
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD_CHAR);
+  gtk_widget_show(text_view);
+
+  gtk_container_add(GTK_CONTAINER(scrwin), text_view);
+  gtk_container_add(GTK_CONTAINER(frame), scrwin);
+  gtk_container_add(GTK_CONTAINER(window), frame);
+  gtk_widget_show(frame);
+  gtk_widget_show(scrwin);
+  gtk_widget_show(text_view);
+}
+
+/* copied from uim-cand-win-gtk.c */
+static void
+uim_cand_win_gtk_layout_sub_window(UIMCandidateWindow *cwin)
+{
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gint x, y, w, h, sw, sh, x2, y2, w2, h2;
+#else
+  gint x, y, w, h, sw, sh, x2, y2, w2, h2, d, d2;
+#endif
+  GdkRectangle rect;
+  GtkTreePath *path;
+  GtkTreeViewColumn *focus_column;
+
+  if (!cwin->sub_window.window)
+    return;
+
+  gtk_tree_view_get_cursor(GTK_TREE_VIEW(cwin->view), &path, &focus_column);
+  gtk_tree_view_get_cell_area(GTK_TREE_VIEW(cwin->view), path, NULL, &rect);
+  gtk_tree_path_free(path);
+
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gdk_window_get_geometry(gtk_widget_get_window(GTK_WIDGET(cwin)),
+                          &x, &y, &w, &h);
+#else
+  gdk_window_get_geometry(gtk_widget_get_window(GTK_WIDGET(cwin)),
+                          &x, &y, &w, &h, &d);
+#endif
+  gdk_window_get_origin(gtk_widget_get_window(GTK_WIDGET(cwin)), &x, &y);
+
+  sw = gdk_screen_get_width  (gdk_screen_get_default ());
+  sh = gdk_screen_get_height (gdk_screen_get_default ());
+#if GTK_CHECK_VERSION(2, 90, 0)
+  gdk_window_get_geometry(gtk_widget_get_window(cwin->sub_window.window),
+                          &x2, &y2, &w2, &h2);
+#else
+  gdk_window_get_geometry(gtk_widget_get_window(cwin->sub_window.window),
+                          &x2, &y2, &w2, &h2, &d2);
+#endif
+  if (x + w + w2 > sw)
+    x = x - w2;
+  else
+    x = x + w;
+
+  if ((y + rect.y + h2) > sh)
+    y = sh - h2;
+  else
+    y = y + rect.y;
+
+  gtk_window_move(GTK_WINDOW(cwin->sub_window.window), x, y);
 }
 
 static gboolean
