@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -52,8 +52,8 @@ static uim_dict_class *classes[] = {
 #ifdef USE_CANNA
   &uim_dict_class_canna,
 #endif
+  NULL
 };
-static unsigned int nr_classes = sizeof(classes) / sizeof(uim_dict_class *);
 
 uim_dict *
 uim_dict_open(const char *identifier)
@@ -62,7 +62,7 @@ uim_dict_open(const char *identifier)
   uim_dict *dict;
 
   /* FIXME! use hash table? */
-  for (i = 0; i < nr_classes; i++) {
+  for (i = 0; classes[i]; i++) {
     dict = classes[i]->open(identifier);
     if (dict)
       return dict;
