@@ -1,7 +1,7 @@
 /*
  *  $Id:$
  *  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
- *                2005-2008 uim Project http://code.google.com/p/uim/
+ *                2005-2011 uim Project http://code.google.com/p/uim/
  *
  *  All rights reserved.
  *
@@ -46,7 +46,11 @@
 static void word_list_view_class_init   (WordListViewClass *klass);
 static void word_list_view_init         (WordListView *view);
 static void word_list_view_finalize     (GObject *object);
+#if GTK_CHECK_VERSION(2, 90, 0)
+static void word_list_view_destroy      (GtkWidget *object);
+#else
 static void word_list_view_destroy      (GtkObject *object);
+#endif
 
 static void word_list_view_set_property (GObject *object,
 					 guint prop_id,
@@ -114,7 +118,11 @@ static void
 word_list_view_class_init(WordListViewClass *klass)
 {
     GObjectClass   *gobject_class = G_OBJECT_CLASS(klass);
+#if GTK_CHECK_VERSION(2, 90, 0)
+    GtkWidgetClass *object_class = GTK_WIDGET_CLASS(klass);
+#else
     GtkObjectClass *object_class = GTK_OBJECT_CLASS(klass);
+#endif
 
     parent_class = g_type_class_peek_parent(klass);
 
@@ -153,7 +161,7 @@ word_list_view_class_init(WordListViewClass *klass)
 	 PROP_EDITABLE,
 	 g_param_spec_boolean("editable",
 			      _("to be written"),
-			      _("to be written "),
+			      _("to be written"),
 			      FALSE,
 			      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
     g_object_class_install_property
@@ -175,7 +183,12 @@ word_list_view_init(WordListView *view)
     GtkCellRenderer *renderer;
     GtkListStore *store;
 
+#if GTK_CHECK_VERSION(2, 22, 0)
+    gtk_widget_set_can_focus(GTK_WIDGET(view), TRUE);
+    gtk_widget_set_receives_default(GTK_WIDGET(view), TRUE);
+#else
     GTK_WIDGET_SET_FLAGS(view, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
+#endif
     gtk_scrolled_window_set_policy(&view->container,
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(&view->container),
@@ -340,7 +353,11 @@ word_list_view_finalize(GObject *object)
 }
 
 static void
+#if GTK_CHECK_VERSION(2, 90, 0)
+word_list_view_destroy(GtkWidget *object)
+#else
 word_list_view_destroy(GtkObject *object)
+#endif
 {
    WordListView *view = WORD_LIST_VIEW(object);
 
@@ -349,9 +366,15 @@ word_list_view_destroy(GtkObject *object)
      view->dict = NULL;
    }
 
+#if GTK_CHECK_VERSION(2, 90, 0)
+   if (GTK_WIDGET_CLASS(parent_class)->destroy) {
+     GTK_WIDGET_CLASS(parent_class)->destroy(object);
+   }
+#else
    if (GTK_OBJECT_CLASS(parent_class)->destroy) {
      GTK_OBJECT_CLASS(parent_class)->destroy(object);
    }
+#endif
 }
 
 /*
