@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,7 +34,12 @@
 #include <config.h>
 
 #include <stdlib.h>
-#include <gdk/gdkkeysyms.h>
+# include <gtk/gtk.h>
+#if GTK_CHECK_VERSION(2, 90, 0)
+# include <gdk/gdkkeysyms-compat.h>
+#else
+# include <gdk/gdkkeysyms.h>
+#endif
 
 #include "gettext.h"
 
@@ -224,7 +229,7 @@ warn_dict_open()
 		  		  GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_OK,
-				  _(message));
+				  "%s", _(message));
   gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(GTK_WIDGET(dialog));
 }
@@ -277,13 +282,13 @@ word_list_window_init (WordListWindow *window)
   gtk_widget_show(word_list);
   gtk_box_pack_start(GTK_BOX(vbox), word_list, TRUE, TRUE, 0);
 
-  g_signal_connect(G_OBJECT(GTK_BIN(window->word_list)->child),
+  g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(window->word_list))),
 		   "button-press-event",
 		   G_CALLBACK(word_list_button_press_cb), window);
-  g_signal_connect(G_OBJECT(GTK_BIN(window->word_list)->child),
+  g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(window->word_list))),
 		   "row-activated",
 		   G_CALLBACK(word_list_row_activated_cb), window);
-  g_signal_connect(G_OBJECT(GTK_BIN(window->word_list)->child),
+  g_signal_connect(G_OBJECT(gtk_bin_get_child(GTK_BIN(window->word_list))),
 		   "key-press-event",
 		   G_CALLBACK(word_list_key_press_cb), window);
   g_signal_connect(G_OBJECT(WORD_LIST_VIEW(window->word_list)->selection),
@@ -474,7 +479,7 @@ edit_remove_word_action_cb(GtkAction *action, WordListWindow *window)
 				  GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_QUESTION,
 				  GTK_BUTTONS_OK_CANCEL,
-				  message);
+				  "%s", message);
   gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
   gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
   g_signal_connect(G_OBJECT(dialog), "response",
@@ -497,13 +502,13 @@ edit_remove_word_action_cb(GtkAction *action, WordListWindow *window)
 				      GTK_DIALOG_MODAL,
 				      GTK_MESSAGE_INFO,
 				      GTK_BUTTONS_CLOSE,
-				      _("Word deletion succeded."));
+				      "%s", _("Word deletion succeded."));
     } else {
       dialog = gtk_message_dialog_new(NULL,
 				      GTK_DIALOG_MODAL,
 				      GTK_MESSAGE_ERROR,
 				      GTK_BUTTONS_CLOSE,
-				      _("Word deletion failed."));
+				      "%s", _("Word deletion failed."));
     }
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
@@ -559,7 +564,7 @@ popup_menu_action_cb(GtkAction *action, WordListWindow *window)
   }
 }
 
-#if GTK_CHECK_VERSION(2, 6, 0)
+#if GTK_CHECK_VERSION(2, 6, 0) && !GTK_CHECK_VERSION(2, 18, 0)
 static void
 activate_url(GtkAboutDialog *about, const gchar *link, gpointer data)
 {
@@ -582,7 +587,7 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
   };
   const gchar *copyright = N_(
     "Copyright (C) 2003-2004 Masahito Omote\n"
-    "Copyright (C) 2004-2009 uim Project\n"
+    "Copyright (C) 2004-2011 uim Project\n"
     "All rights reserved.");
 
   transparent = NULL;
@@ -592,7 +597,9 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
     g_object_unref(pixbuf);
   }
   
+#if !GTK_CHECK_VERSION(2, 18, 0)
   gtk_about_dialog_set_url_hook (activate_url, NULL, NULL);
+#endif
   gtk_show_about_dialog (GTK_WINDOW(window),
 			 "name", name,
 			 "version", VERSION,
@@ -606,7 +613,7 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
   GtkWidget *about_dialog, *label1;
   const gchar *copyright = N_(
     "Copyright 2003-2004 Masahito Omote &lt;omote@utyuuzin.net&gt;\n"
-    "Copyright 2004-2009 uim Project http://code.google.com/p/uim/\n"
+    "Copyright 2004-2011 uim Project http://code.google.com/p/uim/\n"
     "All rights reserved.");
   gchar *about_name =
     g_strdup_printf("<span size=\"20000\">%s %s </span>\n\n<span size=\"14000\">%s </span>\n", _(name), VERSION, _(copyright));
@@ -621,8 +628,9 @@ help_about_action_cb(GtkAction *action, WordListWindow *window)
   gtk_widget_show(label1);
   gtk_label_set_markup(GTK_LABEL(label1), about_name);
   g_free(about_name);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about_dialog)->vbox),
-		     label1, FALSE, FALSE, 0);
+  gtk_box_pack_start(
+      GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(about_dialog))),
+      label1, FALSE, FALSE, 0);
 
   gtk_window_set_transient_for(GTK_WINDOW(about_dialog),
 			       GTK_WINDOW(window));
@@ -639,13 +647,10 @@ activate_radio_action(GtkAction *action, GtkRadioAction *current,
 		      gpointer data)
 {
   WordListWindow *window;
-  const gchar *name, *typename;
   gboolean active;
   guint value;
 
   window = WORD_LIST_WINDOW(data);
-  name = gtk_action_get_name(GTK_ACTION(current));
-  typename = G_OBJECT_TYPE_NAME(GTK_ACTION(current));
   active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(current));
   value = gtk_radio_action_get_current_value(GTK_RADIO_ACTION(current));
 
