@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -179,7 +179,7 @@ word_window_init(WordWindow *window)
 
   /* action area */
   check = gtk_check_button_new_with_label(_("continuance"));
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(window))),
 		     check, FALSE, FALSE, 0);
   gtk_widget_show(check);
   window->continuance = check;
@@ -211,8 +211,8 @@ word_window_init(WordWindow *window)
 			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			 GTK_STOCK_ADD,WORD_WINDOW_RESPONSE_ADD,
 			 NULL);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), vbox1,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(window))),
+      vbox1, TRUE, TRUE, 0);
   gtk_widget_show_all(vbox1);
 
   window->pane = GTK_WIDGET(vbox1);
@@ -299,13 +299,22 @@ word_window_necessary_create(WordWindow *window)
     gint pos_num;
 
     pos_num = sizeof(pos_broad) / sizeof(pos_broad[0]);
+#if GTK_CHECK_VERSION(2, 24, 0)
+    combobox_pos_broad = gtk_combo_box_text_new();
+#else
     combobox_pos_broad = gtk_combo_box_new_text();
+#endif
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), combobox_pos_broad);
     gtk_widget_show(combobox_pos_broad);
 
     for (i = 0; i < pos_num; i++) {
+#if GTK_CHECK_VERSION(2, 24, 0)
+      gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combobox_pos_broad),
+				_(pos_broad[i]));
+#else
       gtk_combo_box_append_text(GTK_COMBO_BOX(combobox_pos_broad),
 				_(pos_broad[i]));
+#endif
     }
 
     alignment_pos_broad = gtk_alignment_new(0, 0.5, 0, 0);
@@ -357,7 +366,7 @@ word_window_additional_create(WordWindow *window)
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 5, 5);
 
   entry = gtk_entry_new();
-  gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+  gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
   gtk_widget_show(entry);
   gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 0, 1, GTK_FILL, 0, 5, 5);
@@ -497,14 +506,14 @@ word_window_add(WordWindow *window)
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_CLOSE,
-				    _("Word registration failed."));
+				    "%s", _("Word registration failed."));
   } else {
     g_signal_emit(G_OBJECT(window), word_window_signals[WORD_ADDED_SIGNAL], 0);
     dialog = gtk_message_dialog_new(NULL,
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_INFO,
 				    GTK_BUTTONS_CLOSE,
-				    _("Word registration succeded."));
+				    "%s", _("Word registration succeded."));
   }
 
   g_signal_connect(G_OBJECT(dialog), "response",
@@ -516,7 +525,7 @@ word_window_add(WordWindow *window)
 
   /* do not destroy the window when the continuance check box is checked */
   if (!word_window_is_continuance_mode(window)) {
-    gtk_idle_add(idle_wordwin_destroy, window);
+    g_idle_add(idle_wordwin_destroy, window);
     uim_dict_unref(window->dict);
   }
 }
@@ -534,7 +543,7 @@ word_window_validate_values(WordWindow *window)
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_CLOSE,
-				    _("Phonetic is emtpy!"));
+				    "%s", _("Phonetic is emtpy!"));
     g_signal_connect(G_OBJECT(dialog), "response",
 		     G_CALLBACK(message_dialog_response_cb), window);
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
@@ -549,7 +558,7 @@ word_window_validate_values(WordWindow *window)
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_CLOSE,
-				    _("Literal is emtpy!"));
+				    "%s", _("Literal is emtpy!"));
     g_signal_connect(G_OBJECT(dialog), "response",
 		     G_CALLBACK(message_dialog_response_cb), window);
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
@@ -567,7 +576,7 @@ word_window_validate_values(WordWindow *window)
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_CLOSE,
-				    _("Part of speech is emtpy!"));
+				    "%s", _("Part of speech is emtpy!"));
     g_signal_connect(G_OBJECT(dialog), "response",
 		     G_CALLBACK(message_dialog_response_cb), window);
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
@@ -603,7 +612,7 @@ word_window_response(GtkDialog *dialog, gint arg)
     break;
   case GTK_RESPONSE_CLOSE:
   case GTK_RESPONSE_CANCEL:
-    gtk_idle_add(idle_wordwin_destroy, dialog);
+    g_idle_add(idle_wordwin_destroy, dialog);
     uim_dict_unref(WORD_WINDOW(dialog)->dict);
     break;
   default:
@@ -617,7 +626,7 @@ message_dialog_response_cb(GtkDialog *dialog, gint arg, WordWindow *window)
   switch (arg)
   {
   case GTK_RESPONSE_CLOSE:
-    gtk_idle_add(idle_wordwin_destroy, dialog);
+    g_idle_add(idle_wordwin_destroy, dialog);
     break;
   default:
     break;
