@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -43,6 +43,8 @@
 
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
+#include <uim/uim-custom.h>
+#include <uim/uim-scm.h>
 #include "uim/gettext.h"
 
 
@@ -50,6 +52,7 @@ static unsigned int read_tag;
 static int uim_fd; /* file descriptor to connect helper message bus */
 static gchar *im_list_str_old; /* To compare new im_list_str */
 static GtkWidget *switcher_tree_view;
+static int custom_enabled;
 
 static gboolean
 reload_im_list(GtkWindow *window, gpointer user_data);
@@ -193,11 +196,9 @@ get_selected_im_name(void)
 {
   GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(switcher_tree_view));
   GtkTreeModel *model;
-  GtkTreeStore *store;
   GtkTreeIter iter;
   gchar *str_data;
   if (gtk_tree_selection_get_selected (sel, &model, &iter) == TRUE) {
-    store = GTK_TREE_STORE(model);
     gtk_tree_model_get (model, &iter,
 			NAME_COLUMN, &str_data,
 			-1);
@@ -235,12 +236,28 @@ toggle_coverage(GtkToggleButton *togglebutton, gpointer user_data)
   }
 }
 
+static void
+save_default_im()
+{
+  if (custom_enabled) {
+    gchar *im_name = get_selected_im_name();
+
+    uim_scm_callf("custom-set-value!",
+		  "yy",
+		  "custom-preserved-default-im-name",
+		  im_name);
+    uim_custom_save_custom("custom-preserved-default-im-name");
+    g_free(im_name);
+  }
+}
+
 static void
 change_input_method(GtkButton *button, gpointer user_data)
 {
   switch (coverage) {
   case IMSW_COVERAGE_WHOLE_DESKTOP:
     send_message_im_change("im_change_whole_desktop\n");
+    save_default_im();
     break;
   case IMSW_COVERAGE_THIS_APPLICATION_ONLY:
     send_message_im_change("im_change_this_application_only\n");
@@ -249,6 +266,12 @@ change_input_method(GtkButton *button, gpointer user_data)
     send_message_im_change("im_change_this_text_area_only\n");
     break;
   }
+}
+
+static void
+change_input_method_and_quit(GtkButton *button, gpointer user_data)
+{
+  change_input_method(button, user_data);
   gtk_main_quit();
 }
 
@@ -283,7 +306,7 @@ create_switcher_treeview(void)
   column = gtk_tree_view_column_new_with_attributes(_("InputMethodName"),
 						    renderer,
 						    "text", NAME_COLUMN,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sort_column_id(column, 0);
   gtk_tree_view_append_column(GTK_TREE_VIEW(switcher_tree_view), column);
 
@@ -292,7 +315,7 @@ create_switcher_treeview(void)
   column = gtk_tree_view_column_new_with_attributes(_("Language"),
 						    renderer,
 						    "text", LANG_COLUMN,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sort_column_id(column, 1);
   gtk_tree_view_append_column(GTK_TREE_VIEW(switcher_tree_view), column);
 
@@ -301,7 +324,7 @@ create_switcher_treeview(void)
   column = gtk_tree_view_column_new_with_attributes(_("Description"),
 						    renderer,
 						    "text", DESC_COLUMN,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sort_column_id(column, 2);
   gtk_tree_view_append_column(GTK_TREE_VIEW(switcher_tree_view), column);
 
@@ -385,8 +408,14 @@ create_switcher(void)
   gtk_button_box_set_layout(GTK_BUTTON_BOX(setting_button_box), GTK_BUTTONBOX_END);
   gtk_box_set_spacing(GTK_BOX(setting_button_box), 8);
 
+  /* Apply button */
+  button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
+  g_signal_connect(G_OBJECT(button), "clicked",
+		   G_CALLBACK(change_input_method), NULL);
+  gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 2);
+
   /* Cancel button */
-  button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(gtk_main_quit), NULL);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 2);
@@ -394,7 +423,7 @@ create_switcher(void)
   /* OK button */
   button = gtk_button_new_from_stock(GTK_STOCK_OK);
   g_signal_connect(G_OBJECT(button), "clicked",
-		   G_CALLBACK(change_input_method), NULL);
+		   G_CALLBACK(change_input_method_and_quit), NULL);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 2);
 
   gtk_box_pack_start(GTK_BOX(vbox3), setting_button_box, FALSE, FALSE, 8);
@@ -430,6 +459,10 @@ parse_helper_str(const char *sent_str)
 {
   if (g_str_has_prefix(sent_str, "im_list") == TRUE) {
     parse_helper_str_im_list(sent_str);
+  } else if (g_str_has_prefix(sent_str, "im_switcher_start") == TRUE) {
+    uim_helper_send_message(uim_fd, "im_switcher_quit\n"); 
+  } else if (g_str_has_prefix(sent_str, "im_switcher_quit") == TRUE) {
+    gtk_main_quit();
   }
 }
 
@@ -537,14 +570,29 @@ main(int argc, char *argv[])
 {
   gint result;
   setlocale(LC_ALL, "");
-  gtk_set_locale();
   bindtextdomain( PACKAGE, LOCALEDIR );
   textdomain( PACKAGE );
   bind_textdomain_codeset( PACKAGE, "UTF-8");
   parse_arg(argc, argv);
 
+  /* connect to uim helper message bus */
+  uim_fd = -1;
+  check_helper_connection();
+
+  /* To check if another uim-im-switcher exists */
+  uim_helper_send_message(uim_fd, "im_switcher_start\n"); 
+
+  /* To load input method list */
+  uim_helper_send_message(uim_fd, "im_list_get\n");
+
   gtk_init(&argc, &argv);
 
+  if (uim_init() < 0) {
+    fprintf(stderr, "uim_init() failed.\n");
+    exit(EXIT_FAILURE);
+  }
+  custom_enabled = uim_custom_enable();
+
   result = create_switcher();
 
   if (result == -1) {
@@ -552,13 +600,6 @@ main(int argc, char *argv[])
     exit(EXIT_FAILURE);
   }
 
-  /* connect to uim helper message bus */
-  uim_fd = -1;
-  check_helper_connection();
-
-  /* To load input method list */
-  uim_helper_send_message(uim_fd, "im_list_get\n");
-
   load_configration(NULL);
   gtk_main ();
   return 0;
