@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -33,7 +33,7 @@
 
 #include <config.h>
 
-#include <pref-gtk-custom-widgets.h>
+#include "pref-gtk-custom-widgets.h"
 #include <gdk/gdkkeysyms.h>
 
 #include <string.h>
@@ -259,9 +259,10 @@ sync_value_int(GtkSpinButton *spin)
   custom = uim_custom_get(custom_sym);
   g_return_if_fail(custom && custom->type == UCustom_Int);
 
-  gtk_widget_set_sensitive(GTK_WIDGET(spin)->parent, custom->is_active);
-  if (custom->range->as_int.min != (int) adj->lower ||
-      custom->range->as_int.max != (int) adj->upper)
+  gtk_widget_set_sensitive(gtk_widget_get_parent(GTK_WIDGET(spin)),
+      custom->is_active);
+  if (custom->range->as_int.min != (int) gtk_adjustment_get_lower(adj) ||
+      custom->range->as_int.max != (int) gtk_adjustment_get_upper(adj))
     gtk_spin_button_set_range(spin, custom->range->as_int.min, custom->range->as_int.max);
   if (custom->value->as_int != (int) gtk_spin_button_get_value(spin)) {
     gtk_spin_button_set_value(spin, custom->value->as_int);
@@ -382,7 +383,8 @@ sync_value_string(GtkEntry *entry)
 		   (custom->type == UCustom_Str ||
 		    custom->type == UCustom_Pathname));
 
-  gtk_widget_set_sensitive(GTK_WIDGET(entry)->parent, custom->is_active);
+  gtk_widget_set_sensitive(gtk_widget_get_parent(GTK_WIDGET(entry)),
+      custom->is_active);
   if (custom->type == UCustom_Str) {
     gtk_entry_set_text(GTK_ENTRY(entry), custom->value->as_str);
   } else if (custom->type == UCustom_Pathname) {
@@ -451,7 +453,7 @@ custom_pathname_button_clicked_cb(GtkWidget *button, GtkWidget *entry)
 					action,
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-					NULL);
+					(const gchar *)NULL);
 
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
     char *filename;
@@ -600,7 +602,12 @@ sync_value_choice(GtkComboBox *combobox)
   default_symbol = custom->value->as_choice->symbol;
 
   while (*item) {
+#if GTK_CHECK_VERSION(2, 24, 0)
+    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combobox),
+        (*item)->label);
+#else
     gtk_combo_box_append_text(GTK_COMBO_BOX(combobox), (*item)->label);
+#endif
     if (!strcmp(default_symbol, (*item)->symbol))
       default_index = i;
     i++;
@@ -608,7 +615,8 @@ sync_value_choice(GtkComboBox *combobox)
   }
   gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), default_index);
 
-  gtk_widget_set_sensitive(GTK_WIDGET(combobox)->parent, custom->is_active);
+  gtk_widget_set_sensitive(gtk_widget_get_parent(GTK_WIDGET(combobox)),
+      custom->is_active);
 
   g_signal_handlers_unblock_by_func(G_OBJECT(combobox),
 				    (gpointer)(uintptr_t) custom_combo_box_changed, NULL);
@@ -635,7 +643,11 @@ add_custom_type_choice(GtkWidget *vbox, struct uim_custom *custom)
   label = gtk_label_new(custom->label);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(2, 24, 0)
+  combobox = gtk_combo_box_text_new();
+#else
   combobox = gtk_combo_box_new_text();
+#endif
   gtk_box_pack_start (GTK_BOX (hbox), combobox, FALSE, TRUE, 0);
 
   g_object_set_data_full(G_OBJECT(combobox),
@@ -688,7 +700,8 @@ sync_value_olist(GtkEntry *entry)
   }
 
   gtk_entry_set_text(GTK_ENTRY(entry), str->str);
-  gtk_widget_set_sensitive(GTK_WIDGET(entry)->parent, custom->is_active);
+  gtk_widget_set_sensitive(gtk_widget_get_parent(GTK_WIDGET(entry)),
+      custom->is_active);
 
   g_string_free(str, TRUE);
   uim_custom_free(custom);
@@ -1183,8 +1196,8 @@ choose_olist_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
 
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+      hbox, TRUE, TRUE, 0);
   gtk_widget_show(hbox);
 
   /* left tree view */
@@ -1222,7 +1235,7 @@ choose_olist_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   column = gtk_tree_view_column_new_with_attributes(_("Enabled items"),
 						    renderer,
 						    "text", 0,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
 
   g_object_unref(store);
@@ -1326,7 +1339,7 @@ choose_olist_clicked_cb(GtkWidget *widget, GtkEntry *olist_entry)
   column = gtk_tree_view_column_new_with_attributes(_("Disabled items"),
 						    renderer,
 						    "text", 0,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
 
   g_object_unref(store);
@@ -1358,7 +1371,7 @@ add_custom_type_orderedlist(GtkWidget *vbox, struct uim_custom *custom)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
 
   entry = gtk_entry_new();
-  gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+  gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label(_("Edit..."));
@@ -1508,13 +1521,12 @@ static void
 choose_key_button_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
 {
   GtkWidget *dialog;
-  gint rv;
 
   dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)),
 				  GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_INFO,
 				  GTK_BUTTONS_CANCEL,
-				  _("Press keys to grab (e.g. <Control>a)"));
+				  "%s", _("Press keys to grab (e.g. <Control>a)"));
   gtk_window_set_title(GTK_WINDOW(dialog), _("Grabbing keys"));
   g_signal_connect(G_OBJECT(dialog), "key-press-event",
 		   G_CALLBACK(grab_win_key_press_cb), key_entry);
@@ -1522,11 +1534,11 @@ choose_key_button_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
 		   G_CALLBACK(grab_win_key_release_cb), key_entry);
 
   gtk_widget_realize(dialog);
-  gdk_keyboard_grab(GTK_WIDGET(dialog)->window,
+  gdk_keyboard_grab(gtk_widget_get_window(GTK_WIDGET(dialog)),
 		    TRUE, GDK_CURRENT_TIME);
 
 
-  rv = gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_dialog_run(GTK_DIALOG(dialog));
 
   gtk_widget_destroy(dialog);
 }
@@ -1562,7 +1574,8 @@ sync_value_key(GtkEntry *entry)
   }
 
   gtk_entry_set_text(GTK_ENTRY(entry), str->str);
-  gtk_widget_set_sensitive(GTK_WIDGET(entry)->parent, custom->is_active);
+  gtk_widget_set_sensitive(gtk_widget_get_parent(GTK_WIDGET(entry)),
+      custom->is_active);
 
   g_string_free(str, TRUE);
   uim_custom_free(custom);
@@ -1725,7 +1738,6 @@ choose_key_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
   struct uim_custom *custom;
   struct uim_custom_key *key;
   gint i;
-  GtkIMContext *im_context;
   gchar title[256];
 
   g_return_if_fail(GTK_IS_ENTRY(key_entry));
@@ -1754,8 +1766,8 @@ choose_key_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
 
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,
-		     TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+      hbox, TRUE, TRUE, 0);
   gtk_widget_show(hbox);
 
   scrwin = gtk_scrolled_window_new(NULL, NULL);
@@ -1778,7 +1790,7 @@ choose_key_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
   column = gtk_tree_view_column_new_with_attributes(_("Key preference"),
 						    renderer,
 						    "text", 0,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
@@ -1817,7 +1829,8 @@ choose_key_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
   gtk_widget_show(button);
 
   hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
+      hbox, FALSE, FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
   gtk_widget_show(hbox);
 
@@ -1826,13 +1839,7 @@ choose_key_clicked_cb(GtkWidget *widget, GtkEntry *key_entry)
   gtk_widget_show(label);
 
   entry = gtk_entry_new();
-
-  /* XXX hack alert!  This modifies private part of gtk_entry */
-  im_context = gtk_im_context_simple_new();
-  g_object_unref(GTK_ENTRY(entry)->im_context);
-  GTK_ENTRY(entry)->im_context = im_context;
-
-  gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+  gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
   key_pref_win.keycode_entry = entry;
   gtk_widget_set_size_request(GTK_WIDGET(entry), 100, -1);
   gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 4);
@@ -1891,7 +1898,7 @@ add_custom_type_key(GtkWidget *vbox, struct uim_custom *custom)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
 
   entry = gtk_entry_new();
-  gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+  gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
   g_object_set_data_full(G_OBJECT(entry),
 			 OBJECT_DATA_UIM_CUSTOM_SYM, g_strdup(custom->symbol),
 			 (GDestroyNotify) g_free);
