@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,7 +34,12 @@
 #include <config.h>
 
 #include <glib.h>
-#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+#if GTK_CHECK_VERSION(2, 90, 0)
+# include <gdk/gdkkeysyms-compat.h>
+#else
+# include <gdk/gdkkeysyms.h>
+#endif
 
 #include <string.h>
 #include <stdlib.h>
@@ -43,6 +48,7 @@
 
 #include "uim/uim.h"
 #include "uim/uim-custom.h"
+#include "uim/uim-scm.h"
 #include "uim/gettext.h"
 #include "pref-gtk-custom-widgets.h"
 #include "../gtk/key-util-gtk.h"
@@ -223,7 +229,7 @@ quit_confirm(void)
 				    GTK_DIALOG_MODAL,
 				    GTK_MESSAGE_QUESTION,
 				    GTK_BUTTONS_YES_NO,
-				    _("Some value(s) have been changed.\n"
+				    "%s", _("Some value(s) have been changed.\n"
 				      "Do you really quit this program?"));
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(pref_window));
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
@@ -278,7 +284,7 @@ create_pref_treeview(void)
   column = gtk_tree_view_column_new_with_attributes(_("Group"),
 						    renderer,
 						    "text", GROUP_COLUMN,
-						    NULL);
+						    (const gchar *)NULL);
   gtk_tree_view_column_set_sort_column_id(column, 0);
   gtk_tree_view_append_column(GTK_TREE_VIEW(pref_tree_view), column);
 
@@ -370,7 +376,6 @@ create_setting_button_box(const char *group_name)
 {
   GtkWidget *setting_button_box;
   GtkWidget *button;
-  GtkTooltips *tooltip;
 
   setting_button_box = gtk_hbutton_box_new();
   gtk_button_box_set_layout(GTK_BUTTON_BOX(setting_button_box), GTK_BUTTONBOX_END);
@@ -381,8 +386,7 @@ create_setting_button_box(const char *group_name)
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(defaults_button_clicked), (gpointer) group_name);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
-  tooltip = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltip, button, _("Revert all changes to default"), NULL);
+  gtk_widget_set_tooltip_text(button, _("Revert all changes to default"));
 
 
   /* Apply button */
@@ -391,16 +395,14 @@ create_setting_button_box(const char *group_name)
 		   G_CALLBACK(apply_button_clicked), (gpointer) group_name);
   gtk_widget_set_sensitive(pref_apply_button, FALSE);
   gtk_box_pack_start(GTK_BOX(setting_button_box), pref_apply_button, TRUE, TRUE, 8);
-  tooltip = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltip, pref_apply_button, _("Apply all changes"), NULL);
+  gtk_widget_set_tooltip_text(pref_apply_button, _("Apply all changes"));
 
   /* Cancel button */
   button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(quit_confirm), NULL);
   gtk_box_pack_start(GTK_BOX(setting_button_box), button, TRUE, TRUE, 8);
-  tooltip = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltip, button, _("Quit this application without applying changes"), NULL);
+  gtk_widget_set_tooltip_text(button, _("Quit this application without applying changes"));
 
   /* OK button */
   pref_ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
@@ -410,8 +412,7 @@ create_setting_button_box(const char *group_name)
 #if USE_CHANGES_SENSITIVE_OK_BUTTON
   gtk_widget_set_sensitive(pref_ok_button, FALSE);
 #endif
-  tooltip = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltip, pref_ok_button, _("Quit this application with applying changes"), NULL);
+  gtk_widget_set_tooltip_text(pref_ok_button, _("Quit this application with applying changes"));
 
   return setting_button_box;
 }
@@ -567,14 +568,23 @@ create_pref_window(void)
 static gboolean
 check_dot_uim_file(void)
 {
-  GString *dot_uim = g_string_new(g_get_home_dir());
+  GString *dot_uim;
   GtkWidget *dialog;
   const gchar *message =
     N_("The user customize file \"~/.uim\" is found.\n"
        "This file will override all conflicted settings set by\n"
        "this tool (stored in ~/.uim.d/customs/*.scm).\n"
-       "Please check the file if you find your settings aren't applied.");
+       "Please check the file if you find your settings aren't applied.\n\n"
+       "(To suppress this dialog, add following line to ~/.uim)\n"
+       "(define uim-pref-suppress-dot-uim-warning-dialog? #t)");
+  uim_bool suppress_dialog;
 
+  suppress_dialog = uim_scm_symbol_value_bool("uim-pref-suppress-dot-uim-warning-dialog?");
+  if (suppress_dialog) {
+    return FALSE;
+  }
+
+  dot_uim = g_string_new(g_get_home_dir());
   g_string_append(dot_uim, "/.uim");
 
   if (!g_file_test(dot_uim->str, G_FILE_TEST_EXISTS)) {
@@ -587,18 +597,12 @@ check_dot_uim_file(void)
 				  GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_OK,
-				  _(message));
+				  "%s", _(message));
   if (pref_window) {
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(pref_window));
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
   }
 
-  /*
-   *  2005-02-07 Takuro Ashie <ashie@homa.ne.jp>
-   *    FIXME! We shoud add a check box like
-   *    "Show this message every time on start up.".
-   */
-
   gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(GTK_WIDGET(dialog));
 
@@ -606,15 +610,13 @@ check_dot_uim_file(void)
 }
 
 int
-main (int argc, char *argv[])
+main(int argc, char *argv[])
 {
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
 
-  gtk_set_locale();
-
   gtk_init(&argc, &argv);
 
   if (uim_init() < 0) {
@@ -626,11 +628,8 @@ main (int argc, char *argv[])
     GtkWidget *pref;
 
     im_uim_init_modifier_keys();
-
-    gtk_idle_add((GtkFunction) check_dot_uim_file, NULL);
-
+    g_idle_add((GSourceFunc) check_dot_uim_file, NULL);
     pref = create_pref_window();
-
     gtk_widget_show_all(pref);
 
     gtk_main();
