@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,10 +37,12 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 
 #include <uim/uim.h>
 #include "uim/uim-helper.h"
 #include "uim/uim-scm.h"
+#include "uim/uim-custom.h"
 #include "uim/gettext.h"
 
 #define OBJECT_DATA_PROP_BUTTONS "PROP_BUTTONS"
@@ -56,6 +58,7 @@ GtkWidget *uim_toolbar_trayicon_new(void);
 GtkWidget *uim_toolbar_applet_new(void);
 void uim_toolbar_check_helper_connection(GtkWidget *widget);
 void uim_toolbar_get_im_list(void);
+void uim_toolbar_launch_helper_application(const char *command);
 
 
 enum {
@@ -84,7 +87,11 @@ static struct _CommandEntry command_entry[] = {
     N_("Switch input method"),
     NULL,
     "im_switcher",
-    "uim-im-switcher-gtk &",
+#if GTK_CHECK_VERSION(2, 90, 0)
+    "uim-im-switcher-gtk3",
+#else
+    "uim-im-switcher-gtk",
+#endif
     "toolbar-show-switcher-button?",
     UIM_FALSE
   },
@@ -93,7 +100,11 @@ static struct _CommandEntry command_entry[] = {
     N_("Preference"),
     NULL,
     GTK_STOCK_PREFERENCES,
-    "uim-pref-gtk &",
+#if GTK_CHECK_VERSION(2, 90, 0)
+    "uim-pref-gtk3",
+#else
+    "uim-pref-gtk",
+#endif
     "toolbar-show-pref-button?",
     UIM_FALSE
   },
@@ -102,7 +113,11 @@ static struct _CommandEntry command_entry[] = {
     N_("Japanese dictionary editor"),
     NULL,
     "uim-dict",
-    "uim-dict-gtk &",
+#if GTK_CHECK_VERSION(2, 90, 0)
+    "uim-dict-gtk3",
+#else
+    "uim-dict-gtk",
+#endif
     "toolbar-show-dict-button?",
     UIM_FALSE
   },
@@ -111,7 +126,11 @@ static struct _CommandEntry command_entry[] = {
     N_("Input pad"),
     NULL,
     GTK_STOCK_BOLD,
-    "uim-input-pad-ja &",
+#if GTK_CHECK_VERSION(2, 90, 0)
+    "uim-input-pad-ja-gtk3",
+#else
+    "uim-input-pad-ja",
+#endif
     "toolbar-show-input-pad-button?",
     UIM_FALSE
   },
@@ -124,7 +143,7 @@ static struct _CommandEntry command_entry[] = {
 #else
     NULL,
 #endif
-    "uim-tomoe-gtk &",
+    "uim-tomoe-gtk",
     "toolbar-show-handwriting-input-pad-button?",
     UIM_FALSE
   },
@@ -133,7 +152,7 @@ static struct _CommandEntry command_entry[] = {
     N_("Help"),
     NULL,
     GTK_STOCK_HELP,
-    "uim-help &",
+    "uim-help",
     "toolbar-show-help-button?",
     UIM_FALSE
   }
@@ -149,10 +168,35 @@ static int uim_fd;
 static GtkIconFactory *uim_factory;
 static GList *uim_icon_list;
 static gboolean prop_menu_showing = FALSE;
+static gboolean custom_enabled;
+static gboolean with_dark_bg;
 
+static void set_button_style(GtkWidget *button);
 static const char *safe_gettext(const char *msgid);
 static gboolean has_n_strs(gchar **str_list, guint n);
 static gboolean register_icon(const gchar *name);
+static void reset_icon(void);
+
+static void
+set_button_style(GtkWidget *button)
+{
+#if GTK_CHECK_VERSION(2, 90, 0)
+    GtkStyleContext *context = gtk_widget_get_style_context(button);
+    GtkCssProvider *provider = gtk_css_provider_new();
+    gtk_css_provider_load_from_data(provider,
+                                    "#uim-systray-button {\n"
+                                    " -GtkWidget-focus-line-width: 0;\n"
+                                    " -GtkWidget-focus-padding: 0;\n"
+                                    " padding-top: 0;\n"
+                                    " padding-bottom: 0;\n"
+                                    "}\n", -1, NULL);
+    gtk_style_context_add_provider(context,
+                                   GTK_STYLE_PROVIDER(provider),
+                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_object_unref(provider);
+#endif
+    gtk_widget_set_name(button, "uim-systray-button");
+}
 
 static const char *
 safe_gettext(const char *msgid)
@@ -181,15 +225,6 @@ has_n_strs(gchar **str_list, guint n)
   return TRUE;
 }
 
-static void
-destroy_tooltips(GtkWidget *widget)
-{
-  GtkTooltipsData *ttd = gtk_tooltips_data_get(widget);
-
-  if (ttd)
-   g_signal_emit_by_name(ttd->tooltips, "destroy");
-}
-
 static void
 calc_menu_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		   GtkWidget *button)
@@ -200,11 +235,23 @@ calc_menu_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
   g_return_if_fail(x && y);
   g_return_if_fail(GTK_IS_BUTTON(button));
 
-  gdk_window_get_origin(button->window, x, y);
-  gdk_drawable_get_size(button->window, NULL, &button_height);
+  gdk_window_get_origin(gtk_widget_get_window(button), x, y);
+#if GTK_CHECK_VERSION(2, 90, 0)
+  button_height = gdk_window_get_height(gtk_widget_get_window(button));
+#else
+  gdk_drawable_get_size(gtk_widget_get_window(button), NULL, &button_height);
+#endif
 
+#if GTK_CHECK_VERSION(2, 18, 0)
+  if (!gtk_widget_get_has_window(button)) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(button, &allocation);
+    *x += allocation.x;
+  }
+#else
   if (GTK_WIDGET_NO_WINDOW(button))
     *x += button->allocation.x;
+#endif
 
   sc_height = gdk_screen_get_height(gdk_screen_get_default());
   sc_width = gdk_screen_get_width(gdk_screen_get_default());
@@ -233,13 +280,25 @@ right_click_menu_quit_activated(GtkMenu *menu_item, gpointer data)
   gtk_main_quit();
 }
 
+void
+uim_toolbar_launch_helper_application(const char *command)
+{
+  if (command) {
+    if (!g_spawn_command_line_async(command, NULL)) {
+      GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+          GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+          _("Cannot launch '%s'."), command);
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(GTK_WIDGET(dialog));
+    }
+  }
+}
+
 static void
 right_click_menu_activated(GtkMenu *menu_item, gpointer data)
 {
   const char *command = data;
-
-  if (command)
-    system(command);
+  uim_toolbar_launch_helper_application(command);
 }
 
 static gboolean
@@ -253,16 +312,66 @@ right_button_pressed(GtkButton *button, GdkEventButton *event, gpointer data)
   return FALSE;
 }
 
+static void
+save_default_im_internal(const char *im)
+{
+  uim_scm_callf("custom-set-value!",
+		"yy",
+		"custom-preserved-default-im-name",
+		im);
+  uim_custom_save_custom("custom-preserved-default-im-name");
+}
+
+static void
+save_default_im(const char *im)
+{
+  if (custom_enabled)
+    uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)save_default_im_internal, (void *)im);
+}
+
+static gboolean
+is_msg_imsw(const gchar *str)
+{
+  return g_str_has_prefix(str, "action_imsw_");
+}
+
+static gboolean
+is_imsw_coverage_system_global()
+{
+  char *coverage;
+  gboolean ret;
+
+  coverage = uim_scm_symbol_value_str("imsw-coverage");
+
+  ret =  (gboolean)!strcmp(coverage, "system-global");
+  free(coverage);
+
+  return ret;
+}
+
+static const char*
+get_imsw_im(const gchar *str)
+{
+  /* skip "action_imsw_" */
+  return str + strlen("action_imsw_");
+}
+
 static void
 prop_menu_activate(GtkMenu *menu_item, gpointer data)
 {
   GString *msg;
+  const gchar *str;
 
-  msg = g_string_new((gchar *)g_object_get_data(G_OBJECT(menu_item),
-		     "prop_action"));
+  str = g_object_get_data(G_OBJECT(menu_item), "prop_action");
+  msg = g_string_new(str);
   g_string_prepend(msg, "prop_activate\n");
   g_string_append(msg, "\n");
   uim_helper_send_message(uim_fd, msg->str);
+  if (is_msg_imsw(str) && is_imsw_coverage_system_global()) {
+    const char *im = get_imsw_im(str);
+    save_default_im(im);
+  }
+
   g_string_free(msg, TRUE);
 }
 
@@ -279,7 +388,6 @@ popup_prop_menu(GtkButton *prop_button, GdkEventButton *event,
 		GtkWidget *widget)
 {
   GtkWidget *menu_item, *hbox, *label, *img;
-  GtkTooltips *tooltip;
   GList *menu_item_list, *icon_list, *label_list, *tooltip_list, *action_list,
 	*state_list, *list;
   int i, selected = -1;
@@ -295,7 +403,6 @@ popup_prop_menu(GtkButton *prop_button, GdkEventButton *event,
 
   list = menu_item_list;
   while (list) {
-    destroy_tooltips(list->data);
     gtk_widget_destroy(list->data);
     list = list->next;
   }
@@ -340,13 +447,19 @@ popup_prop_menu(GtkButton *prop_button, GdkEventButton *event,
       if (i == selected)
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
     } else {
-      menu_item = gtk_menu_item_new_with_label(label_list->data);
+      menu_item = gtk_image_menu_item_new_with_label(label_list->data);
+      if (register_icon(icon_list->data)) {
+	img = gtk_image_new_from_stock(icon_list->data, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item), img); 
+#if GTK_CHECK_VERSION(2, 16, 0)
+	gtk_image_menu_item_set_always_show_image(GTK_IMAGE_MENU_ITEM(menu_item), TRUE);
+#endif
+      }
     }
 
     /* tooltips */
-    tooltip = gtk_tooltips_new();
-    gtk_tooltips_set_tip(tooltip, menu_item,
-			 tooltip_list ? tooltip_list->data : NULL, NULL);
+    gtk_widget_set_tooltip_text(menu_item,
+			 tooltip_list ? tooltip_list->data : NULL);
 
     /* add to the menu */
     gtk_menu_shell_append(GTK_MENU_SHELL(prop_menu), menu_item);
@@ -431,7 +544,13 @@ tool_button_clicked_cb(GtkButton *tool_button, GtkWidget *widget)
 
   command = g_object_get_data(G_OBJECT(tool_button), OBJECT_DATA_COMMAND);
   if (command)
-    system(command);
+    if (!g_spawn_command_line_async(command, NULL)) {
+      GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+          GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+          _("Cannot launch '%s'."), command);
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(GTK_WIDGET(dialog));
+    }
 }
 
 
@@ -467,7 +586,6 @@ prop_data_flush(gpointer data)
 static void
 prop_button_destroy(gpointer data, gpointer user_data)
 {
-  destroy_tooltips(data);
   prop_data_flush(data);
   gtk_widget_destroy(GTK_WIDGET(data));
 }
@@ -475,7 +593,6 @@ prop_button_destroy(gpointer data, gpointer user_data)
 static void
 tool_button_destroy(gpointer data, gpointer user_data)
 {
-  destroy_tooltips(data);
   gtk_widget_destroy(GTK_WIDGET(data));
 }
 
@@ -484,7 +601,6 @@ prop_button_create(GtkWidget *widget, const gchar *icon_name,
 		   const gchar *label, const gchar *tip_text)
 {
   GtkWidget *button;
-  GtkTooltips *tooltip;
   GtkSizeGroup *sg;
 
   sg = g_object_get_data(G_OBJECT(widget), OBJECT_DATA_SIZE_GROUP);
@@ -499,14 +615,13 @@ prop_button_create(GtkWidget *widget, const gchar *icon_name,
 
   if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget),
 		      OBJECT_DATA_TOOLBAR_TYPE)) == TYPE_ICON)
-    gtk_widget_set_name(button, "uim-systray-button");
+    set_button_style(button);
 
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_size_group_add_widget(sg, button);
   g_object_set_data(G_OBJECT(button), OBJECT_DATA_BUTTON_TYPE,
 		    GINT_TO_POINTER(BUTTON_PROP));
-  tooltip = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltip, button, tip_text, NULL);
+  gtk_widget_set_tooltip_text(button, tip_text);
 
   g_signal_connect(G_OBJECT(button), "button-press-event",
 		   G_CALLBACK(button_pressed), widget);
@@ -669,7 +784,6 @@ helper_toolbar_prop_list_update(GtkWidget *widget, gchar **lines)
   /* FIXME! command menu and buttons should be customizable. */
   for (i = 0; i < command_entry_len; i++) {
     GtkWidget *tool_button;
-    GtkTooltips *tooltip;
     GtkWidget *img;
 
     if (!command_entry[i].show_button)
@@ -679,7 +793,7 @@ helper_toolbar_prop_list_update(GtkWidget *widget, gchar **lines)
 
     if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget),
 			OBJECT_DATA_TOOLBAR_TYPE)) == TYPE_ICON)
-      gtk_widget_set_name(tool_button, "uim-systray-button");
+      set_button_style(tool_button);
 
     g_object_set_data(G_OBJECT(tool_button), OBJECT_DATA_BUTTON_TYPE,
 		      GINT_TO_POINTER(BUTTON_TOOL));
@@ -702,8 +816,7 @@ helper_toolbar_prop_list_update(GtkWidget *widget, gchar **lines)
 		     G_CALLBACK(tool_button_clicked_cb), widget);
 
     /* tooltip */
-    tooltip = gtk_tooltips_new();
-    gtk_tooltips_set_tip(tooltip, tool_button, _(command_entry[i].desc), NULL);
+    gtk_widget_set_tooltip_text(tool_button, _(command_entry[i].desc));
 
     append_tool_button(widget, tool_button);
   }
@@ -720,6 +833,9 @@ helper_toolbar_check_custom()
   for (i = 0; i < command_entry_len; i++)
     command_entry[i].show_button =
       uim_scm_symbol_value_bool(command_entry[i].custom_button_show_symbol);
+
+  with_dark_bg =
+    uim_scm_symbol_value_bool("toolbar-icon-for-dark-background?");
 }
 
 static void
@@ -734,6 +850,7 @@ helper_toolbar_parse_helper_str(GtkWidget *widget, gchar *str)
     else if (!strcmp("custom_reload_notify", lines[0])) {
       uim_prop_reload_configs();
       helper_toolbar_check_custom();
+      reset_icon();
     }
     g_strfreev(lines);
   }
@@ -845,6 +962,7 @@ register_icon(const gchar *name)
   GtkIconSet *icon_set;
   GdkPixbuf *pixbuf;
   GString *filename;
+  struct stat st;
 
   g_return_val_if_fail(uim_factory, FALSE);
 
@@ -853,8 +971,18 @@ register_icon(const gchar *name)
 
   filename = g_string_new(UIM_PIXMAPSDIR "/");
   g_string_append(filename, name);
+  if (with_dark_bg) {
+    g_string_append(filename, "_dark_background");
+  }
   g_string_append(filename, ".png");
 
+  if (with_dark_bg && stat(filename->str, &st) == -1) {
+    g_string_free(filename, TRUE);
+    filename = g_string_new(UIM_PIXMAPSDIR "/");
+    g_string_append(filename, name);
+    g_string_append(filename, ".png");
+  }
+
   pixbuf = gdk_pixbuf_new_from_file(filename->str, NULL);
   if (!pixbuf) {
     g_string_free(filename, TRUE);
@@ -888,6 +1016,21 @@ init_icon(void)
   register_icon("null");
 }
 
+static void
+reset_icon(void)
+{
+  g_list_foreach(uim_icon_list, (GFunc)g_free, NULL);
+  g_list_free(uim_icon_list);
+  uim_icon_list = NULL;
+
+  if (GTK_IS_ICON_FACTORY(uim_factory)) {
+    gtk_icon_factory_remove_default(uim_factory);
+    uim_factory = NULL;
+    init_icon();
+  }
+}
+
+
 static GtkWidget *
 toolbar_new(gint type)
 {
@@ -896,6 +1039,16 @@ toolbar_new(gint type)
   GList *prop_buttons = NULL;
   GtkSizeGroup *sg;
 
+  /*
+   * Set uim-toolbar-save-default-im? #t in ~/.uim enable this if you'd like to
+   * save default IM into ~/.uim.d/custom/custom-global.scm upon system global
+   * IM switch.  However, using uim-custom consumes quite amount of memory, and
+   * requires additional startup time.
+   */
+  if (uim_scm_symbol_value_bool("uim-toolbar-save-default-im?"))
+    custom_enabled = (gboolean)uim_custom_enable();
+
+  helper_toolbar_check_custom();
   init_icon();
 
   /* create widgets */
@@ -916,7 +1069,7 @@ toolbar_new(gint type)
   }
 
   if (type == TYPE_ICON)
-    gtk_widget_set_name(button, "uim-systray-button");
+    set_button_style(button);
 
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_size_group_add_widget(sg, button);
@@ -933,8 +1086,6 @@ toolbar_new(gint type)
   g_object_set_data(G_OBJECT(hbox), OBJECT_DATA_TOOLBAR_TYPE,
 		    GINT_TO_POINTER(type));
 
-  helper_toolbar_check_custom();
-
   uim_fd = -1;
 
   if (type != TYPE_ICON) {
@@ -962,6 +1113,7 @@ uim_toolbar_applet_new(void)
 GtkWidget *
 uim_toolbar_trayicon_new(void)
 {
+#if !GTK_CHECK_VERSION(2, 90, 0)
   gtk_rc_parse_string("\n"
 		      "   style \"uim-systray-button-style\"\n"
 		      "   {\n"
@@ -971,6 +1123,7 @@ uim_toolbar_trayicon_new(void)
 		      "   }\n" "\n"
 		      "    widget \"*.uim-systray-button\" style \"uim-systray-button-style\"\n"
 		      "\n");
+#endif
 
   return toolbar_new(TYPE_ICON);
 }
