@@ -2,7 +2,7 @@
 
   toolbar-standalone-gtk.c: toolbar implementation with GTK+
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -47,6 +47,8 @@ extern GtkWidget *uim_toolbar_standalone_new(void);
 static gboolean toolbar_dragging = FALSE;
 static gint window_drag_start_x = -1, window_drag_start_y = -1;
 static gint pointer_drag_start_x = -1, pointer_drag_start_y = -1;
+static void size_request_cb(GtkWidget *widget, GtkRequisition *req,
+                            gpointer data);
 
 #if GLIB_CHECK_VERSION(2, 6, 0)
 
@@ -74,12 +76,12 @@ button_press_event_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
   switch (event->type) {
   case GDK_BUTTON_PRESS:
     cursor = gdk_cursor_new(GDK_FLEUR);
-    gdk_pointer_grab(widget->window, FALSE,
+    gdk_pointer_grab(gtk_widget_get_window(widget), FALSE,
 		     GDK_BUTTON_RELEASE_MASK |
 		     GDK_POINTER_MOTION_MASK,
 		     NULL,
 		     cursor, event->time);
-    gdk_cursor_destroy(cursor);
+    gdk_cursor_unref(cursor);
 
     gtk_window_get_position(GTK_WINDOW(widget),
 		    	    &window_drag_start_x,
@@ -90,14 +92,30 @@ button_press_event_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
     break;
   case GDK_2BUTTON_PRESS:
     toolbar = GTK_WIDGET(data);
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_get_visible(toolbar)) {
+#else
     if (GTK_WIDGET_VISIBLE(toolbar)) {
+#endif
       gtk_window_get_size(GTK_WINDOW(widget), &width, &height);
       gtk_widget_hide(toolbar);
     } else {
       height = -1;
       gtk_widget_show(toolbar);
     }
+#if GTK_CHECK_VERSION(2, 90, 0)
+    {
+      GtkRequisition minimum_size, natural_size;
+      gtk_widget_get_preferred_size(widget, &minimum_size, &natural_size);
+      if (height > 0)
+        natural_size.height = height;
+      else if (height == 0)
+        natural_size.height = minimum_size.height;
+      size_request_cb(widget, &natural_size, NULL);
+    }
+#else
     gtk_widget_set_size_request(widget, -1, height);
+#endif
     break;
   default:
     break;
@@ -115,7 +133,12 @@ helper_win_set_position(GtkWidget *window, gint x, gint y)
   sc_w = gdk_screen_width();
   sc_h = gdk_screen_height();
 
-  gdk_window_get_size(window->window, &w, &h);
+#if GTK_CHECK_VERSION(2, 90, 0)
+  w = gdk_window_get_width(gtk_widget_get_window(window));
+  h = gdk_window_get_height(gtk_widget_get_window(window));
+#else
+  gdk_drawable_get_size(gtk_widget_get_window(window), &w, &h);
+#endif
 
   if (wx < 0)
     wx = 0;
@@ -134,12 +157,6 @@ motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
   if (toolbar_dragging) {
     gint wx, wy;
-    gint w, h, sc_w, sc_h;
-
-    sc_w = gdk_screen_width();
-    sc_h = gdk_screen_height();
-
-    gdk_window_get_size(widget->window, &w, &h);
 
     wx = window_drag_start_x + ((gint)event->x_root - pointer_drag_start_x);
     wy = window_drag_start_y + ((gint)event->y_root - pointer_drag_start_y);
@@ -167,27 +184,55 @@ button_release_event_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
   return FALSE;
 }
 
+#if GTK_CHECK_VERSION(2, 90, 0)
+static gboolean
+handle_draw_cb(GtkWidget *widget, cairo_t *cr)
+{
+  gtk_render_handle(gtk_widget_get_style_context(widget), cr,
+		   0, 0,
+		   gtk_widget_get_allocated_width(widget),
+		   gtk_widget_get_allocated_height(widget));
+  return FALSE;
+}
+
+#else
 static gboolean
 handle_expose_event_cb(GtkWidget *widget, GdkEventExpose *event)
 {
   GdkRectangle *rect = &event->area;
 
-  gtk_paint_handle(widget->style, widget->window,
+#if GTK_CHECK_VERSION(2, 18, 0)
+  GtkAllocation allocation;
+  gtk_widget_get_allocation(widget, &allocation);
+  gtk_paint_handle(gtk_widget_get_style(widget), gtk_widget_get_window(widget),
 		   GTK_STATE_NORMAL, GTK_SHADOW_OUT,
 		   rect, widget, "handlebox",
-		   widget->allocation.x, widget->allocation.y,
-		   widget->allocation.width, widget->allocation.height,
+		   allocation.x, allocation.y,
+		   allocation.width, allocation.height,
 		   GTK_ORIENTATION_VERTICAL);
+#else
+  gtk_paint_handle(widget->style, widget->window,
+                   GTK_STATE_NORMAL, GTK_SHADOW_OUT,
+                   rect, widget, "handlebox",
+                   widget->allocation.x, widget->allocation.y,
+                   widget->allocation.width, widget->allocation.height,
+                   GTK_ORIENTATION_VERTICAL);
+#endif
 
   return FALSE;
 }
+#endif
 
 static void
 size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
 {
   gint x, y;
 
+#if GTK_CHECK_VERSION(2, 20, 0)
+  if (gtk_widget_get_mapped(widget)) {
+#else
   if (GTK_WIDGET_MAPPED(widget)) {
+#endif
     gtk_window_get_position(GTK_WINDOW(widget), &x, &y);
     helper_win_set_position(widget, x, y);
   }
@@ -196,7 +241,12 @@ size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, gpointer user_dat
 static void
 size_request_cb(GtkWidget *widget, GtkRequisition *req, gpointer data)
 {
+
+#if GTK_CHECK_VERSION(2, 20, 0)
+  if (gtk_widget_get_mapped(widget)) {
+#else
   if (GTK_WIDGET_MAPPED(widget)) {
+#endif
     gint width, height;
     gtk_window_get_size(GTK_WINDOW(widget), &width, &height);
 
@@ -222,8 +272,6 @@ main(int argc, char *argv[])
 
   uim_init();
 
-  gtk_set_locale();
-
   gtk_init(&argc, &argv);
 
   window = gtk_window_new(GTK_WINDOW_POPUP);
@@ -250,8 +298,13 @@ main(int argc, char *argv[])
   toolbar = (GtkWidget*)uim_toolbar_standalone_new();
   gtk_box_pack_start(GTK_BOX(hbox), toolbar, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(2, 90, 0)
+  g_signal_connect(G_OBJECT(handle), "draw",
+		   G_CALLBACK(handle_draw_cb), NULL);
+#else
   g_signal_connect(G_OBJECT(handle), "expose-event",
 		   G_CALLBACK(handle_expose_event_cb), NULL);
+#endif
 
   g_signal_connect(G_OBJECT(window), "delete_event",
 		   G_CALLBACK(delete_event), NULL);
@@ -263,8 +316,10 @@ main(int argc, char *argv[])
 		   G_CALLBACK(motion_notify_event_cb), NULL);
   g_signal_connect(G_OBJECT(window), "size-allocate",
 		   G_CALLBACK(size_allocate_cb), NULL);
+#if !GTK_CHECK_VERSION(2, 90, 0)
   g_signal_connect(G_OBJECT(window), "size-request",
 		   G_CALLBACK(size_request_cb), NULL);
+#endif
 
   gtk_widget_show_all(GTK_WIDGET(window));
 
