@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -76,13 +76,12 @@ main(int argc, char *argv[])
   textdomain(PACKAGE);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
 
-  gtk_set_locale();
-
   uim_init();
 
   gtk_init(&argc, &argv);
 
   tray = egg_tray_icon_new("uim");
+  gtk_window_set_wmclass(GTK_WINDOW(tray), "ibus-ui-gtk", "ibus-ui-gtk");
 
   icon = uim_toolbar_trayicon_new();
   g_signal_connect(G_OBJECT(tray), "embedded", G_CALLBACK(embedded_cb), icon);
