@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -42,10 +42,11 @@
 
 #include "uim.h"  /* for uim_bool */
 #include "uim-notify.h"
+#include "gettext.h"
 
 #define UIM_ICON UIM_PIXMAPSDIR "/uim-icon.png"
+#define UGETTEXT(str)	(dgettext(GETTEXT_PACKAGE, (str)))
 
-/* FIXME: gettext body */
 static uim_bool
 uim_libnotify_notify(int urgency, int timeout, const char *body)
 {
@@ -56,9 +57,9 @@ uim_libnotify_notify(int urgency, int timeout, const char *body)
   gchar *gmsg;
   gsize read, written;
 
-  strlcpy(body_short, body, sizeof(body_short));
+  strlcpy(body_short, UGETTEXT(body), sizeof(body_short));
 
-  fprintf(stderr, "libuim: %s\n", body);
+  fprintf(stderr, "libuim: %s\n", UGETTEXT(body));
 
   gmsg = g_locale_to_utf8(body_short, -1, &read, &written, NULL);
 
@@ -72,7 +73,13 @@ uim_libnotify_notify(int urgency, int timeout, const char *body)
     return UIM_FALSE;
   }
 
+#ifdef NOTIFY_CHECK_VERSION
+  /* libnotify 0.7.0 contains this API change
+     and introduced the macro NOTIFY_CHECK_VERSION. */
+  notification = notify_notification_new("uim", gmsg, UIM_ICON);
+#else
   notification = notify_notification_new("uim", gmsg, UIM_ICON, NULL);
+#endif
 
   if (!notification) {
     fprintf(stderr, "notify_notification_new: can not create notification object\n");
