@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <locale.h>
 #include <errno.h>
 
@@ -362,7 +363,7 @@ nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 		switch (c) {
 		case '\\':
 		case '"':
-		    *p++ = c;
+		    *p++ = (char)c;
 		    len++;
 		    break;
 		case 'n':
@@ -426,12 +427,12 @@ nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 		    token = ERROR;
 		    goto string_error;
 		default:
-		    *p++ = c;
+		    *p++ = (char)c;
 		    len++;
 		    break;
 		}
 	    } else {
-		*p++ = c;
+		*p++ = (char)c;
 		len++;
 	    }
 	}
@@ -454,7 +455,7 @@ nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 		*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
 	    }
 	    p = *tokenbuf;
-	    *p++ = c;
+	    *p++ = (char)c;
 	    len++;
 	    c = nextch(fp, lastch);
 	    while (isalnum(c) || c == '_' || c == '-') {
@@ -463,7 +464,7 @@ nexttoken(FILE *fp, char **tokenbuf, int *lastch, size_t *buflen)
 			*tokenbuf = (char *)realloc(*tokenbuf,  *buflen);
 			p = *tokenbuf + len;
 		}
-		*p++ = c;
+		*p++ = (char)c;
 		len++;
 		c = nextch(fp, lastch);
 	    }
@@ -510,42 +511,19 @@ modmask(char *name)
     return(mask);
 }
 
-char *
-QUimInputContext::TransFileName(char *name)
+int
+QUimInputContext::TransFileName(char *transname, const char *name, size_t len)
 {
-    char *home = NULL, *lcCompose = NULL;
-    char *i = name, *ret, *j;
-    int l = 0;
+    char *home = NULL;
+    char lcCompose[MAXPATHLEN];
+    const char *i = name;
+    char *j;
+    char ret[MAXPATHLEN];
 
-    while (*i) {
-	if (*i == '%') {
-	    i++;
-	    switch (*i) {
-	    case '%':
-		l++;
-		break;
-	    case 'H':
-		home = getenv("HOME");
-		if (home)
-		    l += strlen(home);
-		break;
-	    case 'L':
-		lcCompose = get_compose_filename();
-		if (lcCompose)
-		    l += strlen(lcCompose);
-		break;
-	    }
-	} else {
-	    l++;
-	}
-	i++;
-    }
-
-    j = ret = (char *)malloc(l + 1);
-    if (ret == NULL)
-	return ret;
+    j = ret;
     i = name;
-    while (*i) {
+    lcCompose[0] = ret[0] = '\0';
+    while (*i && j - ret < MAXPATHLEN - 1) {
 	if (*i == '%') {
 	    i++;
 	    switch (*i) {
@@ -553,16 +531,17 @@ QUimInputContext::TransFileName(char *name)
 		*j++ = '%';
 		break;
 	    case 'H':
+		home = getenv("HOME");
 		if (home) {
-		    strcpy(j, home);
+		    strlcat(ret, home, sizeof(ret));
 		    j += strlen(home);
 		}
 		break;
 	    case 'L':
-		if (lcCompose) {
-		    strcpy(j, lcCompose);
+		get_compose_filename(lcCompose, sizeof(lcCompose));
+		if (lcCompose[0] != '\0') {
+		    strlcat(ret, lcCompose, sizeof(ret));
 		    j += strlen(lcCompose);
-		    free(lcCompose);
 		}
 		break;
 	    }
@@ -570,9 +549,11 @@ QUimInputContext::TransFileName(char *name)
 	} else {
 	    *j++ = *i++;
 	}
+	*j = '\0';
     }
-    *j = '\0';
-    return ret;
+    strlcpy(transname, ret, len);
+
+    return 1;
 }
 
 #ifndef MB_LEN_MAX
@@ -582,28 +563,26 @@ QUimInputContext::TransFileName(char *name)
 const char *
 QUimInputContext::get_encoding()
 {
-   QTextCodec *codec = QTextCodec::codecForLocale();
+    QTextCodec *codec = QTextCodec::codecForLocale();
 
-   return codec->name();
+    return codec->name();
 }
 
-char *
-QUimInputContext::get_lang_region()
+int
+QUimInputContext::get_lang_region(char *locale, size_t len)
 {
-    char *locale, *p;
+    char *p;
 
-    locale = setlocale(LC_CTYPE, NULL);
-    if (locale) {
-        locale = strdup(locale);
-    } else {
-        return NULL;
+    strlcpy(locale, setlocale(LC_CTYPE, NULL), len);
+    if (locale[0] == '\0') {
+        return 0;
     }
 
     p = strrchr(locale, '.');
     if (p)
         *p = '\0';
 
-    return locale;   
+    return 1;
 }
 
 int
@@ -614,7 +593,7 @@ QUimInputContext::get_mb_string(char *buf, unsigned int ks)
     unsigned int ucs;
 
     ucs = KeySymToUcs4(ks);
-    QString qs = QString(QChar(ks));
+    QString qs = QString(QChar(ucs));
     mb = (const char *)qs.local8Bit();
     if (!mb)
 	return 0;
@@ -668,16 +647,15 @@ QUimInputContext::parse_compose_line(FILE *fp, char **tokenbuf, size_t *buflen)
     n = 0;
     do {
 	if ((token == KEY) && (strcmp("include", *tokenbuf) == 0)) {
-	    char *filename;
+	    char filename[MAXPATHLEN];
 	    FILE *infp;
 	    token = nexttoken(fp, tokenbuf, &lastch, buflen);
 	    if (token != KEY && token != STRING)
 		goto error;
 
-	    if ((filename = TransFileName(*tokenbuf)) == NULL)
+	    if (!TransFileName(filename, *tokenbuf, sizeof(filename)) || filename[0] == '\0')
 		goto error;
 	    infp = fopen(filename, "r");
-	    free(filename);
 	    if (infp == NULL)
 		goto error;
 	    ParseComposeStringFile(infp);
@@ -822,11 +800,9 @@ QUimInputContext::parse_compose_line(FILE *fp, char **tokenbuf, size_t *buflen)
 	}
     }
 
-    if (p->mb != NULL)
-	free(p->mb);
+    free(p->mb);
     p->mb = rhs_string_mb;
-    if (p->utf8 != NULL)
-	free(p->utf8);
+    free(p->utf8);
     p->utf8 = rhs_string_utf8;
     p->ks = rhs_keysym;
     return n;
@@ -875,119 +851,81 @@ QUimInputContext::ParseComposeStringFile(FILE *fp)
 void QUimInputContext::create_compose_tree()
 {
     FILE *fp = NULL;
-    char *name, *tmpname = NULL;
-    char *lang_region;
+    char name[MAXPATHLEN];
+    char lang_region[BUFSIZ];
     const char *encoding;
+    char *compose_env;
+    int ret;
 
-    name = getenv("XCOMPOSEFILE");
+    name[0] = '\0';
+    compose_env = getenv("XCOMPOSEFILE");
 
-    if (name == NULL) {
+    if (compose_env != NULL) {
+	strlcpy(name, compose_env, sizeof(name));
+    } else {
 	char *home = getenv("HOME");
 	if (home != NULL) {
-	    int hl = strlen(home);
-	    tmpname = name = (char *)malloc(hl + 10 + 1);
-	    if (name != NULL) {
-		strcpy(name, home);
-		strcpy(name + hl, "/.XCompose");
-		fp = fopen(name, "r");
-		if (fp == NULL) {
-		    free(name);
-		    name = tmpname = NULL;
-		}
-	    }
+	    snprintf(name, sizeof(name), "%s/.XCompose", home);
+	    fp = fopen(name, "r");
+	    if (fp == NULL)
+		name[0] = '\0';
 	}
     }
 
-    if (name == NULL) {
-	tmpname = name = get_compose_filename();
-    }
-
-    if (name == NULL)
-	return;
-    if (fp == NULL) {
-	fp = fopen(name, "r");
-    }
-    if (tmpname != NULL) {
-	free(tmpname);
-    }
-    if (fp == NULL)
+    if (name[0] == '\0' && !get_compose_filename(name, sizeof(name)))
 	return;
 
-    lang_region = get_lang_region();
+    if (fp == NULL && ((fp = fopen(name, "r")) == NULL))
+	return;
+
+    ret = get_lang_region(lang_region, sizeof(lang_region));
     encoding = get_encoding();
-    if (lang_region == NULL || encoding == NULL) {
+    if (!ret || encoding == NULL) {
 	fprintf(stderr, "Warning: locale name is NULL\n");
 	fclose(fp);
-	free(lang_region);
 	return;
     }
 
     ParseComposeStringFile(fp);
     fclose(fp);
-    free(lang_region);
 }
 
-char *QUimInputContext::get_compose_filename()
+int QUimInputContext::get_compose_filename(char *filename, size_t len)
 {
-    char *compose_dir_file;
-    char *locale;
-    char *lang_region;
+    char compose_dir_file[MAXPATHLEN], name[MAXPATHLEN];
+    char locale[BUFSIZ];
     const char *encoding;
+    char lang_region[BUFSIZ];
+    int ret;
 
     FILE *fp;
-    char buf[XLC_BUFSIZE], *name = NULL, *filename = NULL;
+    char buf[XLC_BUFSIZE];
     const char *xlib_dir = XLIB_DIR ;
 
-    lang_region = get_lang_region();
+    ret = get_lang_region(lang_region, sizeof(lang_region));
     encoding = get_encoding();
 
-    if (lang_region == NULL || encoding == NULL) {
-	free(lang_region);
-	return NULL;
-    }
+    if (!ret || encoding == NULL)
+	return 0;
 
-    locale = (char *)malloc(strlen(lang_region) + strlen(encoding) + 2);
-    if (locale == NULL) {
-	free(lang_region);
-	return NULL;
-    }
-    sprintf(locale, "%s.%s", lang_region, encoding);
-    free(lang_region);
-
-    compose_dir_file = (char *)malloc(strlen(XLIB_DIR) + strlen(COMPOSE_DIR_FILE) + 2);
-    if (compose_dir_file == NULL) {
-	free(locale);
-	return NULL;
-    }
-    sprintf(compose_dir_file, "%s/%s", XLIB_DIR, COMPOSE_DIR_FILE);
+    snprintf(locale, sizeof(locale), "%s.%s", lang_region, encoding);
+    snprintf(compose_dir_file, sizeof(compose_dir_file), "%s/%s", XLIB_DIR, COMPOSE_DIR_FILE);
 
     fp = fopen(compose_dir_file, "r");
     if (fp == NULL) {
 	/* retry with fallback file */
 	if (strcmp(FALLBACK_XLIB_DIR, XLIB_DIR)) {
-	    compose_dir_file = (char *)realloc(compose_dir_file,
-			    strlen(FALLBACK_XLIB_DIR) +
-			    strlen(COMPOSE_DIR_FILE) + 2);
-	    if (compose_dir_file == NULL) {
-		free(locale);
-		return NULL;
-	    }
-	    sprintf(compose_dir_file, "%s/%s",
+	    snprintf(compose_dir_file, sizeof(compose_dir_file), "%s/%s",
 			    FALLBACK_XLIB_DIR, COMPOSE_DIR_FILE);
 	    fp = fopen(compose_dir_file, "r");
-	    if (fp == NULL) {
-		free(locale);
-		free(compose_dir_file);
-		return NULL;
-	    }
+	    if (fp == NULL)
+		return 0;
 	    xlib_dir = FALLBACK_XLIB_DIR;
-	} else {
-	    free(locale);
-	    free(compose_dir_file);
-	    return NULL;
-	}
+	} else
+	    return 0;
     }
 
+    name[0] = '\0';
     while (fgets(buf, XLC_BUFSIZE, fp) != NULL) {
 	char *p = buf;
 	int n;
@@ -1004,28 +942,18 @@ char *QUimInputContext::get_compose_filename()
 	}
 	from = args[1], to = args[0];
 	if (!strcmp(from, locale)) {
-	    name = (char *)malloc(strlen(to) + 1);
-	    if (name != NULL) {
-		strcpy(name, to);
-	    }
+	    strlcpy(name, to, sizeof(name));
 	    break;
 	}
     }
     fclose(fp);
-    free(locale);
-    free(compose_dir_file);
 
-    if (name == NULL)
-	return NULL;
+    if (name[0] == '\0')
+	return 0;
 
-    filename = (char *)malloc(strlen(xlib_dir) + strlen(XLOCALE_DIR) +
-		    strlen(name) + 3);
-    if (filename == NULL)
-	return NULL;
-    sprintf(filename, "%s/%s/%s", xlib_dir, XLOCALE_DIR, name);
-    free(name);
+    snprintf(filename, len, "%s/%s/%s", xlib_dir, XLOCALE_DIR, name);
 
-    return filename;
+    return 1;
 }
 
 static int
