@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -48,11 +48,14 @@
 #include <stdlib.h>
 #include <locale.h>
 
+#include <uim/uim-scm.h>
+#include <uim/uim-custom.h>
 #include "qtgettext.h"
 
 #define NAME_COLUMN 0
 
 static int uim_fd;
+static bool customEnabled;
 static QSocketNotifier *notifier = NULL;
 
 int main( int argc, char **argv )
@@ -83,9 +86,15 @@ UimImSwitcher::UimImSwitcher( QWidget *parent, const char *name )
     uim_fd = -1;
     checkHelperConnection();
 
+    /* to check if another uim-im-switcher exists */
+    uim_helper_send_message( uim_fd, "im_switcher_start\n" );
+
     /* to load input method list */
     uim_helper_send_message( uim_fd, "im_list_get\n" );
 
+    uim_init();
+    customEnabled = uim_custom_enable();
+
     /* create GUI */
     createGUI();
 }
@@ -170,6 +179,7 @@ void UimImSwitcher::slotChangeInputMethod()
     {
     case ID_CHANGE_WHOLE_DESKTOP:
         sendMessageImChange( "im_change_whole_desktop\n" );
+        saveDefaultIm();
         break;
     case ID_CHANGE_THIS_APPLICATION_ONLY:
         sendMessageImChange( "im_change_this_application_only\n" );
@@ -201,6 +211,22 @@ void UimImSwitcher::sendMessageImChange( const QString &change_type )
     uim_helper_send_message( uim_fd, ( const char* ) msg.utf8() );
 }
 
+void UimImSwitcher::saveDefaultIm()
+{
+    if ( customEnabled )
+    {
+        QString imName = selectedImName();
+        if ( imName.isEmpty() )
+            return ;
+
+        uim_scm_callf( "custom-set-value!",
+                       "yy",
+                       "custom-preserved-default-im-name",
+                       ( const char* ) imName.utf8() );
+        uim_custom_save_custom( "custom-preserved-default-im-name" );
+    }
+}
+
 QString UimImSwitcher::selectedImName() const
 {
     QListViewItem * selectedItem = listview->selectedItem();
@@ -240,6 +266,10 @@ void UimImSwitcher::slotStdinActivated( int /*socket*/ )
             reloadImList();
         else if ( msg.startsWith( "im_list" ) )
             parseHelperStrImList( msg );
+        else if ( msg.startsWith( "im_switcher_start" ) )
+            uim_helper_send_message( uim_fd,  "im_switcher_quit\n" );
+        else if ( msg.startsWith( "im_switcher_quit" ) )
+            qApp->quit();
     }
 }
 
