@@ -1,7 +1,7 @@
 /*
 
   Copyright (c) 2004-2005 Kazuki Ohta <mover@hct.zaq.ne.jp>
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,24 +34,22 @@
 
 #include "plugin.h"
 
-#include <qapplication.h>
-#include <qinputcontextplugin.h>
-#include <qinputcontext.h>
-#include <qstringlist.h>
+#include <QtCore/QStringList>
+#include <QtGui/QInputContext>
 #ifdef Q_WS_X11
-#include <QX11Info>
+#include <QtGui/QX11Info>
 #endif
 
 #include "uim/uim.h"
 #include "uim/uim-x-util.h"
+#include "uim/counted-init.h"
 
-#include "debug.h"
 #include "quiminfomanager.h"
 #include "quiminputcontext_with_slave.h"
 
 #define UIM_QT_LIST_SUBIM_AS_QTIM 0
 
-QUimInfoManager *UimInputContextPlugin::infoManager = NULL;
+QUimInfoManager *UimInputContextPlugin::infoManager = 0;
 
 
 UimInputContextPlugin::UimInputContextPlugin()
@@ -72,19 +70,17 @@ QStringList UimInputContextPlugin::keys() const
 
 QInputContext *UimInputContextPlugin::create( const QString & key )
 {
-    QString imname = QString::null;
+    QString imname;
 
 #if UIM_QT_LIST_SUBIM_AS_QTIM
-    if ( key.startsWith( "uim-" ) )
-	imname = key.mid( 4 );
+    if ( key.startsWith( QLatin1String( "uim-" ) ) )
+        imname = key.mid( 4 );
     else
 #endif
     if ( key == "uim" )
-        imname = uim_get_default_im_name( setlocale( LC_ALL, NULL ) );
+        imname = uim_get_default_im_name( setlocale( LC_ALL, 0 ) );
 
-    QStringList langs = createLanguageList( key );
-    QUimInputContext *uic = new QUimInputContext( imname.toUtf8(),
-						  langs[ 0 ].toUtf8() );
+    QUimInputContext *uic = new QUimInputContext( imname.toUtf8().data() );
 
     return uic;
 }
@@ -96,7 +92,7 @@ QStringList UimInputContextPlugin::languages( const QString & key )
 
 QString UimInputContextPlugin::displayName( const QString & key )
 {
-    return QString( key ) + " (" + languages( key ) [ 0 ] + ")";
+    return key;
 }
 
 QString UimInputContextPlugin::description( const QString & key )
@@ -112,11 +108,11 @@ UimInputContextPlugin::getQUimInfoManager()
 
 void UimInputContextPlugin::uimInit()
 {
-    if ( !uim_init() ) {
+    if ( !uim_counted_init() ) {
         if (!infoManager)
             infoManager = new QUimInfoManager();
 #if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
-	uim_x_kana_input_hack_init( QX11Info::display() );
+        uim_x_kana_input_hack_init( QX11Info::display() );
 #endif
         uimReady = true;
     }
@@ -126,7 +122,7 @@ void UimInputContextPlugin::uimQuit()
 {
     if ( uimReady )
     {
-        uim_quit();
+        uim_counted_quit();
         delete infoManager;
         uimReady = false;
     }
@@ -140,11 +136,13 @@ QStringList UimInputContextPlugin::createImList() const
 
     // default
     lst.append( "uim" );
+#ifdef ENABLE_DEBUG
     qDebug( "name = uim" );
+#endif
 
 #if UIM_QT_LIST_SUBIM_AS_QTIM
-    uim_context tmp_uc = uim_create_context( NULL, "UTF-8",
-                         NULL, NULL, uim_iconv, NULL );
+    uim_context tmp_uc = uim_create_context( 0, "UTF-8",
+                         0, 0, uim_iconv, 0 );
     int nr = uim_get_nr_im( tmp_uc );
     if ( uimReady )
     {
@@ -155,7 +153,9 @@ QStringList UimInputContextPlugin::createImList() const
             qs = "uim-" + qs;
             lst << qs;
 
-            qDebug( "name = %s", ( const char* ) qs.toUtf8() );
+#ifdef ENABLE_DEBUG
+            qDebug( "name = %s", qs.toUtf8().data() );
+#endif
         }
     }
     uim_release_context( tmp_uc );
@@ -167,11 +167,11 @@ QStringList UimInputContextPlugin::createImList() const
 QStringList UimInputContextPlugin::createLanguageList( const QString &key ) const
 {
     if ( key == "uim" )
-	return QStringList() << "ja" << "ko" << "zh" << "*";
+        return QStringList() << "ja" << "ko" << "zh" << "*";
 
 #if UIM_QT_LIST_SUBIM_AS_QTIM
-    uim_context tmp_uc = uim_create_context( NULL, "UTF-8",
-                         NULL, NULL, uim_iconv, NULL );
+    uim_context tmp_uc = uim_create_context( 0, "UTF-8",
+                         0, 0, uim_iconv, 0 );
     int nr = uim_get_nr_im( tmp_uc );
     if ( uimReady )
     {
@@ -183,7 +183,7 @@ QStringList UimInputContextPlugin::createLanguageList( const QString &key ) cons
             if ( key == QString( "uim-" ) + name )
             {
                 // ":" separated languages for future extension
-                QStringList langs = QString( lang ).split( ":" );
+                QStringList langs = QString( lang ).split( ':' );
                 return langs;
             }
         }
