@@ -1,7 +1,7 @@
 /*
 
   Copyright (c) 2004-2005 Kazuki Ohta <mover@hct.zaq.ne.jp>
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,9 +31,12 @@
   SUCH DAMAGE.
 
 */
+#ifndef UIM_QT4_IMMODULE_PLUGIN_H
+#define UIM_QT4_IMMODULE_PLUGIN_H
 
-#include <qinputcontextplugin.h>
-#include <qstringlist.h>
+#include <QtGui/QInputContextPlugin>
+
+class QStringList;
 
 class QUimInfoManager;
 
@@ -63,3 +66,4 @@ protected:
     bool uimReady;
 };
 
+#endif /* Not def: UIM_QT4_IMMODULE_PLUGIN_H */
