@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
+Copyright (c) 2006-2011 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -32,28 +32,24 @@ SUCH DAMAGE.
 */
 
 // FIXME: Support Qt4 TextEdit
-
-//#include <config.h>
-
-#include <stdlib.h>
-
-#include <qwidget.h>
-#include <qlineedit.h>
-//#include <qtextedit.h>
-#include <Q3TextEdit>
-#include <qapplication.h>
-#include <qclipboard.h>
-
-#include "uim/uim.h"
-
 #include "qtextutil.h"
-#include "quiminputcontext.h"
 
+#include <cstdlib>
+
+#include <QtGui/QApplication>
+#include <QtGui/QClipboard>
+#include <QtGui/QLineEdit>
+#include <QtGui/QTextEdit>
+#ifdef ENABLE_QT4_QT3SUPPORT
+# include <Qt3Support/Q3TextEdit>
+#endif
+
+#include "quiminputcontext.h"
 
 QUimTextUtil::QUimTextUtil( QObject *parent )
         : QObject( parent )
 {
-    mIc = (QUimInputContext *)parent;
+    mIc = static_cast<QUimInputContext *>( parent );
     mPreeditSaved = false;
 }
 
@@ -68,7 +64,7 @@ QUimTextUtil::acquire_text_cb( void *ptr, enum UTextArea text_id,
                                char **former, char **latter )
 {
     int err;
-    QUimInputContext *ic = (QUimInputContext *)ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext *>( ptr );
     QUimTextUtil *tu = ic->textUtil();
 
     switch ( text_id ) {
@@ -98,7 +94,7 @@ QUimTextUtil::delete_text_cb( void *ptr, enum UTextArea text_id,
                               int former_req_len, int latter_req_len )
 {
     int err;
-    QUimInputContext *ic = (QUimInputContext *)ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext *>( ptr );
     QUimTextUtil *tu = ic->textUtil();
 
     switch ( text_id ) {
@@ -125,17 +121,22 @@ QUimTextUtil::acquirePrimaryText( enum UTextOrigin origin,
 {
     int err;
 #if defined(Q_WS_X11)
-    mWidget = mIc->focusWidget();
+    mWidget = QApplication::focusWidget();
 #else
     return -1;
 #endif
 
-    if ( mWidget->inherits( "QLineEdit" ) )
+    if ( qobject_cast<QLineEdit *>( mWidget ) )
         err = acquirePrimaryTextInQLineEdit( origin, former_req_len,
                                              latter_req_len, former, latter );
-    else if ( mWidget->inherits( "Q3TextEdit" ) )
+    else if ( qobject_cast<QTextEdit *>( mWidget ) )
         err = acquirePrimaryTextInQTextEdit( origin, former_req_len,
                                              latter_req_len, former, latter );
+#ifdef ENABLE_QT4_QT3SUPPORT
+    else if ( qobject_cast<Q3TextEdit *>( mWidget ) )
+        err = acquirePrimaryTextInQ3TextEdit( origin, former_req_len,
+                                              latter_req_len, former, latter );
+#endif
     else
         // FIXME other widgets?
         err = -1;
@@ -149,7 +150,7 @@ QUimTextUtil::acquirePrimaryTextInQLineEdit( enum UTextOrigin origin,
                                              int latter_req_len,
                                              char **former, char **latter )
 {
-    QLineEdit *edit = (QLineEdit *)mWidget;
+    QLineEdit *edit = static_cast<QLineEdit *>( mWidget );
     QString text, former_text, latter_text;
     int cursor_index, len, precedence_len, following_len, offset;
     int preedit_len, preedit_cursor_pos;
@@ -174,7 +175,8 @@ QUimTextUtil::acquirePrimaryTextInQLineEdit( enum UTextOrigin origin,
             if (! ( ~former_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
         }
-        *former = strdup( text.mid( offset, precedence_len - offset ).utf8() );
+        *former = strdup(
+            text.mid( offset, precedence_len - offset ).toUtf8().data() );
 
         offset = 0;
         if ( latter_req_len >= 0 ) {
@@ -186,11 +188,12 @@ QUimTextUtil::acquirePrimaryTextInQLineEdit( enum UTextOrigin origin,
                 return -1;
             }
         }
-        *latter = strdup( text.mid( precedence_len + preedit_len, following_len - offset ).utf8() );
+        *latter = strdup( text.mid( precedence_len + preedit_len,
+            following_len - offset ).toUtf8().data() );
         break;
 
     case UTextOrigin_Beginning:
-        *former = NULL;
+        *former = 0;
         if ( latter_req_len >= 0 ) {
             if ( precedence_len >= latter_req_len )
                 text = text.left( latter_req_len );
@@ -210,7 +213,7 @@ QUimTextUtil::acquirePrimaryTextInQLineEdit( enum UTextOrigin origin,
             latter_text = text.mid( precedence_len + preedit_len, following_len );
             text = former_text + latter_text;
         }
-        *latter = strdup( text.utf8() );
+        *latter = strdup( text.toUtf8().data() );
         break;
 
     case UTextOrigin_End:
@@ -233,8 +236,8 @@ QUimTextUtil::acquirePrimaryTextInQLineEdit( enum UTextOrigin origin,
             latter_text = text.right( following_len );
             text = former_text + latter_text;
         }
-        *former = strdup( text.utf8() );
-        *latter = NULL;
+        *former = strdup( text.toUtf8().data() );
+        *latter = 0;
         break;
 
     case UTextOrigin_Unspecified:
@@ -251,10 +254,25 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
                                              int latter_req_len,
                                              char **former, char **latter )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
+    // FIXME: Implement this
+    Q_UNUSED( origin )
+    Q_UNUSED( former_req_len )
+    Q_UNUSED( latter_req_len )
+    Q_UNUSED( former )
+    Q_UNUSED( latter )
+    return -1; 
+}
+
+#ifdef ENABLE_QT4_QT3SUPPORT
+int
+QUimTextUtil::acquirePrimaryTextInQ3TextEdit( enum UTextOrigin origin,
+                                              int former_req_len,
+                                              int latter_req_len,
+                                              char **former, char **latter )
+{
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     QString text;
 
-    int i;
     int start_para, start_index, end_para, end_index, para, index;
     int n_para;
     int preedit_len, preedit_cursor_pos;
@@ -282,8 +300,8 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
         end_para = para;
 
         if ( former_req_len >= 0 ) {
-            for ( i = 0; i < former_req_len; i++ )
-                QTextEditPositionBackward( &start_para, &start_index );
+            for ( int i = 0; i < former_req_len; i++ )
+                Q3TextEditPositionBackward( &start_para, &start_index );
         } else {
             if ( former_req_len == UTextExtent_Line )
                 start_index = 0;
@@ -296,11 +314,11 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
             }
         }
         edit->setSelection( start_para, start_index, para, index - preedit_cursor_pos, 0 );
-        *former = strdup( edit->selectedText().utf8() );
+        *former = strdup( edit->selectedText().toUtf8().data() );
 
         if ( latter_req_len >= 0 ) {
-            for ( i = 0; i < latter_req_len; i++ )
-                QTextEditPositionForward( &end_para, &end_index );
+            for ( int i = 0; i < latter_req_len; i++ )
+                Q3TextEditPositionForward( &end_para, &end_index );
         } else {
             if ( latter_req_len == UTextExtent_Line ) {
                 end_index = edit->paragraphLength( end_para );
@@ -314,11 +332,11 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
         }
         edit->setSelection( para, index - preedit_cursor_pos + preedit_len,
                             end_para, end_index, 0 );
-        *latter = strdup( edit->selectedText().utf8() );
+        *latter = strdup( edit->selectedText().toUtf8().data() );
         break;
 
     case UTextOrigin_Beginning:
-        *former = NULL;
+        *former = 0;
 
         start_para = 0;
         start_index = 0;
@@ -326,8 +344,8 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
         end_index = start_index;
 
         if ( latter_req_len >= 0 ) {
-            for ( i = 0; i < latter_req_len; i++ )
-                QTextEditPositionForward( &end_para, &end_index );
+            for ( int i = 0; i < latter_req_len; i++ )
+                Q3TextEditPositionForward( &end_para, &end_index );
         } else {
             if ( latter_req_len == UTextExtent_Line )
                 end_index = edit->paragraphLength( end_para );
@@ -348,7 +366,7 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
             edit->setSelection( para, index - preedit_cursor_pos + preedit_len, end_para, end_index, 0 );
             text += edit->selectedText();
         }
-        *latter = strdup( text.utf8() );
+        *latter = strdup( text.toUtf8().data() );
         break;
 
     case UTextOrigin_End:
@@ -359,8 +377,8 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
         start_index = end_index;
 
         if ( former_req_len >= 0 ) {
-            for ( i = 0; i < former_req_len; i++ )
-                QTextEditPositionBackward( &start_para, &start_index );
+            for ( int i = 0; i < former_req_len; i++ )
+                Q3TextEditPositionBackward( &start_para, &start_index );
         } else {
             if ( former_req_len == UTextExtent_Line )
                 start_index = 0;
@@ -383,8 +401,8 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
             edit->setSelection( para, index - preedit_cursor_pos + preedit_len, end_para, end_index, 0 );
             text += edit->selectedText();
         }
-        *former = strdup( text.utf8() );
-        *latter = NULL;
+        *former = strdup( text.toUtf8().data() );
+        *latter = 0;
         break;
 
     case UTextOrigin_Unspecified:
@@ -404,6 +422,7 @@ QUimTextUtil::acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
     edit->setTextFormat( format );
     return 0; 
 }
+#endif
 
 int
 QUimTextUtil::acquireSelectionText( enum UTextOrigin origin,
@@ -412,17 +431,23 @@ QUimTextUtil::acquireSelectionText( enum UTextOrigin origin,
 {
     int err;
 #if defined(Q_WS_X11)
-    mWidget = mIc->focusWidget();
+    mWidget = QApplication::focusWidget();
 #else
     return -1;
 #endif
 
-    if ( mWidget->inherits( "QLineEdit" ) )
+    if ( qobject_cast<QLineEdit *>( mWidget ) )
         err = acquireSelectionTextInQLineEdit( origin, former_req_len,
                                                latter_req_len, former, latter );
-    else if ( mWidget->inherits( "Q3TextEdit" ) )
+    else if ( qobject_cast<QTextEdit *>( mWidget ) )
         err = acquireSelectionTextInQTextEdit( origin, former_req_len,
                                                latter_req_len, former, latter );
+#ifdef ENABLE_QT4_QT3SUPPORT
+    else if ( qobject_cast<Q3TextEdit *>( mWidget ) )
+        err = acquireSelectionTextInQ3TextEdit( origin, former_req_len,
+                                                latter_req_len, former,
+                                                latter );
+#endif
     else
         // FIXME other widgets?
         err = -1;
@@ -436,7 +461,7 @@ QUimTextUtil::acquireSelectionTextInQLineEdit( enum UTextOrigin origin,
                                                int latter_req_len,
                                                char **former, char **latter )
 {
-    QLineEdit *edit = (QLineEdit *)mWidget;
+    QLineEdit *edit = static_cast<QLineEdit *>( mWidget );
     QString text;
     int len, offset, start, current;
     bool cursor_at_beginning = false;
@@ -455,7 +480,7 @@ QUimTextUtil::acquireSelectionTextInQLineEdit( enum UTextOrigin origin,
 
     if ( origin == UTextOrigin_Beginning ||
          ( origin == UTextOrigin_Cursor && cursor_at_beginning ) ) {
-        *former = NULL;
+        *former = 0;
         offset = 0;
         if ( latter_req_len >= 0 ) {
             if ( len > latter_req_len )
@@ -464,7 +489,7 @@ QUimTextUtil::acquireSelectionTextInQLineEdit( enum UTextOrigin origin,
             if (! ( ~latter_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
         }
-        *latter = strdup( text.left( len - offset ).utf8() );
+        *latter = strdup( text.left( len - offset ).toUtf8().data() );
     } else if ( origin == UTextOrigin_End ||
                 ( origin == UTextOrigin_Cursor && !cursor_at_beginning ) ) {
         offset = 0;
@@ -475,8 +500,8 @@ QUimTextUtil::acquireSelectionTextInQLineEdit( enum UTextOrigin origin,
             if (! ( ~former_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
         }
-        *former = strdup( text.mid( offset, len - offset ).utf8() );
-        *latter = NULL;
+        *former = strdup( text.mid( offset, len - offset ).toUtf8().data() );
+        *latter = 0;
     } else {
         return -1;
     }
@@ -490,7 +515,23 @@ QUimTextUtil::acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
                                                int latter_req_len,
                                                char **former, char **latter )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
+    // FIXME: Implement this
+    Q_UNUSED( origin )
+    Q_UNUSED( former_req_len )
+    Q_UNUSED( latter_req_len )
+    Q_UNUSED( former )
+    Q_UNUSED( latter )
+    return -1;
+}
+
+#ifdef ENABLE_QT4_QT3SUPPORT
+int
+QUimTextUtil::acquireSelectionTextInQ3TextEdit( enum UTextOrigin origin,
+                                                int former_req_len,
+                                                int latter_req_len,
+                                                char **former, char **latter )
+{
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     QString text;
     int len, offset, newline;
     int start_para, start_index, end_para, end_index;
@@ -515,7 +556,7 @@ QUimTextUtil::acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
 
     if ( origin == UTextOrigin_Beginning ||
          ( origin == UTextOrigin_Cursor && cursor_at_beginning ) ) {
-        *former = NULL;
+        *former = 0;
         offset = 0;
         if ( latter_req_len >= 0 ) {
             if ( len > latter_req_len )
@@ -526,10 +567,10 @@ QUimTextUtil::acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
                 return -1;
             }
 
-            if ( latter_req_len == UTextExtent_Line && ( ( newline = text.find( '\n' ) ) != -1 ) )
+            if ( latter_req_len == UTextExtent_Line && ( ( newline = text.indexOf( '\n' ) ) != -1 ) )
                 offset = len - newline;
         }
-        *latter = strdup( text.left( len - offset ).utf8() );
+        *latter = strdup( text.left( len - offset ).toUtf8().data() );
     } else if ( origin == UTextOrigin_End ||
                 ( origin == UTextOrigin_Cursor && !cursor_at_beginning ) ) {
         offset = 0;
@@ -542,11 +583,11 @@ QUimTextUtil::acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
                 return -1;
             }
 
-            if ( former_req_len == UTextExtent_Line && ( ( newline = text.findRev( '\n' ) ) != -1 ) )
+            if ( former_req_len == UTextExtent_Line && ( ( newline = text.lastIndexOf( '\n' ) ) != -1 ) )
                 offset = newline + 1;
         }
-        *former = strdup( text.mid( offset, len - offset ).utf8() );
-        *latter = NULL;
+        *former = strdup( text.mid( offset, len - offset ).toUtf8().data() );
+        *latter = 0;
     } else {
         edit->setTextFormat( format );
         return -1;
@@ -555,6 +596,7 @@ QUimTextUtil::acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
     edit->setTextFormat( format );
     return 0;
 }
+#endif
 
 int
 QUimTextUtil::acquireClipboardText( enum UTextOrigin origin,
@@ -582,15 +624,16 @@ QUimTextUtil::acquireClipboardText( enum UTextOrigin origin,
             if (! ( ~former_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
 
-            if ( former_req_len == UTextExtent_Line && ( ( newline = text.findRev( '\n' ) ) != -1 ) )
+            if ( former_req_len == UTextExtent_Line
+                && ( ( newline = text.lastIndexOf( '\n' ) ) != -1 ) )
                 offset = newline + 1;
         }
-        *former = strdup( text.mid( offset, len - offset ).utf8() );
-        *latter = NULL;
+        *former = strdup( text.mid( offset, len - offset ).toUtf8().data() );
+        *latter = 0;
         break;
 
     case UTextOrigin_Beginning:
-        *former = NULL;
+        *former = 0;
         offset = 0;
         if ( latter_req_len >= 0 ) {
             if ( latter_req_len < len )
@@ -599,10 +642,11 @@ QUimTextUtil::acquireClipboardText( enum UTextOrigin origin,
             if (! ( ~latter_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
 
-            if ( latter_req_len == UTextExtent_Line && ( ( newline = text.find( '\n' ) ) != -1 ) )
+            if ( latter_req_len == UTextExtent_Line
+                && ( ( newline = text.indexOf( '\n' ) ) != -1 ) )
                 offset = len - newline;
         }
-        *latter = strdup( text.left( len - offset ).utf8() );
+        *latter = strdup( text.left( len - offset ).toUtf8().data() );
         break;
 
     case UTextOrigin_Unspecified:
@@ -619,17 +663,22 @@ QUimTextUtil::deletePrimaryText( enum UTextOrigin origin, int former_req_len,
 {
     int err;
 #if defined(Q_WS_X11)
-    mWidget = mIc->focusWidget();
+    mWidget = QApplication::focusWidget();
 #else
     return -1;
 #endif
 
-    if ( mWidget->inherits( "QLineEdit" ) )
+    if ( qobject_cast<QLineEdit *>( mWidget ) )
         err = deletePrimaryTextInQLineEdit( origin, former_req_len,
                                             latter_req_len );
-    else if ( mWidget->inherits( "Q3TextEdit" ) )
+    else if ( qobject_cast<QTextEdit *>( mWidget ) )
         err = deletePrimaryTextInQTextEdit( origin, former_req_len,
-                                            latter_req_len );
+                                             latter_req_len );
+#ifdef ENABLE_QT4_QT3SUPPORT
+    else if ( qobject_cast<Q3TextEdit *>( mWidget ) )
+        err = deletePrimaryTextInQ3TextEdit( origin, former_req_len,
+                                             latter_req_len );
+#endif
     else
         // FIXME other widgets?
         err = -1;
@@ -642,7 +691,7 @@ QUimTextUtil::deletePrimaryTextInQLineEdit( enum UTextOrigin origin,
                                             int former_req_len,
                                             int latter_req_len )
 {
-    QLineEdit *edit = (QLineEdit *)mWidget;
+    QLineEdit *edit = static_cast<QLineEdit *>( mWidget );
     QString text;
     int cursor_index, len, precedence_len, following_len;
     int preedit_len, preedit_cursor_pos;
@@ -723,8 +772,20 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
                                             int former_req_len,
                                             int latter_req_len )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
-    int i;
+    // FIXME: Implement this
+    Q_UNUSED( origin )
+    Q_UNUSED( former_req_len )
+    Q_UNUSED( latter_req_len )
+    return -1; 
+}
+
+#ifdef ENABLE_QT4_QT3SUPPORT
+int
+QUimTextUtil::deletePrimaryTextInQ3TextEdit( enum UTextOrigin origin,
+                                             int former_req_len,
+                                             int latter_req_len )
+{
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     int start_para, start_index, end_para, end_index, para, index;
     int n_para;
 
@@ -741,8 +802,8 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
         end_para = para;
 
         if ( former_req_len >= 0 ) {
-            for ( i = 0; i < former_req_len; i++ )
-                QTextEditPositionBackward( &start_para, &start_index );
+            for ( int i = 0; i < former_req_len; i++ )
+                Q3TextEditPositionBackward( &start_para, &start_index );
         } else {
             if ( former_req_len == UTextExtent_Line ) {
                 start_index = 0;
@@ -755,8 +816,8 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
             }
         }
         if ( latter_req_len >= 0 ) {
-            for ( i = 0; i < latter_req_len; i++ )
-                QTextEditPositionForward( &end_para, &end_index );
+            for ( int i = 0; i < latter_req_len; i++ )
+                Q3TextEditPositionForward( &end_para, &end_index );
         } else {
             if ( latter_req_len == UTextExtent_Line ) {
                 end_index = edit->paragraphLength( end_para );
@@ -777,8 +838,8 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
         end_index = start_index;
 
         if ( latter_req_len >= 0 ) {
-            for ( i = 0; i < latter_req_len; i++ )
-                QTextEditPositionForward( &end_para, &end_index );
+            for ( int i = 0; i < latter_req_len; i++ )
+                Q3TextEditPositionForward( &end_para, &end_index );
         } else {
             if ( latter_req_len == UTextExtent_Line ) {
                 end_index = edit->paragraphLength( end_para );
@@ -799,8 +860,8 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
         start_index = end_index;
 
         if ( former_req_len >= 0 ) {
-            for ( i = 0; i < former_req_len; i++ )
-                QTextEditPositionBackward( &start_para, &start_index );
+            for ( int i = 0; i < former_req_len; i++ )
+                Q3TextEditPositionBackward( &start_para, &start_index );
         } else {
             if ( former_req_len == UTextExtent_Line )
                 start_index = 0;
@@ -826,6 +887,7 @@ QUimTextUtil::deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
 
     return 0; 
 }
+#endif
 
 int
 QUimTextUtil::deleteSelectionText( enum UTextOrigin origin,
@@ -833,17 +895,22 @@ QUimTextUtil::deleteSelectionText( enum UTextOrigin origin,
 {
     int err;
 #if defined(Q_WS_X11)
-    mWidget = mIc->focusWidget();
+    mWidget = QApplication::focusWidget();
 #else
     return -1;
 #endif
 
-    if ( mWidget->inherits( "QLineEdit" ) )
+    if ( qobject_cast<QLineEdit *>( mWidget ) )
         err = deleteSelectionTextInQLineEdit( origin, former_req_len,
                                               latter_req_len );
-    else if ( mWidget->inherits( "Q3TextEdit" ) )
+    else if ( qobject_cast<QTextEdit *>( mWidget ) )
         err = deleteSelectionTextInQTextEdit( origin, former_req_len,
                                               latter_req_len );
+#ifdef ENABLE_QT4_QT3SUPPORT
+    else if ( qobject_cast<Q3TextEdit *>( mWidget ) )
+        err = deleteSelectionTextInQ3TextEdit( origin, former_req_len,
+                                               latter_req_len );
+#endif
     else
         // FIXME other widgets?
         err = -1;
@@ -856,7 +923,7 @@ QUimTextUtil::deleteSelectionTextInQLineEdit( enum UTextOrigin origin,
                                               int former_req_len,
                                               int latter_req_len )
 {
-    QLineEdit *edit = (QLineEdit *)mWidget;
+    QLineEdit *edit = static_cast<QLineEdit *>( mWidget );
     QString text;
     int len, start, end, current;
     bool cursor_at_beginning = false;
@@ -905,9 +972,22 @@ QUimTextUtil::deleteSelectionTextInQTextEdit( enum UTextOrigin origin,
                                               int former_req_len,
                                               int latter_req_len )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
+    // FIXME: Implement this
+    Q_UNUSED( origin )
+    Q_UNUSED( former_req_len )
+    Q_UNUSED( latter_req_len )
+    return -1;
+}
+
+#ifdef ENABLE_QT4_QT3SUPPORT
+int
+QUimTextUtil::deleteSelectionTextInQ3TextEdit( enum UTextOrigin origin,
+                                               int former_req_len,
+                                               int latter_req_len )
+{
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     QString text;
-    int len, newline, i;
+    int len, newline;
     int para, index;
     int sel_para_from, sel_index_from, sel_para_to, sel_index_to;
     int start_para, start_index, end_para, end_index;
@@ -937,14 +1017,14 @@ QUimTextUtil::deleteSelectionTextInQTextEdit( enum UTextOrigin origin,
             if ( len > latter_req_len ) {
                 end_para = sel_para_from;
                 end_index = sel_index_from;
-                for ( i = 0; i < latter_req_len; i++)
-                    QTextEditPositionForward( &end_para, &end_index );
+                for ( int i = 0; i < latter_req_len; i++)
+                    Q3TextEditPositionForward( &end_para, &end_index );
             }
         } else {
             if (! ( ~latter_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
 
-            if ( latter_req_len == UTextExtent_Line && ( ( newline = text.find('\n') ) != -1 ) ) {
+            if ( latter_req_len == UTextExtent_Line && ( ( newline = text.indexOf('\n') ) != -1 ) ) {
                 end_para = sel_para_from;
                 end_index = sel_index_from + newline;
             }
@@ -955,14 +1035,14 @@ QUimTextUtil::deleteSelectionTextInQTextEdit( enum UTextOrigin origin,
             if ( len > former_req_len ) {
                 start_para = sel_para_to;
                 start_index = sel_index_to;
-                for ( i = 0; i < former_req_len; i++)
-                    QTextEditPositionBackward( &start_para, &start_index );
+                for ( int i = 0; i < former_req_len; i++)
+                    Q3TextEditPositionBackward( &start_para, &start_index );
             }
         } else {
             if (! ( ~former_req_len & ( ~UTextExtent_Line | ~UTextExtent_Full ) ) )
                 return -1;
 
-            if ( former_req_len == UTextExtent_Line && ( ( newline = text.findRev( '\n' ) ) != -1 ) ) {
+            if ( former_req_len == UTextExtent_Line && ( ( newline = text.lastIndexOf( '\n' ) ) != -1 ) ) {
                 start_para = sel_para_to;
                 start_index = 0;
             }
@@ -977,9 +1057,9 @@ QUimTextUtil::deleteSelectionTextInQTextEdit( enum UTextOrigin origin,
 }
 
 void
-QUimTextUtil::QTextEditPositionBackward( int *cursor_para, int *cursor_index )
+QUimTextUtil::Q3TextEditPositionBackward( int *cursor_para, int *cursor_index )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     int preedit_len, preedit_cursor_pos;
     int para, index;
     int current_para, current_index;
@@ -1013,9 +1093,9 @@ QUimTextUtil::QTextEditPositionBackward( int *cursor_para, int *cursor_index )
 }
 
 void
-QUimTextUtil::QTextEditPositionForward( int *cursor_para, int *cursor_index )
+QUimTextUtil::Q3TextEditPositionForward( int *cursor_para, int *cursor_index )
 {
-    Q3TextEdit *edit = (Q3TextEdit *)mWidget;
+    Q3TextEdit *edit = static_cast<Q3TextEdit *>( mWidget );
     int n_para = edit->paragraphs();
     int preedit_len, preedit_cursor_pos;
     int current_para_len;
@@ -1053,6 +1133,7 @@ QUimTextUtil::QTextEditPositionForward( int *cursor_para, int *cursor_index )
     *cursor_para = current_para;
     *cursor_index = current_index;
 }
+#endif
 
 void QUimTextUtil::savePreedit()
 {
