@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
+Copyright (c) 2006-2011 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -37,7 +37,6 @@ SUCH DAMAGE.
 
 #include "uim/uim.h"
 
-
 class QUimInputContext;
 
 class QUimTextUtil : QObject
@@ -45,7 +44,7 @@ class QUimTextUtil : QObject
     Q_OBJECT
 
 public:
-    QUimTextUtil( QObject *parent = 0 );
+    explicit QUimTextUtil( QObject *parent = 0 );
     ~QUimTextUtil();
 
     static int acquire_text_cb( void *ptr, enum UTextArea text_id,
@@ -66,18 +65,25 @@ private:
     int acquirePrimaryTextInQTextEdit( enum UTextOrigin origin,
                                        int former_req_len, int latter_req_len,
                                        char **former, char **latter );
+    int acquirePrimaryTextInQ3TextEdit( enum UTextOrigin origin,
+                                        int former_req_len, int latter_req_len,
+                                        char **former, char **latter );
 
     int acquireSelectionText( enum UTextOrigin origin,
                               int former_req_len, int latter_req_len,
                               char **former, char **latter );
     int acquireSelectionTextInQLineEdit( enum UTextOrigin origin,
                                          int former_req_len,
-					 int latter_req_len,
+                                         int latter_req_len,
                                          char **former, char **latter );
     int acquireSelectionTextInQTextEdit( enum UTextOrigin origin,
                                          int former_req_len,
-					 int latter_req_len,
+                                         int latter_req_len,
                                          char **former, char **latter );
+    int acquireSelectionTextInQ3TextEdit( enum UTextOrigin origin,
+                                          int former_req_len,
+                                          int latter_req_len,
+                                          char **former, char **latter );
 
     int acquireClipboardText( enum UTextOrigin origin,
                               int former_req_len, int latter_req_len,
@@ -89,17 +95,24 @@ private:
                                       int former_req_len, int latter_req_len );
     int deletePrimaryTextInQTextEdit( enum UTextOrigin origin,
                                       int former_req_len, int latter_req_len );
+    int deletePrimaryTextInQ3TextEdit( enum UTextOrigin origin,
+                                       int former_req_len,
+                                       int latter_req_len );
 
     int deleteSelectionText( enum UTextOrigin origin,
                              int former_req_len, int latter_req_len );
     int deleteSelectionTextInQLineEdit( enum UTextOrigin origin,
                                        int former_req_len, int latter_req_len );
     int deleteSelectionTextInQTextEdit( enum UTextOrigin origin,
-                                       int former_req_len, int latter_req_len );
+                                        int former_req_len,
+                                        int latter_req_len );
+    int deleteSelectionTextInQ3TextEdit( enum UTextOrigin origin,
+                                         int former_req_len,
+                                         int latter_req_len );
 
-    void QTextEditPositionBackward( int *para, int *index );
+    void Q3TextEditPositionBackward( int *para, int *index );
 
-    void QTextEditPositionForward( int *para, int *index );
+    void Q3TextEditPositionForward( int *para, int *index );
 
     void savePreedit();
     void restorePreedit();
