@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
+Copyright (c) 2006-2011 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,15 +30,9 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-
-//#include <config.h>
-
-#include <Q3ValueList>
-
-#include "uim/uim.h"
-
 #include "quiminfomanager.h"
 
+#include "uim/uim.h"
 
 QUimInfoManager::QUimInfoManager()
 {
@@ -49,8 +43,8 @@ QUimInfoManager::~QUimInfoManager()
 {
 }
 
-Q3ValueList<uimInfo>
-QUimInfoManager::getUimInfo()
+QList<uimInfo>
+QUimInfoManager::getUimInfo() const
 {
     return info;
 }
@@ -60,7 +54,7 @@ QUimInfoManager::initUimInfo()
 {
     info.clear();
 
-    uim_context tmp_uc = uim_create_context( NULL, "UTF-8", NULL, NULL, NULL, NULL );
+    uim_context tmp_uc = uim_create_context( 0, "UTF-8", 0, 0, 0, 0 );
     struct uimInfo ui;
     int nr = uim_get_nr_im( tmp_uc );
     for ( int i = 0; i < nr; i++ )
@@ -75,12 +69,9 @@ QUimInfoManager::initUimInfo()
 }
 
 QString
-QUimInfoManager::imLang( const QString &imname )
+QUimInfoManager::imLang( const QString &imname ) const
 {
-    int i, n;
-
-    n = info.count();
-    for (i = 0; i < n; i++) {
+    for (int i = 0, n = info.count(); i < n; i++) {
         if ( info[i].name == imname )
             return info[i].lang;
     }
