@@ -1,7 +1,7 @@
 /*
 
   Copyright (c) 2004-2005 Kazuki Ohta <mover@hct.zaq.ne.jp>
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -31,32 +31,29 @@
   SUCH DAMAGE.
 
 */
-
-#include <qnamespace.h>
-#include <qevent.h>
-#include <qglobal.h>
-#include <qapplication.h>
-#include <qwidget.h>
-#include <qpoint.h>
-#include <qlabel.h>
-#include <QInputMethodEvent>
-#include <QTextCharFormat>
-
-#include <ctype.h>
-#include <string.h>
-
-#include "debug.h"
 #include "quiminputcontext.h"
-#include "quiminputcontext_compose.h"
-#include "plugin.h"
+
+#include <cctype>
+#include <cstring>
+
+#include <QtCore/QPoint>
+#include <QtGui/QApplication>
+#include <QtGui/QInputMethodEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QTextCharFormat>
+
+#include <uim/uim-helper.h>
+#include <uim/uim-im-switcher.h>
+#include <uim/uim-scm.h>
+
+#include "candidatetablewindow.h"
 #include "candidatewindow.h"
-#include "quiminfomanager.h"
+#include "caretstateindicator.h"
+#include "plugin.h"
 #include "qhelpermanager.h"
 #include "qtextutil.h"
-
-#include <uim/uim.h>
-#include <uim/uim-scm.h>
-#include <uim/uim-im-switcher.h>
+#include "quiminfomanager.h"
+#include "quiminputcontext_compose.h"
 
 #if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
 #include <X11/Xlib.h>
@@ -64,16 +61,16 @@
 #include "uim/uim-x-util.h"
 #endif
 
-#define DEFAULT_SEPARATOR_STR "|"
+static const char DEFAULT_SEPARATOR_STR[] = "|";
 
-QUimInputContext *focusedInputContext = NULL;
+QUimInputContext *focusedInputContext = 0;
 bool disableFocusedContext = false;
 
 QList<QUimInputContext*> contextList;
 
-QUimHelperManager * QUimInputContext::m_HelperManager = 0L;
+QUimHelperManager * QUimInputContext::m_HelperManager = 0;
 #ifdef Q_WS_X11
-DefTree *QUimInputContext::mTreeTop = NULL;
+DefTree *QUimInputContext::mTreeTop = 0;
 #endif
 
 static int unicodeToUKey(ushort c);
@@ -82,29 +79,26 @@ static int unicodeToUKey(ushort c);
 // input method name is useless and should be redesigned. I will
 // suggest the change in future. -- YamaKen 2004-07-28
 
-QUimInputContext::QUimInputContext( const char *imname, const char *lang )
-        : QInputContext(), m_imname( imname ), m_lang( lang ), m_uc( 0 ),
-        candwinIsActive( false ),
-        m_isComposing( false )
+QUimInputContext::QUimInputContext( const char *imname )
+        : candwinIsActive( false ), m_isComposing( false ), m_uc( 0 )
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+        , focusedWidget( 0 )
+#endif
 {
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext()" );
+#endif
 
     contextList.append( this );
 
     // must be initialized before createUimContext() call
     if ( !m_HelperManager )
-        m_HelperManager = new QUimHelperManager();
+        m_HelperManager = new QUimHelperManager;
 
     if ( imname )
         m_uc = createUimContext( imname );
 
-    while ( !psegs.isEmpty() )
-        delete psegs.takeFirst();
-    psegs.clear();
-
-    cwin = new CandidateWindow( 0 );
-    cwin->setQUimInputContext( this );
-    cwin->hide();
+    createCandidateWindow();
 
 #ifdef Q_WS_X11
     if ( !mTreeTop )
@@ -115,20 +109,32 @@ QUimInputContext::QUimInputContext( const char *imname, const char *lang )
 
     // read configuration
     readIMConf();
+
+    m_indicator = new CaretStateIndicator;
 }
 
 QUimInputContext::~QUimInputContext()
 {
+#ifdef ENABLE_DEBUG
     qDebug( "~QUimInputContext()" );
+#endif
 
-    contextList.remove( this );
+    contextList.removeAll( this );
 
     if ( m_uc )
         uim_release_context( m_uc );
+    delete cwin;
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    foreach ( const uim_context uc, m_ucHash )
+        if ( uc )
+            uim_release_context( uc );
+    foreach ( const AbstractCandidateWindow* window, cwinHash )
+        delete window;
+#endif
 
     if ( this == focusedInputContext )
     {
-        focusedInputContext = NULL;
+        focusedInputContext = 0;
         disableFocusedContext = true;
     }
 
@@ -139,11 +145,9 @@ QUimInputContext::~QUimInputContext()
 
 uim_context QUimInputContext::createUimContext( const char *imname )
 {
-    m_imname = imname;
-
     uim_context uc = uim_create_context( this, "UTF-8",
-                                         NULL, ( char * ) imname,
-                                         NULL,
+                                         0, imname,
+                                         0,
                                          QUimInputContext::commit_cb );
 
     m_HelperManager->checkHelperConnection();
@@ -177,32 +181,45 @@ uim_context QUimInputContext::createUimContext( const char *imname )
     return uc;
 }
 
+void QUimInputContext::createCandidateWindow()
+{
+    char *candwinprog = uim_scm_symbol_value_str( "uim-candwin-prog" );
+    if ( candwinprog && !strncmp( candwinprog, "uim-candwin-tbl", 15 ) )
+        cwin = new CandidateTableWindow( 0 );
+    else
+        cwin = new CandidateWindow( 0 );
+    free( candwinprog );
+    cwin->setQUimInputContext( this );
+    cwin->hide();
+}
+
 #ifdef Q_WS_X11
 bool QUimInputContext::x11FilterEvent( QWidget *keywidget, XEvent *event )
 {
-    // to suppress warning
-    keywidget = keywidget;
-    event = event;
+    Q_UNUSED( keywidget )
 
 #if UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK
     return uim_x_kana_input_hack_filter_event( m_uc, event );
 #else
-    return FALSE;
+    Q_UNUSED( event )
+    return false;
 #endif
 }
 #endif // Q_WS_X11
 
 bool QUimInputContext::filterEvent( const QEvent *event )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "filterEvent" );
+#endif
 
     int type = event->type();
 
     if ( type != QEvent::KeyPress &&
             type != QEvent::KeyRelease )
-        return FALSE;
+        return false;
 
-    QKeyEvent *keyevent = ( QKeyEvent * ) event;
+    const QKeyEvent *keyevent = static_cast<const QKeyEvent *>( event );
     int qkey = keyevent->key();
 
     int modifier = 0;
@@ -227,9 +244,9 @@ bool QUimInputContext::filterEvent( const QEvent *event )
         }
         else
         {
-            if ( keyevent->state() & Qt::ControlButton &&
+            if ( keyevent->modifiers() & Qt::ControlModifier &&
                  ( ascii >= 0x01 && ascii <= 0x1a ) )
-                if ( keyevent->state() & Qt::ShiftButton )
+                if ( keyevent->modifiers() & Qt::ShiftModifier )
                     key = ascii + 0x40;
                 else
                     key = ascii + 0x60;
@@ -257,7 +274,7 @@ bool QUimInputContext::filterEvent( const QEvent *event )
             key = qkey - Qt::Key_F1 + UKey_F1;
         }
         else if ( qkey >= Qt::Key_Dead_Grave && qkey <= Qt::Key_Dead_Horn )
-	{
+        {
             key = qkey - Qt::Key_Dead_Grave + UKey_Dead_Grave;
         }
         else if ( qkey >= Qt::Key_Kanji && qkey <= Qt::Key_Eisu_toggle )
@@ -273,7 +290,7 @@ bool QUimInputContext::filterEvent( const QEvent *event )
             switch ( qkey )
             {
             case Qt::Key_Tab: key = UKey_Tab; break;
-            case Qt::Key_BackSpace: key = UKey_Backspace; break;
+            case Qt::Key_Backspace: key = UKey_Backspace; break;
             case Qt::Key_Escape: key = UKey_Escape; break;
             case Qt::Key_Delete: key = UKey_Delete; break;
             case Qt::Key_Return: key = UKey_Return; break;
@@ -281,8 +298,8 @@ bool QUimInputContext::filterEvent( const QEvent *event )
             case Qt::Key_Up: key = UKey_Up; break;
             case Qt::Key_Right: key = UKey_Right; break;
             case Qt::Key_Down: key = UKey_Down; break;
-            case Qt::Key_Prior: key = UKey_Prior; break;
-            case Qt::Key_Next: key = UKey_Next; break;
+            case Qt::Key_PageUp: key = UKey_Prior; break;
+            case Qt::Key_PageDown: key = UKey_Next; break;
             case Qt::Key_Home: key = UKey_Home; break;
             case Qt::Key_End: key = UKey_End; break;
             case Qt::Key_Multi_key: key = UKey_Multi_key; break;
@@ -300,8 +317,8 @@ bool QUimInputContext::filterEvent( const QEvent *event )
                 break;
             case Qt::Key_Control: key = UKey_Control_key;
                 if ( type == QEvent::KeyPress )
-        	    modifier &= ~UMod_Control;
-		break;
+                    modifier &= ~UMod_Control;
+                break;
             case Qt::Key_Alt: key = UKey_Alt_key;
                 if ( type == QEvent::KeyPress )
                     modifier &= ~UMod_Alt;
@@ -329,7 +346,7 @@ bool QUimInputContext::filterEvent( const QEvent *event )
             return mCompose->handle_qkey( keyevent );
 #else
         if ( notFiltered )
-            return FALSE;
+            return false;
 #endif
     }
     else if ( type == QEvent::KeyRelease )
@@ -340,34 +357,45 @@ bool QUimInputContext::filterEvent( const QEvent *event )
             return mCompose->handle_qkey( keyevent );
 #else
         if ( notFiltered )
-            return FALSE;
+            return false;
 #endif
     }
 
-    return TRUE;
+    return true;
 }
 
 void QUimInputContext::setFocusWidget( QWidget *w )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext::setFocusWidget() w = %p", w );
+#endif
 
     QInputContext::setFocusWidget( w );
 
     if ( w )
-	setFocus();
+        setFocus();
     else
-	unsetFocus();
+        unsetFocus();
 }
 
 // Qt4 does not have QInputContext::setFocus()
 void QUimInputContext::setFocus()
 {
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext: %p->setFocus(), focusWidget()=%p",
-            this, focusWidget() );
+            this, QApplication::focusWidget() );
+#endif
 
     focusedInputContext = this;
     disableFocusedContext = false;
 
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    focusedWidget = QApplication::focusWidget();
+    if ( isPreeditPreservationEnabled()
+            && m_ucHash.contains( focusedWidget ) )
+        restorePreedit();
+    else
+#endif
     if ( candwinIsActive )
         cwin->popup();
 
@@ -382,19 +410,15 @@ void QUimInputContext::setFocus()
 // Qt4 does not have QInputContext::unsetFocus()
 void QUimInputContext::unsetFocus()
 {
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext: %p->unsetFocus(), focusWidget()=%p",
-            this, focusWidget() );
+            this, QApplication::focusWidget() );
+#endif
 
     uim_focus_out_context( m_uc );
 
-    // Don't reset Japanese input context here. Japanese input context
-    // sometimes contains a whole paragraph and has minutes of
-    // lifetime different to ephemeral one in other languages. The
-    // input context should be survived until focused again.
-    if ( ! isPreeditPreservationEnabled() )
-        reset();
-
     cwin->hide();
+    m_indicator->hide();
 
     m_HelperManager->checkHelperConnection();
 
@@ -406,36 +430,6 @@ QUimInputContext * QUimInputContext::focusedIC()
     return focusedInputContext;
 }
 
-void QUimInputContext::reloadUim()
-{
-    QList<QUimInputContext*>::iterator it;
-    QUimInfoManager *infoManager = UimInputContextPlugin::getQUimInfoManager();
-
-    for ( it = contextList.begin(); it != contextList.end(); ++it )
-    {
-        ( *it )->reset();
-        uim_release_context( ( *it )->m_uc );
-    }
-
-    uim_quit();
-    uim_init();
-    infoManager->initUimInfo();
-
-    for ( it = contextList.begin(); it != contextList.end(); ++it )
-    {
-        ( *it )->m_uc = ( *it )->createUimContext( ( *it )->m_imname );
-    }
-}
-
-void QUimInputContext::setMicroFocus( int x, int y, int w, int h, QFont *f )
-{
-    qDebug("IC setMicroFocus (%d, %d), (%d, %d)", x, y, w, h);
-
-    f = f;  // to suppress warning
-
-    cwin->layoutWindow( x, y, w, h );
-}
-
 void QUimInputContext::mouseHandler( int x, QMouseEvent *e )
 {
     switch ( e->type() )
@@ -444,8 +438,12 @@ void QUimInputContext::mouseHandler( int x, QMouseEvent *e )
     case QEvent::MouseButtonRelease:
     case QEvent::MouseButtonDblClick:
     case QEvent::MouseMove:
+#ifdef ENABLE_DEBUG
         qDebug( "QUimInputContext::mouseHandler: "
                 "x=%d, type=%d, button=%d ", x, e->type(), e->button() );
+#else
+        Q_UNUSED( x )
+#endif
         break;
     default:
         break;
@@ -454,9 +452,23 @@ void QUimInputContext::mouseHandler( int x, QMouseEvent *e )
 
 void QUimInputContext::reset()
 {
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext::reset()" );
+#endif
 
-    candwinIsActive = FALSE;
+    candwinIsActive = false;
+
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    // Japanese input context sometimes contains a whole paragraph
+    // and has minutes of lifetime different to ephemeral one
+    // in other languages. The input context should be survived
+    // until focused again.
+    if ( isPreeditPreservationEnabled()
+            && !m_ucHash.contains( focusedWidget ) ) {
+        psegs.isEmpty() ? cwin->hide() : savePreedit();
+        return;
+    }
+#endif
     cwin->hide();
     uim_reset_context( m_uc );
 #ifdef Q_WS_X11
@@ -468,14 +480,18 @@ void QUimInputContext::reset()
 
 void QUimInputContext::update()
 {
-    QWidget *w = focusWidget();
+    QWidget *w = QApplication::focusWidget();
 
+#ifdef ENABLE_DEBUG
     qDebug( "QUimInputContext::update() w = %p", w );
+#endif
 
     if ( w ) {
-	QRect mf = w->inputMethodQuery( Qt::ImMicroFocus ).toRect();
-	QPoint p = w->mapToGlobal( mf.topLeft() );
-	setMicroFocus( p.x(), p.y(), mf.width(), mf.height() );
+        QRect mf = w->inputMethodQuery( Qt::ImMicroFocus ).toRect();
+        QPoint p = w->mapToGlobal( mf.topLeft() );
+        cwin->layoutWindow( p, mf );
+        m_indicator->move( w->mapToGlobal( mf.bottomLeft() )
+            + QPoint( 0, CaretStateIndicator::SPACING ) );
     }
 }
 
@@ -486,31 +502,40 @@ QString QUimInputContext::identifierName()
 
 QString QUimInputContext::language()
 {
-    return m_lang;
+    const QUimInfoManager *manager
+        = UimInputContextPlugin::getQUimInfoManager();
+    const QString name = QString::fromUtf8( uim_get_current_im_name( m_uc ) );
+    return manager->imLang( name );
 }
 
 // callbacks for uim
 void QUimInputContext::commit_cb( void *ptr, const char *str )
 {
     QString qs = QString::fromUtf8( str );
-    qDebug( "commit_cb : str = |%s|", ( const char* ) qs.toLocal8Bit() );
+#ifdef ENABLE_DEBUG
+    qDebug( "commit_cb : str = |%s|", qs.toLocal8Bit().data() );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext * ) ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext *>( ptr );
     ic->commitString( qs );
 }
 
 void QUimInputContext::clear_cb( void *ptr )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "clear_cb" );
+#endif
 
-    QUimInputContext* ic = ( QUimInputContext* ) ptr;
+    QUimInputContext* ic = static_cast<QUimInputContext*>( ptr );
     ic->clearPreedit();
 }
 
 void QUimInputContext::pushback_cb( void *ptr, int attr, const char *str )
 {
     QString qs = QString::fromUtf8( str );
-    qDebug( "pushback_cb :  str = |%s|", ( const char* ) qs.toLocal8Bit() );
+#ifdef ENABLE_DEBUG
+    qDebug( "pushback_cb :  str = |%s|", qs.toLocal8Bit().data() );
+#endif
 
     if ( !str )
         return ;
@@ -520,59 +545,71 @@ void QUimInputContext::pushback_cb( void *ptr, int attr, const char *str )
     if ( !strcmp( str, "" ) && !( attr & ( UPreeditAttr_Cursor | UPreeditAttr_Separator ) ) )
         return ;
 
-    QUimInputContext* ic = ( QUimInputContext* ) ptr;
+    QUimInputContext* ic = static_cast<QUimInputContext*>( ptr );
     ic->pushbackPreeditString( attr, qs );
 }
 
 void QUimInputContext::update_cb( void *ptr )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "update_cb" );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
     ic->updatePreedit();
 }
 
 void QUimInputContext::cand_activate_cb( void *ptr, int nr, int displayLimit )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "cand_activate_cb" );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    ic->candidateActivate( nr, displayLimit );
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
+    ic->cwin->candidateActivate( nr, displayLimit );
+    ic->candwinIsActive = true;
 }
 
 void QUimInputContext::cand_select_cb( void *ptr, int index )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "cand_select_cb" );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    ic->candidateSelect( index );
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
+    ic->cwin->candidateSelect( index );
 }
 
 void QUimInputContext::cand_shift_page_cb( void *ptr, int forward )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "cand_shift_page_cb" );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    ic->candidateShiftPage( (bool)forward );
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
+    ic->cwin->candidateShiftPage( forward );
 }
 
 void QUimInputContext::cand_deactivate_cb( void *ptr )
 {
+#ifdef ENABLE_DEBUG
     qDebug( "cand_deactivate_cb" );
+#endif
 
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
-    ic->candidateDeactivate();
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
+    ic->cwin->deactivateCandwin();
+    ic->candwinIsActive = false;
 }
 
 void QUimInputContext::switch_app_global_im_cb( void *ptr, const char *name )
 {
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
     ic->switch_app_global_im( name );
 }
 
 void QUimInputContext::switch_system_global_im_cb( void *ptr, const char *name )
 {
-    QUimInputContext *ic = ( QUimInputContext* ) ptr;
+    QUimInputContext *ic = static_cast<QUimInputContext*>( ptr );
     ic->switch_system_global_im( name );
 }
 
@@ -587,17 +624,12 @@ void QUimInputContext::commitString( const QString& str )
 
 void QUimInputContext::clearPreedit()
 {
-    // delete first
-    while ( !psegs.isEmpty() )
-        delete psegs.takeFirst();
-
-    // and clear
     psegs.clear();
 }
 
 void QUimInputContext::pushbackPreeditString( int attr, const QString& str )
 {
-    PreeditSegment * ps = new PreeditSegment( attr, str );
+    PreeditSegment ps( attr, str );
     psegs.append( ps );
 }
 
@@ -606,29 +638,58 @@ void QUimInputContext::updatePreedit()
     QString newString = getPreeditString();
 
     if ( !isComposing() ) {
-	if ( newString.isEmpty() )
-	    return;
+        if ( newString.isEmpty() )
+            return;
 
-	// Start conversion
-	m_isComposing = true;
+        // Start conversion
+        m_isComposing = true;
     }
 
     if ( !newString.isEmpty() ) {
         QInputMethodEvent e( newString, getPreeditAttrs() );
         sendEvent( e );
-	// Qt4.3.1 does not call back update() here
-	update();
+        // Qt4.3.1 does not call back update() here
+        update();
     } else {
-	// Complete conversion implicitly since the preedit is empty
-	commitString( "" );
+        // Complete conversion implicitly since the preedit is empty
+        commitString( "" );
     }
 }
 
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+void QUimInputContext::savePreedit()
+{
+    m_ucHash.insert( focusedWidget, m_uc );
+    psegsHash.insert( focusedWidget, psegs );
+    cwinHash.insert( focusedWidget, cwin );
+    visibleHash.insert( focusedWidget, cwin->isVisible() );
+    cwin->hide();
+
+    const char *imname = uim_get_current_im_name( m_uc );
+    if ( imname )
+        m_uc = createUimContext( imname );
+    psegs.clear();
+    createCandidateWindow();
+}
+
+void QUimInputContext::restorePreedit()
+{
+    if ( m_uc )
+        uim_release_context( m_uc );
+    delete cwin;
+    m_uc = m_ucHash.take( focusedWidget );
+    psegs = psegsHash.take( focusedWidget );
+    cwin = cwinHash.take( focusedWidget );
+    if ( visibleHash.take( focusedWidget ) )
+        cwin->popup();
+}
+#endif
+
 void QUimInputContext::saveContext()
 {
     // just send IMEnd and keep preedit string
     if ( isComposing() )
-	commitString( "" );
+        commitString( "" );
 }
 
 void QUimInputContext::restoreContext()
@@ -636,11 +697,6 @@ void QUimInputContext::restoreContext()
     updatePreedit();
 }
 
-bool QUimInputContext::isPreeditRelocationEnabled()
-{
-    return ( language() == "ja" );
-}
-
 bool QUimInputContext::isPreeditPreservationEnabled()
 {
     return ( language() == "ja" );
@@ -650,17 +706,17 @@ QString QUimInputContext::getPreeditString()
 {
     QString pstr;
 
-    QList<PreeditSegment*>::ConstIterator seg = psegs.begin();
-    const QList<PreeditSegment*>::ConstIterator end = psegs.end();
+    QList<PreeditSegment>::ConstIterator seg = psegs.begin();
+    const QList<PreeditSegment>::ConstIterator end = psegs.end();
     for ( ; seg != end; ++seg )
     {
-        if ( ( *seg ) ->attr & UPreeditAttr_Separator && ( *seg ) ->str.isEmpty() )
+        if ( ( *seg ).attr & UPreeditAttr_Separator && ( *seg ).str.isEmpty() )
         {
             pstr += DEFAULT_SEPARATOR_STR;
         }
         else
         {
-            pstr += ( *seg ) ->str;
+            pstr += ( *seg ).str;
         }
     }
 
@@ -673,22 +729,22 @@ int QUimInputContext::getPreeditCursorPosition()
         return 0;
 
     int cursorPos = 0;
-    QList<PreeditSegment*>::ConstIterator seg = psegs.begin();
-    const QList<PreeditSegment*>::ConstIterator end = psegs.end();
+    QList<PreeditSegment>::ConstIterator seg = psegs.begin();
+    const QList<PreeditSegment>::ConstIterator end = psegs.end();
     for ( ; seg != end; ++seg )
     {
-        if ( ( *seg ) ->attr & UPreeditAttr_Cursor )
+        if ( ( *seg ).attr & UPreeditAttr_Cursor )
         {
             return cursorPos;
         }
-        else if ( ( *seg ) ->attr & UPreeditAttr_Separator
-                  && ( *seg ) ->str.isEmpty() )
+        else if ( ( *seg ).attr & UPreeditAttr_Separator
+                  && ( *seg ).str.isEmpty() )
         {
             cursorPos += QString( DEFAULT_SEPARATOR_STR ).length();
         }
         else
         {
-            cursorPos += ( *seg ) ->str.length();
+            cursorPos += ( *seg ).str.length();
         }
     }
 
@@ -699,8 +755,8 @@ int QUimInputContext::getPreeditSelectionLength()
 {
     int selectionLength = 0;
 
-    QList<PreeditSegment*>::ConstIterator seg = psegs.begin();
-    const QList<PreeditSegment*>::ConstIterator end = psegs.end();
+    QList<PreeditSegment>::ConstIterator seg = psegs.begin();
+    const QList<PreeditSegment>::ConstIterator end = psegs.end();
     for ( ; seg != end; ++seg )
     {
         // In converting state, uim encodes UPreeditAttr_Cursor into
@@ -710,9 +766,9 @@ int QUimInputContext::getPreeditSelectionLength()
         // UPreeditAttr_Underline or UPreeditAttr_Reverse to detect
         // logical selection length. They are sometimes disabled by
         // user preference.
-        if ( ( *seg ) ->attr & UPreeditAttr_Cursor )
+        if ( ( *seg ).attr & UPreeditAttr_Cursor )
         {
-            selectionLength = ( *seg ) ->str.length();
+            selectionLength = ( *seg ).str.length();
             return selectionLength;
         }
     }
@@ -720,6 +776,14 @@ int QUimInputContext::getPreeditSelectionLength()
     return 0;
 }
 
+static QColor getUserDefinedColor( const char *symbol )
+{
+    char *literal = uim_scm_symbol_value_str( symbol );
+    QColor color( QString::fromAscii( literal ) );
+    free( literal );
+    return color;
+}
+
 QList<QInputMethodEvent::Attribute> QUimInputContext::getPreeditAttrs()
 {
     const int HIDE_CARET = 0;
@@ -727,166 +791,77 @@ QList<QInputMethodEvent::Attribute> QUimInputContext::getPreeditAttrs()
     const int DUMMY = 0;
     QList<QInputMethodEvent::Attribute> attrs;
 
-    QList<PreeditSegment *>::ConstIterator seg = psegs.begin();
-    const QList<PreeditSegment *>::ConstIterator end = psegs.end();
+    QList<PreeditSegment>::ConstIterator seg = psegs.begin();
+    const QList<PreeditSegment>::ConstIterator end = psegs.end();
     int segPos = 0;
     for ( ; seg != end; ++seg ) {
-	int uimAttr = ( *seg )->attr;
-	int segStrLen = ( *seg )->str.length();
-	QTextCharFormat segFmt;
+        int uimAttr = ( *seg ).attr;
+        int segStrLen = ( *seg ).str.length();
+        QTextCharFormat segFmt;
 
         if ( uimAttr & UPreeditAttr_Cursor ) {
-	    // Transparent cursor is required to set microfocus even
-	    // if the caret is invisible to user.
-	    //
-	    // FIXME: Segment string may be outframed if the cursor is
-	    // located near the right frame.
-	    int visibility = ( segStrLen ) ? HIDE_CARET : SHOW_CARET;
-	    QInputMethodEvent::Attribute cursor( QInputMethodEvent::Cursor,
-						 segPos, visibility, DUMMY );
-	    attrs << cursor;
-	} else if ( uimAttr & UPreeditAttr_Separator ) {
-	    if ( !segStrLen )
-		segStrLen = QString( DEFAULT_SEPARATOR_STR ).length();
-	}
+            // Transparent cursor is required to set microfocus even
+            // if the caret is invisible to user.
+            //
+            // FIXME: Segment string may be outframed if the cursor is
+            // located near the right frame.
+            int visibility = ( segStrLen ) ? HIDE_CARET : SHOW_CARET;
+            QInputMethodEvent::Attribute cursor( QInputMethodEvent::Cursor,
+                                                 segPos, visibility, DUMMY );
+            attrs << cursor;
+        } else if ( uimAttr & UPreeditAttr_Separator ) {
+            if ( !segStrLen )
+                segStrLen = QString( DEFAULT_SEPARATOR_STR ).length();
+            if ( !( uimAttr & UPreeditAttr_Reverse ) ) {
+                QColor color = getUserDefinedColor( "separator-foreground" );
+                if ( color.isValid() )
+                    segFmt.setForeground( color );
+                color = getUserDefinedColor( "separator-background" );
+                if ( color.isValid() )
+                    segFmt.setBackground( color );
+    
+            }
+        }
 
-	if ( segStrLen ) {
-	    if ( uimAttr & UPreeditAttr_Reverse ) {
+        if ( segStrLen ) {
+            if ( uimAttr & UPreeditAttr_Reverse ) {
 #if 0
-		// FIXME: fmt.foreground() is white (expecting black)
-		QTextFormat fmt = standardFormat( PreeditFormat );
-		segFmt.setForeground( fmt.background() );
-		segFmt.setBackground( fmt.foreground() );
+                // FIXME: fmt.foreground() is white (expecting black)
+                QTextFormat fmt = standardFormat( PreeditFormat );
+                segFmt.setForeground( fmt.background() );
+                segFmt.setBackground( fmt.foreground() );
 #else
-		// FIXME: Retrieve customized colors from the text widget
-		segFmt.setForeground( Qt::white );
-		segFmt.setBackground( Qt::black );
+                // FIXME: Retrieve customized colors from the text widget
+                // foreground symbol
+                const char *fgSymbol;
+                // background symbol
+                const char *bgSymbol;
+                if ( uimAttr & UPreeditAttr_Separator ) {
+                    fgSymbol = "reversed-separator-foreground";
+                    bgSymbol = "reversed-separator-background";
+                } else {
+                    fgSymbol = "reversed-preedit-foreground";
+                    bgSymbol = "reversed-preedit-background";
+                }
+                QColor color = getUserDefinedColor( fgSymbol );
+                segFmt.setForeground( color.isValid() ? color : Qt::white );
+                color = getUserDefinedColor( bgSymbol );
+                segFmt.setBackground( color.isValid() ? color : Qt::black );
 #endif
-	    }
-	    if ( uimAttr & UPreeditAttr_UnderLine ) {
-		segFmt.setFontUnderline( TRUE );
-	    }
-	    QInputMethodEvent::Attribute segAttr( QInputMethodEvent::TextFormat,
-						  segPos, segStrLen, segFmt );
-	    attrs << segAttr;
-	    segPos += segStrLen;
-	}
+            }
+            if ( uimAttr & UPreeditAttr_UnderLine ) {
+                segFmt.setFontUnderline( true );
+            }
+            QInputMethodEvent::Attribute segAttr( QInputMethodEvent::TextFormat,
+                                                  segPos, segStrLen, segFmt );
+            attrs << segAttr;
+            segPos += segStrLen;
+        }
     }
 
     return attrs;
 }
 
-#if UIM_QT_USE_NEW_PAGE_HANDLING
-void QUimInputContext::prepare_page_candidates( int page )
-{
-    QList<uim_candidate> list;
-    list.clear();
-
-    if ( page < 0 )
-	return;
-
-    if ( pageFilled[ page ] )
-	return;
-
-    // set page candidates
-    uim_candidate cand;
-    int pageNr, start, nrCandidates, displayLimit;
-
-    nrCandidates = cwin->nrCandidates;
-    displayLimit = cwin->displayLimit;
-    start = page * displayLimit;
-
-    if ( displayLimit && ( nrCandidates - start ) > displayLimit )
-	pageNr = displayLimit;
-    else
-	pageNr = nrCandidates - start;
-
-    for ( int i = start; i < ( pageNr + start ); i++ )
-    {
-	cand = uim_get_candidate( m_uc, i, displayLimit ? i % displayLimit : i );
-	list.append( cand );
-    }
-    pageFilled[ page ] = true;
-    cwin->setPageCandidates( page, list );
-}
-#endif /* UIM_QT_USE_NEW_PAGE_HANDLING */
-
-void QUimInputContext::candidateActivate( int nr, int displayLimit )
-{
-    QList<uim_candidate> list;
-    list.clear();
-
-#if !UIM_QT_USE_NEW_PAGE_HANDLING
-    cwin->activateCandwin( displayLimit );
-
-    // set candidates
-    uim_candidate cand;
-    for ( int i = 0; i < nr; i++ )
-    {
-        cand = uim_get_candidate( m_uc, i, displayLimit ? i % displayLimit : i );
-        list.append( cand );
-    }
-    cwin->setCandidates( displayLimit, list );
-
-#else /* !UIM_QT_USE_NEW_PAGE_HANDLING */
-    nrPages = displayLimit ? ( nr - 1 ) / displayLimit + 1 : 1;
-    pageFilled.clear();
-    for ( int i = 0; i < nrPages; i++ )
-	pageFilled.append( false );
-    
-    cwin->setNrCandidates( nr, displayLimit );
-
-    // set page candidates
-    prepare_page_candidates( 0 );
-    cwin->setPage( 0 );
-#endif /* !UIM_QT_USE_NEW_PAGE_HANDLING */
-    cwin->popup();
-    candwinIsActive = true;
-}
-
-void QUimInputContext::candidateSelect( int index )
-{
-#if UIM_QT_USE_NEW_PAGE_HANDLING
-    int new_page;
-    
-    if ( index >= cwin->nrCandidates )
-	index = 0;
-
-    if ( index >= 0 && cwin->displayLimit )
-	new_page = index / cwin->displayLimit;
-    else
-	new_page = cwin->pageIndex;
-
-    prepare_page_candidates( new_page );
-#endif /* UIM_QT_USE_NEW_PAGE_HANDLING */
-    cwin->setIndex( index );
-}
-
-void QUimInputContext::candidateShiftPage( bool forward )
-{
-#if UIM_QT_USE_NEW_PAGE_HANDLING
-    int new_page, index;
-
-    index = forward ? cwin->pageIndex + 1 : cwin->pageIndex - 1;
-    if ( index < 0 )
-	new_page = nrPages - 1;
-    else if ( index >= nrPages )
-	new_page = 0;
-    else
-	new_page = index;
-
-    prepare_page_candidates( new_page );
-#endif /* UIM_QT_USE_NEW_PAGE_HANDLING */
-    cwin->shiftPage( forward );
-}
-
-
-void QUimInputContext::candidateDeactivate()
-{
-    cwin->deactivateCandwin();
-    candwinIsActive = false;
-}
-
 void QUimInputContext::switch_app_global_im( const char *name )
 {
     QList<QUimInputContext*>::iterator it;
@@ -901,7 +876,7 @@ void QUimInputContext::switch_app_global_im( const char *name )
             ( *it )->readIMConf();
         }
     }
-    uim_prop_update_custom(this->uimContext(), "custom-preserved-default-im-name", ( const char* ) im_name_sym.toUtf8() );
+    uim_prop_update_custom(this->uimContext(), "custom-preserved-default-im-name", im_name_sym.toUtf8().data() );
 }
 
 void QUimInputContext::switch_system_global_im( const char *name )
@@ -913,13 +888,15 @@ void QUimInputContext::switch_system_global_im( const char *name )
 void QUimInputContext::readIMConf()
 {
     char * leftp = uim_scm_symbol_value_str( "candidate-window-position" );
-    if ( leftp && !strcmp( leftp, "left" ) )
-        cwin->setAlwaysLeftPosition( true );
-    else
-        cwin->setAlwaysLeftPosition( false );
+    cwin->setAlwaysLeftPosition( leftp && !strcmp( leftp, "left" ) );
     free( leftp );
 }
 
+void QUimInputContext::updateIndicator( const QString &str )
+{
+    m_indicator->update( str );
+}
+
 static int unicodeToUKey (ushort c) {
     int sym;
 
