@@ -1,7 +1,7 @@
 /*
 
   Copyright (c) 2004-2005 Kazuki Ohta <mover@hct.zaq.ne.jp>
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -35,44 +35,45 @@
 #ifndef UIM_QT4_IMMODULE_QUIMINPUTCONTEXT_H
 #define UIM_QT4_IMMODULE_QUIMINPUTCONTEXT_H
 
-#include <qinputcontext.h>
-#include <qevent.h>
+#include <QtGui/QInputContext>
 
 #ifdef Q_WS_X11
 #define UIM_QT_USE_JAPANESE_KANA_KEYBOARD_HACK 1
 #endif
-#define UIM_QT_USE_NEW_PAGE_HANDLING 1
 
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
-#include <uim/uim-util.h>
 
-class CandidateWindow;
+class QEvent;
+
+class AbstractCandidateWindow;
+class CaretStateIndicator;
 class QUimHelperManager;
+class QUimTextUtil;
 #ifdef Q_WS_X11
 typedef struct _DefTree DefTree;
 class Compose;
 #endif
-class QUimTextUtil;
 
-class PreeditSegment
+struct PreeditSegment
 {
-public:
-    PreeditSegment( int p_attr, const QString &p_str )
+    PreeditSegment( int attr, const QString &str )
     {
-        attr = p_attr;
-        str = p_str;
+        this->attr = attr;
+        this->str = str;
     }
 
     int attr;
     QString str;
 };
 
+#define WORKAROUND_BROKEN_RESET_IN_QT4
+
 class QUimInputContext : public QInputContext
 {
     Q_OBJECT
 public:
-    QUimInputContext( const char *imname = 0, const char *lang = 0 );
+    explicit QUimInputContext( const char *imname = 0 );
     ~QUimInputContext();
 
     virtual QString identifierName();
@@ -88,12 +89,9 @@ public:
     virtual bool isComposing() const { return m_isComposing; }
     virtual void setFocusWidget( QWidget *w );
 
-    virtual bool isPreeditRelocationEnabled();
-
     uim_context uimContext() { return m_uc; }
 
     static QUimInputContext *focusedIC();
-    static void reloadUim();
 
     void commitString( const QString& str );
 
@@ -107,16 +105,22 @@ public:
     void saveContext();
     void restoreContext();
 
+    void updateIndicator( const QString &str );
+
 protected:
     uim_context createUimContext( const char *imname );
-    virtual bool isPreeditPreservationEnabled();  // not a QInputContext func
+    void createCandidateWindow();
+    bool isPreeditPreservationEnabled();  // not a QInputContext func
     virtual void setFocus();    // not a QInputContext func
     virtual void unsetFocus();  // not a QInputContext func
 
 private:
-    void setMicroFocus( int x, int y, int w, int h, QFont *f = 0 );
     int getPreeditSelectionLength();
     QList<QInputMethodEvent::Attribute> getPreeditAttrs();
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    void savePreedit();
+    void restorePreedit();
+#endif
 
     /* callbacks for uim */
     static void commit_cb( void *ptr, const char *str );
@@ -137,51 +141,47 @@ private:
     void clearPreedit();
     void pushbackPreeditString( int attr, const QString& str );
     void updatePreedit();
-    //candidate
-    void candidateActivate( int nr, int displayLimit );
-    void candidateSelect( int index );
-    void candidateShiftPage( bool forward );
-    void candidateDeactivate();
     //imsw
     void switch_app_global_im( const char *str );
     void switch_system_global_im( const char *str );
 
-#if UIM_QT_USE_NEW_PAGE_HANDLING
-    void prepare_page_candidates( int page );
-#endif
 #ifdef Q_WS_X11
     // for X11 Compose
     static DefTree *mTreeTop;
     static void create_compose_tree( void );
-    static char *get_compose_filename( void );
-    static char *TransFileName( char *name );
+    static int get_compose_filename( char *filename, size_t len );
+    static int TransFileName( char *transname, const char *name, size_t len );
     static void ParseComposeStringFile( FILE *fp );
     static void FreeComposeTree( DefTree *top );
     static int parse_compose_line( FILE *fp, char **tokenbuf, size_t *buflen );
     static int get_mb_string( char *buf, unsigned int ks );
     static const char *get_encoding( void );
-    static char *get_lang_region( void );
+    static int get_lang_region( char *lang_region, size_t len );
 
     Compose *mCompose;
 #endif
     QUimTextUtil *mTextUtil;
 
+    CaretStateIndicator *m_indicator;
+
 protected:
-    QString m_imname;
-    QString m_lang;
-    uim_context m_uc;
     bool candwinIsActive;
     bool m_isComposing;
 
-    QList<PreeditSegment*> psegs;
+    uim_context m_uc;
+    QList<PreeditSegment> psegs;
+    AbstractCandidateWindow *cwin;
 
-    CandidateWindow *cwin;
-    static QUimHelperManager *m_HelperManager;
+#ifdef WORKAROUND_BROKEN_RESET_IN_QT4
+    QHash<QWidget*, uim_context> m_ucHash;
+    QHash<QWidget*, QList<PreeditSegment> > psegsHash;
+    QHash<QWidget*, AbstractCandidateWindow*> cwinHash;
+    QHash<QWidget*, bool> visibleHash;
 
-#if UIM_QT_USE_NEW_PAGE_HANDLING
-    QList<bool> pageFilled;
-    int nrPages;
+    QWidget *focusedWidget;
 #endif
+
+    static QUimHelperManager *m_HelperManager;
 };
 
 #endif /* Not def: UIM_QT4_IMMODULE_QUIMINPUTCONTEXT_H */
