@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
 All rights reserved.
 
@@ -30,59 +30,58 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
 */
-
-//#include <config.h>
-
-#include <qlabel.h>
-#include <qtextbrowser.h>
-#include <qtimer.h>
-#include <qapplication.h>
-#include <QDesktopWidget>
-#include <qrect.h>
-#include <qpoint.h>
-
 #include "subwindow.h"
 
-const Qt::WFlags subwindowFlag = ( Qt::Window
-                                   | Qt::WindowStaysOnTopHint
-                                   | Qt::FramelessWindowHint
-                                   | Qt::Tool
+#include <QtCore/QPoint>
+#include <QtCore/QRect>
+#include <QtGui/QApplication>
+#include <QtGui/QDesktopWidget>
+#include <QtGui/QTextBrowser>
+#include <QtGui/QVBoxLayout>
+
+
+const Qt::WindowFlags subwindowFlag = ( Qt::Window
+                                        | Qt::WindowStaysOnTopHint
+                                        | Qt::FramelessWindowHint
+                                        | Qt::Tool
 #if defined(Q_WS_X11)
-                                   | Qt::X11BypassWindowManagerHint
+                                        | Qt::X11BypassWindowManagerHint
 #endif
                                  );
 
 static const int TIMER_INTERVAL = 1000; // 1000ms = 1second
 
-SubWindow::SubWindow( QWidget *parent, const char *name )
-        : Q3VBox( parent, name, subwindowFlag )
+SubWindow::SubWindow( QWidget *parent )
+        : QFrame( parent, subwindowFlag )
 {
-    m_titleLabel = new QLabel( this );
-    m_titleLabel->setAlignment( Qt::AlignHCenter );
-    m_titleLabel->setPaletteBackgroundColor( Qt::darkGray );
-    m_titleLabel->setPaletteForegroundColor( Qt::white );
-
     m_contentsEdit = new QTextBrowser( this );
 
     m_hookTimer = new QTimer( this );
     connect( m_hookTimer, SIGNAL( timeout() ), this, SLOT( timerDone() ) );
 
+    QVBoxLayout *layout = new QVBoxLayout;
+    layout->setMargin( 0 );
+    layout->addWidget( m_contentsEdit );
+    setLayout( layout );
+
+    adjustSize();
+
     hide();
 }
 
 SubWindow::~SubWindow()
 {}
 
-void SubWindow::hookPopup( const QString &title, const QString contents )
+void SubWindow::hookPopup( const QString &contents )
 {
     // stop now running timer
     if ( m_hookTimer->isActive() )
         m_hookTimer->stop();
 
-    m_titleLabel->setText( title );
     m_contentsEdit->setText( contents );
 
-    m_hookTimer->start( TIMER_INTERVAL, TRUE );
+    m_hookTimer->setSingleShot( true );
+    m_hookTimer->start( TIMER_INTERVAL );
 }
 
 void SubWindow::popup()
@@ -102,25 +101,21 @@ void SubWindow::timerDone()
     popup();
 }
 
-void SubWindow::layoutWindow( int x, int y )
+void SubWindow::layoutWindow( const QRect &rect )
 {
-    QRect focusRect = QRect( QPoint( x, y ), frameSize() );
-    QRect screenRect = QRect( 0, 0,
-                              QApplication::desktop() ->screenGeometry().width(),
-                              QApplication::desktop() ->screenGeometry().height() );
+    const QRect screenRect = QApplication::desktop()->screenGeometry();
 
-    QPoint p = forceInside( screenRect, focusRect );
-    move( p );
-}
+    const int w = width();
+    const int candX = rect.x();
+    int destX = candX + rect.width();
+    if ( destX + w > screenRect.width() )
+        destX = candX - w;
 
-QPoint SubWindow::forceInside( const QRect &enclosure, const QRect &prisoner )
-{
-    int new_x, new_y;
+    const int h = height();
+    const int screenH = screenRect.height();
+    int destY = rect.y();
+    if ( destY + h > screenH )
+        destY = screenH - h;
 
-    new_x = QMIN( enclosure.right(), prisoner.right() ) - prisoner.width() + 1;
-    new_x = QMAX( enclosure.left(), new_x );
-    new_y = QMIN( enclosure.bottom(), prisoner.bottom() ) - prisoner.height() + 1;
-    new_y = QMAX( enclosure.top(), new_y );
-
-    return QPoint( new_x, new_y );
+    move( destX, destY );
 }
