@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -37,6 +37,7 @@
 
 #include "qtgettext.h"
 
+#include <QtCore/QPointer>
 #include <QtGui/QFileDialog>
 #include <QtGui/QLabel>
 #include <QtGui/QTreeWidget>
@@ -167,7 +168,7 @@ void CustomLineEdit::slotCustomTextChanged( const QString &text )
     Q_ASSERT( m_custom->type == UCustom_Str );
 
     free( m_custom->value->as_str );
-    m_custom->value->as_str = strdup( (const char*)text.toUtf8() );
+    m_custom->value->as_str = strdup( text.toUtf8().data() );
 
     setCustom( m_custom );
 }
@@ -203,6 +204,7 @@ CustomPathnameEdit::CustomPathnameEdit( struct uim_custom *c, QWidget *parent )
                       this, SLOT(slotPathnameButtonClicked()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 3 );
     layout->addWidget( m_lineEdit );
     layout->addWidget( m_fileButton );
@@ -259,7 +261,7 @@ void CustomPathnameEdit::slotCustomTextChanged( const QString & text )
     Q_ASSERT( m_custom->type == UCustom_Pathname );
 
     free( m_custom->value->as_pathname->str );
-    m_custom->value->as_pathname->str = strdup( (const char*)text.toUtf8() );
+    m_custom->value->as_pathname->str = strdup( text.toUtf8().data() );
 
     setCustom( m_custom );
 }
@@ -359,6 +361,7 @@ CustomOrderedListEdit::CustomOrderedListEdit( struct uim_custom *c, QWidget *par
                       this, SLOT(slotEditButtonClicked()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 3 );
     layout->addWidget( m_lineEdit );
     layout->addWidget( m_editButton );
@@ -638,6 +641,7 @@ CustomKeyEdit::CustomKeyEdit( struct uim_custom *c, QWidget *parent )
                       this, SLOT(slotKeyButtonClicked()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 3 );
     layout->addWidget( m_lineEdit );
     layout->addWidget( m_editButton );
@@ -763,7 +767,7 @@ void CustomKeyEdit::slotKeyButtonClicked()
 
         for( int i = 0; i < num; i++ )
         {
-            const char *keystr = (const char *)keyStrList[i].toAscii();
+            const char *keystr = keyStrList[i].toAscii().data();
 
             struct uim_custom_key *item = (struct uim_custom_key *)malloc(sizeof(struct uim_custom_key));
             item->type        = UCustomKey_Regular;
@@ -825,7 +829,7 @@ const QStringList KeyEditForm::getKeyStrList()
 
 void KeyEditForm::slotAddClicked()
 {
-    KeyGrabDialog *d = new KeyGrabDialog( this );
+    QPointer<KeyGrabDialog> d = new KeyGrabDialog( this );
     if( d->exec() == KeyGrabDialog::Accepted )
     {
         QString keystr = d->getKeyStr();
@@ -852,7 +856,7 @@ void KeyEditForm::slotEditClicked()
     QList<QTreeWidgetItem *> selectedItems = m_listView->selectedItems();
     if( selectedItems.isEmpty() )
         return;
-    KeyGrabDialog *d = new KeyGrabDialog( this );
+    QPointer<KeyGrabDialog> d = new KeyGrabDialog( this );
     if( d->exec() == KeyGrabDialog::Accepted )
     {
         QString keystr = d->getKeyStr();
@@ -883,8 +887,7 @@ KeyGrabDialog::KeyGrabDialog( QWidget *parent )
     : QDialog( parent ),
       pressed_keyval( 0 ),
       pressed_keystate( Qt::NoModifier ),
-      pressed_unichar ( 0 ),
-      m_keystr( 0 )
+      pressed_unichar ( 0 )
 {
     QLabel *l = new QLabel( _("Press keys to grab (e.g. <Control>a)"), this );
 
@@ -901,8 +904,9 @@ void KeyGrabDialog::keyPressEvent( QKeyEvent *e )
     pressed_unichar = e->text().at(0);
 }
 
-void KeyGrabDialog::keyReleaseEvent( QKeyEvent * /* e */ )
+void KeyGrabDialog::keyReleaseEvent( QKeyEvent *e )
 {
+    Q_UNUSED( e )
     // create keystr
     setKeyStr();
 
@@ -977,7 +981,7 @@ void KeyGrabDialog::setKeyStr()
     case Qt::Key_End:
         keystr += "end";
         break;
-#ifdef QT_IMMODULE
+#ifdef QT4_IMMODULE
     case Qt::Key_Multi_key:
         keystr += "Multi_key";
         break;
@@ -1134,7 +1138,7 @@ void KeyGrabDialog::setKeyStr()
     case Qt::Key_Dead_Horn:
         keystr += "dead-horn";
         break;
-#endif /* Def: QT_IMMODULE */
+#endif /* Def: QT4_IMMODULE */
     case Qt::Key_Shift:
         keystr += "Shift_key";
         break;
