@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -34,7 +34,6 @@
 
 #include "qt4.h"
 #include "customwidgets.h"
-#include "kseparator.h"
 
 #include <QtCore/QFile>
 #include <QtCore/QSettings>
@@ -175,8 +174,12 @@ void UimPrefDialog::createMainWidgets()
     buttonHLayout->addWidget( m_applyButton );
     buttonHLayout->addWidget( cancelButton );
 
+    QFrame *separator = new QFrame( this );
+    separator->setFrameShape( QFrame::HLine );
+    separator->setFrameShadow( QFrame::Sunken );
+    mainVLayout->setMargin( 0 );
     mainVLayout->addWidget( mainSplitter );
-    mainVLayout->addWidget( new KSeparator( this ) );
+    mainVLayout->addWidget( separator );
     mainVLayout->addWidget( buttonHWidget );
 
     mainSplitter->setStretchFactor( 1, 1 );
@@ -294,7 +297,7 @@ void UimPrefDialog::slotSetDefault()
     QWidget *w = m_groupWidgetStack->currentWidget();
     if( w )
     {
-        ((GroupPageWidget*)w)->setDefault();
+        (static_cast<GroupPageWidget*>(w))->setDefault();
     }
 }
 
@@ -353,7 +356,9 @@ void QConfirmDialog::setupWidgets( const QString&msg )
     vLayout->setSpacing( 6 );
     vLayout->setMargin( 6 );
     QLabel *msgLabel = new QLabel( msg, this );
-    KSeparator *sep = new KSeparator( this );
+    QFrame *sep = new QFrame( this );
+    sep->setFrameShape( QFrame::HLine );
+    sep->setFrameShadow( QFrame::Sunken );
     vLayout->addWidget( msgLabel );
     vLayout->addWidget( sep );
 
@@ -415,7 +420,9 @@ void GroupPageWidget::setupWidgets()
     groupLabel->setAlignment( Qt::AlignLeft );
     vLayout->addWidget( groupLabel );
     
-    KSeparator *separator = new KSeparator( this );
+    QFrame *separator = new QFrame( this );
+    separator->setFrameShape( QFrame::HLine );
+    separator->setFrameShadow( QFrame::Sunken );
     vLayout->addWidget( separator );
 
     /* default QVGroupBox */
@@ -568,6 +575,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypeInteger( QGroupBox *vbox, stru
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addStretch();
@@ -589,6 +597,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypeString( QGroupBox *vbox, struc
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addWidget( lineEdit );
@@ -609,6 +618,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypePathname( QGroupBox *vbox, str
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addWidget( pathnameEdit );
@@ -630,6 +640,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypeChoice( QGroupBox *vbox, struc
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addWidget( choiceCombo );
@@ -650,6 +661,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypeOrderedList( QGroupBox *vbox,
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addWidget( olistEditBox );
@@ -670,6 +682,7 @@ UimCustomItemIface *GroupPageWidget::addCustomTypeKey( QGroupBox *vbox, struct u
                       this, SLOT(slotCustomValueChanged()) );
 
     QHBoxLayout *layout = new QHBoxLayout;
+    layout->setMargin( 0 );
     layout->setSpacing( 6 );
     layout->addWidget( label );
     layout->addWidget( keyEditBox );
@@ -696,6 +709,9 @@ int main( int argc, char **argv )
     
     QApplication a( argc, argv );
 
+    QCoreApplication::setOrganizationName( "uim" );
+    QCoreApplication::setApplicationName( "uim" );
+
     UimPrefDialog *dlg = new UimPrefDialog();
     dlg->resize( DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT );
     dlg->show();
