@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -51,11 +51,14 @@
 #include <cstdlib>
 #include <clocale>
 
+#include <uim/uim-scm.h>
+#include <uim/uim-custom.h>
 #include "qtgettext.h"
 
 static const int NAME_COLUMN = 0;
 
 static int uim_fd;
+static bool customEnabled;
 static QSocketNotifier *notifier = 0;
 
 int main( int argc, char **argv )
@@ -85,9 +88,15 @@ UimImSwitcher::UimImSwitcher( QWidget *parent )
     uim_fd = -1;
     checkHelperConnection();
 
+    /* to check if another uim-im-switcher exists */
+    uim_helper_send_message( uim_fd, "im_switcher_start\n" );
+
     /* to load input method list */
     uim_helper_send_message( uim_fd, "im_list_get\n" );
 
+    uim_init();
+    customEnabled = uim_custom_enable();
+
     /* create GUI */
     createGUI();
 }
@@ -124,20 +133,25 @@ void UimImSwitcher::createGUI()
     QGroupBox *groupBox = new QGroupBox(  _( "Effective coverage" ) );
     groupBox->setLayout( vbox );
 
-    /* cancel & ok button */
-    okButton = new QPushButton( this );
+    /* cancel, apply & ok button */
+    QPushButton *okButton = new QPushButton( this );
     okButton->setText( _( "OK" ) );
-    okButton->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
     connect( okButton, SIGNAL( clicked() ),
+                      this, SLOT( slotChangeInputMethodAndQuit() ) );
+
+    QPushButton *applyButton = new QPushButton( this );
+    applyButton->setText( _( "Apply" ) );
+    connect( applyButton, SIGNAL( clicked() ),
                       this, SLOT( slotChangeInputMethod() ) );
-    cancelButton = new QPushButton( this );
+
+    QPushButton *cancelButton = new QPushButton( this );
     cancelButton->setText( _( "Cancel" ) );
-    cancelButton->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
     connect( cancelButton, SIGNAL( clicked() ),
-                      qApp, SLOT( quit() ) );
+                      QApplication::instance(), SLOT( quit() ) );
     QHBoxLayout *buttonLayout = new QHBoxLayout;
     buttonLayout->addStretch( 0 );
     buttonLayout->addWidget( okButton );
+    buttonLayout->addWidget( applyButton );
     buttonLayout->addWidget( cancelButton );
 
     // main layout
@@ -161,7 +175,7 @@ void UimImSwitcher::checkHelperConnection()
                 delete notifier;
             notifier = new QSocketNotifier( uim_fd, QSocketNotifier::Read );
             connect( notifier, SIGNAL( activated( int ) ),
-                              this, SLOT( slotStdinActivated( int ) ) );
+                              this, SLOT( slotStdinActivated() ) );
         }
     }
 }
@@ -172,16 +186,23 @@ void UimImSwitcher::helper_disconnect_cb()
     disconnect( notifier, SIGNAL( activated( int ) ), 0, 0 );
 }
 
+void UimImSwitcher::slotChangeInputMethodAndQuit()
+{
+    slotChangeInputMethod();
+    QApplication::instance()->quit();
+}
+
 void UimImSwitcher::slotChangeInputMethod()
 {
     if ( wholeButton->isChecked() )
+    {
         sendMessageImChange( "im_change_whole_desktop\n" );
+        saveDefaultIm();
+    }
     else if ( applicationButton->isChecked() )
         sendMessageImChange( "im_change_this_application_only\n" );
     else if ( textButton->isChecked() )
         sendMessageImChange( "im_change_this_text_area_only\n" );
-
-    qApp->quit();
 }
 
 void UimImSwitcher::sendMessageImChange( const QString &change_type )
@@ -198,9 +219,25 @@ void UimImSwitcher::sendMessageImChange( const QString &change_type )
     msg.append( imName );
     msg.append( "\n" );
 
-    uim_helper_send_message( uim_fd, ( const char* ) msg.toUtf8() );
+    uim_helper_send_message( uim_fd, msg.toUtf8().data() );
 }
 
+void UimImSwitcher::saveDefaultIm()
+{
+    if ( customEnabled )
+    {
+        QString imName = selectedImName();
+        if ( imName.isEmpty() )
+            return ;
+
+        uim_scm_callf( "custom-set-value!",
+                       "yy",
+                       "custom-preserved-default-im-name",
+                       imName.toUtf8().data() );
+        uim_custom_save_custom( "custom-preserved-default-im-name" );
+    }
+}
+ 
 QString UimImSwitcher::selectedImName() const
 {
     QList<QTableWidgetItem *> selectedItems = listview->selectedItems();
@@ -212,7 +249,7 @@ QString UimImSwitcher::selectedImName() const
     return QString();
 }
 
-void UimImSwitcher::slotStdinActivated( int /*socket*/ )
+void UimImSwitcher::slotStdinActivated()
 {
     uim_helper_read_proc( uim_fd );
 
@@ -244,6 +281,10 @@ void UimImSwitcher::slotStdinActivated( int /*socket*/ )
             reloadImList();
         else if ( msg.startsWith( QLatin1String( "im_list" ) ) )
             parseHelperStrImList( msg );
+        else if ( msg.startsWith( QLatin1String( "im_switcher_start" ) ) )
+            uim_helper_send_message( uim_fd,  "im_switcher_quit\n" );
+        else if ( msg.startsWith( QLatin1String( "im_switcher_quit" ) ) )
+            QApplication::instance()->quit();
     }
 }
 
@@ -270,8 +311,10 @@ void UimImSwitcher::parseHelperStrImList( const QString &message )
             if (iminfoList[1].isEmpty())
                 lang = QString("-");
             else
-                lang = QString::fromUtf8(gettext(iminfoList[1].toUtf8()));
-            short_desc = QString::fromUtf8(gettext(iminfoList[2].toUtf8()));
+                lang = QString::fromUtf8(
+                    gettext(iminfoList[1].toUtf8().data()));
+            short_desc = QString::fromUtf8(
+                gettext(iminfoList[2].toUtf8().data()));
 
             // add new item to listview
             int row = listview->rowCount();
@@ -292,6 +335,9 @@ void UimImSwitcher::parseHelperStrImList( const QString &message )
                     && iminfoList[ 3 ] == QLatin1String( "selected" ) )
                 for ( int j = 0; j < listview->columnCount(); j++ )
                     listview->item( row, j )->setSelected( true );
+
+            listview->setRowHeight(
+                row, QFontMetrics( listview->font() ).height() + 2 );
         }
     }
     listview->sortItems( 0 );
