@@ -1,6 +1,6 @@
 /*
 
- Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+ Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
  All rights reserved.
 
@@ -33,28 +33,42 @@
 #include <config.h>
 
 #include "common-uimstateindicator.h"
+#include <uim/uim-scm.h>
 
 #include <QtCore/QSocketNotifier>
 #include <QtCore/QString>
 #include <QtCore/QStringList>
 #include <QtCore/QTextCodec>
 #include <QtGui/QHBoxLayout>
-#include <QtGui/QImage>
+#include <QtGui/QMouseEvent>
 #include <QtGui/QPixmap>
 
 #include <cstring>
 #include <cstdlib>
+#include <sys/stat.h>
+
+#include "qtgettext.h"
 
 static const QString ICONDIR = UIM_PIXMAPSDIR;
 static int uim_fd;
 static QHelperToolbarButton *fallbackButton = 0;
 static QSocketNotifier *notifier = 0;
 
+static inline QString qstring_(const QString &string)
+{
+#if ENABLE_NLS
+    return mygettext(string.toUtf8().data());
+#else
+    return string;
+#endif
+}
+
 UimStateIndicator::UimStateIndicator( QWidget *parent )
         : QFrame( parent )
 {
     m_layout = new QHBoxLayout;
     m_layout->setMargin( 0 );
+    m_layout->setSpacing( 0 );
 
     if ( !fallbackButton )
     {
@@ -62,10 +76,8 @@ UimStateIndicator::UimStateIndicator( QWidget *parent )
         m_layout->addWidget( fallbackButton );
         QPixmap icon = QPixmap( ICONDIR + '/' + "uim-icon.png" );
         if ( !icon.isNull() ) {
-            QImage image = icon.toImage();
-            QPixmap scaledIcon = QPixmap::fromImage(
-                image.scaled( ICON_SIZE, ICON_SIZE,
-                    Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
+            QPixmap scaledIcon = icon.scaled( ICON_SIZE, ICON_SIZE,
+                    Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
             fallbackButton->setIcon( QIcon( scaledIcon ) );
         } else {
             fallbackButton->setText( "?" );
@@ -108,7 +120,7 @@ void UimStateIndicator::checkHelperConnection()
                 delete notifier;
             notifier = new QSocketNotifier( uim_fd, QSocketNotifier::Read );
             connect( notifier, SIGNAL( activated( int ) ),
-                              this, SLOT( slotStdinActivated( int ) ) );
+                              this, SLOT( slotStdinActivated() ) );
         }
     }
 }
@@ -119,6 +131,8 @@ void UimStateIndicator::parseHelperStr( const QString& str )
     {
         if ( lines[ 0 ] == "prop_list_update" )
             propListUpdate( lines );
+        else if (lines[0] == "custom_reload_notify" )
+            uim_prop_reload_configs();
     }
 }
 
@@ -128,7 +142,9 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
         return;
 
     QHelperPopupMenu *popupMenu = 0;
+#ifdef PLASMA_APPLET_UIM
     int prevCount = m_layout->count();
+#endif
     foreach ( QHelperToolbarButton *button, buttons )
     {
         if ( m_layout->indexOf( button ) >= 0 )
@@ -158,12 +174,20 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
                 QHelperToolbarButton *button = new QHelperToolbarButton;
                 m_layout->addWidget( button );
                 buttons.append( button );
-                QPixmap icon = QPixmap( ICONDIR + '/' + fields[1] + ".png" );
+
+                uim_bool isDarkBg =
+                    uim_scm_symbol_value_bool("toolbar-icon-for-dark-background?");
+                const QString append = isDarkBg ? "_dark_background" : "";
+                QString fileName = ICONDIR + '/' + fields[1] + append + ".png";
+                struct stat st;
+                if ( isDarkBg && stat( fileName.toUtf8().data(), &st ) == -1 )
+		{
+                  fileName = ICONDIR + '/' + fields[1] + ".png";
+                }
+                QPixmap icon = QPixmap( fileName );
                 if (!icon.isNull()) {
-                    QImage image = icon.toImage();
-                    QPixmap scaledIcon = QPixmap::fromImage(
-                        image.scaled( ICON_SIZE, ICON_SIZE,
-                            Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
+                    QPixmap scaledIcon = icon.scaled( ICON_SIZE, ICON_SIZE,
+                            Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
                     button->setIcon( QIcon( scaledIcon ) );
                 } else {
                     button->setText( fields[ 2 ] );
@@ -171,11 +195,17 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
                 button->setToolTip( fields[ 3 ] );
 
                 // create popup
+#ifdef PLASMA_APPLET_UIM
+                popupMenu = new QHelperPopupMenu( 0 );
+#else
                 popupMenu = new QHelperPopupMenu( button );
+#endif
                 connect( popupMenu, SIGNAL( aboutToShow() ),
                     this, SLOT( slotPopupMenuAboutToShow() ) );
                 connect( popupMenu, SIGNAL( aboutToHide() ),
                     this, SLOT( slotPopupMenuAboutToHide() ) );
+                connect( button, SIGNAL( menuRequested( QMenu* ) ),
+                    this, SIGNAL( menuRequested( QMenu* ) ) );
                 button->setMenu( popupMenu );
                 button->setPopupMode( QToolButton::InstantPopup );
 
@@ -190,7 +220,8 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
                         && !fields[ 5 ].isEmpty() )
                 {
                     QAction *action = popupMenu->insertHelperItem(
-                        fields[1], fields[ 3 ], fields[ 4 ], fields[ 5 ] );
+                        fields[1], qstring_( fields[ 3 ] ),
+                        fields[ 4 ], fields[ 5 ] );
                     // check the item which is now used
                     if ( fields.count() > 6 && fields[ 6 ] == "*" )
                         action->setChecked( true );
@@ -199,10 +230,10 @@ void UimStateIndicator::propListUpdate( const QStringList& lines )
         }
     }
 
+#ifdef PLASMA_APPLET_UIM
     if ( m_layout->count() != prevCount )
+#endif
         emit indicatorResized();
-
-    parentWidget()->show();
 }
 
 void UimStateIndicator::helper_disconnect_cb()
@@ -211,7 +242,7 @@ void UimStateIndicator::helper_disconnect_cb()
     disconnect( notifier, SIGNAL( activated( int ) ), 0, 0 );
 }
 
-void UimStateIndicator::slotStdinActivated( int /*socket*/ )
+void UimStateIndicator::slotStdinActivated()
 {
     uim_helper_read_proc( uim_fd );
 
@@ -263,6 +294,30 @@ void UimStateIndicator::clearButtons()
     }
 }
 
+/**/
+QHelperToolbarButton::QHelperToolbarButton( QWidget *parent )
+    : QToolButton( parent )
+{
+    setAutoRaise( true );
+}
+
+QSize QHelperToolbarButton::sizeHint() const
+{
+    return QSize( BUTTON_SIZE, BUTTON_SIZE );
+}
+
+void QHelperToolbarButton::mousePressEvent( QMouseEvent *event )
+{
+#ifdef PLASMA_APPLET_UIM
+    QMenu *popupMenu = menu();
+    if ( event->button() == Qt::LeftButton && popupMenu ) {
+        emit menuRequested( popupMenu );
+        return;
+    }
+#endif
+    QToolButton::mousePressEvent( event );
+}
+
 /**/
 
 QHelperPopupMenu::QHelperPopupMenu( QWidget *parent )
@@ -284,13 +339,20 @@ QAction *QHelperPopupMenu::insertHelperItem( const QString &indicationIdStr,
                                         const QString &menucommandStr )
 {
     QAction *action;
-    QPixmap icon = QPixmap( ICONDIR + '/' + indicationIdStr + ".png" );
+    uim_bool isDarkBg =
+	    uim_scm_symbol_value_bool("toolbar-icon-for-dark-background?");
+    const QString append = isDarkBg ? "_dark_background" : "";
+    QString fileName = ICONDIR + '/' + indicationIdStr + append + ".png";
+    struct stat st;
+    if ( isDarkBg && stat( fileName.toUtf8().data(), &st ) == -1 )
+    {
+        fileName = ICONDIR + '/' + indicationIdStr + ".png";
+    }
+    QPixmap icon = QPixmap ( fileName );
 
     if (!icon.isNull()) {
-        QImage image = icon.toImage();
-        QPixmap scaledIcon = QPixmap::fromImage(
-            image.scaled( ICON_SIZE, ICON_SIZE,
-                Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
+        QPixmap scaledIcon = icon.scaled( ICON_SIZE, ICON_SIZE,
+                Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
         action = addAction( scaledIcon, menulabelStr );
     } else {
         action = addAction( menulabelStr );
