@@ -1,6 +1,6 @@
 ;;; action.scm: Handles user-operable actions of input methods
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -332,7 +332,11 @@
 ;; returns action-id list that can be passed to context-update-widget-states!
 ;; TODO: write test
 (define context-current-widget-states
-  (let ((widget-act-id (compose action-id widget-activity)))
+  (let ((widget-act-id (compose
+                         (lambda (activity) (if activity
+                                              (action-id activity)
+                                              #f))
+                         widget-activity)))
     (lambda (context)
       (map widget-act-id (context-widgets context)))))
 
@@ -353,12 +357,19 @@
       (if (not (null? (filter-map widget-update-state! widgets)))
           (context-propagate-widget-states context)))))
 
+(define bridge-show-input-state-mode-on? #f)
+
 (define context-propagate-prop-list-update
   (lambda (context)
     (let* ((widgets (context-widgets context))
 	   (branches (map widget-compose-live-branch
 			  widgets))
 	   (widget-config-tree (apply string-append branches)))
+      (if (eq? bridge-show-with?
+               'mode)
+          (if (eq? (context-current-mode context) 0)
+              (set! bridge-show-input-state-mode-on? #f)
+              (set! bridge-show-input-state-mode-on? #t)))
       (im-update-prop-list context widget-config-tree))))
 
 ;; API for uim developers
