@@ -1,6 +1,6 @@
 ;;; anthy-custom.scm: Customization variables for anthy.scm
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -43,7 +43,7 @@
 		     (N_ "Anthy (advanced)")
 		     (N_ "Advanced settings for Anthy"))
 
-(define-custom-group 'prediction
+(define-custom-group 'anthy-prediction
 		     (N_ "Prediction")
 		     (N_ "long description will be here."))
 
@@ -167,13 +167,23 @@
 	 'ja_azik
 	 ""
 	 (N_ "AZIK")
-	 (N_ "AZIK extended romaji input mode"))))
+	 (N_ "AZIK extended romaji input mode"))
+   (list 'action_anthy_act
+	 'ja_act
+	 ""
+	 (N_ "ACT")
+	 (N_ "ACT extended romaji input mode"))
+   (list 'action_anthy_kzik
+	 'ja_kzik
+	 ""
+	 (N_ "KZIK")
+	 (N_ "KZIK extended romaji input mode"))))
 
 ;;; Buttons
 
 (define-custom 'anthy-widgets '(widget_anthy_input_mode
 				widget_anthy_kana_input_method)
-  '(anthy toolbar)
+  '(anthy toolbar-widget)
   (list 'ordered-list
 	(list 'widget_anthy_input_mode
 	      (N_ "Input mode")
@@ -196,7 +206,7 @@
 ;;; Input mode
 
 (define-custom 'default-widget_anthy_input_mode 'action_anthy_direct
-  '(anthy toolbar)
+  '(anthy toolbar-widget)
   (cons 'choice
 	(map indication-alist-entry-extract-choice
 	     anthy-input-mode-indication-alist))
@@ -205,7 +215,7 @@
 
 (define-custom 'anthy-input-mode-actions
                (map car anthy-input-mode-indication-alist)
-  '(anthy toolbar)
+  '(anthy toolbar-widget)
   (cons 'ordered-list
 	(map indication-alist-entry-extract-choice
 	     anthy-input-mode-indication-alist))
@@ -247,7 +257,7 @@
 ;;; Kana input method
 
 (define-custom 'default-widget_anthy_kana_input_method 'action_anthy_roma
-  '(anthy toolbar)
+  '(anthy toolbar-widget)
   (cons 'choice
 	(map indication-alist-entry-extract-choice
 	     anthy-kana-input-method-indication-alist))
@@ -256,7 +266,7 @@
 
 (define-custom 'anthy-kana-input-method-actions
                (map car anthy-kana-input-method-indication-alist)
-  '(anthy toolbar)
+  '(anthy toolbar-widget)
   (cons 'ordered-list
 	(map indication-alist-entry-extract-choice
 	     anthy-kana-input-method-indication-alist))
@@ -296,23 +306,29 @@
 		   (anthy-configure-widgets)))
 
 (define-custom 'anthy-use-prediction? #f
-  '(anthy-advanced prediction)
+  '(anthy-advanced anthy-prediction)
   '(boolean)
   (N_ "Enable input prediction")
   (N_ "long description will be here."))
 
 (define-custom 'anthy-select-prediction-by-numeral-key? #f
-  '(anthy-advanced prediction)
+  '(anthy-advanced anthy-prediction)
   '(boolean)
   (N_ "Select prediction candidate by numeral keys")
   (N_ "long description will be here."))
 
 (define-custom 'anthy-use-implicit-commit-prediction? #t
-  '(anthy-advanced prediction)
+  '(anthy-advanced anthy-prediction)
   '(boolean)
   (N_ "Show selected prediction candidate in preedit area")
   (N_ "long description will be here."))
 
+(define-custom 'anthy-prediction-start-char-count 1
+  '(anthy-advanced anthy-prediction)
+  '(integer 1 65535)
+  (N_ "Character count to start input prediction")
+  (N_ "long description will be here."))
+
 (custom-add-hook 'anthy-use-candidate-window?
 		 'custom-get-hooks
 		 (lambda ()
@@ -334,12 +350,23 @@
 		 (lambda ()
 		   anthy-use-prediction?))
 
+(custom-add-hook 'anthy-prediction-start-char-count
+		 'custom-activity-hooks
+		 (lambda ()
+		   anthy-use-prediction?))
+
 (define-custom 'anthy-use-with-vi? #f
   '(anthy-advanced special-op)
   '(boolean)
   (N_ "Enable vi-cooperative mode")
   (N_ "long description will be here."))
 
+(define-custom 'anthy-auto-start-henkan? #f
+  '(anthy-advanced special-op)
+  '(boolean)
+  (N_ "Enable auto conversion with punctuation marks")
+  (N_ "long description will be here."))
+
 (define-custom 'anthy-use-mode-transition-keys-in-off-mode? #f
   '(anthy-advanced mode-transition)
   '(boolean)
