@@ -1,6 +1,6 @@
 ;;; anthy-key-custom.scm: Customization variables for anthy.scm
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -271,13 +271,13 @@
 	       (N_ "long description will be here"))
 
 (define-custom 'anthy-next-prediction-key '("tab" "down" "<IgnoreCase><Control>n" "<IgnoreCase><Control>i")
-               '(anthy-keys4 prediction)
+               '(anthy-keys4 anthy-prediction)
 	       '(key)
 	       (N_ "[Anthy] Next prediction candidate")
 	       (N_ "long description will be here"))
 
 (define-custom 'anthy-prev-prediction-key '(generic-prev-candidate-key)
-               '(anthy-keys4 prediction)
+               '(anthy-keys4 anthy-prediction)
 	       '(key)
 	       (N_ "[Anthy] Previous prediction candidate")
 	       (N_ "long description will be here"))
