@@ -1,6 +1,6 @@
 ;;; byeoru-custom.scm: Customization variables for byeoru.scm
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -86,6 +86,10 @@
   (N_ "Properties")
   (N_ "long description will be here."))
 
+(define-custom-group 'byeoru-dict
+  (N_ "Byeoru dictionaries")
+  (N_ "long description will be here."))
+
 (define-custom-group 'byeoru-workarounds
   (N_ "Workarounds")
   (N_ "long description will be here."))
@@ -96,8 +100,8 @@
   (cons 'choice
 	(map (lambda (entry)
 	       (let ((id (car entry))
-		     (label (nth 2 entry))
-		     (short-desc (nth 3 entry)))
+		     (label (list-ref entry 2))
+		     (short-desc (list-ref entry 3)))
 		 (list id
 		       label
 		       short-desc)))
@@ -157,10 +161,37 @@
 
 (define-custom 'byeoru-symbol-cache-size 5
   '(byeoru candwin)
-  '(integer 1 20)
+  '(integer 0 20)
   (N_ "Symbol cache size")
   (N_ "long description will be here."))
 
+(define-custom 'byeoru-conversion-history-size 1000
+  '(byeoru byeoru-dict)
+  '(integer 0 99999)
+  (N_ "Length of conversion history to keep")
+  (N_ "long description will be here."))
+
+(define-custom 'byeoru-conversion-history-path
+  (string-append (or (home-directory (user-name)) "") "/.byeoru-uim-history")
+  '(byeoru byeoru-dict)
+  '(pathname regular-file)
+  (N_ "Conversion history file")
+  (N_ "long description will be here."))
+
+(define-custom 'byeoru-personal-dict-path
+  (string-append (or (home-directory (user-name)) "") "/.byeoru-uim-dict")
+  '(byeoru byeoru-dict)
+  '(pathname regular-file)
+  (N_ "Personal dictionary file")
+  (N_ "long description will be here."))
+
+(define-custom 'byeoru-sys-dict-path
+  (string-append (sys-pkgdatadir) "/byeoru-data/byeoru-dict")
+  '(byeoru byeoru-dict)
+  '(pathname regular-file)
+  (N_ "System dictionary file")
+  (N_ "long description will be here."))
+
 ;; Encoding of the composing character should be changed accordingly.
 (define-custom 'byeoru-compatibility-jamos-for-incomplete-syllables? #t
   '(byeoru byeoru-workarounds)
