@@ -1,7 +1,7 @@
 ;;; custom-rt.scm: Partial customization support for runtime input
 ;;;                processes
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -40,6 +40,8 @@
 
 ;; TODO: write test-custom-rt.scm
 
+(require-extension (srfi 6 34))
+
 (require "util.scm")
 (require "key.scm")
 
@@ -61,8 +63,11 @@
 (define custom-file-path
   (lambda (gsym)
     (let* ((group-name (symbol->string gsym))
-	   (path (string-append (getenv "HOME")
-				"/.uim.d/customs/custom-"
+           (config-path (get-config-path #f))
+           (path (string-append (if config-path
+                                  config-path
+                                  "")
+				"/customs/custom-"
 				group-name
 				".scm")))
       path)))
@@ -70,7 +75,9 @@
 ;; experimental
 (define custom-update-group-conf-freshness
   (lambda (gsym)
-    (let ((mtime (file-mtime (custom-file-path gsym))))
+    (let ((mtime (guard (err
+			 (else #f))
+		   (file-mtime (custom-file-path gsym)))))
       (set! custom-group-conf-freshnesses
 	    (alist-replace (cons gsym mtime)
 			   custom-group-conf-freshnesses))
@@ -81,8 +88,10 @@
   (lambda (gsym)
     (let ((prev-mtime (assq-cdr gsym custom-group-conf-freshnesses)))
       (or (not prev-mtime)
-	  (not (= (file-mtime (custom-file-path gsym))
-		  prev-mtime))))))
+	  (guard (err
+		  (else #f))
+	    (not (= (file-mtime (custom-file-path gsym))
+		    prev-mtime)))))))
 
 ;; experimental
 (define custom-load-updated-group-conf
@@ -113,10 +122,11 @@
       (let* ((post-groups (custom-list-primary-groups))
 	     (new-groups (list-tail post-groups (length pre-groups))))
 	(if (and (not (getenv "LIBUIM_VANILLA"))
-		 (not (is-set-ugid?)))
+		 (not (setugid?)))
 	    (for-each (lambda (gsym)
 			(custom-load-group-conf gsym)
-			(custom-update-group-conf-freshness gsym))
+			(if custom-enable-mtime-aware-user-conf-reloading?
+			    (custom-update-group-conf-freshness gsym)))
 		      (reverse new-groups)))))))
 
 ;; full implementation
@@ -130,7 +140,7 @@
   (lambda (keys)
     (map (lambda (key)
 	   (if (symbol? key)
-	       (symbolconc key '?)
+	       (symbol-append key '?)
 	       key))
 	 keys)))
 
@@ -190,7 +200,7 @@
 ;; lightweight implementation
 (define custom-key-exist?
   (lambda (sym)
-    (let ((key-sym (symbolconc sym '?)))
+    (let ((key-sym (symbol-append sym '?)))
       (and (symbol-bound? sym)
 	   (list? (symbol-value sym))
 	   (symbol-bound? key-sym)
@@ -209,7 +219,7 @@
 	  ((custom-key-exist? sym)
 	   (set-symbol-value! sym val)
 	   (let ((key-val (custom-modify-key-predicate-names val)))
-	     (eval (list 'define (symbolconc sym '?)
+	     (eval (list 'define (symbol-append sym '?)
 			 (list 'make-key-predicate (list 'quote key-val)))
 		   (interaction-environment)))
 	   #t)
@@ -241,15 +251,23 @@
 		(begin
 		  (if (eq? (car type)
 			   'key)
-		      (eval (list 'define (symbolconc sym '?) list)
+		      (eval (list 'define (symbol-append sym '?) list)
 			    (interaction-environment)))
 		  (custom-set-value! sym default))))))))  ;; to apply hooks
 
-;; lightweight implementation
-;; warning: no validation performed
+;; warning: no validation is performed by custom-set-value! on custom-rt.scm
 (define custom-prop-update-custom-handler
-  (lambda (context custom-sym val)
-    (custom-set-value! custom-sym val)))
+  (let ((READ-ERR (list 'read-err))) ;; unique id
+    (lambda (context custom-sym val-str)
+      (let* ((val (guard (err
+			  (else READ-ERR))
+		    (read (open-input-string val-str))))
+	     (unquoted-val (or (and (pair? val)
+				    (eq? (car val) 'quote)
+				    (cadr val))
+			       val)))
+	(and (not (eq? unquoted-val READ-ERR))
+	     (custom-set-value! custom-sym unquoted-val))))))
 
 ;; custom-reload-user-configs can switch its behavior by
 ;; custom-enable-mtime-aware-user-conf-reloading? since the
@@ -267,7 +285,7 @@
 (define custom-reload-user-configs
   (lambda ()
     (and (not (getenv "LIBUIM_VANILLA"))
-	 (not (is-set-ugid?))
+	 (not (setugid?))
 	 (let ((load-conf (if custom-enable-mtime-aware-user-conf-reloading?
 			      custom-load-updated-group-conf
 			      custom-load-group-conf)))  ;; original behavior
