@@ -1,6 +1,6 @@
 ;;; custom.scm: Customization support
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -183,7 +183,7 @@
 ;; TODO: write test
 (define map-key-list-letter
   (lambda (letter-mapper key-list)
-    (let ((letter (string->letter (find string? key-list))))
+    (let ((letter (string->alphabetic-ichar (find string? key-list))))
       (map-key-list-body (lambda (elem)
 			   (if letter
 			       (charcode->string (letter-mapper letter))
@@ -201,12 +201,12 @@
 ;; TODO: write test
 (define key-list-upcase
   (lambda (key-list)
-    (map-key-list-letter char-upcase key-list)))
+    (map-key-list-letter ichar-upcase key-list)))
 
 ;; TODO: write test
 (define key-list-downcase
   (lambda (key-list)
-    (map-key-list-letter char-downcase key-list)))
+    (map-key-list-letter ichar-downcase key-list)))
 
 ;; TODO: write test
 (define key-list-visualize-space
@@ -231,20 +231,20 @@
   (lambda (key-list)
     (let* ((has-shift? (memq 'Shift_key key-list))
 	   (str (find string? key-list))
-	   (printable (string->printable-char str))
-	   (letter (string->letter str)))
+	   (printable (string->printable-ichar str))
+	   (letter (string->alphabetic-ichar str)))
       (filter-map (lambda (elem)
 		    (cond
 		     ((and (eq? elem 'Shift_key)
-			   (char-graphic? printable))
+			   (ichar-graphic? printable))
 		      #f)
 		     ((and (string? elem)
 			   has-shift?
 			   letter)
-		      (charcode->string (char-upcase letter)))
+		      (charcode->string (ichar-upcase letter)))
 		     ((and (string? elem)
 			   has-shift?
-			   (char-graphic? printable))
+			   (ichar-graphic? printable))
 		      str)
 		     (else
 		      elem)))
@@ -253,9 +253,9 @@
 ;; TODO: write test
 (define key-list-decode-shift
   (lambda (key-list)
-    (let* ((letter (string->letter (find string? key-list)))
+    (let* ((letter (string->alphabetic-ichar (find string? key-list)))
 	   (upper-case? (and letter
-			     (char-upper-case? letter)))
+			     (ichar-upper-case? letter)))
 	   (has-shift? (memq 'Shift_key key-list))
 	   (stripped (key-list-downcase key-list)))
       (if (and (not has-shift?)
@@ -266,15 +266,15 @@
 ;; TODO: write test
 (define key-list-ignore-regular-shift
   (lambda (key-list)
-    (let ((printable (string->printable-char (find string? key-list))))
-      (if (char-graphic? printable)
+    (let ((printable (string->printable-ichar (find string? key-list))))
+      (if (ichar-graphic? printable)
 	  (cons 'IgnoreRegularShift key-list)
 	  key-list))))
 
 ;; TODO: write test
 (define key-list-ignore-letter-shift
   (lambda (key-list)
-    (let ((letter (string->letter (find string? key-list))))
+    (let ((letter (string->alphabetic-ichar (find string? key-list))))
       (if letter
 	  (cons 'IgnoreShift key-list)
 	  key-list))))
@@ -283,16 +283,16 @@
 (define key-list-ignore-punct-numeric-shift
   (lambda (key-list)
     (let* ((str (find string? key-list))
-	   (c (string->printable-char str)))
-      (if (and (char-graphic? c)
-	       (not (char-alphabetic? c)))
+	   (c (string->printable-ichar str)))
+      (if (and (ichar-graphic? c)
+	       (not (ichar-alphabetic? c)))
 	  (cons 'IgnoreShift key-list)
 	  key-list))))
 
 ;; TODO: write test
 (define key-list-ignore-case
   (lambda (key-list)
-    (let ((letter (string->letter (find string? key-list))))
+    (let ((letter (string->alphabetic-ichar (find string? key-list))))
       (if letter
 	  (cons 'IgnoreCase key-list)
  	  key-list))))
@@ -306,8 +306,8 @@
 (define key-list-strip-regular-shift
   (lambda (key-list)
     (let* ((str (find string? key-list))
-	   (printable (string->printable-char str)))
-      (if (char-graphic? printable)
+	   (printable (string->printable-ichar str)))
+      (if (ichar-graphic? printable)
 	  (key-list-strip-shift key-list)
 	  key-list))))
 
@@ -568,7 +568,7 @@
 	   (if (eq? (custom-type sym)
 		    'key)
 	       (let ((key-val (custom-modify-key-predicate-names val)))
-		 (eval (list 'define (symbolconc sym '?)
+		 (eval (list 'define (symbol-append sym '?)
 			     (list 'make-key-predicate (list 'quote key-val)))
 		       (interaction-environment))))
 	   (custom-call-hook-procs sym custom-set-hooks)
@@ -675,7 +675,7 @@
 				 (else
 				  "")))
 			      lst)))
-      (string-append "'(" (string-join " " canonicalized) ")"))))
+      (string-append "'(" (string-join canonicalized " ") ")"))))
 
 ;; API
 (define custom-value-as-literal
@@ -684,7 +684,7 @@
 	  (type (custom-type sym)))
       (cond
        ((eq? type 'integer)
-	(digit->string val))
+	(number->string val))
        ((eq? type 'string)
 	(string-escape val))
        ((eq? type 'pathname)
@@ -707,7 +707,7 @@
 	  (val (custom-value-as-literal sym))
 	  (hooked (custom-call-hook-procs sym custom-literalize-hooks)))
       (if (not (null? hooked))
-	  (string-join "\n" hooked)
+	  (string-join hooked "\n")
 	  (apply string-append
 		 (append
 		  (list "(define " var " " val ")")
@@ -724,7 +724,7 @@
 ;; TODO: implement after uim 0.4.6 depending on scm-nested-eval
 (define custom-broadcast-custom
   (lambda (sym)
-    ))
+  #f))
 
 ;; API
 ;; #f means 'any group'
@@ -734,10 +734,6 @@
     (let ((custom-syms (custom-collect-by-group group)))
       (for-each custom-broadcast-custom custom-syms))))
 
-(define custom-prop-update-custom-handler
-  (lambda (context custom-sym val)
-    (custom-set-value! custom-sym val)))
-
 (define custom-register-cb
   (lambda (hook valid? custom-sym ptr gate-func func)
     (and (valid? custom-sym)
@@ -749,12 +745,12 @@
 ;;
 
 (define-custom-group 'main
-		     (_ "-")
-		     (_ "Main settings of this group"))
+		     (N_ "-")
+		     (N_ "Main settings of this group"))
 
 (define-custom-group 'hidden
-		     (_ "Hidden settings")
-		     (_ "Hidden settings of this group. This group is invisible from uim_custom clients. Exists for internal variable management."))
+		     (N_ "Hidden settings")
+		     (N_ "Hidden settings of this group. This group is invisible from uim_custom clients. Exists for internal variable management."))
 
 
 (prealloc-heaps-for-heavy-job)
