@@ -1,6 +1,6 @@
 ;;; im-custom.scm: Customization variables for im.scm
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -49,6 +49,22 @@
 		     (N_ "Toolbar")
 		     (N_ "long description will be here."))
 
+(define-custom-group 'annotation
+                     (N_ "Annotation")
+                     (N_ "long description will be here."))
+
+(define-custom-group 'xim
+		     (N_ "XIM")
+		     (N_ "long description will be here."))
+
+(define-custom-group 'notify
+		     (N_ "Notify")
+		     (N_ "long description will be here."))
+
+(define-custom-group 'http
+		     (N_ "Http")
+		     (N_ "long description will be here."))
+
 ;; subgroup
 (define-custom-group 'advanced
 		     (N_ "Advanced settings")
@@ -69,11 +85,6 @@
 		     (N_ "Candidate window")
 		     (N_ "long description will be here."))
 
-;; subgroup
-(define-custom-group 'annotation
-                     (N_ "Annotation")
-                     (N_ "long description will be here."))
-
 ;; subgroup
 (define-custom-group 'dictionary
                      (N_ "Dictionary")
@@ -109,6 +120,27 @@
 		     (N_ "Visual preference")
 		     (N_ "long description will be here."))
 
+;; subgroup
+(define-custom-group 'xim-preedit
+		     (N_ "Preedit settings of XIM")
+		     (N_ "long description will be here."))
+
+;; subgroup
+(define-custom-group 'toolbar-help
+                     (N_ "Help")
+                     (N_ "long description will be here."))
+
+;; subgroup
+(define-custom-group 'toolbar-icon
+                     (N_ "Icon")
+                     (N_ "long description will be here."))
+
+;; subgroup
+(define-custom-group 'toolbar-widget
+                     (N_ "Toolbar")
+                     (N_ "long description will be here."))
+
+
 ;; 
 ;; default-im-name
 ;;
@@ -274,34 +306,38 @@
 ;;
 ;; im-switching
 ;;
+(define-custom-group 'im-switching
+  (N_ "Input method switching")
+  (N_ "long description will be here."))
 
-;; I think that current "im-switching by hotkey" feature is not
-;; useful. So commented out them to avoid confusion of users.
-;;   -- YamaKen 2005-02-01
+(define-custom 'enable-im-switch? #f
+  '(global im-switching)
+  '(boolean)
+  (N_ "Enable IM switching by hotkey")
+  (N_ "long description will be here."))
 
-;;(define-custom-group 'im-switching
-;;		     (N_ "Input method switching")
-;;		     (N_ "long description will be here."))
-;;
-;;(define-custom 'enable-im-switch #f
-;;  '(global im-switching)
-;;  '(boolean)
-;;  (N_ "Enable IM switching by hotkey")
-;;  (N_ "long description will be here."))
-;;
-;;(define-custom 'switch-im-key '("<Control>Shift_key" "<Shift>Control_key")
-;;  '(global im-switching)
-;;  '(key)
-;;  (N_ "IM switching key")
-;;  (N_ "long description will be here."))
-;;
-;;;; activity dependency
-;;(custom-add-hook 'switch-im-key?
-;;		 'custom-activity-hooks
-;;		 (lambda ()
-;;		   enable-im-switch))
+(define-custom 'switch-im-key '("<Control>Shift_key" "<Shift>Control_key")
+  '(global im-switching)
+  '(key)
+  (N_ "IM switching key")
+  (N_ "long description will be here."))
 
-(define-key switch-im-key? '())
+(define-custom 'switch-im-skip-direct-im? #f
+  '(global im-switching)
+  '(boolean)
+  (N_ "Skip direct method for IM switching by hotkey")
+  (N_ "long description will be here."))
+
+;; activity dependency
+(custom-add-hook 'switch-im-key
+		 'custom-activity-hooks
+		 (lambda ()
+		   enable-im-switch?))
+
+(custom-add-hook 'switch-im-skip-direct-im?
+		 'custom-activity-hooks
+		 (lambda ()
+		   enable-im-switch?))
 
 ;; im-toggle 
 (define-custom-group 'im-toggle
@@ -476,12 +512,58 @@
   (N_ "help")
   (N_ "long description will be here."))
 
+(define-custom 'toolbar-help-browser 'system
+  '(toolbar toolbar-help)
+  (list 'choice
+        (list 'system
+              (N_ "System")
+              (N_ "long description will be here."))
+        (list 'manual
+              (N_ "Manual")
+              (N_ "long description will be here.")))
+  (N_ "Document browser")
+  (N_ "long description will be here."))
+
+(define-custom 'toolbar-help-browser-name "firefox"
+  '(toolbar toolbar-help)
+  '(string ".*")
+  (N_ "Browser name")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'toolbar-help-browser-name
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? toolbar-help-browser 'manual)))
+
+(define-custom 'toolbar-icon-for-dark-background? #f
+  '(toolbar toolbar-icon)
+  '(boolean)
+  (N_ "Use icon for dark background")
+  (N_ "long description will be here."))
+
 (define-custom 'bridge-show-input-state? #f
   '(global visual-preference)
   '(boolean)
   (N_ "Show input mode nearby cursor")
   (N_ "long description will be here."))
 
+(define-custom 'bridge-show-with? 'time
+  '(global visual-preference)
+  (list 'choice
+        (list 'mode
+              (N_ "With mode")
+              (N_ "long description will be here."))
+        (list 'time
+              (N_ "With time")
+              (N_ "long description will be here.")))
+  (N_ "Show input mode")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'bridge-show-with?
+		 'custom-activity-hooks
+		 (lambda ()
+                   bridge-show-input-state?))
+
 (define-custom 'bridge-show-input-state-time-length 3
   '(global visual-preference)
   '(integer 0 100)
@@ -491,50 +573,236 @@
 (custom-add-hook 'bridge-show-input-state-time-length
 		 'custom-activity-hooks
 		 (lambda ()
-		   bridge-show-input-state?))
+		   (and bridge-show-input-state?
+                        (eq? bridge-show-with?
+                             'time))))
+
+;;
+;; Annotation
+;;
+
+(define-custom 'enable-annotation? #t
+  '(annotation candwin)
+  '(boolean)
+  (N_ "Enable annotation")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-agent 'eb
+  '(annotation candwin)
+  (list 'choice
+    (list 'eb
+      (N_ "EB library")
+      (N_ "EB library"))
+    (list 'dict
+      (N_ "dict server")
+      (N_ "dict server"))
+    (list 'filter
+      (N_ "Custom filter")
+      (N_ "Custom filter"))
+    (list 'im
+      (N_ "Input method itself")
+      (N_ "long description will be here.")))
+  (N_ "Annotation agent name")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'annotation-agent
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+
+(custom-add-hook 'annotation-agent
+                 'custom-set-hooks
+                 (lambda ()
+                   (annotation-unload)
+                   (annotation-load
+                     (if enable-annotation?
+                       (symbol->string annotation-agent)
+                       #f))))
 
 ;; EB Library support
-;; 2005-02-08 Takuro Ashie <ashie@homa.ne.jp>
-;; FIXME! Here isn't suitable position for EB support preference
 (define-custom-group 'eb
 		     (N_ "EB library")
 		     (N_ "long description will be here."))
 
+;; eb-enable-for-annotation? exists only for compatibility.
+;; You shouldn't add similar variables to other annotation agents.
 (define-custom 'eb-enable-for-annotation? #f
-  '(eb candwin)
+  '(annotation eb)
   '(boolean)
   (N_ "Use EB library to search annotations")
   (N_ "long description will be here."))
 
-(define-custom 'eb-dic-path
+(custom-add-hook 'eb-enable-for-annotation?
+  'custom-activity-hooks
+  (lambda ()
+    (and enable-annotation?
+      (eq? annotation-agent 'eb))))
+
+(define-custom 'annotation-eb-dic-path
   (string-append (sys-datadir) "/dict")
-  '(eb candwin)
+  '(annotation eb)
   '(pathname directory)
   (N_ "The directory which contains EB dictionary file")
   (N_ "long description will be here."))
 
-(custom-add-hook 'eb-dic-path
+(custom-add-hook 'annotation-eb-dic-path
+  'custom-activity-hooks
+  (lambda ()
+    (and enable-annotation?
+         eb-enable-for-annotation?
+         (eq? annotation-agent 'eb))))
+
+;; dict server support
+(define-custom-group 'dict
+		     (N_ "dict server")
+		     (N_ "long description will be here."))
+
+(define-custom 'annotation-dict-server
+  "dict.org"
+  '(annotation dict)
+  '(string ".*")
+  (N_ "Server address of dict")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-dict-servname
+  2628
+  '(annotation dict)
+  '(integer 0 65535)
+  (N_ "Server port of dict")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-dict-database
+  "web1913"
+  '(annotation dict)
+  '(string ".*")
+  (N_ "Database name of dict")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-dict-cache-words
+  256
+  '(annotation dict)
+  '(integer 0 65535)
+  (N_ "Number of cache of annotation")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'annotation-dict-server
+		 'custom-activity-hooks
+                 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'dict))))
+
+(custom-add-hook 'annotation-dict-servname
+		 'custom-activity-hooks
+                 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'dict))))
+
+(custom-add-hook 'annotation-dict-database
+		 'custom-activity-hooks
+                 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'dict))))
+
+(custom-add-hook 'annotation-dict-cache-words
+		 'custom-activity-hooks
+                 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'dict))))
+
+(define-custom-group 'filter
+		     (N_ "Custom filter")
+		     (N_ "long description will be here."))
+
+(define-custom 'annotation-filter-server-setting? 'pipe
+  '(annotation filter)
+  (list 'choice
+        (list 'unixdomain
+              (N_ "Unix domain")
+              (N_ "Use UNIX domain socket to communicate with custom filter."))
+        (list 'tcpserver
+              (N_ "TCP server")
+              (N_ "Use tcp server to communicate with custom filter"))
+        (list 'pipe
+              (N_ "Pipe")
+              (N_ "Use pipe. spawn new annotation-filter process to communicate with custom filter")))
+  (N_ "Custom filter connection setting")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'annotation-filter-server-setting?
+  'custom-activity-hooks
+  (lambda ()
+    (and enable-annotation?
+         (eq? annotation-agent 'filter))))
+
+
+(define-custom 'annotation-filter-unix-domain-socket-path "/path/of/socket"
+  '(annotation filter)
+  '(pathname regular-file)
+  (N_ "Path of custom filter socket")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-filter-tcpserver-name "localhost"
+  '(annotation filter)
+  '(string ".*")
+  (N_ "Custom filter server address")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-filter-tcpserver-port 6789
+  '(annotation filter)
+  '(integer 0 65535)
+  (N_ "Custom filter server port")
+  (N_ "long description will be here."))
+
+(define-custom 'annotation-filter-command "/path/of/filter-program"
+  '(annotation filter)
+  '(pathname regular-file)
+  (N_ "Path of custom filter")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'annotation-filter-unix-domain-socket-path
 		 'custom-activity-hooks
 		 (lambda ()
-		   eb-enable-for-annotation?))
+                   (and enable-annotation?
+                        (eq? annotation-agent 'filter)
+                        (eq? annotation-filter-server-setting?
+                             'unixdomain))))
 
+(custom-add-hook 'annotation-filter-tcpserver-name
+		 'custom-activity-hooks
+		 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'filter)
+                        (eq? annotation-filter-server-setting?
+                             'tcpserver))))
+
+(custom-add-hook 'annotation-filter-tcpserver-port
+		 'custom-activity-hooks
+		 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'filter)
+                        (eq? annotation-filter-server-setting?
+                             'tcpserver))))
+
+(custom-add-hook 'annotation-filter-command
+		 'custom-activity-hooks
+                 (lambda ()
+                   (and enable-annotation?
+                        (eq? annotation-agent 'filter)
+                        (eq? annotation-filter-server-setting?
+                             'pipe))))
+
+;;
 ;; uim-xim specific custom
-(define-custom-group 'xim
-		     (N_ "XIM")
-		     (N_ "long description will be here."))
-
-(define-custom-group 'preedit
-		     (N_ "Preedit settings of XIM")
-		     (N_ "long description will be here."))
+;;
 
 (define-custom 'uim-xim-use-xft-font? #f
-  '(xim preedit)
+  '(xim xim-preedit)
   '(boolean)
   (N_ "Use anti-aliased fonts for Over-the-Spot/Root-Window preedit")
   (N_ "long description will be here."))
 
 (define-custom 'uim-xim-xft-font-name "Sans"
-  '(xim preedit)
+  '(xim xim-preedit)
   '(string ".*")
   (N_ "Font name for preedit area (anti-aliased)")
   (N_ "long description will be here."))
@@ -543,11 +811,9 @@
 		 'custom-activity-hooks
 		 (lambda ()
 		   uim-xim-use-xft-font?))
-
-
-(define-custom-group 'notify
-		     (N_ "Notify")
-		     (N_ "long description will be here."))
+;;
+;; Notify
+;;
 
 (define-custom 'notify-agent 'stderr
   '(notify)
@@ -556,6 +822,65 @@
   (N_ "Notify agent name")
   (N_ "long description will be here."))
 
+(custom-add-hook 'notify-agent
+                 'custom-set-hooks
+                 (lambda ()
+                   (if (symbol-bound? 'uim-notify-load)
+                     (uim-notify-load (symbol->string
+                                        notify-agent)))))
+
+;;
+;; Http
+;;
+
+(define-custom 'http-proxy-setting 'direct
+  '(http)
+  (list 'choice
+        (list 'direct
+              (N_ "Direct connection")
+              (N_ "Direct connection, no proxy."))
+        (list 'user
+              (N_ "User")
+              (N_ "Use proxy."))
+        ;;(list 'system
+        ;;      (N_ "System")
+        ;;      (N_ "Use proxy with system setting.")))
+        )
+  (N_ "Proxy setting")
+  (N_ "long description will be here."))
+
+(define-custom 'http-proxy-hostname "localhost"
+  '(http)
+  '(string ".*")
+  (N_ "Http proxy hostname")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'http-proxy-hostname
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? http-proxy-setting
+                        'user)))
+
+(define-custom 'http-proxy-port 8080
+  '(http)
+  '(integer 0 65535)
+  (N_ "Http proxy port")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'http-proxy-port
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? http-proxy-setting
+                        'user)))
+
+(define-custom 'http-timeout 3000
+  '(http)
+  '(integer 0 65535)
+  (N_ "Timeout (msec)")
+  (N_ "Timeout of http connection (msec)."))
+
+(load "predict-custom.scm")
+
 
 (if custom-full-featured?
     (for-each require-module installed-im-module-list))
