@@ -1,6 +1,6 @@
 ;;; im-switcher.scm: Action-based IM switcher
 ;;;
-;;; Copyright (c) 2006-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2006-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -38,19 +38,36 @@
   '())
 
 (define imsw-iconic-label-alist
-  '((direct . "-")
-    (anthy  . "A")
-    (canna  . "C")
-    (wnn    . "W")
-    (mana   . "M")
-    (sj3    . "3")
-    (skk    . "S")
-    (tcode  . "T")
-    (tutcode . "U")
-    (byeoru . "B")
-    (anthy-utf8 . "A")
-    (elatin . "E")
-    (look   . "L")))
+  '((direct      . "-")
+    (ajax-ime    . "Aj")
+    (anthy       . "An")
+    (anthy-utf8  . "An")
+    (byeoru      . "B")
+    (canna       . "Ca")
+    (chewing     . "Ch")
+    (elatin      . "E")
+    (hangul2     . "H2")
+    (hangul3     . "H3")
+    (ipa-x-sampa . "I")
+    (latin       . "Lt")
+    (look        . "Lo")
+    (mana        . "Ma")
+    (mozc        . "Mz")
+    (pinyin-big5 . "Pi")
+    (pyunihan    . "Py")
+    (prime       . "Pm")
+    (romaja      . "R")
+    (sj3         . "Sj")
+    (skk         . "Sk")
+    (social-ime  . "Si")
+    (tcode       . "Tc")
+    (tutcode     . "Tu")
+    (trycode     . "Tr")
+    (viqr        . "V")
+    (wb86        . "Wb")
+    (wnn         . "Wn")
+    (yahoo-jp    . "Yj")
+    (zm          . "Zm")))
 
 (define imsw-default-iconic-label "IM")
 
