@@ -1,6 +1,6 @@
 ;;; im.scm: Core IM management functions for uim
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -36,6 +36,7 @@
 (require "util.scm")
 (require "i18n.scm")
 (require "load-action.scm")
+(require "annotation.scm")
 
 ;; config
 (define default-im-name #f)
@@ -353,11 +354,11 @@
 (define create-context
   (lambda (uc lang name)
     (let* ((im (find-im name lang))
-	   (arg (and im (im-init-arg im))))
+           (arg (and im (im-init-arg im))))
       (im-set-encoding uc (im-encoding im))
       (let* ((handler (im-init-handler im))
-	     (c (handler uc im arg)))
-	(register-context c)
+             (c (handler uc im arg)))
+        (register-context c)
         ;; im-* procedures that require uc->sc must not called here since it
         ;; is not filled yet. Place such procedures to setup-context.
         c))))
@@ -514,7 +515,13 @@
 
 (define get-candidate
   (lambda (uc idx accel-enum-hint)
-    (invoke-handler im-get-candidate-handler uc idx accel-enum-hint)))
+    (let ((c (invoke-handler im-get-candidate-handler uc idx accel-enum-hint)))
+      (if (and (not enable-annotation?)
+               (not (string=? (last c) "")))
+          (set-cdr! (cdr c) (list ""))
+          (and (string=? (last c) "")
+               (set-cdr! (cdr c) (list (annotation-get-text (car c) (uim-context-encoding uc))))))
+      c)))
 
 (define set-candidate-index
   (lambda (uc idx)
