@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -36,6 +36,11 @@
 
 (require-extension (srfi 23 34 60) (siod))
 
+;; FIXME: Temporary workaround to resolve circular dependency of
+;; util.scm, deprecated-util.scm and light-record.scm. Record users
+;; should explicitly require light-record.scm.  -- YamaKen 2008-04-29
+(require "light-record.scm")
+
 ;; Disable SIOD compatibilities.
 (undefine the-environment)
 (undefine bit-and)
@@ -126,6 +131,10 @@
 (if (symbol-bound? 'uim-notify-load)
     (uim-notify-load (symbol->string notify-agent)))
 
+;; redefine annotation-related procedures to use an annotation agent
+(and enable-annotation?
+  (annotation-load (symbol->string annotation-agent)))
+
 (or (getenv "LIBUIM_VANILLA")
     (load-user-conf)
     (load "default.scm"))
