@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -218,7 +218,7 @@
 
 (define ipa-x-sampa-init-handler
   (lambda (id im arg)
-    (generic-context-new id im ipa-x-sampa-rule #t)))
+    (generic-context-new id im ipa-x-sampa-rule #f)))
 
 (generic-register-im
  'ipa-x-sampa
