@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -528,3 +528,7 @@
 ))
 
 (define ja-azik-rule (append ja-azik-rule-basic ja-rk-rule-basic))
+
+(define ja-azik-skk-okuri-char-alist '((";" . "t")))
+(define ja-azik-skk-downcase-alist (alist->icharlist '(("+" . ";"))))
+(define ja-azik-skk-set-henkan-point-key (map string->charcode '(";")))
