@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,6 +29,9 @@
 ;;;;
 
 ;; Japanese EUC
+
+(require-custom "japanese-custom.scm")
+
 (define ja-rk-rule-basic
   '(
     ((("-"). ())("" "" ""))
@@ -130,6 +133,12 @@
     ((("g" "y" "e"). ())(("" "" "") ("" "" "")))
     ((("g" "y" "o"). ())(("" "" "") ("" "" "")))
 
+    ((("q" "a"). ())(("" "" "") ("" "" "")))
+    ((("q" "i"). ())(("" "" "") ("" "" "")))
+    ((("q" "u"). ())("" "" ""))
+    ((("q" "e"). ())(("" "" "") ("" "" "")))
+    ((("q" "o"). ())(("" "" "") ("" "" "")))
+
     ((("s" "s"). ("s"))("" "" ""))
 
     ((("s" "a"). ())("" "" ""))
@@ -677,7 +686,9 @@
    ("py" "pi")
    ("my" "mi")
    ("ly" "li")
+   ("ry" "ri")
    ("wh" "wu")
+   ("ds" "du")
    ("dh" "de")
    ("dw" "do")
    ("kw" "ku")
@@ -687,10 +698,18 @@
    ("th" "te")
    ("hw" "hu")
    ("fw" "fu")
+   ("xw" "wa")
+   ("wy" "wi")
    ("ch" "ti")
    ("qw" "ku")
    ("qy" "ku")
    ("gw" "gu")
+   ("zw" "zu")
+   ("bw" "bu")
+   ("pw" "pu")
+   ("mw" "mu")
+   ("yw" "yu")
+   ("rw" "ru")
    ))
 
 (define ja-default-small-tsu-roma "ltu")
@@ -879,32 +898,6 @@
 			 (get-str-by-type sl))
 	  ""))))
     
-;;
-;; 2004-08-30 Takuro Ashie <ashie@homa.ne.jp>
-;;
-;;   In the future, these APIs should be marked as "deprecated" because these
-;;   APIs breaks original "ja-rk-rule" table, and no longer can use it after
-;;   breaks it.
-;;
-(define using-kana-table? #f)
-
-(define load-kana-table
-  (lambda ()
-    (require "japanese-kana.scm")
-    (set! ja-rk-rule ja-kana-hiragana-rule)
-    (set! using-kana-table? #t)
-    ;(define-key anthy-kana-toggle-key? "")
-    ))
-
-(define load-azik-table
-  (lambda ()
-    (require "japanese-azik.scm")
-    (set! ja-rk-rule ja-azik-rule)
-    (define-key skk-prev-candidate-key? 'generic-prev-candidate-key?)
-    (define-key skk-kana-toggle-key? "@")
-    (define-key skk-kanji-mode-key? "`")
-    ))
-
 (define japanese-roma-set-yen-representation
   (lambda ()
     ;; Since ordinary Japanese users press the "yen sign" key on
@@ -915,5 +908,29 @@
     (set-symbol-value! 'yen "\\")
     ))
 
+;; TODO: Support new custom type string-list.
+(define japanese-auto-start-henkan-keyword-list '("" "" "" "" "" "" "" "" "" ")" ";" ":" "" "" "" "" "" "" "" "" "" "}" "]" "?" "." "," "!"))
+
+(define ja-rk-rule-consonant-to-keep
+  (map (lambda (c)
+         (if (= (string-length c) 1)
+           (list (cons (list c) '()) (list c c c))
+           (let ((lst (reverse (string-to-list c))))
+             (list (cons lst '()) (list (list (car lst) (car lst) (car lst))
+                                        (list (cadr lst) (cadr lst) (cadr lst)))))))
+       (filter (lambda (x) (not (string=? "n" x)))
+               (map car ja-consonant-syllable-table))))
+
+(define ja-rk-rule-update
+  (lambda ()
+    (if ja-rk-rule-keep-consonant?
+      (set! ja-rk-rule (append ja-rk-rule-consonant-to-keep
+                               ja-rk-rule-basic
+                               ja-rk-rule-additional))
+      (set! ja-rk-rule (append ja-rk-rule-basic
+                               ja-rk-rule-additional)))))
+
 ;;
 (require "rk.scm")
+
+(ja-rk-rule-update)
