@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -28,9 +28,13 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
+(require-extension (srfi 95))
+
 (require-custom "generic-key-custom.scm")
 (require-custom "look-custom.scm")
 
+(require "annotation.scm")
+
 ;; widgets
 (define look-widgets '(widget_look_input_mode))
 
@@ -119,21 +123,6 @@
               l)))
 (define (look-internal:make-string n c)
   (apply string-append (map (lambda (x) (symbol->string c)) (iota n))))
-;; XXX: slow quick-sort
-(define (look-internal:qsort! data proc)
-  (let ((pivot 0)
-        (left '())
-        (right '()))
-    (if (< (length data) 2)
-        data
-        (begin
-          (set! pivot (car data))
-          (for-each (lambda (x)
-                      (if (proc x pivot)
-                          (set! left (cons x left ))
-                          (set! right (cons x right))))
-                    (cdr data))
-          (append (look-internal:qsort! left proc) (cons pivot (look-internal:qsort! right proc)))))))
 (define (look-to-lower-string str)
   (apply string-append
          (map (lambda (c)
@@ -144,9 +133,7 @@
 
 (define (look-history-sort li lessf)
   ;;(map car li))
-  (map car (look-internal:qsort!
-            li
-            (lambda (x y) (lessf (cdr x) (cdr y))))))
+  (map car (sort! li (lambda (x y) (lessf (cdr x) (cdr y))))))
 
 (define (look-history-eow? x)
   (eq? #t (car x)))
@@ -348,6 +335,8 @@
 (define (look-context-new . args)
   (let ((lc (apply look-context-new-internal args)))
     (look-context-set-widgets! lc look-widgets)
+    (if look-use-annotation?
+        (annotation-init))
     lc))
 
 (define (look-context-clean lc)
@@ -377,6 +366,8 @@
     lc))
 
 (define (look-release-handler lc)
+  (if look-use-annotation?
+      (annotation-release))
   #f)
 
 (define (look-alphabetic-char? key state)
@@ -394,9 +385,9 @@
 
 (define (look-look lc look-dict str)
   (let* ((learned (look-search-learned lc str))
-         (looked (look-lib-look look-dict str)))
+         (looked (look-lib-look #t #t look-candidates-max look-dict str)))
     (look-context-set-dictlen! lc (length learned))
-    (append learned looked)))
+    (append learned (if looked looked '()))))
 
 (define (look-update lc)
   (let ((str (look-context-left lc)))
@@ -436,6 +427,29 @@
                        (number->string candidates)
                        "]"))))
 
+(define (look-format-annotation lc)
+  (define (annotation-format-entry str lines)
+    (let loop ((l (string->list str))
+             (lines lines)
+             (rest '()))
+      (cond ((or (null? l)
+                 (= 0 lines))
+             (list->string (reverse rest)))
+            ((eq? #\newline (car l))
+           (loop (cdr l) (- lines 1) (cons #\space rest)))
+            (else
+             (loop (cdr l) lines (cons (car l) rest))))))
+  (let ((candidates (look-context-candidates lc)))
+    (if (or (= 0 (string-length (look-context-left lc)))
+            (<= (length candidates) (look-context-nth lc)))
+        ""
+        (annotation-format-entry (annotation-get-text (string-append
+                                                       (look-context-left lc)
+                                                       (nth (look-context-nth lc) candidates))
+                                                      "UTF-8")
+                                 look-annotation-show-lines))))
+
+
 (define (look-update-preedit lc)
   (im-clear-preedit lc)
   (im-pushback-preedit
@@ -451,6 +465,10 @@
       (im-pushback-preedit
        lc preedit-reverse
        (look-format-candidates-nth lc)))
+  (if look-use-annotation?
+      (im-pushback-preedit
+       lc preedit-none
+       (look-format-annotation lc)))
   (im-update-preedit lc))
 
 (define (look-key-press-state-look lc key state)
@@ -554,7 +572,7 @@
 
 (register-im
  'look
- "*"  ;; wildcard language. see i18n.scm
+ ""
  "UTF-8"
  (N_ "Look")
  (N_ "Tiny predictive input method")
