@@ -1,7 +1,7 @@
 ;;; mana.scm: mana for uim.
 ;;; charset: EUC-JP
 ;;;
-;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -33,8 +33,6 @@
 (require "util.scm")
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "mana-custom.scm")
 (require-custom "mana-key-custom.scm")
@@ -95,7 +93,7 @@
                                   (else
                                     "")))
                               lst)))
-      (string-append "(" (string-join " " canonicalized) ")"))))
+      (string-append "(" (string-join canonicalized " ") ")"))))
 
 (define mana-set-string!
   (lambda (mc yomi yomi-len)
@@ -142,8 +140,11 @@
 	   (right-str (ustr-latter-seq raw-str))
 	   (left-str (ustr-former-seq raw-str)))
      (append left-str
-	     (if (not (null? residual-kana))
-		 (list pending))
+	     (if residual-kana
+               (if (list? (car residual-kana))
+                 (reverse (string-to-list pending))
+                 (list pending))
+               '())
 	     right-str))))
 
 (define mana-get-raw-candidate
@@ -157,8 +158,8 @@
 	   (preconv (ja-join-vu (string-to-list yomi)))
 	   (unconv (ja-join-vu (sublist
 				(string-to-list yomi)
-				(- (- yomi-len 1) (+ pos (- len 1)))
-				(- (- yomi-len 1) pos))))
+				(- yomi-len (+ pos len))
+				(- yomi-len pos))))
 	   (raw-str (reverse (mana-get-raw-str-seq mc))))
      (cond
       ((= cand-idx mana-candidate-type-hiragana)
@@ -172,9 +173,11 @@
 	   (if (member (car unconv) preconv)
 	       (let ((start (list-seq-contained? preconv unconv))
 		     (len (length unconv)))
-		 (if start
+		 (if (and
+                       start
+                       (= (length raw-str) (length preconv))) ;; sanity check
 		     (mana-make-raw-string
-		      (reverse (sublist raw-str start (+ start (- len 1))))
+		      (reverse (sublist-rel raw-str start len))
 		      (if (or
 			   (= cand-idx mana-candidate-type-halfwidth-alnum)
 			   (= cand-idx
@@ -200,7 +203,7 @@
   (lambda (mc seg-idx)
     (let* ((segment-list (mana-context-segment-list mc))
            (segment (list-ref segment-list seg-idx)))
-      (if (not (mana-segment-candidate-list segment))
+      (if (null? (mana-segment-candidate-list segment))
           (mana-set-candidate-list! mc seg-idx))
       (mana-segment-nr-candidates segment))))
 
@@ -276,6 +279,8 @@
 (define mana-input-rule-roma 0)
 (define mana-input-rule-kana 1)
 (define mana-input-rule-azik 2)
+(define mana-input-rule-act 3)
+(define mana-input-rule-kzik 4)
 
 (define mana-candidate-type-katakana -2)
 (define mana-candidate-type-hiragana -3)
@@ -464,6 +469,7 @@
                       mana-input-rule-kana))
                  (lambda (mc)
                    (mana-prepare-input-rule-activation mc)
+                   (require "japanese-kana.scm")
                    (mana-context-set-input-rule! mc mana-input-rule-kana)
                    (mana-context-change-kana-mode!
 		    mc (mana-context-kana-mode mc))
@@ -486,10 +492,47 @@
                       mana-input-rule-azik))
                  (lambda (mc)
                    (mana-prepare-input-rule-activation mc)
+                   (require "japanese-azik.scm")
                    (rk-context-set-rule! (mana-context-rkc mc)
                                          ja-azik-rule)
                    (mana-context-set-input-rule! mc mana-input-rule-azik)))
 
+(register-action 'action_mana_kzik
+                 ;;              (indication-alist-indicator 'action_mana_kzik
+                 ;;                                          mana-kana-input-method-indication-alist)
+                 (lambda (mc)
+                   '(ja_kzik
+                      ""
+                      "KZIK"
+                      "KZIKĥ޻ϥ⡼"))
+                 (lambda (mc)
+                   (= (mana-context-input-rule mc)
+                      mana-input-rule-kzik))
+                 (lambda (mc)
+                   (mana-prepare-input-rule-activation mc)
+                   (require "japanese-kzik.scm")
+                   (rk-context-set-rule! (mana-context-rkc mc)
+                                         ja-kzik-rule)
+                   (mana-context-set-input-rule! mc mana-input-rule-kzik)))
+
+(register-action 'action_mana_act
+                 (lambda (mc)
+                   '(ja_act
+                      ""
+                      "ACT"
+                      "ACTĥ޻ϥ⡼"))
+                 (lambda (mc)
+                   (= (mana-context-input-rule mc)
+                      mana-input-rule-act))
+                 (lambda (mc)
+                   (mana-prepare-input-rule-activation mc)
+                   (require "japanese-act.scm")
+                   (rk-context-set-rule! (mana-context-rkc mc)
+                                         ja-act-rule)
+                   (mana-context-set-input-rule! mc mana-input-rule-act)))
+
+
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -538,18 +581,9 @@
           (set! mana-lib-initialized? (mana-lib-init)))
       (mana-context-set-widgets! mc mana-widgets)
       (mana-context-set-rkc! mc rkc)
-      (mana-context-set-preconv-ustr! mc (ustr-new))
-      (mana-context-set-raw-ustr! mc (ustr-new))
-      (mana-context-set-segments! mc (ustr-new))
-
-      ;; 2004-08-26 Takuro Ashie <ashie@homa.ne.jp>
-      ;;   * I think load-kana-table should be marked as depracated.
-      ;;     Because it is a little violent (it overwrites ja-rk-rule table).
-      ;;     We should prepare a custom entry like "uim-default-input-rule"
-      ;;     instead of using-kana-table.
-      (if using-kana-table?
-          (mana-context-set-input-rule! mc mana-input-rule-kana)
-          (mana-context-set-input-rule! mc mana-input-rule-roma))
+      (mana-context-set-preconv-ustr! mc (ustr-new '()))
+      (mana-context-set-raw-ustr! mc (ustr-new '()))
+      (mana-context-set-segments! mc (ustr-new '()))
       mc)))
 
 (define mana-commit-raw
@@ -602,8 +636,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-		      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
 	      (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -611,8 +647,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
 	   (if convert-pending-into-kana?
 	       (if residual-kana
-		   (extract-kana residual-kana)
-                   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+		   (extract-kana residual-kana))
+                 "")
 	       pending)
 	   (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -624,7 +662,7 @@
 	      (ja-string-list-to-wide-alphabet 
 		(if upper?
 		    (map charcode->string
-			 (map char-upcase
+			 (map ichar-upcase
 			      (map string->charcode
 				   (string-to-list (car raw-str-list)))))
 		    (string-to-list (car raw-str-list))))
@@ -633,7 +671,7 @@
 	      (if upper?
 		  (string-list-concat
 		   (map charcode->string
-			(map char-upcase
+			(map ichar-upcase
 			     (map string->charcode
 				  (string-to-list (car raw-str-list))))))
 		  (car raw-str-list))
@@ -845,7 +883,7 @@
         (direct
           (im-commit mc direct))
 
-	;; spcae key => commit
+	;; space key => commit
 	((mana-space-key? key key-state)
 	 (if (mana-context-alnum mc)
 	     (im-commit mc (list-ref
@@ -870,12 +908,13 @@
 	     (let* ((key-str (charcode->string
 			      (if (= rule mana-input-rule-kana)
 				  key
-				  (to-lower-char key))))
+				  (ichar-downcase key))))
 		    (res (rk-push-key! rkc key-str)))
 	       (if res
 		   (begin
-		     (ustr-insert-elem! (mana-context-preconv-ustr mc)
-					res)
+                     (if (list? (car res))
+                       (ustr-insert-seq! (mana-context-preconv-ustr mc) res)
+                       (ustr-insert-elem! (mana-context-preconv-ustr mc) res))
 		     (ustr-insert-elem! (mana-context-raw-ustr mc)
 					key-str))
 		   (if (null? (rk-context-seq rkc))
@@ -920,24 +959,27 @@
 	    (if (and lst
 	    	     (not (null? (cdr lst))))
 		(set! state (car (cdr lst)))
-		(set! state (mana-rotate-transposing-alnum-type
-			     (mana-context-transposing-type mc)
-			     (car rotate-list)))))
+		(if (not (null? rotate-list))
+		    (set! state (mana-rotate-transposing-alnum-type
+				 (mana-context-transposing-type mc)
+				 (car rotate-list))))))
 	  (begin
 	    (mana-context-set-transposing! mc #t)
 	    (set! state (car rotate-list))))
 
       (cond
-       ((or
-	 (= state mana-type-hiragana)
-	 (= state mana-type-katakana)
-	 (= state mana-type-halfkana))
+       ((and state
+	     (or
+	      (= state mana-type-hiragana)
+	      (= state mana-type-katakana)
+	      (= state mana-type-halfkana)))
 	(mana-context-set-transposing-type! mc state))
-       ((or
-	 (= state mana-type-halfwidth-alnum)
-	 (= state mana-candidate-type-upper-halfwidth-alnum)
-	 (= state mana-type-fullwidth-alnum)
-	 (= state mana-candidate-type-upper-fullwidth-alnum))
+       ((and state
+	     (or
+	      (= state mana-type-halfwidth-alnum)
+	      (= state mana-candidate-type-upper-halfwidth-alnum)
+	      (= state mana-type-fullwidth-alnum)
+	      (= state mana-candidate-type-upper-fullwidth-alnum)))
 	(if (not (= (mana-context-input-rule mc)
 		    mana-input-rule-kana))
 	    (mana-context-set-transposing-type! mc state)))
@@ -990,6 +1032,12 @@
 
 (define mana-proc-input-state-with-preedit
   (lambda (mc key key-state)
+    (define (check-auto-conv str)
+      (and
+	str
+	mana-auto-start-henkan?
+	(string-find japanese-auto-start-henkan-keyword-list str)
+	(mana-begin-conv mc)))
     (let ((preconv-str (mana-context-preconv-ustr mc))
           (raw-str (mana-context-raw-ustr mc))
           (rkc (mana-context-rkc mc))
@@ -1011,8 +1059,8 @@
 	      (if (and
 		   (= (mana-context-input-rule mc) mana-input-rule-roma)
 		   (not (null? (ustr-former-seq preconv-str)))
-		   (not (char-printable?	;; check for kana
-			 (string->char
+		   (not (ichar-printable?	;; check for kana
+			 (string->ichar
 			  (car (last (ustr-former-seq preconv-str)))))))
 		  (ja-fix-deleted-raw-str-to-valid-roma! raw-str)))))
 
@@ -1162,22 +1210,6 @@
         #f)
 
        (else
-	;; handle "n1" sequence as "1"
-	(if (and (not (mana-context-alnum mc))
-		 (not (alphabet-char? key))
-		 (not (string-find
-		       (rk-expect rkc)
-		       (charcode->string
-			(if (= rule mana-input-rule-kana)
-			    key
-			    (to-lower-char key))))))
-	    (let ((pend (rk-pending rkc))
-		  (residual-kana (rk-push-key-last! rkc)))
-	      (if residual-kana
-		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))))
-
 	(if (mana-context-alnum mc)
 	    (let ((key-str (charcode->string key))
 		  (pend (rk-pending rkc))
@@ -1185,19 +1217,26 @@
 	      (rk-flush rkc) ;; OK to reset rkc here.
 	      (if residual-kana
 		  (begin
-		    (ustr-insert-elem! preconv-str residual-kana)
-		    (ustr-insert-elem! raw-str pend)))
+                    (if (list? (car residual-kana))
+                      (begin
+                        (ustr-insert-seq! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str (reverse
+                                                     (string-to-list pend))))
+                      (begin
+                        (ustr-insert-elem! preconv-str residual-kana)
+                        (ustr-insert-elem! raw-str pend)))))
 	      (ustr-insert-elem! preconv-str
 				 (if (= (mana-context-alnum-type mc)
 					mana-type-halfwidth-alnum)
 				     (list key-str key-str key-str)
 				     (list (ja-wide key-str) (ja-wide key-str)
 					   (ja-wide key-str))))
-	      (ustr-insert-elem! raw-str key-str))
+	      (ustr-insert-elem! raw-str key-str)
+	      (check-auto-conv key-str))
 	    (let* ((key-str (charcode->string 
 			     (if (= rule mana-input-rule-kana)
 				 key
-				 (to-lower-char key))))
+				 (ichar-downcase key))))
 		   (pend (rk-pending rkc))
 		   (res (rk-push-key! rkc key-str)))
 	      (if (and res
@@ -1209,14 +1248,25 @@
 			(ustr-insert-elem! preconv-str res))
 		    (if (and next-pend
 			     (not (string=? next-pend "")))
-			(ustr-insert-elem! raw-str pend)
+			(ustr-insert-seq! raw-str
+                                          (reverse (string-to-list pend)))
 			(if (list? (car res))
 			    (begin
-			      (ustr-insert-elem! raw-str pend)
+                              (if (member pend
+                                          (map car
+                                               ja-consonant-syllable-table))
+                                ;; treat consonant having more than one
+                                ;; charactear as one raw-str in this case
+                                (ustr-insert-elem! raw-str pend)
+                                (ustr-insert-seq! raw-str (reverse
+                                                            (string-to-list
+                                                              pend))))
+                              ;; assume key-str as a vowel
 			      (ustr-insert-elem! raw-str key-str))
 			    (ustr-insert-elem!
 			     raw-str
-			     (string-append pend key-str)))))))))))))
+			     (string-append pend key-str))))))
+	      (check-auto-conv (if res (car res) #f)))))))))
 
 (define mana-context-confirm-kana!
   (lambda (mc)
@@ -1227,7 +1277,9 @@
                (residual-kana (rk-peek-terminal-match rkc)))
           (if residual-kana
               (begin
-                (ustr-insert-elem! preconv-str residual-kana)
+                (if (list? (car residual-kana))
+                  (ustr-insert-seq! preconv-str residual-kana)
+                  (ustr-insert-elem! preconv-str residual-kana))
                 (rk-flush rkc)))))))
 
 (define mana-proc-input-state
@@ -1238,8 +1290,8 @@
 
 (define mana-separator
   (lambda (mc)
-    (let ((attr (bit-or preedit-separator
-                        preedit-underline)))
+    (let ((attr (bitwise-ior preedit-separator
+			     preedit-underline)))
       (if mana-show-segment-separator?
           (cons attr mana-segment-separator)
           #f))))
@@ -1277,8 +1329,8 @@
       (append-map
         (lambda (seg-idx cand-idx)
           (let* ((attr (if (= seg-idx cur-seg)
-                           (bit-or preedit-reverse
-                                   preedit-cursor)
+                           (bitwise-ior preedit-reverse
+					preedit-cursor)
                            preedit-underline))
                  (cand (mana-get-nth-candidate mc seg-idx cand-idx))
                  (seg (list (cons attr cand))))
@@ -1404,7 +1456,7 @@
            (cur-page (if (= mana-nr-candidate-max 0)
                          0
                          (quotient n mana-nr-candidate-max)))
-           (pageidx (- (numeral-char->number numeralc) 1))
+           (pageidx (- (numeric-ichar->integer numeralc) 1))
            (compensated-pageidx (cond
                                   ((< pageidx 0) ; pressing key_0
                                    (+ pageidx 10))
@@ -1544,7 +1596,7 @@
        (mana-cancel-conv mc))
 
       ((and mana-select-candidate-by-numeral-key?
-            (numeral-char? key)
+            (ichar-numeric? key)
             (mana-context-candidate-window mc))
        (mana-move-candidate-in-page mc key))
 
@@ -1565,7 +1617,7 @@
 
 (define mana-press-key-handler
   (lambda (mc key key-state)
-    (if (control-char? key)
+    (if (ichar-control? key)
         (im-commit-raw mc)
         (if (mana-context-on mc)
             (if (mana-context-transposing mc)
@@ -1580,7 +1632,7 @@
 
 (define mana-release-key-handler
   (lambda (mc key key-state)
-    (if (or (control-char? key)
+    (if (or (ichar-control? key)
             (not (mana-context-on mc)))
         ;; don't discard key release event for apps
         (mana-commit-raw mc))))
