@@ -1,6 +1,6 @@
 ;;; prime-custom.scm: Customization variables for prime.scm
 ;;;
-;;; Copyright (c) 2003-2005 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -35,44 +35,104 @@
 (define prime-im-short-desc (N_ "Japanese predictive input method"))
 
 (define-custom-group 'prime
-  (ugettext prime-im-name-label)
-  (ugettext prime-im-short-desc))
+  prime-im-name-label
+  prime-im-short-desc)
 
 (define-custom-group 'prime-advanced
-  (_ "PRIME (advanced)")
-  (_ "Advanced settings for PRIME"))
+  (N_ "PRIME (advanced)")
+  (N_ "Advanced settings for PRIME"))
+
+(define-custom-group 'language
+  (N_ "Language choice")
+  (N_ "Language settings"))
 
 (define-custom-group 'japanese
-  (_ "Japanese")
-  (_ "Japanese specific settings"))
+  (N_ "Japanese")
+  (N_ "Japanese specific settings"))
+
+(define-custom-group 'english
+  (N_ "English")
+  (N_ "English specific settings"))
+
 
 (define-custom 'prime-custom-default-language 'Japanese
-  '(prime)
+  '(prime language)
   (list 'choice
-	(list 'Japanese (_ "Japanese") (_ "Japanese"))
-	(list 'English  (_ "English")  (_ "English")))
-  (_ "Default language")
-  (_ "long description will be here."))
+	(list 'Japanese (N_ "Japanese") (N_ "Japanese"))
+	(list 'English  (N_ "English")  (N_ "English")))
+  (N_ "Default language")
+  (N_ "long description will be here."))
+
+(define-custom 'prime-language-toggle-key '("F11")
+  '(prime language)
+  '(key)
+  (N_ "Language toggle key")
+  (N_ "long description will be here."))
 
 (define-custom 'prime-auto-register-mode? #t
   '(prime)
   '(boolean)
-  (_ "Enable auto register mode")
-  (_ "long description will be here."))
+  (N_ "Enable auto register mode")
+  (N_ "long description will be here."))
 
 (define-custom 'prime-custom-japanese-space 'wide
   '(prime japanese)
   (list 'choice
-	(list 'wide (_ "Wide width (Zenkaku)") (_ "Wide width (Zenkaku)"))
-	(list 'half (_ "Half width (Hankaku)") (_ "Half width (Hankaku)")))
-  (_ "Space character")
-  (_ "long description will be here."))
+	(list 'wide (N_ "Wide width (Zenkaku)") (N_ "Wide width (Zenkaku)"))
+	(list 'half (N_ "Half width (Hankaku)") (N_ "Half width (Hankaku)")))
+  (N_ "Space character")
+  (N_ "long description will be here."))
 
-(define-custom 'prime-use-unixdomain? #f
+(define-custom 'prime-altspace-key '("<Control> " "<Alt> ")
+  '(prime japanese)
+  '(key)
+  (N_ "Alternative space character key")
+  (N_ "long description will be here."))
+
+(define-custom 'prime-server-setting? 'unixdomain
   '(prime advanced)
-  '(boolean)
-  (_ "Use UNIX domain socket to communicate with PRIME")
-  (_ "long description will be here."))
+  (list 'choice
+        (list 'unixdomain
+              (N_ "Unix domain")
+              (N_ "Use UNIX domain socket to communicate with PRIME."))
+        (list 'tcpserver
+              (N_ "TCP server")
+              (N_ "Use tcp server to communicate with PRIME"))
+        (list 'pipe
+              (N_ "Pipe")
+              (N_ "Use pipe. spawn new prime process to communicate with PRIME")))
+  (N_ "Prime connection setting")
+  (N_ "long description will be here."))
+
+(define-custom 'prime-command-path "prime"
+  '(prime advanced)
+  '(pathname regular-file)
+  (N_ "Prime command path")
+  (N_ "long description will be here."))
+
+(define-custom 'prime-tcpserver-name "localhost"
+  '(prime advanced)
+  '(string ".*")
+  (N_ "PRIME server address")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-tcpserver-name
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? prime-server-setting?
+                        'tcpserver)))
+
+(define-custom 'prime-tcpserver-port 1180
+  '(prime advanced)
+  '(integer 0 65535)
+  (N_ "PRIME server port")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-tcpserver-port
+		 'custom-activity-hooks
+		 (lambda ()
+                   (eq? prime-server-setting?
+                        'tcpserver)))
 
 ;(define-custom 'prime-use-candidate-window? #t
 ;  '(prime candwin)
@@ -89,62 +149,80 @@
 (define-custom 'prime-nr-candidate-max 10
   '(prime-advanced candwin)
   '(integer 1 20)
-  (_ "Number of candidates in candidate window at a time")
-  (_ "long description will be here."))
+  (N_ "Number of candidates in candidate window at a time")
+  (N_ "long description will be here."))
 
 (define-custom 'prime-always-show-window? #t
   '(prime-advanced candwin)
   '(boolean)
-  (_ "Always showing candidate window")
-  (_ "long description will be here."))
+  (N_ "Always showing candidate window")
+  (N_ "long description will be here."))
 
 ;; If #t a candidate window displays usage examples of candidate words.
 (define-custom 'prime-custom-display-usage? #t
   '(prime-advanced annotation)
   '(boolean)
-  (_ "Show usage examples of candidate words")
-  (_ "long description will be here."))
+  (N_ "Show usage examples of candidate words")
+  (N_ "long description will be here."))
 
 ;; If #t a candidate window displays comments of candidate words.
 (define-custom 'prime-custom-display-comment? #t
   '(prime-advanced annotation)
   '(boolean)
-  (_ "Show candidate annotations")
-  (_ "long description will be here."))
+  (N_ "Show candidate annotations")
+  (N_ "long description will be here."))
 
 ;; If #t a candidate window displays forms of candidate words such as
 ;; 'l (small L)', 'I (large i)'.
 (define-custom 'prime-custom-display-form? #t
   '(prime-advanced annotation)
   '(boolean)
-  (_ "Show candidate forms")
-  (_ "long description will be here."))
+  (N_ "Show candidate forms")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-custom-display-usage?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+(custom-add-hook 'prime-custom-display-comment?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
+(custom-add-hook 'prime-custom-display-form?
+  'custom-activity-hooks
+  (lambda ()
+    enable-annotation?))
 
 ;; ------------------------------------------------------------
 
 (define-custom 'prime-custom-number-selection? #f
   '(prime-advanced special-op)
   '(boolean)
-  (_ "Select candidate by numeral keys")
-  (_ "long description will be here."))
+  (N_ "Select candidate by numeral keys")
+  (N_ "long description will be here."))
 
 (define-custom 'prime-custom-app-mode-vi? #f
   '(prime-advanced special-op)
   '(boolean)
-  (_ "Enable vi-cooperative mode")
-  (_ "long description will be here."))
+  (N_ "Enable vi-cooperative mode")
+  (N_ "long description will be here."))
+
+(custom-add-hook 'prime-custom-app-mode-vi?
+  'custom-set-hooks
+  (lambda ()
+    (prime-configure-app-mode-vi)))
 
 (define-custom 'prime-pseudo-mode-cursor? #f
   '(prime-advanced special-op)
   '(boolean)
-  (_ "Enable pseudo mode cursor")
-  (_ "long description will be here."))
+  (N_ "Enable pseudo mode cursor")
+  (N_ "long description will be here."))
 
 ;(define-custom 'prime-mask-pending-preedit? #f
 ;  '(prime)
 ;  '(boolean)
-;  (_ "Mask preedit strings (For T-Code users)")
-;  (_ "long description will be here."))
+;  (N_ "Mask preedit strings (For T-Code users)")
+;  (N_ "long description will be here."))
 
 ;;
 ;; toolbar
@@ -155,20 +233,30 @@
 (define prime-input-mode-indication-alist
   (list
    (list 'action_prime_mode_latin
-	 'figure_prime_mode_latin
-	 "P"
+	 'ja_direct
+	 "--"
 	 (N_ "Direct input")
 	 (N_ "PRIME off"))
    (list 'action_prime_mode_hiragana
-	 'figure_prime_mode_hiragana
+	 'ja_hiragana
 	 ""
 	 (N_ "Japanese")
 	 (N_ "PRIME on"))
+   (list 'action_prime_mode_english
+	 'ja_halfwidth_alnum
+	 "A"
+	 (N_ "English")
+	 (N_ "PRIME on"))
    (list 'action_prime_mode_wide_latin
-	 'figure_prime_mode_wide_latin
+	 'ja_fullwidth_alnum
 	 ""
 	 (N_ "Fullwidth Alphanumeric")
-	 (N_ "Fullwidth Alphanumeric input mode"))))
+	 (N_ "Fullwidth Alphanumeric input mode"))
+   (list 'action_prime_mode_application
+	 'prime_mode_application
+	 ""
+	 (N_ "Peculiar")
+	 (N_ "Application specific input mode"))))
 
 (define prime-widgets '(widget_prime_input_mode))
 
@@ -178,24 +266,24 @@
 ;; Users don't care this option.
 ;; (2005-03-03) <Hiroyuki Komatsu>
 ;(define-custom 'default-widget_prime_input_mode 'action_prime_mode_latin
-;  '(prime-advanced toolbar)
+;  '(prime-advanced toolbar-widget)
 ;  (cons 'choice
 ;	(map indication-alist-entry-extract-choice
 ;	     prime-input-mode-indication-alist))
-;  (_ "Default input mode")
-;  (_ "long description will be here."))
+;  (N_ "Default input mode")
+;  (N_ "long description will be here."))
 
 (define prime-input-mode-actions (map car prime-input-mode-indication-alist))
 ;; Users don't care this option.
 ;; (2005-03-03) <Hiroyuki Komatsu>
 ;(define-custom 'prime-input-mode-actions
 ;               (map car prime-input-mode-indication-alist)
-;   '(prime-advanced toolbar)
+;   '(prime-advanced toolbar-widget)
 ;  (cons 'ordered-list
 ;	(map indication-alist-entry-extract-choice
 ;	     prime-input-mode-indication-alist))
-;  (_ "Input mode menu items")
-;  (_ "long description will be here."))
+;  (N_ "Input mode menu items")
+;  (N_ "long description will be here."))
 
 ;; value dependency
 ;(if custom-full-featured?
