@@ -1,6 +1,6 @@
 ;;; sj3.scm: SJ3 for uim.
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -33,8 +33,8 @@
 
 (require "ustr.scm")
 (require "japanese.scm")
-(require "japanese-kana.scm")
-(require "japanese-azik.scm")
+(require "generic-predict.scm")
+(require "sj3v2-socket.scm")
 (require-custom "generic-key-custom.scm")
 (require-custom "sj3-custom.scm")
 (require-custom "sj3-key-custom.scm")
@@ -51,9 +51,6 @@
       (sj3-lib-open "" sj3-user)))
 
 ;; error recovery
-(define (sj3-lib-error? result)
-  (and (pair? result)
-       (eq? (car result) 'error)))
 (define (sj3-connect-wait sc w)
   (let ((stime (time)))
     (let loop ((now (time)))
@@ -72,26 +69,31 @@
   (set! sj3-init-lib-ok? #f)
   (sj3-cancel-conv sc)
   (let loop ((fib1 1) (fib2 1))
-    (if (sj3-lib-error? (sj3-lib-init sj3-server-name))
-        (begin
-          (uim-notify-info (N_ "Reconnecting to sj3 server."))
-          (im-clear-preedit sc)
-          (im-pushback-preedit
-           sc preedit-reverse
-           (N_ "[Reconnecting...]"))
-          (im-update-preedit sc)
-          (sleep 1)
-          (sj3-connect-wait sc fib1)
-          (loop fib2 (+ fib1 fib2)))
-        (set! sj3-init-lib-ok? #t))))
+    (uim-notify-info (N_ "Reconnecting to sj3 server."))
+    (guard (err (else
+                 (uim-notify-info (N_ "Reconnecting to sj3 server."))
+                 (im-clear-preedit sc)
+                 (im-pushback-preedit
+                  sc preedit-reverse
+                  (N_ "[Reconnecting...]"))
+                 (im-update-preedit sc)
+                 (sleep 1)
+                 (sj3-connect-wait sc fib1)
+                 (loop fib2 (+ fib1 fib2))))
+           (sj3-lib-init sj3-server-name))
+    (set! sj3-init-lib-ok? #t)))
 (define (sj3-lib-funcall sc f . args)
-  (let ((ret (apply f args)))
-    (if (sj3-lib-error? ret)
-        (begin (if sj3-init-lib-ok?
-                   (sj3-lib-close))
-               (sj3-connect-retry sc)
-               (apply f args))
-        ret)))
+  (or
+   (guard (err (else #f))
+          (apply f args))
+   (begin
+     (if sj3-init-lib-ok?
+         (guard (close-err
+                 (else
+                  #f))
+                (sj3-lib-close)))
+     (sj3-connect-retry sc)
+     (apply f args))))
 
 (define (sj3-lib-alloc-context)
   #t)
@@ -111,7 +113,8 @@
 (define (sj3-lib-get-nth-candidate-without-muhenkan sc seg nth)
   (let* ((yomi (sj3-get-nth-yomi sc seg))
          (cnt (sj3-lib-funcall sc sj3-lib-douoncnt yomi)))
-    (if (< nth cnt)
+    (if (and (< nth cnt)
+             (<= 0 nth))
         ;; henkan
         (cons (sj3-lib-get-nth-candidate sc seg nth) #t)
         ;; muhenkan
@@ -119,9 +122,12 @@
 (define (sj3-lib-get-nth-candidate sc seg nth)
   (let* ((yomi (sj3-get-nth-yomi sc seg))
          (cnt (sj3-lib-funcall sc sj3-lib-douoncnt yomi)))
-    (if (< nth cnt)
-        (car (sj3-lib-funcall sc sj3-lib-get-nth-douon yomi nth))
-        (list-ref (sj3-make-map-from-kana-string yomi) (- nth cnt)))))
+    (cond ((< nth 0)
+           (list-ref (sj3-make-map-from-kana-string yomi) 0))
+          ((< nth cnt)
+           (car (sj3-lib-funcall sc sj3-lib-get-nth-douon yomi nth)))
+          (else
+           (list-ref (sj3-make-map-from-kana-string yomi) (- nth cnt))))))
 (define (sj3-lib-release-context sc)
   #t)
 (define (sj3-lib-get-unconv-candidate sc seg-idx)
@@ -185,14 +191,20 @@
            (sj3-context-set-sc-ctx! sc (cdr ret))
            (- (length ret) 1))
           (else
-           (length (sj3-make-map-from-kana-string str))))))
+           (sj3-context-set-sc-ctx! sc (list (sj3-make-map-from-kana-string str))) ; XXX hack
+           1))))
 (define (sj3-lib-commit-segment sc seg delta)
   ;; segment learnining
-  (let ((douon (sj3-getdouon sc (sj3-get-nth-yomi sc seg))))
-    (if (< delta (length douon))
-        (let ((entry (list-ref douon delta)))
-          (if (= 2 (length entry))
-              (sj3-lib-funcall sc sj3-lib-gakusyuu (list-ref entry 1)))))))
+  (if (sj3-lib-opened?)
+      (let* ((yomi (sj3-get-nth-yomi sc seg))
+             (douon (sj3-getdouon sc yomi)))
+        (if (< delta (length douon))
+            (let ((entry (list-ref douon delta)))
+              (if (= 2 (length entry))
+                  (begin
+                    (predict-meta-commit (sj3-context-prediction-ctx sc) yomi (list-ref entry 0) "")
+                    (sj3-lib-funcall sc sj3-lib-gakusyuu (list-ref entry 1)))))))
+      #f))
 
 ;; return alist of cons'ed offset and length
 ;; ("abc" "d" "efgh") => ((3 . 3) (4 . 1) (8 . 4))
@@ -268,47 +280,47 @@
          (iota (length segs))
          segs))
   ;; segment-length learnining
-  (let* ((sc-ctx (sj3-context-sc-ctx sc))
-         ;; revert from hell
-         (new-segments (restore-segments))
-         (orig (apply string-append (map car sc-ctx)))
-         ;; revert too
-         (old-string-list (map car (cdr (sj3-lib-funcall sc sj3-lib-getkan orig)))))
-    (receive (new-string-list new-dcid-list)
-             (unzip2 new-segments)
-      (let ((old-offset (sj3-get-seg-offset old-string-list))
-            (new-offset (sj3-get-seg-offset new-string-list)))
-        ;; split case
-        (for-each (lambda (l)
-                    (let ((yomi1 (list-ref new-string-list l))
-                          (yomi2 (list-ref new-string-list (+ 1 l)))
-                          (dcid  (list-ref new-dcid-list   (+ 1 l))))
-                      (if dcid
-                          (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
-                  (sj3-filter-split-segment old-offset new-offset))
-        ;; merge case
-        (for-each (lambda (l)
-                    (let ((yomi1 (list-ref new-string-list l))
-                          (yomi2 #f)
-                          (dcid  #f))
-                      (if dcid
-                          (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
-                  (sj3-filter-merge-segment old-offset new-offset))
-        ;; move case
-        (for-each (lambda (l)
-                    (let ((yomi1 (list-ref new-string-list l))
-                          (yomi2 (list-ref new-string-list (+ 1 l)))
-                          (dcid  (list-ref new-dcid-list   (+ 1 l))))
-                      (if dcid
-                          (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
-                  (sj3-filter-move-segment old-offset new-offset))
-        )))
+  (if (sj3-lib-opened?)
+      (let* ((sc-ctx (sj3-context-sc-ctx sc))
+             ;; revert from hell
+             (new-segments (restore-segments))
+             (orig (apply string-append (map car sc-ctx)))
+             ;; revert too
+             (old-string-list (map car (cdr (sj3-lib-funcall sc sj3-lib-getkan orig)))))
+        (receive (new-string-list new-dcid-list)
+                 (unzip2 new-segments)
+          (let ((old-offset (sj3-get-seg-offset old-string-list))
+                (new-offset (sj3-get-seg-offset new-string-list)))
+            ;; split case
+            (for-each (lambda (l)
+                        (let ((yomi1 (list-ref new-string-list l))
+                              (yomi2 (list-ref new-string-list (+ 1 l)))
+                              (dcid  (list-ref new-dcid-list   (+ 1 l))))
+                          (if dcid
+                              (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
+                      (sj3-filter-split-segment old-offset new-offset))
+            ;; merge case
+            (for-each (lambda (l)
+                        (let ((yomi1 (list-ref new-string-list l))
+                              (yomi2 #f)
+                              (dcid  #f))
+                          (if dcid
+                              (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
+                      (sj3-filter-merge-segment old-offset new-offset))
+            ;; move case
+            (for-each (lambda (l)
+                        (let ((yomi1 (list-ref new-string-list l))
+                              (yomi2 (list-ref new-string-list (+ 1 l)))
+                              (dcid  (list-ref new-dcid-list   (+ 1 l))))
+                          (if dcid
+                              (sj3-lib-gakusyuu2 yomi1 yomi2 dcid))))
+                      (sj3-filter-move-segment old-offset new-offset))
+            ))))
   #t)
 
 (define (sj3-lib-reset-conversion sc)
   #f)
 
-
 (define sj3-init-lib-ok? #f)
 
 (define sj3-type-direct	   ja-type-direct)
@@ -321,6 +333,8 @@
 (define sj3-input-rule-roma 0)
 (define sj3-input-rule-kana 1)
 (define sj3-input-rule-azik 2)
+(define sj3-input-rule-act 3)
+(define sj3-input-rule-kzik 4)
 
 (define sj3-candidate-type-katakana -2)
 (define sj3-candidate-type-hiragana -3)
@@ -495,6 +509,7 @@
 		      sj3-input-rule-kana))
 		 (lambda (sc)
 		   (sj3-prepare-input-rule-activation sc)
+                   (require "japanese-kana.scm")
 		   (sj3-context-set-input-rule! sc sj3-input-rule-kana)
                    (sj3-context-change-kana-mode!
                      sc (sj3-context-kana-mode sc))
@@ -511,10 +526,43 @@
 		      sj3-input-rule-azik))
 		 (lambda (sc)
 		   (sj3-prepare-input-rule-activation sc)
+                   (require "japanese-azik.scm")
 		   (rk-context-set-rule! (sj3-context-rkc sc)
 					 ja-azik-rule)
 		   (sj3-context-set-input-rule! sc sj3-input-rule-azik)))
 
+(register-action 'action_sj3_kzik
+		 (lambda (sc)
+		   '(ja_kzik
+		     ""
+		     "KZIK"
+		     "KZIKĥ޻ϥ⡼"))
+		 (lambda (sc)
+		   (= (sj3-context-input-rule sc)
+		      sj3-input-rule-kzik))
+		 (lambda (sc)
+		   (sj3-prepare-input-rule-activation sc)
+                   (require "japanese-kzik.scm")
+		   (rk-context-set-rule! (sj3-context-rkc sc)
+					 ja-kzik-rule)
+		   (sj3-context-set-input-rule! sc sj3-input-rule-kzik)))
+
+(register-action 'action_sj3_act
+		 (lambda (sc)
+		   '(ja_act
+		     ""
+		     "ACT"
+		     "ACTĥ޻ϥ⡼"))
+		 (lambda (sc)
+		   (= (sj3-context-input-rule sc)
+		      sj3-input-rule-act))
+		 (lambda (sc)
+		   (sj3-prepare-input-rule-activation sc)
+                   (require "japanese-act.scm")
+		   (rk-context-set-rule! (sj3-context-rkc sc)
+					 ja-act-rule)
+		   (sj3-context-set-input-rule! sc sj3-input-rule-act)))
+
 ;; Update widget definitions based on action configurations. The
 ;; procedure is needed for on-the-fly reconfiguration involving the
 ;; custom API
@@ -537,6 +585,7 @@
     (list 'state              #f)
     (list 'transposing        #f)
     (list 'transposing-type    0)
+    (list 'predicting         #f)
     (list 'sc-ctx             ()) ;; sj3-internal-context
     (list 'preconv-ustr	      #f) ;; preedit strings
     (list 'rkc                ())
@@ -548,10 +597,50 @@
     (list 'alnum-type	      sj3-type-halfwidth-alnum)
     (list 'commit-raw         #t)
     (list 'input-rule         sj3-input-rule-roma)
-    (list 'raw-ustr	      #f))))
+    (list 'raw-ustr	      #f)
+    (list 'prediction-ctx     '())
+    (list 'prediction-word '())
+    (list 'prediction-candidates '())
+    (list 'prediction-appendix '())
+    (list 'prediction-nr '())
+    (list 'prediction-window  #f)
+    (list 'prediction-index   #f)
+    (list 'prediction-cache   '()))))
 (define-record 'sj3-context sj3-context-rec-spec)
 (define sj3-context-new-internal sj3-context-new)
 
+(define (sj3-predict sc str)
+  (predict-meta-search
+   (sj3-context-prediction-ctx sc)
+   str))
+(define (sj3-lib-set-prediction-src-string sc str)
+  (let* ((ret (sj3-predict sc str))
+         (word  (predict-meta-word? ret))
+         (cands (predict-meta-candidates? ret))
+         (appendix (predict-meta-appendix? ret)))
+    (sj3-context-set-prediction-word! sc word)
+    (sj3-context-set-prediction-candidates! sc cands)
+    (sj3-context-set-prediction-appendix! sc appendix)
+    (sj3-context-set-prediction-nr! sc (length cands)))
+  #f)
+(define (sj3-lib-get-nr-predictions sc)
+  (sj3-context-prediction-nr sc))
+(define (sj3-lib-get-nth-word sc nth)
+  (let ((word (sj3-context-prediction-word sc)))
+    (list-ref word nth)))
+(define (sj3-lib-get-nth-prediction sc nth)
+  (let ((cands (sj3-context-prediction-candidates sc)))
+    (list-ref cands nth)))
+(define (sj3-lib-get-nth-appendix sc nth)
+  (let ((appendix (sj3-context-prediction-candidates sc)))
+    (list-ref appendix nth)))
+(define (sj3-lib-commit-nth-prediction sc nth)
+  (predict-meta-commit
+   (sj3-context-prediction-ctx sc)
+   (sj3-lib-get-nth-word sc nth)
+   (sj3-lib-get-nth-prediction sc nth)
+   (sj3-lib-get-nth-appendix sc nth)))
+
 (define (sj3-context-new id im)
   (let ((sc (sj3-context-new-internal id im))
 	(rkc (rk-context-new ja-rk-rule #t #f)))
@@ -563,9 +652,11 @@
     (sj3-context-set-preconv-ustr! sc (ustr-new '()))
     (sj3-context-set-raw-ustr! sc (ustr-new '()))
     (sj3-context-set-segments! sc (ustr-new '()))
-    (if using-kana-table?
-        (sj3-context-set-input-rule! sc sj3-input-rule-kana)
-        (sj3-context-set-input-rule! sc sj3-input-rule-roma))
+    (if sj3-use-prediction?
+        (begin
+          (sj3-context-set-prediction-ctx! sc (predict-make-meta-search))
+          (predict-meta-open (sj3-context-prediction-ctx sc) "sj3")
+          (predict-meta-set-external-charset! (sj3-context-prediction-ctx sc) "EUC-JP")))
     sc))
 
 (define (sj3-commit-raw sc)
@@ -614,8 +705,10 @@
 	      (string-append-map-ustr-former extract-kana preconv-str)
 	      (if convert-pending-into-kana?
 		  (if residual-kana
-		      (extract-kana residual-kana)
-                      pending)
+                    (if (list? (car residual-kana))
+                      (string-append-map extract-kana residual-kana)
+		      (extract-kana residual-kana))
+                    pending)
 		  pending)
               (string-append-map-ustr-latter extract-kana preconv-str))))
 	   kana)
@@ -623,8 +716,10 @@
 	   (string-append-map-ustr-former extract-kana preconv-str)
            (if convert-pending-into-kana?
                (if residual-kana
-                   (extract-kana residual-kana)
-                   "")
+                 (if (list? (car residual-kana))
+                   (string-append-map extract-kana residual-kana)
+                   (extract-kana residual-kana))
+                 "")
                pending)
            (string-append-map-ustr-latter extract-kana preconv-str))))))
 
@@ -674,9 +769,11 @@
   (ustr-clear! (sj3-context-segments sc))
   (sj3-context-set-transposing! sc #f)
   (sj3-context-set-state! sc #f)
-  (if (sj3-context-candidate-window sc)
+  (if (or (sj3-context-candidate-window sc)
+          (sj3-context-prediction-window sc))
       (im-deactivate-candidate-selector sc))
   (sj3-context-set-candidate-window! sc #f)
+  (sj3-context-set-prediction-window! sc #f)
   (sj3-context-set-candidate-op-count! sc 0))
 
 (define (sj3-begin-input sc key key-state)
@@ -731,8 +828,10 @@
 			      (sj3-context-transposing-state-preedit sc)
 			      (if (sj3-context-state sc)
 				  (sj3-compose-state-preedit sc)
-				  (sj3-input-state-preedit sc)))
-			  ())))
+                                  (if (sj3-context-predicting sc)
+                                      (sj3-predicting-state-preedit sc)
+                                      (sj3-input-state-preedit sc))))
+                          ())))
 	(context-update-preedit sc segments))
       (sj3-context-set-commit-raw! sc #f)))
 
@@ -872,7 +971,9 @@
 	         (res (rk-push-key! rkc key-str)))
 	    (if res
 	        (begin
-	          (ustr-insert-elem! (sj3-context-preconv-ustr sc) res)
+                  (if (list? (car res))
+                    (ustr-insert-seq! (sj3-context-preconv-ustr sc) res)
+                    (ustr-insert-elem! (sj3-context-preconv-ustr sc) res))
 	          (ustr-insert-elem! (sj3-context-raw-ustr sc) key-str))
 	        (if (null? (rk-context-seq rkc))
 		    (sj3-commit-raw sc)))))))))
@@ -985,7 +1086,185 @@
 	   (sj3-flush sc)
 	   (sj3-proc-input-state sc key key-state))))))))
 
-(define (sj3-proc-input-state-with-preedit sc key key-state)
+(define (sj3-move-prediction sc offset)
+  (let* ((nr (sj3-lib-get-nr-predictions sc))
+         (idx (sj3-context-prediction-index sc))
+         (n (if (not idx)
+                0
+                (+ idx offset)))
+         (compensated-n (cond
+                         ((>= n nr)
+                          0)
+                        ((< n 0)
+                         (- nr 1))
+                        (else
+                         n))))
+    (im-select-candidate sc compensated-n)
+    (sj3-context-set-prediction-index! sc compensated-n)))
+
+(define (sj3-move-prediction-in-page sc numeralc)
+  (let* ((nr (sj3-lib-get-nr-predictions sc))
+         (p-idx (sj3-context-prediction-index sc))
+         (n (if (not p-idx)
+                0
+                p-idx))
+         (cur-page (if (= sj3-nr-candidate-max 0)
+                       0
+                       (quotient n sj3-nr-candidate-max)))
+         (pageidx (- (numeric-ichar->integer numeralc) 1))
+         (compensated-pageidx (cond
+                               ((< pageidx 0) ; pressing key_0
+                                (+ pageidx 10))
+                               (else
+                                pageidx)))
+         (idx (+ (* cur-page sj3-nr-candidate-max) compensated-pageidx))
+         (compensated-idx (cond
+                           ((>= idx nr)
+                            #f)
+                           (else
+                            idx)))
+         (selected-pageidx (if (not p-idx)
+                               #f
+                               (if (= sj3-nr-candidate-max 0)
+                                   p-idx
+                                   (remainder p-idx
+                                              sj3-nr-candidate-max)))))
+    (if (and
+         compensated-idx
+         (not (eqv? compensated-pageidx selected-pageidx)))
+        (begin
+          (sj3-context-set-prediction-index! sc compensated-idx)
+          (im-select-candidate sc compensated-idx)
+          #t)
+       #f)))
+
+(define (sj3-prediction-select-non-existing-index? sc numeralc)
+  (let* ((nr (sj3-lib-get-nr-predictions sc))
+         (p-idx (sj3-context-prediction-index sc))
+         (cur-page (if (= sj3-nr-candidate-max 0)
+                       0
+                       (quotient p-idx sj3-nr-candidate-max)))
+         (pageidx (- (numeric-ichar->integer numeralc) 1))
+         (compensated-pageidx (cond
+                               ((< pageidx 0) ; pressing key_0
+                                (+ pageidx 10))
+                               (else
+                                pageidx)))
+         (idx (+ (* cur-page sj3-nr-candidate-max) compensated-pageidx)))
+    (if (>= idx nr)
+        #t
+        #f)))
+
+(define (sj3-prediction-keys-handled? sc key key-state)
+  (cond
+   ((sj3-next-prediction-key? key key-state)
+    (sj3-move-prediction sc 1)
+    #t)
+   ((sj3-prev-prediction-key? key key-state)
+    (sj3-move-prediction sc -1)
+    #t)
+   ((and
+     sj3-select-prediction-by-numeral-key?
+     (ichar-numeric? key))
+    (sj3-move-prediction-in-page sc key))
+   ((and
+     (sj3-context-prediction-index sc)
+     (sj3-prev-page-key? key key-state))
+    (im-shift-page-candidate sc #f)
+    #t)
+   ((and
+     (sj3-context-prediction-index sc)
+     (sj3-next-page-key? key key-state))
+    (im-shift-page-candidate sc #t)
+    #t)
+   (else
+    #f)))
+
+(define (sj3-proc-prediction-state sc key key-state)
+  (cond
+   ;; prediction index change
+   ((sj3-prediction-keys-handled? sc key key-state))
+
+   ;; cancel
+   ((sj3-cancel-key? key key-state)
+    (if (sj3-context-prediction-index sc)
+        (sj3-reset-prediction-window sc)
+        (begin
+          (sj3-reset-prediction-window sc)
+          (sj3-proc-input-state sc key key-state))))
+
+   ;; commit
+   ((and
+     (sj3-context-prediction-index sc)
+     (sj3-commit-key? key key-state))
+    (sj3-do-commit-prediction sc))
+   (else
+    (if (and
+         sj3-use-implicit-commit-prediction?
+         (sj3-context-prediction-index sc))
+        (cond
+         ((or
+           ;; check keys used in sj3-proc-input-state-with-preedit
+           (sj3-begin-conv-key? key key-state)
+           (sj3-backspace-key? key key-state)
+           (sj3-delete-key? key key-state)
+           (sj3-kill-key? key key-state)
+           (sj3-kill-backward-key? key key-state)
+           (and
+            (not (sj3-context-alnum sc))
+            (sj3-commit-as-opposite-kana-key? key key-state))
+           (sj3-transpose-as-hiragana-key? key key-state)
+           (sj3-transpose-as-katakana-key? key key-state)
+           (sj3-transpose-as-halfkana-key? key key-state)
+           (and
+            (not (= (sj3-context-input-rule sc) sj3-input-rule-kana))
+            (or
+             (sj3-transpose-as-halfwidth-alnum-key? key key-state)
+             (sj3-transpose-as-fullwidth-alnum-key? key key-state)))
+           (sj3-hiragana-key? key key-state)
+           (sj3-katakana-key? key key-state)
+           (sj3-halfkana-key? key key-state)
+           (sj3-halfwidth-alnum-key? key key-state)
+           (sj3-fullwidth-alnum-key? key key-state)
+           (and
+            (not (sj3-context-alnum sc))
+            (sj3-kana-toggle-key? key key-state))
+           (sj3-alkana-toggle-key? key key-state)
+           (sj3-go-left-key? key key-state)
+           (sj3-go-right-key? key key-state)
+           (sj3-beginning-of-preedit-key? key key-state)
+           (sj3-end-of-preedit-key? key key-state)
+           (and
+            (modifier-key-mask key-state)
+            (not (shift-key-mask key-state))))
+          ;; go back to unselected prediction
+          (sj3-reset-prediction-window sc)
+          (sj3-check-prediction sc #f))
+         ((and
+           (ichar-numeric? key)
+           sj3-select-prediction-by-numeral-key?
+           (not (sj3-prediction-select-non-existing-index? sc key)))
+          (sj3-context-set-predicting! sc #f)
+          (sj3-context-set-prediction-index! sc #f)
+          (sj3-proc-input-state sc key key-state))
+         (else
+          ;; implicit commit
+          (sj3-do-commit-prediction sc)
+          (sj3-proc-input-state sc key key-state)))
+        (begin
+          (sj3-context-set-predicting! sc #f)
+          (sj3-context-set-prediction-index! sc #f)
+          (sj3-proc-input-state sc key key-state))))))
+
+(define (sj3-proc-input-state-with-preedit sc key key-state) 
+  (define (check-auto-conv str)
+    (and
+      str
+      sj3-auto-start-henkan?
+      (string-find japanese-auto-start-henkan-keyword-list str)
+      (begin
+	(sj3-reset-prediction-window sc)
+	(sj3-begin-conv sc))))
   (let ((preconv-str (sj3-context-preconv-ustr sc))
 	(raw-str (sj3-context-raw-ustr sc))
 	(rkc (sj3-context-rkc sc))
@@ -994,8 +1273,13 @@
     (cond
      ;; begin conversion
      ((sj3-begin-conv-key? key key-state)
+      (sj3-reset-prediction-window sc)
       (sj3-begin-conv sc))
 
+     ;; prediction
+     ((sj3-next-prediction-key? key key-state)
+      (sj3-check-prediction sc #t))
+
      ;; backspace
      ((sj3-backspace-key? key key-state)
       (if (not (rk-backspace rkc))
@@ -1045,6 +1329,7 @@
 	   (or
 	    (sj3-transpose-as-halfwidth-alnum-key? key key-state)
 	    (sj3-transpose-as-fullwidth-alnum-key? key key-state))))
+      (sj3-reset-prediction-window sc)
       (sj3-proc-transposing-state sc key key-state))
 
      ((sj3-hiragana-key? key key-state)
@@ -1145,22 +1430,6 @@
       #f)
 
      (else
-      ;; handle "n1" sequence as "1"
-      (if (and (not (sj3-context-alnum sc))
-	       (not (ichar-alphabetic? key))
-	       (not (string-find
-		     (rk-expect rkc)
-		     (charcode->string
-		      (if (= rule sj3-input-rule-kana)
-			  key
-			  (ichar-downcase key))))))
-	  (let ((pend (rk-pending rkc))
-		(residual-kana (rk-push-key-last! rkc)))
-	    (if residual-kana
-		(begin
-		  (ustr-insert-elem! preconv-str residual-kana)
-		  (ustr-insert-elem! raw-str pend)))))
-
       (if (sj3-context-alnum sc)
           (let ((key-str (charcode->string key))
 	        (pend (rk-pending rkc))
@@ -1168,15 +1437,22 @@
 	    (rk-flush rkc) ;; OK to reset rkc here.
 	    (if residual-kana
 	        (begin
-		  (ustr-insert-elem! preconv-str residual-kana)
-		  (ustr-insert-elem! raw-str pend)))
+                  (if (list? (car residual-kana))
+                    (begin
+                      (ustr-insert-seq! preconv-str residual-kana)
+                      (ustr-insert-elem! raw-str (reverse
+                                                   (string-to-list pend))))
+                    (begin
+                      (ustr-insert-elem! preconv-str residual-kana)
+                      (ustr-insert-elem! raw-str pend)))))
 	    (ustr-insert-elem! preconv-str
 			       (if (= (sj3-context-alnum-type sc)
 				      sj3-type-halfwidth-alnum)
 				   (list key-str key-str key-str)
 				   (list (ja-wide key-str) (ja-wide key-str)
 					 (ja-wide key-str))))
-	    (ustr-insert-elem! raw-str key-str))
+	    (ustr-insert-elem! raw-str key-str)
+	    (check-auto-conv key-str))
 	  (let* ((key-str (charcode->string
 			   (if (= rule sj3-input-rule-kana)
 			       key
@@ -1192,14 +1468,25 @@
 		      (ustr-insert-elem! preconv-str res))
 	          (if (and next-pend
 		           (not (string=? next-pend "")))
-		      (ustr-insert-elem! raw-str pend)
+		      (ustr-insert-seq! raw-str
+                                        (reverse (string-to-list pend)))
 		      (if (list? (car res))
 		          (begin
-			    (ustr-insert-elem! raw-str pend)
+                            (if (member pend
+                                        (map car
+                                             ja-consonant-syllable-table))
+                              ;; treat consonant having more than one
+                              ;; charactear as one raw-str in this case
+                              (ustr-insert-elem! raw-str pend)
+                              (ustr-insert-elem! raw-str (reverse
+                                                           (string-to-list
+                                                             pend))))
+                            ;; assume key-str as a vowel
 			    (ustr-insert-elem! raw-str key-str))
 		          (ustr-insert-elem!
 		           raw-str
-		           (string-append pend key-str))))))))))))
+		           (string-append pend key-str))))))
+	    (check-auto-conv (if res (car res) #f))))))))
 
 (define sj3-context-confirm-kana!
   (lambda (sc)
@@ -1210,13 +1497,59 @@
 	       (residual-kana (rk-peek-terminal-match rkc)))
 	  (if residual-kana
 	      (begin
-		(ustr-insert-elem! preconv-str residual-kana)
+                (if (list? (car residual-kana))
+                  (ustr-insert-seq! preconv-str residual-kana)
+                  (ustr-insert-elem! preconv-str residual-kana))
 		(rk-flush rkc)))))))
 
+(define (sj3-reset-prediction-window sc)
+  (if (sj3-context-prediction-window sc)
+      (im-deactivate-candidate-selector sc))
+  (sj3-context-set-predicting! sc #f)
+  (sj3-context-set-prediction-window! sc #f)
+  (sj3-context-set-prediction-index! sc #f))
+
+(define (sj3-check-prediction sc force-check?)
+  (if (and
+       (not (sj3-context-state sc))
+       (not (sj3-context-transposing sc))
+       (not (sj3-context-predicting sc)))
+      (let* ((use-pending-rk-for-prediction? #t)
+	     (preconv-str
+	      (sj3-make-whole-string
+	       sc
+	       (not use-pending-rk-for-prediction?)
+	       (sj3-context-kana-mode sc)))
+	     (preedit-len (+
+			   (ustr-length (sj3-context-preconv-ustr sc))
+			   (if (not use-pending-rk-for-prediction?)
+			       0
+			       (string-length (rk-pending
+					       (sj3-context-rkc
+						sc)))))))
+	(if (or
+	     (>= preedit-len sj3-prediction-start-char-count)
+	     force-check?)
+	    (begin
+	      (sj3-lib-set-prediction-src-string sc preconv-str)
+	      (let ((nr (sj3-lib-get-nr-predictions sc)))
+		(if (and
+		     nr
+		     (> nr 0))
+		    (begin
+		      (im-activate-candidate-selector
+		       sc nr sj3-nr-candidate-max)
+		      (sj3-context-set-prediction-window! sc #t)
+		      (sj3-context-set-predicting! sc #t))
+		    (sj3-reset-prediction-window sc))))
+	    (sj3-reset-prediction-window sc)))))
+
 (define (sj3-proc-input-state sc key key-state)
   (if (sj3-has-preedit? sc)
       (sj3-proc-input-state-with-preedit sc key key-state)
-      (sj3-proc-input-state-no-preedit sc key key-state)))
+      (sj3-proc-input-state-no-preedit sc key key-state))
+  (if sj3-use-prediction?
+      (sj3-check-prediction sc #f)))
 
 (define sj3-separator
   (lambda (sc)
@@ -1259,8 +1592,10 @@
 	   (left-str (ustr-former-seq raw-str)))
      (append left-str
 	     (if residual-kana
-		 (list pending)
-		 '())
+               (if (list? (car residual-kana))
+                 (reverse (string-to-list pending))
+		 (list pending))
+               '())
 	      right-str))))
 
 (define sj3-get-raw-candidate
@@ -1285,7 +1620,9 @@
 	    (if (member (car unconv) preconv)
 		(let ((start (list-seq-contained? preconv unconv))
 		      (len (length unconv)))
-		  (if start
+		  (if (and
+                        start
+                        (= (length raw-str) (length preconv))) ;; sanity check
 		      (sj3-make-raw-string
 		       (reverse (sublist-rel raw-str start len))
 		       (if (or
@@ -1303,6 +1640,14 @@
 		"???") ;; FIXME
 	    "????")))))) ;; shouldn't happen
 
+(define (sj3-predicting-state-preedit sc)
+  (if (or
+       (not sj3-use-implicit-commit-prediction?)
+       (not (sj3-context-prediction-index sc)))
+      (sj3-input-state-preedit sc)
+      (let ((cand (sj3-get-prediction-string sc)))
+        (list (cons (bitwise-ior preedit-reverse preedit-cursor) cand)))))
+
 (define (sj3-compose-state-preedit sc)
   (let* ((segments (sj3-context-segments sc))
 	 (cur-seg (ustr-cursor-pos segments))
@@ -1383,6 +1728,22 @@
     (sj3-reset-candidate-window sc)
     (sj3-flush sc))
 
+(define (sj3-get-prediction-string sc)
+  (sj3-lib-get-nth-prediction
+   sc
+   (sj3-context-prediction-index sc)))
+
+(define (sj3-learn-prediction-string sc)
+  (sj3-lib-commit-nth-prediction
+   sc
+   (sj3-context-prediction-index sc)))
+
+(define (sj3-do-commit-prediction sc)
+  (im-commit sc (sj3-get-prediction-string sc))
+  (sj3-learn-prediction-string sc)
+  (sj3-reset-prediction-window sc)
+  (sj3-flush sc))
+
 (define sj3-correct-segment-cursor
   (lambda (segments)
     (if (ustr-cursor-at-end? segments)
@@ -1601,7 +1962,9 @@
               (sj3-proc-transposing-state sc key key-state)
               (if (sj3-context-state sc)
                   (sj3-proc-compose-state sc key key-state)
-                  (sj3-proc-input-state sc key key-state)))
+                  (if (sj3-context-predicting sc)
+                      (sj3-proc-prediction-state sc key key-state)
+                      (sj3-proc-input-state sc key key-state))))
 	  (sj3-proc-raw-state sc key key-state)))
   (sj3-update-preedit sc))
 
@@ -1621,13 +1984,19 @@
 ;;;
 (define (sj3-get-candidate-handler sc idx ascel-enum-hint)
   (let* ((cur-seg (ustr-cursor-pos (sj3-context-segments sc)))
-	 (cand (sj3-lib-get-nth-candidate
-		sc cur-seg idx)))
+         (cand (if (sj3-context-state sc)
+                   (sj3-lib-get-nth-candidate sc cur-seg idx)
+                   (sj3-lib-get-nth-prediction sc idx))))
     (list cand (digit->string (+ idx 1)) "")))
 
 (define (sj3-set-candidate-index-handler sc idx)
-  (ustr-cursor-set-frontside! (sj3-context-segments sc) idx)
-  (sj3-update-preedit sc))
+  (cond
+   ((sj3-context-state sc)
+    (ustr-cursor-set-frontside! (sj3-context-segments sc) idx)
+    (sj3-update-preedit sc))
+   ((sj3-context-predicting sc)
+    (sj3-context-set-prediction-index! sc idx)
+    (sj3-update-preedit sc))))
 
 (define (sj3-proc-raw-state sc key key-state)
   (if (not (sj3-begin-input sc key key-state))
