@@ -1,6 +1,6 @@
 ;;; skk-custom.scm: Customization variables for skk.scm
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -205,17 +205,30 @@
 	 ""
 	 (N_ "Romaji")
 	 (N_ "Romaji input mode"))
+
    (list 'action_skk_azik
 	 'ja_azik
 	 ""
 	 (N_ "AZIK")
-	 (N_ "AZIK extended romaji input mode"))))
+	 (N_ "AZIK extended romaji input mode"))
 
+   (list 'action_skk_act
+	 'ja_act
+	 ""
+	 (N_ "ACT")
+	 (N_ "ACT extended romaji input mode"))
+
+   (list 'action_skk_kzik
+	 'ja_kzik
+	 ""
+	 (N_ "KZIK")
+	 (N_ "KZIK extended romaji input mode"))))
+    
 ;;; Buttons
 
 (define-custom 'skk-widgets '(widget_skk_input_mode
 			      widget_skk_kana_input_method)
-  '(skk toolbar)
+  '(skk toolbar-widget)
   (list 'ordered-list
 	(list 'widget_skk_input_mode
 	      (N_ "Input mode")
@@ -238,7 +251,7 @@
 ;;; Input mode
 
 (define-custom 'default-widget_skk_input_mode 'action_skk_latin
-  '(skk toolbar)
+  '(skk toolbar-widget)
   (cons 'choice
 	(map indication-alist-entry-extract-choice
 	     skk-input-mode-indication-alist))
@@ -247,7 +260,7 @@
 
 (define-custom 'skk-input-mode-actions
                (map car skk-input-mode-indication-alist)
-  '(skk toolbar)
+  '(skk toolbar-widget)
   (cons 'ordered-list
 	(map indication-alist-entry-extract-choice
 	     skk-input-mode-indication-alist))
@@ -278,7 +291,7 @@
 ;;; Kana input method
 
 (define-custom 'default-widget_skk_kana_input_method 'action_skk_roma
-  '(skk toolbar)
+  '(skk toolbar-widget)
   (cons 'choice
 	(map indication-alist-entry-extract-choice
 	     skk-kana-input-method-indication-alist))
@@ -287,7 +300,7 @@
 
 (define-custom 'skk-kana-input-method-actions
                (map car skk-kana-input-method-indication-alist)
-  '(skk toolbar)
+  '(skk toolbar-widget)
   (cons 'ordered-list
 	(map indication-alist-entry-extract-choice
 	     skk-kana-input-method-indication-alist))
@@ -454,6 +467,12 @@
   (N_ "Use UNIX look command for completion")
   (N_ "long description will be here."))
 
+(custom-add-hook 'skk-use-look?
+                 'custom-set-hooks
+                 (lambda ()
+                   (if skk-use-look?
+                     (skk-lib-look-open skk-look-dict))))
+
 (define-custom 'skk-look-dict "/usr/share/dict/words"
   '(skk-advanced)
   '(pathname regular-file)
