@@ -1,6 +1,6 @@
 ;;; uim-module-manager.scm: Part of uim-module-manager, it's not a part of libuim.
 ;;;
-;;; Copyright (c) 2005-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -29,6 +29,7 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
+(require "util.scm")
 (require "im.scm")
 (require "lazy-load.scm")
 
@@ -39,12 +40,14 @@
     (let ((orig-enabled-im-list enabled-im-list)
 	  (orig-require require))
       (set! enabled-im-list ())  ;; enable all IMs
+      (set! im-list ()) ;; reset im-list
       ;; XXX temporary solution to register all IM in a file
       (set! require
 	    (lambda (file)
-	      (let* ((file-sym (string->symbol file))
-		     (loaded-sym (symbolconc '* file-sym '-loaded*))
-		     (reloaded-sym (symbolconc '* file-sym '-reloaded*)))
+	      (let* ((loaded-sym (string->symbol
+				  (string-append "*" file "-loaded*")))
+		     (reloaded-sym (string->symbol
+				    (string-append "*" file "-reloaded*"))))
 		(cond
 		 ((symbol-bound? reloaded-sym)
 		  loaded-sym)
@@ -69,16 +72,16 @@
 	;; Test if the module is valid
 	(if (require-module (symbol->string module))
 	    #t
-	    (begin (puts (string-append "Warning: Module "
-					(symbol->string module)
-					" is not a correct module.\n"))
+	    (begin (display (string-append "Warning: Module "
+					   (symbol->string module)
+					   " is not a correct module.\n"))
 		   #f)))
       (remove
        (lambda (module)
 	 (if (memq module current-module-list)
-	     (begin (puts (string-append "Warning: Module "
-					 (symbol->string module)
-					 " is already registered\n"))
+	     (begin (display (string-append "Warning: Module "
+					    (symbol->string module)
+					    " is already registered\n"))
 		    #t)
 	     #f))
        modules))
@@ -96,7 +99,7 @@
 ;; This function is called with 'uim-module-manager --register'
 (define register-modules
   (lambda (module-names)
-    (let* ((modules (map string->symbol (string-split module-names " ")))
+    (let* ((modules (map string->symbol module-names))
 	   (current-module-list (map string->symbol installed-im-module-list))
 	   (revised-module-list (add-modules-to-module-list modules
 				 current-module-list)))
@@ -105,7 +108,7 @@
 ;; This function is called with 'uim-module-manager --unregister'
 (define unregister-modules
   (lambda (module-names)
-    (let* ((modules (map string->symbol (string-split module-names " ")))
+    (let* ((modules (map string->symbol module-names))
 	   (current-module-list (map string->symbol installed-im-module-list))
 	   (revised-module-list (remove-modules-from-module-list 
 				 modules
@@ -136,7 +139,7 @@
       "(define-record 'stub-im stub-im-rec-spec)\n\n"
       "(define stub-im-list\n"
       "  '(\n"
-      (string-join "\n" (stub-im-generate-all-stub-im-list))
+      (string-join (stub-im-generate-all-stub-im-list) "\n")
       "    ))\n\n"
       "(for-each (lambda (stub)\n"
       "            (if (memq (stub-im-name stub) enabled-im-list)\n"
@@ -166,3 +169,4 @@
 
 
 (prealloc-heaps-for-heavy-job)
+(verbose 1)
