@@ -1,6 +1,6 @@
 ;;; util.scm: Utility functions for uim.
 ;;;
-;;; Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -96,6 +96,21 @@
     (take (drop lst start)
 	  len)))
 
+;;; Merged from composer branch
+;;; TODO: Merge with trunk, write test
+;;;
+;;(define sublist
+;;  (lambda (lst start end)
+;;    (list-tail (list-head lst (+ end 1))
+;;	       start)))
+;;
+;;;; .parameter len Length to get. -1 means end of original lst
+;;(define sublist-rel
+;;  (lambda (lst start len)
+;;    (if (negative? len)
+;;	(list-tail lst start)
+;;	(sublist lst start (+ start len)))))
+
 (define alist-replace
   (lambda (kons alist)
     (let* ((id (car kons))
@@ -142,6 +157,36 @@
   (lambda args
     (string->symbol (string-append-map symbol->string args))))
 
+(define call-with-output-string
+  (lambda (proc)
+    (let ((out (open-output-string)))
+      (proc out)
+      (get-output-string out))))
+
+(define call-with-input-string
+  (lambda (str proc)
+    (let ((in (open-input-string str)))
+      (proc in))))
+
+(define call-with-string-io
+  (lambda (str proc)
+    (let ((out (open-output-string))
+          (in  (open-input-string str)))
+      (proc in out)
+      (get-output-string out))))
+
+(define write-to-string
+  (lambda (obj . args)
+    (call-with-output-string
+     (lambda (port) ((if (pair? args) (car args) write) obj port)))))
+
+(define read-from-string
+  (lambda (string)
+    (call-with-input-string
+     string
+     (lambda (port)
+       (read port)))))
+
 ;; only accepts single-arg functions
 ;; (define caddr (compose car cdr cdr))
 ;; FIXME: remove the closure overhead
@@ -176,6 +221,26 @@
     (max bottom
 	 (min x ceiling))))
 
+(define inc
+  (lambda (n)
+    (+ n 1)))
+
+(define dec
+  (lambda (n)
+    (- n 1)))
+
+(define *gensym-counter* 0)
+(define gensym
+  (lambda args
+    (set! *gensym-counter* (inc *gensym-counter*))
+    (let ((new (string->symbol (format "~a~X" (if (pair? args) (car args) "g") *gensym-counter*))))
+      (if (symbol-bound? new)
+          (gensym)
+          new))))
+
+;; TODO: write test
+(define number->symbol
+  (compose string->symbol number->string))
 
 ;;
 ;; uim-specific utilities
@@ -186,28 +251,41 @@
 (define make-scm-pathname
   (lambda (file)
     (if (string-prefix? "/" file)
-	file
-	(string-append (load-path) "/" file))))
+	(list file)
+        (let ((paths (string-split (load-path) ":")))
+          (map (lambda (x) (string-append x "/" file)) paths)))))
 
-;; TODO: write test
 ;; returns succeeded or not
 (define try-load
   (lambda (file)
     (guard (err
-	    (else #f))
-      ;; to suppress error message, check file existence first
-      (and (file-readable? (make-scm-pathname file))
-	   (load file)))))
+             (else #f))
+           (let ((paths (make-scm-pathname file)))
+             (let loop ((path (car paths))
+                        (rest (cdr paths)))
+               ;; to suppress error message, check file existence first
+               (if (file-readable? path)
+                 (begin
+                   (load path)
+                   #t)
+                 (if (not (null? rest))
+                   (loop (car rest) (cdr rest))
+                   #f)))))))
 
-;; TODO: write test
 ;; returns succeeded or not
 (define try-require
   (lambda (file)
     (guard (err
-	    (else #f))
-      ;; to suppress error message, check file existence first
-      (and (file-readable? (make-scm-pathname file))
-	   (require file)))))
+             (else #f))
+           (let ((paths (make-scm-pathname file)))
+             (let loop ((path (car paths))
+                        (rest (cdr paths)))
+               ;; to suppress error message, check file existence first
+               (if (file-readable? path)
+                 (require path)
+                 (if (not (null? rest))
+                   (loop (car rest) (cdr rest))
+                   #f)))))))
 
 ;; used for dynamic environment substitution of closure
 (define %%enclose-another-env
@@ -218,51 +296,6 @@
 	   (definition (list 'lambda args body)))
       (eval definition another-env))))
 
-;; See test/test-util.scm to know what define-record does.
-;; rec-spec requires list of list rather than alist to keep
-;; extensibility (e.g. (nth 2 spec) and so on may be used)
-(define define-record
-  (lambda (rec-sym rec-spec)
-    (for-each (lambda (spec index)
-		(let* ((elem-sym (list-ref spec 0))
-		       (default  (list-ref spec 1))
-		       (getter-sym (symbol-append rec-sym hyphen-sym elem-sym))
-		       (getter (lambda (rec)
-				 (list-ref rec index)))
-		       (setter-sym (symbol-append
-				    rec-sym hyphen-sym 'set- elem-sym '!))
-		       (setter (lambda (rec val)
-				 (set-car! (list-tail rec index)
-					   val))))
-		  (eval (list 'define getter-sym getter)
-			(interaction-environment))
-		  (eval (list 'define setter-sym setter)
-			(interaction-environment))))
-	      rec-spec
-	      (iota (length rec-spec)))
-    (let ((creator-sym (symbol-append rec-sym hyphen-sym 'new))
-	  (creator (let ((defaults (map cadr rec-spec)))
-		     (lambda init-lst
-		       (cond
-			((null? init-lst)
-			 (list-copy defaults))
-			;; fast path
-			((= (length init-lst)
-			    (length defaults))
-			 (list-copy init-lst))
-			;; others
-			((< (length init-lst)
-			    (length defaults))
-			 (let* ((rest-defaults (list-tail defaults
-							  (length init-lst)))
-				(complemented-init-lst (append init-lst
-							       rest-defaults)))
-			   (list-copy complemented-init-lst)))
-			(else
-			 #f))))))
-      (eval (list 'define creator-sym creator)
-	    (interaction-environment)))))
-
 ;; for direct candidate selection
 (define number->candidate-index
   (lambda (n)
@@ -275,6 +308,17 @@
      (else
       n))))
 
+;; TODO: write test
+;; Compensates an index number for an enumerable container
+;; .parameter idx Item index. Negative value instructs (abs idx)
+;; items before from last item
+;; .parameter size Size of the enumerable container
+(define compensate-index
+  (lambda (idx size)
+    (if (negative? idx)
+        (max 0 (+ size idx))
+        (clamp idx 0 (dec size)))))
+
 ;; update style-element vars
 ;; style-spec requires list of (style-element-name . validator)
 (define update-style
