@@ -1,4 +1,4 @@
-;;; Copyright (c) 2004-2008 uim Project http://code.google.com/p/uim/
+;;; Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -38,15 +38,13 @@
 ;; backtrace following an error.
 (define UIM-SH-MULTILINE-ERROR #t)
 
-(if (version<? *gaunit-version* "0.1.1")
-    (error "GaUnit 0.1.1 is required"))
+(if (version<? *gaunit-version* "0.1.6")
+    (error "GaUnit 0.1.6 is required"))
 
-(sys-putenv "LIBUIM_SYSTEM_SCM_FILES" (string-append (sys-realpath ".")
-						     "/sigscheme/lib"))
-(sys-putenv "LIBUIM_SCM_FILES" (string-append (sys-realpath ".") "/scm"))
+(sys-putenv "LIBUIM_SYSTEM_SCM_FILES" (uim-test-source-path "sigscheme" "lib"))
+(sys-putenv "LIBUIM_SCM_FILES" (string-append (uim-test-source-path "scm") ":" (uim-test-build-path "scm")))
 ;; FIXME: '.libs' is hardcoded
-(sys-putenv "LIBUIM_PLUGIN_LIB_DIR"
-	    (string-append (sys-realpath ".") "/uim/.libs"))
+(sys-putenv "LIBUIM_PLUGIN_LIB_DIR" (uim-test-build-path "uim" ".libs"))
 (sys-putenv "LIBUIM_VERBOSE" "2")  ;; must be 1 or 2 (2 enables backtrace)
 (sys-putenv "LIBUIM_VANILLA" "1")
 
@@ -124,15 +122,16 @@
       #t)))
 
 (eval
-   '(begin
+   `(begin
       (define (*uim-sh-setup-proc*)
-        (set! *uim-sh-process* (run-process "uim/uim-sh"
+        (set! *uim-sh-process* (run-process ,(uim-test-build-path "uim" "uim-sh")
                                             "-b"
                                             :input :pipe
                                             :output :pipe))
 	(uim '(%%set-current-error-port! (current-output-port))))
       (define (*uim-sh-teardown-proc*)
         (close-input-port (process-input *uim-sh-process*))
+        (process-wait *uim-sh-process*)
         (set! *uim-sh-process* #f))
 
       (define-syntax define-uim-test-case
