@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -30,40 +30,23 @@
   SUCH DAMAGE.
 */
 
-/*
- * Currently defined as 1 to be compatible with previous code. If no
- * distro packagers need this, please remove
- */
-#define UIM_EDITLINE_SEPARATED_BUILD 1
 
+#include <stdio.h>
 #include <histedit.h>
 
-#if UIM_EDITLINE_SEPARATED_BUILD
-#include <uim/uim.h>
-#include <uim/uim-scm.h>
-#include <uim/uim-compat-scm.h>
-#include <uim/plugin.h>
-#else
 #include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
-#include "plugin.h"
-#endif
+#include "dynlib.h"
 
-#include "editline.h"
 
 static EditLine *el;
 static History *hist;
 static HistEvent hev;
-static uim_lisp uim_editline_readline(void);
+static uim_lisp editline_readline(void);
 static char *prompt(EditLine *e);
 
-#if UIM_SCM_GCC4_READY_GC
-static void *uim_editline_readline_internal(void *dummy);
-#endif
-
 void
-editline_init(void)
+uim_plugin_instance_init(void)
 {
   el = el_init("uim", stdin, stdout, stderr);
   el_set(el, EL_PROMPT, &prompt);
@@ -74,63 +57,41 @@ editline_init(void)
   el_set(el, EL_HIST, history, hist);
   el_source(el, NULL);
 
-  uim_scm_init_subr_0("uim-editline-readline", uim_editline_readline);
+  uim_scm_init_proc0("editline-readline", editline_readline);
+
+  uim_scm_callf("provide", "s", "editline");
 }
 
 void
-editline_quit(void)
+uim_plugin_instance_quit(void)
 {
-    history_end(hist);
-    el_end(el);
+  history_end(hist);
+  el_end(el);
 }
 
 static uim_lisp
-uim_editline_readline(void)
-#if UIM_SCM_GCC4_READY_GC
+editline_readline(void)
 {
-  return (uim_lisp)uim_scm_call_with_gc_ready_stack(uim_editline_readline_internal, NULL);
+  const char *line;
+  int count = 0;
+
+  line = el_gets(el, &count);
+
+  if (count > 0 && line) {
+    history(hist, &hev, H_ENTER, line);
+    return uim_scm_make_str(line);
+  } else {
+    return uim_scm_eof();
+  }
 }
 
-static void *
-uim_editline_readline_internal(void *dummy)
-#endif
+static char *
+prompt(EditLine *e)
 {
-    const char *line;
-    int count = 0;
-#if !UIM_SCM_GCC4_READY_GC
-    uim_lisp stack_start;
-#endif
-    uim_lisp ret;
+  uim_lisp p;
 
-#if !UIM_SCM_GCC4_READY_GC
-    uim_scm_gc_protect_stack(&stack_start);
-#endif
+  p = uim_scm_callf("editline-prompt", "");
 
-    line = el_gets(el, &count);
-
-    if(count > 0 && line) {
-	history(hist, &hev, H_ENTER, line);
-	ret = uim_scm_make_str(line);
-    } else {
-	ret = uim_scm_make_str("");
-    }
-
-#if UIM_SCM_GCC4_READY_GC
-    return (void *)ret;
-#else
-    uim_scm_gc_unprotect_stack(&stack_start);
-
-    return ret;
-#endif
-}
-
-static char *prompt(EditLine *e) {
-    char *p;
-
-    p = uim_scm_symbol_value_str("uim-sh-prompt"); /* XXX */
-
-    if(!p)
-	return "uim-sh> ";
-    else
-	return p;
+  /* libedit does not free the prompt str */
+  return (char *)uim_scm_refer_c_str(p);
 }
