@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -37,6 +37,7 @@
 #include <string.h>
 #include <iconv.h>
 #include <assert.h>
+#include <errno.h>
 
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
@@ -45,7 +46,9 @@
 #include "uim.h"
 #include "uim-internal.h"
 #include "uim-util.h"
-
+#include "uim-scm.h"
+#include "uim-scm-abbrev.h"
+#include "uim-iconv.h"
 
 #define MBCHAR_LEN_MAX 6  /* assumes CESU-8 */
 
@@ -246,45 +249,105 @@ uim_iconv_create(const char *tocode, const char *fromcode)
 }
 
 static char *
-uim_iconv_code_conv(void *obj, const char *str)
+uim_iconv_code_conv(void *obj, const char *instr)
 {
-  iconv_t ic;
-  size_t len, bufsize;
-  char *outbuf, *realbuf, *copied;
-  const char *inbuf, *src;
+  iconv_t cd = (iconv_t)obj;
+  size_t ins;
+  const char *in;
+  size_t outbufsiz, outs;
+  char   *outbuf = NULL, *out;
+  size_t ret = 0;
+  size_t nconv = 0;
+  size_t idx = 0;
+  char *str = NULL;
 
   if (UIM_CATCH_ERROR_BEGIN())
     return NULL;
 
-  do {
-    if (!str) {
-      copied = NULL;
-      break;
-    }
+  if (!obj)
+    return uim_strdup(instr);
 
-    ic = (iconv_t)obj;
-    if (ic) {
-      len = strlen(str);
-      bufsize = (len + sizeof("")) * MBCHAR_LEN_MAX;
-      realbuf = alloca(bufsize);
-      bufsize--;
+  if (!instr)
+    goto err;
 
-      inbuf = str;
-      outbuf = realbuf;
-      iconv(ic, (ICONV_CONST char **)&inbuf, &len, &outbuf, &bufsize);
-      iconv(ic, NULL, NULL, &outbuf, &bufsize);
-      *outbuf = '\0';
-      src = realbuf;
+  ins = strlen(instr);
+  in = instr;
+
+  outbufsiz = (ins + sizeof("")) * MBCHAR_LEN_MAX;
+  out = outbuf = uim_malloc(outbufsiz);
+
+  while (ins > 0) {
+    out = outbuf;
+    outs = outbufsiz;
+
+    ret = iconv(cd, (ICONV_CONST char **)&in, &ins, &out, &outs);
+    nconv = outbufsiz - outs;
+    if (ret == (size_t)-1) {
+      switch (errno) {
+      case EINVAL:
+	goto err;
+      case E2BIG:
+	outbufsiz *= 2;
+	out = uim_realloc(outbuf, outbufsiz);
+	outbuf = out;
+	break;
+      default:
+	goto err;
+      }
     } else {
-      src = str;
+      /* XXX: irreversible characters */
     }
+    if (nconv > 0) {
+      if (str == NULL)
+	str = uim_malloc(nconv + 1);
+      else
+	str = uim_realloc(str, idx + nconv + 1);
+      memcpy(&str[idx], outbuf, nconv);
+      idx += nconv;
+    }
+  }
+  do {
+    out = outbuf;
+    outs = outbufsiz;
 
-    copied = uim_strdup(src);
-  } while (/* CONSTCOND */ 0);
+    ret = iconv(cd, NULL, NULL, &out, &outs);
+    nconv = outbufsiz - outs;
+
+    if (ret == (size_t)-1) {
+      outbufsiz *= 2;
+      out = uim_realloc(outbuf, outbufsiz);
+      outbuf = out;
+    } else {
+      /* XXX: irreversible characters */
+    }
+    if (nconv > 0) {
+      if (str == NULL)
+	str = uim_malloc(nconv + 1);
+      else
+	str = uim_realloc(str, idx + nconv + 1);
+      memcpy(&str[idx], outbuf, nconv);
+      idx += nconv;
+    }
+  } while (ret == (size_t)-1);
+
+  if (str == NULL)
+    str = uim_strdup("");
+  else
+    str[idx] = '\0';
+  free(outbuf);
 
   UIM_CATCH_ERROR_END();
 
-  return copied;
+  return str;
+
+ err:
+
+  free(str);
+  free(outbuf);
+
+  UIM_CATCH_ERROR_END();
+
+  return uim_strdup("");
 }
 
 static void
@@ -300,3 +363,44 @@ uim_iconv_release(void *obj)
 
   UIM_CATCH_ERROR_END();
 }
+
+static uim_lisp
+uim_ext_iconv_open(uim_lisp tocode_, uim_lisp fromcode_)
+{
+  const char *tocode = REFER_C_STR(tocode_);
+  const char *fromcode = REFER_C_STR(fromcode_);
+  iconv_t ic;
+
+  ic = uim_iconv_create(tocode, fromcode);
+  if (!ic)
+    return uim_scm_f();
+
+  return MAKE_PTR(ic);
+}
+
+static uim_lisp
+uim_ext_iconv_code_conv(uim_lisp ic_, uim_lisp inbuf_)
+{
+  char *outbuf;
+
+  outbuf = uim_iconv_code_conv(C_PTR(ic_), REFER_C_STR(inbuf_));
+  if (!outbuf)
+    return uim_scm_f();
+
+  return MAKE_STR_DIRECTLY(outbuf);
+}
+
+static uim_lisp
+uim_ext_iconv_release(uim_lisp ic_)
+{
+  uim_iconv_release(C_PTR(ic_));
+  return uim_scm_t();
+}
+
+void
+uim_init_iconv_subrs(void)
+{
+  uim_scm_init_proc2("iconv-open", uim_ext_iconv_open);
+  uim_scm_init_proc2("iconv-code-conv", uim_ext_iconv_code_conv);
+  uim_scm_init_proc1("iconv-release", uim_ext_iconv_release);
+}
