@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -43,7 +43,7 @@
 #include "uim-scm.h"
 #include "uim-scm-abbrev.h"
 #include "uim-util.h"
-#include "plugin.h"
+#include "dynlib.h"
 
 static int m17nlib_ok;
 static MConverter *converter;
@@ -279,7 +279,7 @@ preedit_changedp(uim_lisp id_)
 static uim_lisp
 get_left_of_cursor(uim_lisp id_)
 {
-  int id, buflen, i;
+  int id, i;
   uim_lisp buf_;
   char *buf, *p;
   MInputContext *ic;
@@ -300,9 +300,7 @@ get_left_of_cursor(uim_lisp id_)
     p = m17nlib_utf8_find_next_char(p);
   *p = '\0';
 
-  buflen = strlen(buf);
-  buf_ = MAKE_STR(buf);
-  free(buf);
+  buf_ = MAKE_STR_DIRECTLY(buf);
 
   return buf_;
 }
@@ -310,7 +308,7 @@ get_left_of_cursor(uim_lisp id_)
 static uim_lisp
 get_right_of_cursor(uim_lisp id_)
 {
-  int id, buflen, i;
+  int id, i;
   uim_lisp buf_;
   char *buf, *p;
   MInputContext *ic;
@@ -327,9 +325,7 @@ get_right_of_cursor(uim_lisp id_)
   for (i = 0; i < ic->cursor_pos ;i++)
     p = m17nlib_utf8_find_next_char(p);
 
-  buflen = strlen(p);
-  buf_ = MAKE_STR(p);
-  free(buf);
+  buf_ = MAKE_STR_DIRECTLY(p);
 
   return buf_;
 }
@@ -337,7 +333,7 @@ get_right_of_cursor(uim_lisp id_)
 static uim_lisp
 get_left_of_candidate(uim_lisp id_)
 {
-  int id, buflen, i;
+  int id, i;
   uim_lisp buf_;
   char *buf, *p;
   MInputContext *ic;
@@ -358,9 +354,7 @@ get_left_of_candidate(uim_lisp id_)
     p = m17nlib_utf8_find_next_char(p);
   *p = '\0';
 
-  buflen = strlen(buf);
-  buf_ = MAKE_STR(buf);
-  free(buf);
+  buf_ = MAKE_STR_DIRECTLY(buf);
 
   return buf_;
 }
@@ -368,7 +362,7 @@ get_left_of_candidate(uim_lisp id_)
 static uim_lisp
 get_selected_candidate(uim_lisp id_)
 {
-  int id, buflen, i;
+  int id, i;
   uim_lisp buf_;
   char *buf, *p, *start;
   MInputContext *ic;
@@ -393,9 +387,7 @@ get_selected_candidate(uim_lisp id_)
     p = m17nlib_utf8_find_next_char(p);
   *p = '\0';
 
-  buflen = strlen(start);
-  buf_ = MAKE_STR(start);
-  free(buf);
+  buf_ = MAKE_STR_DIRECTLY(start);
 
   return buf_;
 }
@@ -403,7 +395,7 @@ get_selected_candidate(uim_lisp id_)
 static uim_lisp
 get_right_of_candidate(uim_lisp id_)
 {
-  int id, buflen, i;
+  int id, i;
   uim_lisp buf_;
   char *buf, *p;
   MInputContext *ic;
@@ -420,9 +412,7 @@ get_right_of_candidate(uim_lisp id_)
   for (i = 0; i < ic->candidate_to ;i++)
     p = m17nlib_utf8_find_next_char(p);
 
-  buflen = strlen(p);
-  buf_ = MAKE_STR(p);
-  free(buf);
+  buf_ = MAKE_STR_DIRECTLY(p);
 
   return buf_;
 }
@@ -436,19 +426,16 @@ get_nr_input_methods()
 static uim_lisp
 get_input_method_name(uim_lisp nth_)
 {
-  int len, nth;
-  char *name;
+  int nth;
+  char name[BUFSIZ];
   
   nth = C_INT(nth_);
 
   if (nth < nr_input_methods) {
-    len = strlen(im_array[nth].lang) + strlen(im_array[nth].name) + 7;
-    name = alloca(len);
-
     if (!strcmp(im_array[nth].lang, "t"))
-      snprintf(name, len, "m17n-%s", im_array[nth].name);
+      snprintf(name, sizeof(name), "m17n-%s", im_array[nth].name);
     else
-      snprintf(name, len, "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
+      snprintf(name, sizeof(name), "m17n-%s-%s", im_array[nth].lang, im_array[nth].name);
 
     return MAKE_STR(name);
   }
@@ -566,9 +553,9 @@ find_im_by_name(const char *name)
     char buf[100];
 
     if (!strcmp(im_array[i].lang, "t"))
-      snprintf(buf, 100, "%s", im_array[i].name);
+      strlcpy(buf, im_array[i].name, sizeof(buf));
     else
-      snprintf(buf, 100, "%s-%s", im_array[i].lang, im_array[i].name);
+      snprintf(buf, sizeof(buf), "%s-%s", im_array[i].lang, im_array[i].name);
 
     if (!strcmp(im_name, buf))
       return im_instance(i);
