@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -42,7 +42,7 @@
 */
 
 
-#include "plugin.h"
+#include "dynlib.h"
 #include "uim-custom.h"
 
 uim_bool uim_custom_init(void);
