@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -98,8 +98,7 @@
     char *buf; \
     form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1); \
     if (form_size != -1) { \
-      buf = malloc(form_size); \
-      snprintf(buf, form_size, sexp_tmpl, arg1); \
+      uim_asprintf(&buf, sexp_tmpl, arg1); \
       UIM_EVAL_STRING_INTERNAL(uc, buf); \
       free(buf); \
     } \
@@ -111,8 +110,7 @@
     char *buf; \
     form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2); \
     if (form_size != -1) { \
-      buf = malloc(form_size); \
-      snprintf(buf, form_size, sexp_tmpl, arg1, arg2); \
+      uim_asprintf(&buf, sexp_tmpl, arg1, arg2); \
       UIM_EVAL_STRING_INTERNAL(uc, buf); \
       free(buf); \
     } \
@@ -124,8 +122,7 @@
     char *buf; \
     form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3); \
     if (form_size != -1) { \
-      buf = malloc(form_size); \
-      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3); \
+      uim_asprintf(&buf, sexp_tmpl, arg1, arg2, arg3); \
       UIM_EVAL_STRING_INTERNAL(uc, buf); \
       free(buf); \
     } \
@@ -137,8 +134,7 @@
     char *buf; \
     form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3, arg4); \
     if (form_size != -1) { \
-      buf = malloc(form_size); \
-      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3, arg4); \
+      uim_asprintf(&buf, sexp_tmpl, arg1, arg2, arg3, arg4); \
       UIM_EVAL_STRING_INTERNAL(uc, buf); \
       free(buf); \
     } \
@@ -150,8 +146,7 @@
     char *buf; \
     form_size = uim_sizeof_sexp_str(sexp_tmpl, arg1, arg2, arg3, arg4, arg5); \
     if (form_size != -1) { \
-      buf = malloc(form_size); \
-      snprintf(buf, form_size, sexp_tmpl, arg1, arg2, arg3, arg4, arg5); \
+      uim_asprintf(&buf, sexp_tmpl, arg1, arg2, arg3, arg4, arg5); \
       UIM_EVAL_STRING_INTERNAL(uc, buf); \
       free(buf); \
     } \
@@ -242,6 +237,7 @@ static uim_bool uim_conf_prepare_dir(const char *subdir);
 static uim_bool for_each_primary_groups(uim_bool (*func)(const char *));
 static uim_bool uim_custom_load_group(const char *group);
 static uim_bool uim_custom_save_group(const char *group);
+static const char *uim_custom_get_primary_group_by_custom(const char *custom_sym);
 
 static const char str_list_arg[] = "uim-custom-c-str-list-arg";
 static const char custom_subdir[] = "customs";
@@ -401,18 +397,18 @@ c_list_to_str(const void *const *list, char *(*mapper)(const void *elem), const
     free(str);
   }
   buf = (char *)malloc(buf_size);
+  buf[0] = '\0';
 
   for (bufp = buf, elem = list; *elem; elem++) {
     if (elem != list) {
-      strcpy(bufp, sep);
+      strlcat(buf, sep, buf_size);
       bufp += strlen(sep);
     }
     str = (*mapper)(*elem);
-    strcpy(bufp, str);
+    strlcat(buf, str, buf_size);
     bufp += strlen(str);
     free(str);
   }
-  buf[buf_size - 1] = '\0';
 
   return buf;
 }
@@ -967,7 +963,7 @@ helper_disconnect_cb(void)
 uim_bool
 uim_custom_enable(void)
 {
-  UIM_EVAL_STRING(NULL, "(load-plugin \"custom-enabler\")");
+  UIM_EVAL_STRING(NULL, "(require-dynlib \"custom-enabler\")");
   return uim_scm_c_bool(uim_scm_return_value());
 }
 
@@ -1269,13 +1265,36 @@ uim_custom_save_group(const char *group)
 uim_bool
 uim_custom_save(void)
 {
-  if(uim_helper_is_setugid() ==UIM_FALSE) {
+  if(uim_helper_is_setugid() == UIM_FALSE) {
     return for_each_primary_groups(uim_custom_save_group);
   } else {
     return UIM_FALSE;
   }
 }
 
+/**
+ * Saves a per-user custom variable configuration. This function saves a
+ * primary custom group values which contains the specified custom variable as
+ * ~/.uim.d/customs/custom-primary-group-file-containing-the-variable.scm. The
+ * directory will be made if not exist. The saved values will be implicitly
+ * loaded at uim_init() or can explicitly be loaded by uim_custom_load().
+ *
+ * @see uim_init()
+ * @see uim_custom_load()
+ * @retval UIM_TRUE succeeded
+ * @retval UIM_FALSE failed
+ */
+uim_bool
+uim_custom_save_custom(const char *custom_sym)
+{
+  if(uim_helper_is_setugid() == UIM_FALSE) {
+    const char *group_sym = uim_custom_get_primary_group_by_custom(custom_sym);
+    return uim_custom_save_group(group_sym);
+  } else {
+    return UIM_FALSE;
+  }
+}
+
 /**
  * Broadcasts custom variable configurations to other uim-enabled application
  * processes via uim-helper-server. This function broadcasts current custom
@@ -1291,7 +1310,6 @@ uim_custom_broadcast(void)
 {
   char **custom_syms, **sym;
   char *value, *msg;
-  size_t msg_size;
 
   if (helper_fd < 0) {
     helper_fd = uim_helper_init_client_fd(helper_disconnect_cb);
@@ -1301,14 +1319,17 @@ uim_custom_broadcast(void)
   for (sym = custom_syms; *sym; sym++) {
     value = uim_custom_value_as_literal(*sym);
     if (value) {
-      msg_size = sizeof(custom_msg_tmpl) + strlen(*sym) + strlen(value);
-      msg = (char *)malloc(msg_size);
-      if (!msg) {
+#if 1
+      uim_asprintf(&msg, custom_msg_tmpl, *sym, value);
+#else
+      /* old behavior: useless since other memory exhaustions disable uim */
+      if (asprintf(&msg, custom_msg_tmpl, *sym, value) < 0 || !msg) {
+	free(msg);
 	free(value);
 	uim_custom_symbol_list_free(custom_syms);
 	return UIM_FALSE;
       }
-      snprintf(msg, msg_size, custom_msg_tmpl, *sym, value);
+#endif
       uim_helper_send_message(helper_fd, msg);
       free(msg);
       free(value);
@@ -1643,6 +1664,15 @@ uim_custom_symbol_list_free(char **symbol_list)
   uim_scm_c_list_free((void **)symbol_list, (uim_scm_c_list_free_func)free);
 }
 
+static const char *
+uim_custom_get_primary_group_by_custom(const char *custom_sym)
+{
+  uim_lisp groups;
+  groups = uim_scm_callf("custom-groups", "y", custom_sym);
+
+  return uim_scm_refer_c_str(uim_scm_car(groups));
+}
+
 static uim_lisp
 uim_custom_cb_update_cb_gate(uim_lisp cb, uim_lisp ptr, uim_lisp custom_sym)
 {
