@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2007-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2007-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -36,6 +36,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <assert.h>
 #if HAVE_SYSEXITS_H
 #include "sysexits.h"
@@ -43,7 +44,7 @@
 
 #include "uim.h"
 #include "uim-internal.h"
-#if UIM_USE_NOTIFY && !UIM_NON_LIBUIM_PROG
+#if UIM_USE_NOTIFY_PLUGINS && !UIM_NON_LIBUIM_PROG
 #include "uim-notify.h"
 #endif
 #include "gettext.h"
@@ -94,7 +95,7 @@ print_caught_error(void)
     }
 
     /* And notify user of it via uim-notify in addition to the stderr msg. */
-#if UIM_USE_NOTIFY && !UIM_NON_LIBUIM_PROG
+#if UIM_USE_NOTIFY_PLUGINS && !UIM_NON_LIBUIM_PROG
     /* Since this function will also be called on hard situations such
      * as memory exhaustion, these uim_notify_*() calls may be failed
      * to notify user of the error, due to the memory shortage.
@@ -107,7 +108,7 @@ print_caught_error(void)
     } else {
       uim_notify_info(err_msg);
     }
-#endif  /* UIM_USE_NOTIFY */
+#endif  /* UIM_USE_NOTIFY_PLUGINS */
   }
 }
 
@@ -215,17 +216,25 @@ uim_strdup(const char *s)
 {
   char *copied;
 
-#if HAVE_STRDUP
   copied = strdup(s);
   if (!copied)
     uim_fatal_error("strdup() failed");
-#else
-  size_t size;
-
-  size = strlen(s) + sizeof("");
-  copied = uim_malloc(size);
-  strcpy(copied, s);
-#endif
 
   return copied;
 }
+
+int
+uim_asprintf(char **ret, const char *fmt, ...)
+{
+  va_list ap;
+  int i;
+
+  va_start(ap, fmt);
+  i = vasprintf(ret, fmt, ap);
+  va_end(ap);
+
+  if (i < 0 || *ret == NULL)
+    uim_fatal_error("asprintf() failed");
+
+  return i;
+}
