@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -105,7 +105,7 @@ void
 uim_helper_send_message(int fd, const char *message)
 {
   int res;
-  int len, out_len;
+  int out_len;
   sig_t old_sigpipe;
   char *buf, *bufp;
 
@@ -123,13 +123,11 @@ uim_helper_send_message(int fd, const char *message)
     return;
 #endif
 
-  len = strlen(message) + 1;
-  buf = uim_malloc(len + 1);
-  snprintf(buf, len + 1, "%s\n", message);
+  uim_asprintf(&buf, "%s\n", message);
 
   old_sigpipe = signal(SIGPIPE, SIG_IGN);
 
-  out_len = len;
+  out_len = strlen(buf);
   bufp = buf;
   while (out_len > 0) {
     if ((res = write(fd, bufp, out_len)) < 0) {
@@ -209,7 +207,11 @@ uim_helper_get_pathname(char *helper_path, int len)
   return UIM_TRUE;
 
  path_error:
-  uim_fatal_error("uim_helper_get_pathname()");
+#if USE_UIM_NOTIFY && !UIM_NON_LIBUIM_PROG
+  uim_notify_fatal("uim_helper_get_pathname() failed");
+#else
+  fprintf(stderr, "uim_helper_get_pathname() failed\n");
+#endif
   helper_path[0] = '\0';
 
   UIM_CATCH_ERROR_END();
@@ -269,12 +271,12 @@ uim_helper_buffer_append(char *buf, const char *fragment, size_t fragment_size)
 {
   size_t buf_size, extended_size;
 
-  buf_size = strlen(buf);
-  extended_size = buf_size + fragment_size + 1;
-  buf = realloc(buf, extended_size);
   if (buf) {
-    memcpy(&buf[buf_size], fragment, fragment_size);
-    buf[extended_size - 1] = '\0';
+	  buf_size = strlen(buf);
+	  extended_size = buf_size + fragment_size + 1;
+	  buf = uim_realloc(buf, extended_size);
+	  memcpy(&buf[buf_size], fragment, fragment_size);
+	  buf[extended_size - 1] = '\0';
   }
 
   return buf;
@@ -297,6 +299,9 @@ uim_helper_buffer_get_message(char *buf)
   if (UIM_CATCH_ERROR_BEGIN())
     return NULL;
 
+  if (!buf)
+    return NULL;
+
   msg_term = strstr(buf, "\n\n");
   if (msg_term) {
     msg_size = msg_term + 2 - buf;
