@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,14 +34,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 #include "uim.h"
 #include "uim-scm.h"
-#include "uim-compat-scm.h"
+#include "uim-scm-abbrev.h"
 #include "uim-internal.h"
 
 
-#define KEYBUF_SIZE (sizeof("'hangul-multiple-candidate     "))
+/* Future version of uim should have uim_filter_key() that returns 'filtered'
+ * as true value. Current semantics is not an ordinary design for IM and felt
+ * unnatural.  -- YamaKen 2007-01-10 */
+enum key_filtering_result {
+  FILTERED = 0,
+  PASSTHROUGH = 1
+};
 
 #define ISASCII(c) (0 <= (c) && (c) <= 127)
 
@@ -270,21 +277,30 @@ static struct key_entry key_tab[] = {
   {0, 0}
 };
 
+static uim_lisp protected;
+
+static void define_valid_key_symbols(void);
+static const char *get_sym(int key);
+static uim_bool filter_key(uim_context uc,
+                           int key, int state, uim_bool is_press);
+static int emergency_key_p(int key, int state);
+
 #if 0
 int uim_key_sym_to_int(uim_lisp sym);
 
 int
 uim_key_sym_to_int(uim_lisp sym_)
 {
-  char *sym = uim_scm_refer_c_str(sym_);
-  int res = 0;
+  char *sym;
   int i;
+
+  sym = REFER_C_STR(sym_);
   for (i = 0; key_tab[i].key; i++) {
-    if (!strcmp(key_tab[i].str, sym)) {
-      res = key_tab[i].key;
-    }
+    if (!strcmp(key_tab[i].str, sym))
+      return key_tab[i].key;
   }
-  return res;
+
+  return 0;
 }
 #endif
 
@@ -292,57 +308,62 @@ static void
 define_valid_key_symbols(void)
 {
   int i;
+  uim_lisp valid_key_symbols;
 
-  UIM_EVAL_STRING(NULL, "(define valid-key-symbols ())");
+  valid_key_symbols = uim_scm_null();
   for (i = 0; key_tab[i].key; i++) {
-    UIM_EVAL_FSTRING1(NULL,
-		      "(set! valid-key-symbols (cons '%s valid-key-symbols))",
-		      key_tab[i].str);
+    valid_key_symbols = CONS(MAKE_SYM(key_tab[i].str), valid_key_symbols);
   }
+  uim_scm_eval(LIST3(MAKE_SYM("define"),
+		     MAKE_SYM("valid-key-symbols"),
+		     QUOTE(valid_key_symbols)));
 }
 
 static const char *
 get_sym(int key)
 {
   int i;
-  const char *res = NULL;
+
   for (i = 0; key_tab[i].key; i++) {
-    if (key_tab[i].key == key) {
-      res = key_tab[i].str;
-      break;
-    }
+    if (key_tab[i].key == key)
+      return key_tab[i].str;
   }
-  return res;
+
+  return NULL;
 }
 
-static int
-keycode_to_sym(int key, char *buf)
+/* FIXME: Replace 'protected' variable with stack protection */
+static uim_bool
+filter_key(uim_context uc, int key, int state, uim_bool is_press)
 {
-  const char *s;
+  uim_lisp key_, filtered;
+  const char *sym, *handler;
+
+  if (!uc)
+    return UIM_FALSE;
+
+  if (is_press
+      && emergency_key_p(key, state)
+      && getenv("LIBUIM_ENABLE_EMERGENCY_KEY"))
+  {
+    uc->is_enabled = !uc->is_enabled;
+    return UIM_TRUE;
+  }
+  if (!uc->is_enabled)
+    return UIM_FALSE;
 
   if (ISASCII(key)) {
-    snprintf(buf, KEYBUF_SIZE, "%d", key);
+    protected = key_ = MAKE_INT(key);
   } else {
-    s = get_sym(key);
-    if (!s)
-      return -1;
-    snprintf(buf, KEYBUF_SIZE, "'%s", s);
+    sym = get_sym(key);
+    if (!sym)
+      return UIM_FALSE;
+    protected = key_ = MAKE_SYM(get_sym(key));
   }
-  return 0;
-}
 
-static void
-handle_key(uim_context uc, const char *p, int key, int state)
-{
-  char keybuf[KEYBUF_SIZE];
-  int rv;
-
-  rv = keycode_to_sym(key, keybuf);
-  if (rv == -1) {
-    uc->commit_raw_flag = 1;
-    return;
-  }
-  UIM_EVAL_FSTRING4(uc, "(key-%s-handler %d %s %d)", p, uc->id, keybuf, state);
+  handler = (is_press) ? "key-press-handler" : "key-release-handler";
+  filtered = uim_scm_callf(handler, "poi", uc, key_, state);
+  return C_BOOL(filtered);
 }
 
 static int
@@ -358,52 +379,48 @@ emergency_key_p(int key, int state)
 int
 uim_press_key(uim_context uc, int key, int state)
 {
-  if (!uc) {
-    return 1;
-  }
-  if (getenv("LIBUIM_ENABLE_EMERGENCY_KEY") && emergency_key_p(key, state)) {
-    uc->is_enable = uc->is_enable ? 0 : 1;
-    return 0;
-  }
-  if (!uc->is_enable) {
-    return 1;
-  }
-  uc->commit_raw_flag = 0;
-  handle_key(uc, "press", key, state);
-  return uc->commit_raw_flag;
+  uim_bool filtered;
+
+  if (UIM_CATCH_ERROR_BEGIN())
+    return PASSTHROUGH;
+
+  assert(uim_scm_gc_any_contextp());
+  assert(uc);
+  assert(key >= 0);
+  assert(state >= 0);
+
+  filtered = filter_key(uc, key, state, UIM_TRUE);
+
+  UIM_CATCH_ERROR_END();
+
+  return (filtered) ? FILTERED : PASSTHROUGH;
 }
 
 int
 uim_release_key(uim_context uc, int key, int state)
 {
-  if (!uc) {
-    return 1;
-  }
-  if (!uc->is_enable) {
-    return 1;
-  }
-  uc->commit_raw_flag = 0;
-  handle_key(uc, "release", key, state);
-  return uc->commit_raw_flag;
-}
+  uim_bool filtered;
 
-static uim_lisp
-define_key(uim_lisp args, uim_lisp env)
-{
-  uim_lisp form, predicate_sym, sources;
+  if (UIM_CATCH_ERROR_BEGIN())
+    return PASSTHROUGH;
 
-  predicate_sym = uim_scm_car(args);
-  sources = uim_scm_nullp(args) ? uim_scm_null_list() : uim_scm_cadr(args);
-  form = uim_scm_list3(uim_scm_make_symbol("define-key-internal"),
-		       uim_scm_quote(predicate_sym),
-		       sources);
+  assert(uim_scm_gc_any_contextp());
+  assert(uc);
+  assert(key >= 0);
+  assert(state >= 0);
 
-  return uim_scm_eval(form);
+  filtered = filter_key(uc, key, state, UIM_FALSE);
+
+  UIM_CATCH_ERROR_END();
+
+  return (filtered) ? FILTERED : PASSTHROUGH;
 }
 
 void
 uim_init_key_subrs(void)
 {
+  protected = uim_scm_f();
+  uim_scm_gc_protect(&protected);
+
   define_valid_key_symbols();
-  uim_scm_init_fsubr("define-key", define_key);
 }
