@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -50,9 +50,9 @@
 #include "uim-scm-abbrev.h"
 #include "uim-internal.h"
 #include "uim-notify.h"
+#include "gettext.h"
 
-
-#define NOTIFY_PLUGIN_PATH LIBDIR "/uim/notify"
+#define NOTIFY_PLUGIN_PATH PKGLIBDIR "/notify"
 #define NOTIFY_PLUGIN_PREFIX "libuimnotify-"
 #define NOTIFY_PLUGIN_SUFFIX ".so"
 
@@ -109,7 +109,7 @@ uim_notify_load(const char *name)
     return UIM_FALSE;
   }
 
-  if (strcmp(agent->desc()->name, name) == 0) {
+  if (getenv("UIM_DISABLE_NOTIFY") != NULL || strcmp(agent->desc()->name, name) == 0) {
     return UIM_TRUE;
   } else if (strcmp(name, "stderr") == 0) {
     agent->quit();
@@ -219,7 +219,7 @@ uim_notify_fatal_raw(const char *msg)
  * Scheme interfaces
  */
 static uim_lisp
-notify_get_plugins(void)
+notify_get_plugins_internal(void)
 {
   uim_lisp ret_;
   DIR *dirp;
@@ -238,6 +238,10 @@ notify_get_plugins(void)
 		    MAKE_STR(desc->name),
 		    MAKE_STR(desc->desc)),
 	      uim_scm_null());
+
+  if (getenv("UIM_DISABLE_NOTIFY") != NULL)
+    return uim_scm_callf("reverse", "o", ret_);
+
   dirp = opendir(NOTIFY_PLUGIN_PATH);
   if (dirp) {
     while ((dp = readdir(dirp)) != NULL) {
@@ -276,6 +280,13 @@ notify_get_plugins(void)
   return uim_scm_callf("reverse", "o", ret_);
 }
 
+static uim_lisp
+notify_get_plugins(void)
+{
+  return (uim_lisp)uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)notify_get_plugins_internal,
+						    NULL);
+}
+
 static uim_lisp
 notify_load(uim_lisp name_)
 {
@@ -356,7 +367,7 @@ static uim_bool
 uim_notify_stderr_info(const char *msg)
 {
   fputs("libuim: [info] ", stderr);
-  fputs(msg, stderr);
+  fputs(dgettext(GETTEXT_PACKAGE, msg), stderr);
   fputs("\n", stderr);
 
   return UIM_TRUE;
@@ -369,7 +380,7 @@ uim_notify_stderr_fatal(const char *msg)
    * exhaustion, printf()s with indirect directives are intentionally
    * avoided here.  -- YamaKen 2008-02-11 */
   fputs("libuim: [fatal] ", stderr);
-  fputs(msg, stderr);
+  fputs(dgettext(GETTEXT_PACKAGE, msg), stderr);
   fputs("\n", stderr);
 
   return UIM_TRUE;
