@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -39,8 +39,8 @@
  */
 /* This file must be included before uim's config.h */
 #include "sigscheme-combined.c"
-#if !SSCM_VERSION_REQUIRE(0, 8, 0)
-#error "SigScheme version 0.8.0 or later is required"
+#if !SSCM_VERSION_REQUIRE(0, 8, 5)
+#error "SigScheme version 0.8.5 or later is required"
 #endif
 
 #include <config.h>
@@ -87,7 +87,7 @@ static void *uim_scm_callf_internal(struct callf_args *args);
 static void *uim_scm_c_int_internal(void *uim_lisp_integer);
 static void *uim_scm_make_int_internal(void *integer);
 static void *uim_scm_c_char_internal(void *uim_lisp_ch);
-static void *uim_scm_make_char_internal(void *ch);
+static void *uim_scm_make_char_internal(intptr_t ch);
 static const char *uim_scm_refer_c_str_internal(void *uim_lisp_str);
 static void *uim_scm_make_str_internal(const char *str);
 static void *uim_scm_make_str_directly_internal(char *str);
@@ -259,12 +259,11 @@ uim_scm_make_char(long ch)
 {
   assert(uim_scm_gc_any_contextp());
 
-  return (uim_lisp)uim_scm_call_with_gc_ready_stack(uim_scm_make_char_internal,
-                                                    (void *)(intptr_t)ch);
+  return (uim_lisp)uim_scm_call_with_gc_ready_stack((uim_gc_gate_func_ptr)uim_scm_make_char_internal, (void *)(intptr_t)ch);
 }
 
 static void *
-uim_scm_make_char_internal(void *ch)
+uim_scm_make_char_internal(intptr_t ch)
 {
   return (void *)SCM_MAKE_CHAR((scm_ichar_t)ch);
 }
@@ -883,6 +882,22 @@ uim_scm_symbolp(uim_lisp obj)
   return (SCM_SYMBOLP((ScmObj)obj));
 }
 
+uim_bool
+uim_scm_ptrp(uim_lisp obj)
+{
+  assert(uim_scm_gc_any_contextp());
+
+  return (SCM_C_POINTERP((ScmObj)obj));
+}
+
+uim_bool
+uim_scm_func_ptrp(uim_lisp obj)
+{
+  assert(uim_scm_gc_any_contextp());
+
+  return (SCM_C_FUNCPOINTERP((ScmObj)obj));
+}
+
 uim_bool
 uim_scm_eq(uim_lisp a, uim_lisp b)
 {
