@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -199,6 +199,8 @@ uim_bool uim_scm_charp(uim_lisp obj);
 uim_bool uim_scm_vectorp(uim_lisp obj);
 uim_bool uim_scm_strp(uim_lisp obj);
 uim_bool uim_scm_symbolp(uim_lisp obj);
+uim_bool uim_scm_ptrp(uim_lisp obj);
+uim_bool uim_scm_func_ptrp(uim_lisp obj);
 uim_bool uim_scm_eq(uim_lisp a, uim_lisp b);
 
 /* constants */
