@@ -1,7 +1,7 @@
 /*
   uim-util.h utility function prototypes for uim.
 
-  Copyright (c) 2004-2007 uim Project http://uim.freedesktop.org/
+  Copyright (c) 2004-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -32,31 +32,62 @@
 
 */
 
-#ifndef _uim_util_h_included_
-#define _uim_util_h_included_
+#ifndef UIM_UTIL_H
+#define UIM_UTIL_H
+
+#include <stdio.h>
+#include <sys/types.h>
+
+#include "uim.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /**
- *  *
- * @param localename locale string. typical format of this string
- * is ll_CC.charset. E.G. for Japanese locale, ja_JP.EUC-JP
- * ll is language code defined in ISO 639, CC is country code defined
- * in ISO 3166.
- * @return language name string. (Now it's untranslated.)
+ * Returns human-readable language name from a locale string.
+ *
+ * @param locale locale string. typical format of this string is
+ * ll_CC.charset. e.g. for Japanese locale, ja_JP.EUC-JP ll is language code
+ * defined in ISO 639-1, CC is country code defined in ISO 3166.
+ *
+ * @return untranslated language name string, or else "-" if no language is
+ * matched. apply gettext() in caller side if needed.
  *
  * @see uim_create_context
  */
 const char *
-uim_get_language_name_from_locale(const char *localename);
+uim_get_language_name_from_locale(const char *locale);
 
+/**
+ * Returns ISO 639-1 language code from a human-readable language name.
+ *
+ * @param language_name a human-readable language name in English such as
+ * "Japanese".
+ *
+ * @return ISO 639-1 language code such as "ja", or else "-" if no language
+ * is matched.
+ */
 const char *
 uim_get_language_code_from_language_name(const char *language_name);
 
-/* uim's iconv_open wrapper */
-void *uim_iconv_open(const char *tocode, const char *fromcode);
+
+/* command execution in pipe-connected subprocess (like popen(3))*/
+pid_t uim_ipc_open_command(pid_t old_pid,
+			   FILE **read_handler, FILE **write_handler,
+			   const char *command);
+pid_t uim_ipc_open_command_with_option(pid_t old_pid,
+				       FILE **read_handler,
+				       FILE **write_handler,
+				       const char *command,
+				       const char *option);
+char *uim_ipc_send_command(pid_t *pid,
+			   FILE **read_handler, FILE **write_handler,
+			   const char *command, const char *str);
+
+/* an uim_code_converter implementation using iconv */
+extern struct uim_code_converter *uim_iconv;
+
 
 #ifdef __cplusplus
 }
