@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -36,19 +36,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#if !UIM_USE_NOTIFY
+#if !UIM_USE_NOTIFY_PLUGINS
 #include <stdarg.h>
 #endif
 
 #include "uim.h"
 #include "uim-internal.h"
 #include "uim-util.h"
+#include "uim-iconv.h"
 #include "uim-posix.h"
 #include "uim-im-switcher.h"
 #include "uim-scm.h"
 #include "uim-scm-abbrev.h"
-#if UIM_USE_NOTIFY
+#if UIM_USE_NOTIFY_PLUGINS
 #include "uim-notify.h"
+#else
+#include "gettext.h"
 #endif
 
 
@@ -79,6 +82,7 @@ void uim_anthy_utf8_plugin_instance_quit(void);
 static uim_bool uim_initialized;
 static uim_lisp protected0, protected1;
 
+unsigned int uim_init_count;
 
 /****************************************************************
  * Core APIs                                                    *
@@ -131,15 +135,16 @@ uim_init_internal(void *dummy)
 
   uim_init_im_subrs();
   uim_init_intl_subrs();
+  uim_init_iconv_subrs();
   uim_init_posix_subrs();
   uim_init_util_subrs();
-#if UIM_USE_NOTIFY
+#if UIM_USE_NOTIFY_PLUGINS
   uim_notify_init();  /* init uim-notify facility */
-  uim_init_notify_subrs();  /* init Scheme interface of uim-notify */
 #endif
+  uim_init_notify_subrs();  /* init Scheme interface of uim-notify */
   uim_init_key_subrs();
   uim_init_rk_subrs();
-  uim_init_plugin();
+  uim_init_dynlib();
 #ifdef ENABLE_ANTHY_STATIC
   uim_anthy_plugin_instance_init();
 #endif
@@ -173,16 +178,18 @@ uim_quit(void)
     return;
   }
 
-  uim_quit_plugin();
 #ifdef ENABLE_ANTHY_STATIC
   uim_anthy_plugin_instance_quit();
 #endif
 #ifdef ENABLE_ANTHY_UTF8_STATIC
   uim_anthy_utf8_plugin_instance_quit();
 #endif
-#if UIM_USE_NOTIFY
+#if UIM_USE_NOTIFY_PLUGINS
   uim_notify_quit();
 #endif
+  uim_scm_callf("annotation-unload", "");
+  uim_scm_callf("dynlib-unload-all", "");
+  uim_quit_dynlib();
   uim_scm_quit();
   uim_initialized = UIM_FALSE;
 }
@@ -718,7 +725,7 @@ uim_get_im_name_for_locale(const char *localename)
   return name;
 }
 
-#if !UIM_USE_NOTIFY
+#if !UIM_USE_NOTIFY_PLUGINS
 uim_bool
 uim_notify_info(const char *msg_fmt, ...)
 {
@@ -748,7 +755,37 @@ uim_notify_fatal(const char *msg_fmt, ...)
 
   return (ret >= 0);
 }
-#endif  /* !UIM_USE_NOTIFY */
+
+static uim_lisp
+notify_get_plugins(void)
+{
+   return CONS(LIST3(MAKE_SYM("builtin"), MAKE_STR(N_("builtin")), MAKE_STR(N_("libuim builtin"))), uim_scm_null());
+}
+
+static uim_lisp
+notify_info(uim_lisp msg_)
+{
+  const char *msg = REFER_C_STR(msg_);
+
+  return MAKE_BOOL(uim_notify_info("%s", dgettext(GETTEXT_PACKAGE, msg)));
+}
+
+static uim_lisp
+notify_fatal(uim_lisp msg_)
+{
+  const char *msg = REFER_C_STR(msg_);
+
+  return MAKE_BOOL(uim_notify_fatal("%s", dgettext(GETTEXT_PACKAGE, msg)));
+}
+
+void
+uim_init_notify_subrs(void)
+{
+  uim_scm_init_proc0("uim-notify-get-plugins", notify_get_plugins);
+  uim_scm_init_proc1("uim-notify-info", notify_info);
+  uim_scm_init_proc1("uim-notify-fatal", notify_fatal);
+}
+#endif  /* !UIM_USE_NOTIFY_PLUGINS */
 
 /****************************************************************
  * Legacy 'mode' API                                            *
