@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2008 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,12 +34,12 @@
 # include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <errno.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstdlib>
 #include <fcntl.h>
+#include <unistd.h>
+#include <sys/types.h>
 
 #include "uim/uim.h"
 #include "uim/uim-util.h"
@@ -53,8 +53,12 @@
 
 #if defined(USE_QT_CANDWIN)
   #define DEFAULT_CANDWIN_PROG	(UIM_LIBEXECDIR "/uim-candwin-qt")
+#elif defined(USE_QT4_CANDWIN)
+  #define DEFAULT_CANDWIN_PROG	(UIM_LIBEXECDIR "/uim-candwin-qt4")
 #elif defined(USE_GTK_CANDWIN) && defined(USE_GTK2)
   #define DEFAULT_CANDWIN_PROG	(UIM_LIBEXECDIR "/uim-candwin-gtk")
+#elif defined(USE_GTK3_CANDWIN)
+  #define DEFAULT_CANDWIN_PROG	(UIM_LIBEXECDIR "/uim-candwin-gtk3")
 #else
   #define DEFAULT_CANDWIN_PROG	NULL
 #endif
@@ -83,7 +87,7 @@ static const char *candwin_command(void)
     if (!user_config)
 	user_config = uim_scm_symbol_value_str("uim-candwin-prog");
 
-    if (user_config) {
+    if (user_config && *user_config) {
 	asprintf(&candwin_prog, UIM_LIBEXECDIR "/%s", user_config);
 	return candwin_prog;
     }
@@ -122,6 +126,26 @@ Canddisp::Canddisp()
 Canddisp::~Canddisp() {
 }
 
+/* XXX: adjust display_limit for uim-candwin-tbl-gtk */
+int Canddisp::adjust_display_limit(uim_context uc, int display_limit)
+{
+    uim_candidate cand;
+    const char *s;
+    int ret = display_limit;
+
+    if (strstr(command, "/uim-candwin-tbl-") == NULL)
+        return display_limit;
+
+    cand = uim_get_candidate(uc, 0, 9999);
+    s = uim_candidate_get_annotation_str(cand);
+#define LEN_DISPLAY_LIMIT 14
+    if (strncmp(s, "display_limit=", LEN_DISPLAY_LIMIT) == 0) {
+      ret = atoi(s + LEN_DISPLAY_LIMIT);
+    }
+    uim_candidate_free(cand);
+    return ret;
+}
+
 void Canddisp::activate(std::vector<const char *> candidates, int display_limit)
 {
     std::vector<const char *>::iterator i;
@@ -129,22 +153,64 @@ void Canddisp::activate(std::vector<const char *> candidates, int display_limit)
     if (!candwin_w)
 	return;
 
-    fprintf(candwin_w, "activate\ncharset=UTF-8\ndisplay_limit=%d\n",
+    fprintf(candwin_w, "activate\fcharset=UTF-8\fdisplay_limit=%d\f",
 		    display_limit);
     for (i = candidates.begin(); i != candidates.end(); ++i)
-	fprintf(candwin_w, "%s\n", *i);
-    fprintf(candwin_w, "\n");
+	fprintf(candwin_w, "%s\f", *i);
+    fprintf(candwin_w, "\f");
     fflush(candwin_w);
     check_connection();
 }
 
+#if UIM_XIM_USE_NEW_PAGE_HANDLING
+void Canddisp::set_nr_candidates(int nr, int display_limit)
+{
+    if (!candwin_w)
+	return;
+
+    fprintf(candwin_w, "set_nr_candidates\f");
+    fprintf(candwin_w, "%d\f", nr);
+    fprintf(candwin_w, "%d\f", display_limit);
+    fprintf(candwin_w, "\f");
+    fflush(candwin_w);
+    check_connection();
+}
+
+void Canddisp::set_page_candidates(int page, CandList candidates)
+{
+    std::vector<const char *>::iterator i;
+
+    if (!candwin_w)
+	return;
+
+    fprintf(candwin_w, "set_page_candidates\fcharset=UTF-8\fpage=%d\f", page);
+    for (i = candidates.begin(); i != candidates.end(); ++i)
+	fprintf(candwin_w, "%s\f", *i);
+    fprintf(candwin_w, "\f");
+    fflush(candwin_w);
+    check_connection();
+}
+
+void Canddisp::show_page(int page)
+{
+    if (!candwin_w)
+	return;
+
+    fprintf(candwin_w, "show_page\f");
+    fprintf(candwin_w, "%d\f", page);
+    fprintf(candwin_w, "\f");
+    fflush(candwin_w);
+    check_connection();
+}
+#endif /* UIM_XIM_USE_NEW_PAGE_HANDLING */
+
 void Canddisp::select(int index, bool need_hilite)
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "select\n");
-    fprintf(candwin_w, "%d\n", index);
-    fprintf(candwin_w, "%d\n\n", need_hilite? 1 : 0);
+    fprintf(candwin_w, "select\f");
+    fprintf(candwin_w, "%d\f", index);
+    fprintf(candwin_w, "%d\f\f", need_hilite? 1 : 0);
     fflush(candwin_w);
     check_connection();
 }
@@ -153,7 +219,7 @@ void Canddisp::deactivate()
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "deactivate\n\n");
+    fprintf(candwin_w, "deactivate\f\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -162,7 +228,7 @@ void Canddisp::show()
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "show\n\n");
+    fprintf(candwin_w, "show\f\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -171,7 +237,7 @@ void Canddisp::hide()
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "hide\n\n");
+    fprintf(candwin_w, "hide\f\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -180,10 +246,10 @@ void Canddisp::move(int x, int y)
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "move\n");
-    fprintf(candwin_w, "%d\n", x);
-    fprintf(candwin_w, "%d\n", y);
-    fprintf(candwin_w, "\n");
+    fprintf(candwin_w, "move\f");
+    fprintf(candwin_w, "%d\f", x);
+    fprintf(candwin_w, "%d\f", y);
+    fprintf(candwin_w, "\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -192,10 +258,10 @@ void Canddisp::show_caret_state(const char *str, int timeout)
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "show_caret_state\n");
-    fprintf(candwin_w, "%d\n", timeout);
-    fprintf(candwin_w, "%s\n", str);
-    fprintf(candwin_w, "\n");
+    fprintf(candwin_w, "show_caret_state\f");
+    fprintf(candwin_w, "%d\f", timeout);
+    fprintf(candwin_w, "%s\f", str);
+    fprintf(candwin_w, "\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -204,7 +270,7 @@ void Canddisp::update_caret_state()
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "update_caret_state\n\n");
+    fprintf(candwin_w, "update_caret_state\f\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -213,7 +279,7 @@ void Canddisp::hide_caret_state()
 {
     if (!candwin_w)
 	return;
-    fprintf(candwin_w, "hide_caret_state\n\n");
+    fprintf(candwin_w, "hide_caret_state\f\f");
     fflush(candwin_w);
     check_connection();
 }
@@ -229,7 +295,7 @@ static void candwin_read_cb(int fd, int /* ev */)
     char buf[1024];
     int n;
 
-    n = read(fd, buf, 1024 - 1);
+    n = static_cast<int>(read(fd, buf, 1024 - 1));
     if (n == 0) {
 	terminate_canddisp_connection();
 	return;
