@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -36,11 +36,11 @@
 # include <config.h>
 #endif
 
-#include <stdio.h>
-#include <string.h>
-#include <locale.h>
+#include <cerrno>
+#include <clocale>
+#include <cstdio>
+#include <cstring>
 #include <iconv.h>
-#include <errno.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include "ximserver.h"
@@ -249,49 +249,20 @@ UTF8_Locale::~UTF8_Locale()
         uim_iconv->release(m_iconv_cd);
 }
 
-char *UTF8_Locale::utf8_to_native_str(char *utf8) {
-    size_t outbufsize = BUFSIZ;
-    char *inbuf, outbuf[BUFSIZ], *convstr = NULL;
-    char *inchar;
-    char *outchar;
-    size_t inbytesleft, outbytesleft;
-    size_t ret_val;
-    
+char *UTF8_Locale::utf8_to_native_str(char *utf8)
+{
+    char *str;
+
     if (m_iconv_cd == (iconv_t)-1)
 	return NULL;
 
-    inbuf = utf8;
-    if (!inbuf)
-	return NULL;
+    str = uim_iconv->convert(m_iconv_cd, utf8);
 
-    // no conversion needed
-    if (!m_iconv_cd)
-	return strdup(inbuf);
-
-    inchar = inbuf;
-    outchar = outbuf;
-    inbytesleft = strlen(inbuf);
-    outbytesleft = outbufsize - 1;
-    ret_val = iconv(m_iconv_cd, (ICONV_CONST char **)&inchar, &inbytesleft, &outchar, &outbytesleft);
-    if (ret_val == (size_t)-1 && errno != E2BIG) {
-	//perror("error in iconv");
-	return NULL;
+    if (strlen(str) == 0) {
+	    free(str);
+	    return NULL;
     }
-    ret_val = iconv(m_iconv_cd, NULL, NULL, &outchar, &outbytesleft);
-    if (ret_val == (size_t)-1 && errno != E2BIG) {
-	//perror("error in iconv");
-	return NULL;
-    }
-
-    convstr = (char *)malloc(outbufsize - outbytesleft);
-    if (!convstr) {
-	return NULL;
-    }
-
-    *outchar = '\0';
-
-    strlcpy(convstr, outbuf, outbufsize - outbytesleft);
-    return convstr;
+    return str;
 }
 
 
@@ -311,7 +282,7 @@ get_valid_locales(const char *locales)
     while ((locale = strsep(&tmpp, ":")) != NULL) {
 	if (setlocale(LC_CTYPE, locale) != NULL) {
 	    asprintf(&validated, "%s:", locale);
-	    len += strlen(validated);
+	    len += static_cast<int>(strlen(validated));
 	    if (valid_locales) {
 		valid_locales = (char *)realloc(valid_locales, len + 1);
 		strcat(valid_locales, validated);
@@ -338,7 +309,7 @@ get_valid_locales(const char *locales)
 
 		    if (setlocale(LC_CTYPE, test_locale) != NULL) {
 			asprintf(&validated, "%s:", locale);
-			len += strlen(validated);
+			len += static_cast<int>(strlen(validated));
 
 			if (valid_locales) {
 			    valid_locales = (char *)realloc(valid_locales, len + 1);
@@ -397,10 +368,10 @@ all_locales(void)
 	free((char *)valid_locales);
 
 	if (locales == NULL) {
-	    len = strlen(tmp);
+	    len = static_cast<int>(strlen(tmp));
 	    locales = strdup(tmp);
 	} else {
-	    len += strlen(tmp);
+	    len += static_cast<int>(strlen(tmp));
 	    locales = (char *)realloc(locales, len + 1);
 	    strcat(locales, tmp);
 	}
