@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -51,7 +51,7 @@ int asprintf(char **ptr, const char *format, ...);
 #endif
 
 #ifndef HAVE_VASPRINTF
-#include <stdarg.h>
+#include <cstdarg>
 int vasprintf(char **ptr, const char *fmt, va_list ap);
 #endif
 
