@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -38,8 +38,8 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/Xos.h>
+#include <cstdio>
 #include <list>
-#include <stdio.h>
 #include "ximserver.h"
 #include "ximpn.h"
 #include "compose.h"
@@ -208,8 +208,8 @@ protected:
     char *mLangRegion;
 
     // for Compose
-    char *get_compose_filename();
-    char *TransFileName(char *name);
+    int get_compose_filename(char *filename, size_t len);
+    int TransFileName(char *transname, const char *name, size_t len);
     void ParseComposeStringFile(FILE *fp);
     void FreeComposeTree(DefTree *top);
     int parse_compose_line(FILE *fp, char **tokenbuf, size_t *buflen);
