@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -34,10 +34,10 @@
 # include <config.h>
 #endif
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <list>
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
 
 #include "xim.h"
 #include "util.h"
@@ -173,10 +173,10 @@ private:
 class TxString : public TxElement {
 public:
     TxString(char *s) {
-	init(s, strlen(s));
+	init(s, static_cast<int>(strlen(s)));
     }
     TxString(char *s, int len) {
-	init(s, len);
+	init(s, static_cast<int>(len));
     }
     virtual ~TxString() {
 	free(m_str);
