@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2011 uim Project http://code.google.com/p/uim/
 
   All rights reserved.
 
@@ -36,9 +36,9 @@
 
 #define UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK 1
 
-#include <stdio.h>
-#include <ctype.h>
-#include <locale.h>
+#include <cctype>
+#include <clocale>
+#include <cstdio>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
@@ -272,7 +272,7 @@ XimServer::strToUstring(uString *d, const char *s)
     int l = 0, nbyte = 0;
     uchar ch;
 
-    len = strlen(s);
+    len = static_cast<int>(strlen(s));
     while (l < len && *s != 0 &&
 	   (nbyte = utf8_mbtowc(&ch, (const unsigned char *)s, len - l)) > 0) {
 	    d->push_back(ch);
@@ -303,8 +303,7 @@ const char *XimServer::getIMLang()
 
 void XimServer::set_im(const char *engine)
 {
-    if (mIMName)
-	free(mIMName);
+    free(mIMName);
 
     mIMName = strdup(engine);
     mIMLang = get_im_lang_from_engine(engine);
@@ -808,6 +807,7 @@ void InputContext::candidate_activate(int nr, int display_limit)
 #if !UIM_XIM_USE_NEW_PAGE_HANDLING
     const char *cand_str;
     const char *heading_label;
+    const char *annotation_str;
     char *str;
 #else
     std::vector<CandList>::iterator slot_it;
@@ -817,6 +817,8 @@ void InputContext::candidate_activate(int nr, int display_limit)
 
     Canddisp *disp = canddisp_singleton();
 
+    if (nr > display_limit)
+        display_limit = disp->adjust_display_limit(mUc, display_limit);
     mDisplayLimit = display_limit;
     if (display_limit)
 	mNumPage = (nr - 1) / display_limit + 1;
@@ -835,15 +837,15 @@ void InputContext::candidate_activate(int nr, int display_limit)
 			display_limit ? i % display_limit : i);
 	cand_str = uim_candidate_get_cand_str(cand);
 	heading_label = uim_candidate_get_heading_label(cand);
-	//annotation_str = uim_candidate_get_annotation(cand);
-	if (cand_str && heading_label) {
-	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + 2);
-	    sprintf(str, "%s\t%s", heading_label, cand_str);
+	annotation_str = uim_candidate_get_annotation(cand);
+	if (cand_str && heading_label && annotation_str) {
+	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + strlen(annotation_str) + 3);
+	    sprintf(str, "%s\a%s\a%s", heading_label, cand_str, annotation_str);
 	    candidates.push_back((const char *)str);
 	}
 	else {
 	    fprintf(stderr, "Warning: cand_str at %d is NULL\n", i);
-	    candidates.push_back((const char *)strdup("\t"));
+	    candidates.push_back((const char *)strdup("\a\a"));
 	}
 	uim_candidate_free(cand);
     }
@@ -901,6 +903,7 @@ void InputContext::prepare_page_candidates(int page)
     int page_nr, start;
     const char *cand_str;
     const char *heading_label;
+    const char *annotation_str;
     char *str;
     CandList candidates;
 
@@ -923,15 +926,15 @@ void InputContext::prepare_page_candidates(int page)
 					(i + start));
 	cand_str = uim_candidate_get_cand_str(cand);
 	heading_label = uim_candidate_get_heading_label(cand);
-	//annotation_str = uim_candidate_get_annotation(cand);
-	if (cand_str && heading_label) {
-	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + 2);
-	    sprintf(str, "%s\t%s", heading_label, cand_str);
+	annotation_str = uim_candidate_get_annotation_str(cand);
+	if (cand_str && heading_label && annotation_str) {
+	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + strlen(annotation_str) + 3);
+	    sprintf(str, "%s\a%s\a%s", heading_label, cand_str, annotation_str);
 	    candidates.push_back((const char *)str);
 	}
 	else {
 	    fprintf(stderr, "Warning: cand_str at %d is NULL\n", i);
-	    candidates.push_back((const char *)strdup("\t"));
+	    candidates.push_back((const char *)strdup("\a\a"));
 	}
 	uim_candidate_free(cand);
     }
@@ -1057,7 +1060,7 @@ char *InputContext::get_caret_state_label_from_prop_list(const char *str)
 	if (p) {
 	    p++;
 	    q = strchr(p, '\t');
-	    len = q - p;
+	    len = static_cast<int>(q - p);
 	    if (q && len < 10) {
 		strlcpy(label, p, len + 1);
 		if (!state_label) {
@@ -1096,18 +1099,31 @@ void InputContext::update_prop_list(const char *str)
     // mismatch during IM switch caused from context-update-widgets.
     uim_bool show_caret_state =
 	uim_scm_symbol_value_bool("bridge-show-input-state?");
-    if (show_caret_state == UIM_TRUE) {
+    char *show_caret_with =
+	uim_scm_c_symbol(uim_scm_symbol_value("bridge-show-with?"));
+    uim_bool show_caret_mode = (strcmp(show_caret_with, "mode") == 0);
+    uim_bool show_caret_mode_on = uim_scm_symbol_value_bool("bridge-show-input-state-mode-on?");
+
+    if (show_caret_state == UIM_TRUE && !(show_caret_mode && !show_caret_mode_on)) {
 	char *label;
 	int timeout;
 	Canddisp *disp = canddisp_singleton();
 
-	timeout =
-	    uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+	if (strcmp(show_caret_with, "time") == 0)
+	    timeout = static_cast<int>(uim_scm_symbol_value_int(
+				    "bridge-show-input-state-time-length"));
+	else
+	    timeout = 0;
+
 	label = get_caret_state_label_from_prop_list(str);
 	disp->show_caret_state(label, timeout);
 	free(label);
 	mCaretStateShown = true;
+    } else if (show_caret_mode && !show_caret_mode_on) {
+	Canddisp *disp = canddisp_singleton();
+	disp->hide_caret_state();
     }
+    free(show_caret_with);
 #endif
 }
 
@@ -1241,18 +1257,19 @@ void keyState::check_key(keyEventX *x)
 	mModifier |= (gMod5Mask & mPreModState);
 
     if (x->key_sym < 128 && x->key_sym >= 32)
-	mKey = x->key_sym;
+	mKey = static_cast<int>(x->key_sym);
     else if (x->key_sym >= XK_F1 && x->key_sym <= XK_F35)
-	mKey = x->key_sym - XK_F1 + UKey_F1;
+	mKey = static_cast<int>(x->key_sym - XK_F1 + UKey_F1);
     // GTK+ and Qt don't support dead_stroke yet
     else if (x->key_sym >= XK_dead_grave && x->key_sym <= XK_dead_horn)
-	mKey = x->key_sym - XK_dead_grave + UKey_Dead_Grave;
+	mKey = static_cast<int>(x->key_sym - XK_dead_grave + UKey_Dead_Grave);
     else if (x->key_sym >= XK_Kanji && x->key_sym <= XK_Eisu_toggle)
-	mKey = x->key_sym - XK_Kanji + UKey_Kanji;
+	mKey = static_cast<int>(x->key_sym - XK_Kanji + UKey_Kanji);
     else if (x->key_sym >= XK_Hangul && x->key_sym <= XK_Hangul_Special)
-	mKey = x->key_sym - XK_Hangul + UKey_Hangul;
+	mKey = static_cast<int>(x->key_sym - XK_Hangul + UKey_Hangul);
     else if (x->key_sym >= XK_kana_fullstop && x->key_sym <= XK_semivoicedsound)
-	mKey = x->key_sym - XK_kana_fullstop + UKey_Kana_Fullstop;
+	mKey = static_cast<int>(
+			x->key_sym - XK_kana_fullstop + UKey_Kana_Fullstop);
     else {
 	switch (x->key_sym) {
 	case XK_yen: mKey = UKey_Yen; break;
@@ -1383,8 +1400,9 @@ void init_modifier_keys() {
     gXNumLockMask = 0;
     XModifierKeymap *map = XGetModifierMapping(XimServer::gDpy);
     XDisplayKeycodes(XimServer::gDpy, &min_keycode, &max_keycode);
-    KeySym *sym = XGetKeyboardMapping(XimServer::gDpy, min_keycode,
-		    (max_keycode - min_keycode + 1), &keysyms_per_keycode);
+    KeySym *sym = XGetKeyboardMapping(XimServer::gDpy,
+                    static_cast<KeyCode>(min_keycode),
+                    (max_keycode - min_keycode + 1), &keysyms_per_keycode);
     for (i = 0; i < 8; i++) {
 	int j;
 	for (j = 0; j < map->max_keypermod; j++) {
